; phat2 - norecess.condense

; IN a = video index
setVideoIndex
	ld ( videoIndex ), a
	ld a, -1
	ld ( frameIndex ), a
    	ret
    
; OUT a = video index
getVideoIndex
	ld a, ( videoIndex )
    	ret
    
; OUT a = frame index
getFrameIndex
	ld a, ( frameIndex )
    	ret

; IN a = video index
; OUT hl = video ptr
getVideoPtr
	ld ( getVideoPtr_videoIndex + 1 ), a
	ld a, &C4
	call pushBank
getVideoPtr_videoIndex
	ld a, 0
	ld c, a
	ld b, 0
	ld hl, videoDataPtr + 1
	add hl, bc
	add hl, bc
	ld a, (hl)
	inc hl
	ld h, (hl)
	ld l, a
	ld ( getVideoPtr_ret + 1 ), hl
	call popBank
getVideoPtr_ret
	ld hl, 0
	ret
    
; OUT a <> 0 if last frame reached
updateFrameIndex
	call getFrameCount
	ld b, a
	ld a, ( frameIndex )
	inc a	
	cp b
	jp z, resetVideoIndex
	ld ( frameIndex ), a
	xor a
	ret
resetVideoIndex
	xor a
	ld ( frameIndex ), a
	inc a
	ret	
    
; OUT a = get video count
getVideoCount
	ld a, &C4
	call pushBank

	ld a, ( videoDataPtr )
	ld ( retVideo + 1 ), a

	call popBank
retVideo
	ld a, 0
	ret

; OUT a <> 0 if overscan
isOverscan
	ld a, &C4
	call pushBank
	call getVideoIndex
	call getVideoPtr
	ld a, ( hl )
	ld ( retIsOverscan + 1 ), a	

	call popBank

retIsOverscan
	ld a, 0
	ret
	    
; OUT a = get frame count in video
getFrameCount
	ld a, &C4
	call pushBank
	call getVideoIndex
	call getVideoPtr
	inc hl
	ld a, (hl)
	ld ( retFrame + 1 ), a	

	call popBank

retFrame
	ld a, 0
	ret	

; OUT HL = ptr
; OUT a = bank
getFrameData
	ld a, &C4
	call pushBank
	call getVideoIndex
	call getVideoPtr

	inc hl ; skip isOverscan
	inc hl ; skip number of frames
	
	ld a, ( frameIndex )
	ld bc, 3
seekToFrame
	or a
	jp z, endSeekToFrame
	add hl, bc
	dec a
	jp seekToFrame
endSeekToFrame

	ld a, (hl)
	ld ( retFrameBank + 1 ), a
	inc hl
	ld a, (hl)
	inc hl
	ld h, (hl)
	ld l, a
	ld ( retFramePtr + 1 ), hl

	call popBank

retFramePtr
	ld hl, 0
retFrameBank
	ld a, 0
	ret

;
drawFrame
	call getScreenPtrOverscan
	ld ( topPtr ), hl
	ld ( bottomPtr ), de

	call getFrameData
	push hl
	call pushBank
	pop hl

	push hl
	ld de, &F800 - 384
	ld bc, &400
	ldir

	call isScreenFlipped
	pop hl

	or a
	jp z, unpackPage2

unpackPage1
	ld a, &C0
	call pushBank

	ld hl, &F800 - 384
	ld de, ( bottomPtr )
	call unpack
    
    	call popBank

	jp endUnpackPage

unpackPage2
	ld de, ( bottomPtr )
	call unpack

endUnpackPage

    	call popBank

	ld ( drawFrameSP + 1 ), sp

startTop
	ld hl, ( topPtr )	
	ld ( innerLoopTop + 1 ), hl
	ld hl, ( bottomPtr )	
	ld a, ( hl )
	inc hl
	ld h, ( hl )
	ld l, a
	ld de, ( bottomPtr )
	add hl, de
	ld ( topCodePtr + 1 ), hl	
	ld a, 7
	
	di
	
innerLoopTop
	ld sp, 0
	ei
	ld hl, endTop
topCodePtr
	jp 0
endTop
	ld hl, ( innerLoopTop + 1 )
	ld bc, - &800
	add hl, bc
	ld ( innerLoopTop + 1 ), hl
	dec a
	jp nz, innerLoopTop

startBottom
	ld a, ( cacheType )
	cp cacheTypeDefault
	jp z, startBottomTypeDefault
	ld a, 7 - cacheTypeBigButSlow
	jp startBottomTypeDone
startBottomTypeDefault
	ld a, 7 - cacheTypeDefault
startBottomTypeDone
	ld hl, ( bottomPtr )	
	ld ( innerLoopBottom + 1 ), hl	
	inc hl
	inc hl
	ld ( bottomCodePtr + 1 ), hl	


innerLoopBottom
	ld sp, 0
	ei
	ld hl, endBottom
bottomCodePtr
	jp 0
endBottom
	ld hl, ( innerLoopBottom + 1 )
	ld bc, - &800
	add hl, bc
	ld ( innerLoopBottom + 1 ), hl
	dec a
	jp nz, innerLoopBottom

drawFrameSP
	ld sp, 0
	ei

	ld a, ( cacheType )
	or a
	jp z, noCopy

	ld de, ( bottomPtr )
	ld h, d
	ld l, e
	ld bc, - &800
	add hl, bc
	ld a, ( cacheType )
	cp cacheTypeDefault
	jp z, drawFrameTypeDefault	
	ld bc, &800 * cacheTypeBigButSlow
	jp drawFrameTypeDone
drawFrameTypeDefault
	ld bc, &800 * cacheTypeDefault
drawFrameTypeDone
	ldir	
noCopy
	ret
	    
;
videoIndex
	defb 0
	
frameIndex
	defb 0

topPtr
	defw 0
   
bottomPtr
	defw 0