
#include <cmath>
#include <string>
#include <vector>
#include <iostream>

#include "SDL.h"

#define XSIZE 512
#define YSIZE 384

#define RGB_Red 0xFF00FFFF
#define RGB_Green 0xFFFFFFFF
#define RGB_Blue 0xFF0080FF
#define RGB_White 0xFF000080
#define RGB_Yellow 0xFF00FF00

int KeyUp = 0;
int KeyDown = 0;
int KeyLeft = 0;
int KeyRight = 0;

SDL_Surface *thescreen;
unsigned int *pixels;

int scrlock()
{
	if(SDL_MUSTLOCK(thescreen))
	{
		if ( SDL_LockSurface(thescreen) < 0 )
		{
			fprintf(stderr, "Couldn't lock display surface: %s\n",
				SDL_GetError());
			return -1;
		}
	}

	pixels = (unsigned int *) thescreen->pixels;

	return 0;
}

void scrunlock(void)
{
	if(SDL_MUSTLOCK(thescreen))
		SDL_UnlockSurface(thescreen);
	SDL_UpdateRect(thescreen, 0, 0, 0, 0);
}

void putPixel( int x, int y, unsigned int color )
{
	if ( x < 0 )
		return;

	if ( y < 0 )
		return;

	if ( x >= XSIZE )
		return;

	if ( y >= YSIZE )
		return;

	pixels[ y * XSIZE + x ] = color; // ARGB 0x00FF0000 = red
}

void drawRect( int x1, int y1, int x2, int y2, unsigned int color )
{
	int x, y;

	for ( y = y1; y < y2; y++ )
	{
		for ( x = x1; x < x2; x++ )
		{
			putPixel( x, y, color );
		}
	}
}

#define mapWidth 24
#define mapHeight 24

unsigned char worldMap[ mapWidth ][ mapHeight ]=
{
  {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,2,2,2,2,2,0,0,0,0,3,0,3,0,3,0,0,0,1},
  {1,0,0,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,2,0,0,0,2,0,0,0,0,3,0,0,0,3,0,0,0,1},
  {1,0,0,0,0,0,2,0,0,0,2,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,2,2,0,2,2,0,0,0,0,3,0,3,0,3,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,4,4,4,4,4,4,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,0,4,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,0,0,0,0,5,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,0,4,0,0,0,0,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,0,4,4,4,4,4,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,4,4,4,4,4,4,4,4,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},
  {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1}
};

float posX = 22.0f;
float posY = 12.0f;
float dirX = -1.0f;
float dirY = 0.0f;
float planeX = 0.0f, planeY = 0.66f;

float time = 0.0f;
float oldTime = 0.0f;

float sideDistX;
float sideDistY;
int stepX;
int stepY;

void playWolf()
{
    // CLS
    drawRect( 0, 0, XSIZE, YSIZE, 0xFF00FF00 );

	int w = XSIZE;
	int h = YSIZE;

    for( int x = 0; x < w; x++ )
    {
		float cameraX = 2 * x / float( w ) - 1;
		float rayPosX = posX;
		float rayPosY = posY;
		float rayDirX = dirX + planeX * cameraX;
		float rayDirY = dirY + planeY * cameraX;

		int mapX = int( rayPosX );
		int mapY = int( rayPosY );
       
		float deltaDistX = sqrt( 1 + ( rayDirY * rayDirY ) / ( rayDirX * rayDirX ) );
		float deltaDistY = sqrt( 1 + ( rayDirX * rayDirX ) / ( rayDirY * rayDirY ) );
		float perpWallDist;
       
	
		int hit = 0;
		int side;
     
		if ( rayDirX < 0 )
		{
			stepX = -1;
			sideDistX = ( rayPosX - mapX ) * deltaDistX;
		}
		else
		{
			stepX = 1;
			sideDistX = ( mapX + 1.0f - rayPosX ) * deltaDistX;
		}
		if ( rayDirY < 0 )
		{
			stepY = -1;
			sideDistY = ( rayPosY - mapY ) * deltaDistY;
		}
		else
		{
			stepY = 1;
			sideDistY = ( mapY + 1.0f - rayPosY ) * deltaDistY;
		}
      
		while ( hit == 0 )
		{
			if ( sideDistX < sideDistY )
			{
				sideDistX += deltaDistX;
				mapX += stepX;
				side = 0;
			}
			else
			{
				sideDistY += deltaDistY;
				mapY += stepY;
				side = 1;
			}
		
			if ( worldMap[ mapX ][ mapY ] > 0 )
			{
				hit = 1;
			}
		} 
		
		if ( side == 0 )
		{
			perpWallDist = fabs( ( mapX - rayPosX + ( 1 - stepX ) / 2 ) / rayDirX );
		}		
		else
		{
			perpWallDist = fabs( ( mapY - rayPosY + ( 1 - stepY ) / 2 ) / rayDirY );
		}
		
		int lineHeight = abs( int( h / perpWallDist ) );

		int drawStart = -lineHeight / 2 + h / 2;
		if( drawStart < 0 )
		{
			drawStart = 0;
		}
		int drawEnd = lineHeight / 2 + h / 2;
		if( drawEnd >= h )
		{
			drawEnd = h - 1;
		}
        
		unsigned int color;
		switch( worldMap[ mapX ][ mapY ] )
		{
			case 1:  color = RGB_Red;  break;
			case 2:  color = RGB_Green;  break;
			case 3:  color = RGB_Blue;   break;
			case 4:  color = RGB_White;  break;
			default: color = RGB_Yellow; break;
		}
       
		if ( side == 1 )
		{
			color = color / 2;
		}

		drawRect( x, drawStart, x + 1, drawEnd, color );
    }
    
	oldTime = time;
    time = (float) SDL_GetTicks( );
    float frameTime = ( time - oldTime ) / 1000.0f;
    
    float moveSpeed = frameTime * 5.0f;
    float rotSpeed = frameTime * 3.0f;
    
    if ( KeyUp )
    {
		if ( worldMap[ int( posX + dirX * moveSpeed ) ][ int( posY ) ] == false )
		{
			posX += dirX * moveSpeed;
		}
		if ( worldMap[ int( posX ) ][ int( posY + dirY * moveSpeed ) ] == false )
		{
			posY += dirY * moveSpeed;
		}
    }

    if (KeyDown)
    {
      if ( worldMap[ int( posX - dirX * moveSpeed ) ][ int( posY ) ] == false )
	  {
		  posX -= dirX * moveSpeed;
	  }
      if ( worldMap[ int( posX ) ][ int( posY - dirY * moveSpeed ) ] == false )
	  {
		  posY -= dirY * moveSpeed;
	  }
    }
    
	if ( KeyRight )
    {
		float oldDirX = dirX;
		dirX = dirX * cos( -rotSpeed ) - dirY * sin( -rotSpeed );
		dirY = oldDirX * sin( -rotSpeed ) + dirY * cos( -rotSpeed );
		float oldPlaneX = planeX;
		planeX = planeX * cos( -rotSpeed ) - planeY * sin( -rotSpeed );
		planeY = oldPlaneX * sin( -rotSpeed ) + planeY * cos( -rotSpeed );
	}

	if ( KeyLeft )
	{
		float oldDirX = dirX;
		dirX = dirX * cos( rotSpeed ) - dirY * sin( rotSpeed );
		dirY = oldDirX * sin( rotSpeed ) + dirY * cos( rotSpeed );
		float oldPlaneX = planeX;
		planeX = planeX * cos( rotSpeed ) - planeY * sin( rotSpeed );
		planeY = oldPlaneX * sin( rotSpeed ) + planeY * cos( rotSpeed );
	}
}


int main(int argc, char *argv[])
{
	std::cout << "Wolf version 1.0" << std::endl;

	if ( argc < 2 )
	{
		std::cerr << "Not enough parameters !" << std::endl << std::endl;

		std::cout << "Usage:" << std::endl;
		std::cout << "  ARG1    = filename to generate ex. phat2.wolf.bin" << std::endl;
		
		std::cout << std::endl;

		std::cout << "This will generate phat2.wolf.bin" << std::endl;
		std::cout << "Output format:    [OFFSET:TYPE:COMMENT]" << std::endl;
		std::cout << "  0:unsigned short:number of frames" << std::endl;
		std::cout << "  1:unsigned short:ptr video 0" << std::endl;
		std::cout << "  1..N:unsigned short:ptr video 1 .. N etc" << std::endl;
		std::cout << " >ptr video 0:" << std::endl;
		std::cout << "  0:unsigned char:number of frames in video" << std::endl;
		std::cout << "  1:unsigned char:bank frame 0 = value between 0xC0 and 0xC7" << std::endl;
		std::cout << "  2:unsigned short:src ptr (packed buffer) for frame 0" << std::endl;

		std::cout << std::endl;

		return 1;
	}

	int done = 0;
	SDL_Event event;

	if ( SDL_Init(SDL_INIT_VIDEO) < 0 )
	{
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}
	
	thescreen = SDL_SetVideoMode(XSIZE, YSIZE, 32, SDL_SWSURFACE|SDL_HWPALETTE);
	if ( thescreen == NULL )
	{
		fprintf(stderr, "Couldn't set display mode: %s\n",
			SDL_GetError());
		SDL_Quit();
		exit(5);
	}

	while(!done)
	{
		scrlock( );

		playWolf( );

		scrunlock( );
		
		while(SDL_PollEvent(&event))
		{
			switch (event.type)
			{
            case SDL_QUIT:
				done = 1;
				break;
			case SDL_KEYDOWN:
				if( event.key.keysym.sym == SDLK_UP ) KeyUp = 1;
				if( event.key.keysym.sym == SDLK_DOWN ) KeyDown = 1;
				if( event.key.keysym.sym == SDLK_LEFT ) KeyLeft = 1;
				if( event.key.keysym.sym == SDLK_RIGHT ) KeyRight = 1;
				break;

			case SDL_KEYUP:
				if( event.key.keysym.sym == SDLK_UP ) KeyUp = 0;
				if( event.key.keysym.sym == SDLK_DOWN ) KeyDown = 0;
				if( event.key.keysym.sym == SDLK_LEFT ) KeyLeft = 0;
				if( event.key.keysym.sym == SDLK_RIGHT ) KeyRight = 0;
				break;

			default:
				break;
			}
		}
	}

	SDL_Quit();
	exit(0);

	return 0;
}
