// delAmsHeader.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <iostream>
#include <vector>
#include <string>
#include <windows.h>

#define AMSDOS_HEADER_SIZE 128

#define VIDEODATAPTR	0x56D0

struct SVideoFrame
{
	unsigned char bank;
	unsigned short ptr;
	
	// internal
	unsigned char *data;
	unsigned short dataSize;
};

struct SVideo
{
	unsigned short ptrInBank;

	unsigned char isOverscan;
	std::vector< SVideoFrame > videoFrames;
};

struct SVideoCollection
{
	// unsigned char nbVideos;
	std::vector< SVideo > videos;
};

std::string filename;

std::string GetFileNameBIN( unsigned char bank )
{
	if ( bank == 0xc0 ) return filename+".c0.bin";
	if ( bank == 0xc4 ) return filename+".c4.bin";
	if ( bank == 0xc5 ) return filename+".c5.bin";
	if ( bank == 0xc6 ) return filename+".c6.bin";
	if ( bank == 0xc7 ) return filename+".c7.bin";

	return "";
}

std::string GetFileNameASM( unsigned char bank )
{
	if ( bank == 0xc0 ) return filename+".c0.asm";
	if ( bank == 0xc4 ) return filename+".c4.asm";
	if ( bank == 0xc5 ) return filename+".c5.asm";
	if ( bank == 0xc6 ) return filename+".c6.asm";
	if ( bank == 0xc7 ) return filename+".c7.asm";

	return "";
}

int main( int argc, char *argv[ ] )
{
	if ( argc < 3 )
	{
		std::cout << "datagen version 1.0 -- jfyeNG data bank maker  (crocolib)" << std::endl;

		std::cerr << "Not enough parameters !" << std::endl << std::endl;

		std::cout << "Usage:" << std::endl;
		std::cout << "  ARG1    = base filename for files to generate ex. phat2.video" << std::endl;
		std::cout << "  ARG2..N = source filename" << std::endl;
		
		std::cout << std::endl;
        
        std::cout << "This will generate phat2.video.c4.asm .. phat2.video.c7.asm  (with intermediate BIN files)" << std::endl;
        std::cout << "Output format:    [OFFSET:TYPE:COMMENT]" << std::endl;
        std::cout << " >from here, always bank 0xC4 !" << std::endl;
        std::cout << "  0:unsigned char:number of videos" << std::endl;
        std::cout << "  1:unsigned short:ptr video 0" << std::endl;
        std::cout << "  1..N:unsigned short:ptr video 1 .. N etc" << std::endl;
        std::cout << " >ptr video 0:" << std::endl;
        std::cout << "  0:unsigned char:number of frames in video" << std::endl;
        std::cout << "  1:unsigned char:bank frame 0 = value between 0xC0 and 0xC7" << std::endl;
        std::cout << "  2:unsigned short:src ptr (packed buffer) for frame 0" << std::endl;
        
        std::cout << std::endl;
		
		return 1;
	}

	filename = argv[ 1 ];


	// GET VIDEOS FROM DISC
	SVideoCollection videoCollection;

	std::vector< std::string > filenames;

	for ( int iFile = 2; iFile < argc; ++iFile )
	{
		char *filename = argv[ iFile ];

		FILE *intHandle = fopen( filename, "rb" );
		if ( !intHandle )
		{
			std::cerr << "Can't open " << filename << " !" << std::endl;
			return 1;
		}

		SVideo video;

		fread( &video.isOverscan, 1, 1, intHandle );

		unsigned char nbFrames;
		fread( &nbFrames, 1, 1, intHandle );

		for ( int iFrame = 0; iFrame < nbFrames; iFrame++ )
		{
			SVideoFrame videoFrame;
			
			fread( &videoFrame.dataSize, sizeof( unsigned short), 1, intHandle );
			videoFrame.data = new unsigned char[ videoFrame.dataSize ];

			fread( videoFrame.data, videoFrame.dataSize, 1, intHandle );

			video.videoFrames.push_back( videoFrame );
		}

		fclose( intHandle );

		videoCollection.videos.push_back( video );
	}

	
	// GET MAIN HEADER SIZE
	unsigned short headerSize = 0;
	
	headerSize += 1 /*videoCount*/ + sizeof(unsigned short)*videoCollection.videos.size() /*1..N:unsigned short:ptr video 1 .. N etc*/;

	for ( int iVideo = 0; iVideo < videoCollection.videos.size(); ++iVideo )
	{
		SVideo &video = videoCollection.videos[ iVideo ];
		video.ptrInBank = VIDEODATAPTR + headerSize;

		headerSize += 1 /*isOverscan*/ + 1 /*frameCount*/ + ( 1 /*bank frame*/ + sizeof(unsigned short)/*frame ptr*/ ) * video.videoFrames.size();
	}
	

	// SET BANK/ADRESS VALUES
	unsigned char bank = 0xC4;
	unsigned short ptr = VIDEODATAPTR + headerSize;

	for ( int iVideo = 0; iVideo < videoCollection.videos.size(); ++iVideo )
	{
		SVideo &video = videoCollection.videos[ iVideo ];

		for ( int iFrame = 0; iFrame < video.videoFrames.size(); ++iFrame )
		{
			SVideoFrame &videoFrame = video.videoFrames[ iFrame ];

			if ( ( ptr + videoFrame.dataSize ) >= 0x8000 )
			{
				ptr = 0x4000;
				bank++;
			}
			
			videoFrame.bank = bank;
			videoFrame.ptr = ptr;

			ptr += videoFrame.dataSize;
		}
	}


	// WRITE FINAL BINARY FILE
	unsigned char currBank = 0xc4;
	FILE *outHandle = fopen( GetFileNameBIN( currBank ).c_str(), "wb" );
	unsigned char nbVideos = videoCollection.videos.size();
	fwrite( &nbVideos, 1, 1, outHandle );
		
	for ( int iVideo = 0; iVideo < videoCollection.videos.size(); ++iVideo )
	{
		SVideo &video = videoCollection.videos[ iVideo ];

		fwrite( &video.ptrInBank, sizeof( unsigned short ), 1, outHandle );
	}

	for ( int iVideo = 0; iVideo < videoCollection.videos.size(); ++iVideo )
	{
		SVideo &video = videoCollection.videos[ iVideo ];

		fwrite( &video.isOverscan, 1, 1, outHandle );

		unsigned char frameCount = video.videoFrames.size();
		fwrite( &frameCount, 1, 1, outHandle );

		for ( int iFrame = 0; iFrame < video.videoFrames.size(); ++iFrame )
		{
			SVideoFrame &videoFrame = video.videoFrames[ iFrame ];

			fwrite( &videoFrame.bank, 1, 1, outHandle );
			fwrite( &videoFrame.ptr, sizeof( unsigned short ), 1, outHandle );
		}
	}

	for ( int iVideo = 0; iVideo < videoCollection.videos.size(); ++iVideo )
	{
		SVideo &video = videoCollection.videos[ iVideo ];

		for ( int iFrame = 0; iFrame < video.videoFrames.size(); ++iFrame )
		{
			SVideoFrame &videoFrame = video.videoFrames[ iFrame ];

			if ( currBank != videoFrame.bank )
			{
				currBank = videoFrame.bank;

				fclose( outHandle );
				outHandle = fopen( GetFileNameBIN( currBank ).c_str(), "wb" );
			}

			fwrite( videoFrame.data, videoFrame.dataSize, 1, outHandle );
		}
	}
	
	fclose( outHandle );

	currBank++;
	while ( currBank <= 0xC7 )
	{
		remove( GetFileNameBIN( currBank ).c_str() );

		currBank++;
	}


	// CONVERT BINARY FILES TO ASM
	currBank = 0xc4;
	FILE *binHandle;
	while ( ( binHandle = fopen( GetFileNameBIN( currBank ).c_str(), "rb" ) ) )
	{
		fseek( binHandle, 0, SEEK_END );
		int binSize = ftell( binHandle );
		fseek( binHandle, 0, SEEK_SET );

		char *binData = new char[ binSize ];
		fread( binData, binSize, 1, binHandle );

		fclose( binHandle );

		FILE *asmHandle = fopen( GetFileNameASM( currBank ).c_str(), "wb" );

		unsigned short binOffset = 0;
		unsigned char lineOffset = 0;
		while ( binOffset < binSize )
		{
			if ( lineOffset == 0 )
			{
				fprintf( asmHandle, "    defb " );
			}

			unsigned char value = binData[ binOffset ];
			fprintf( asmHandle, "&%02X", value );

			binOffset++;
			
			lineOffset++;
			if ( lineOffset == 16 )
			{
				fprintf( asmHandle, "\n" );

				lineOffset = 0;
			}
			else if ( binOffset < binSize )
			{
				fprintf( asmHandle, ", " );
			}
		}

		fprintf( asmHandle, "\n" );

		fclose( asmHandle );

		delete [] binData;

		currBank++;
	}

	while ( currBank <= 0xC7 )
	{
		FILE *asmHandle = fopen( GetFileNameASM( currBank ).c_str(), "wb" );

		fprintf( asmHandle, "\n" );
		
		fclose( asmHandle );

		currBank++;
	}

	
	std::cout << "datagen - OK.";

	return 0;
}

