/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwo.c - LWO Object                                                         *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "system/cmdline.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoparse.h"
#include "lightwave/lwo/lwoinit.h"

/*---------------------------------------------------------------------------- GLOBAL VARIABLES */
File *lwoFile = NULL;
Lwo *lwoCurr = NULL;
UINT8 *lwoPtr = NULL;
INT32 chunkID = 0;
INT32 chunkSize = 0;
INT32 subChunkID = 0;
INT32 subChunkSize = 0;
INT32 subSubChunkID = 0;
INT32 subSubChunkSize = 0;

/*---------------------------------------------------------------------------- LOAD */
RESULT 
LwoLoad(Lwo *lwo, INT8 *fileName, const FLOAT32 sizeCoef, void (*progress)(INT32 i))
{
    RESULT res = OK;

    if (lwo==NULL || fileName==NULL)
    {
        return ERR_INVALID_PTR;
    }
    
    strcpy(lwo->fileName, fileName);
    
	lwo->sizeCoef = sizeCoef;
    lwo->offsetUV = 0.0f;
    lwo->hasNormals = 0;
    lwo->dissolveAmount = 100.0f;

    lwo->endoMorphCount = 0;
    memset(lwo->endoMorph, 0, sizeof(EndoMorph)*LWO_MAX_ENDOMORPH);
    
    lwo->txuvCount = 0;
    memset((void*)&lwo->txuvCount, 0, sizeof(TexUV)*LWO_MAX_TXUV);
    
    lwoFile = (File *) MemAlloc(sizeof(File));

    res = FileLoad(lwoFile, fileName);

    if (res!=OK)
    {
        ConsoleAdd("Error while loading %s !", fileName);
        
        return ERR_LWO_LOAD;
    }

    lwoCurr = lwo;

    res = LwoParse(progress);
    if (res!=OK)
    {
        FileFree(lwoFile);

        return ERR;
    }

	res = LwoInit();
    if (res!=OK)
    {
        FileFree(lwoFile);

        return ERR;
    }

    FileFree(lwoFile);

    MemFree(lwoFile);

	return OK;
}

/*---------------------------------------------------------------------------- FREE */
RESULT
LwoFree(Lwo *lwo)
{
    INT32 i = 0;
    
    if (lwo==NULL)
    {
        return ERR_INVALID_PTR;
    }

    for (i=0; i<LWO_MAX_ENDOMORPH; i++)
    {
        if (lwo->endoMorph[i].vertTable!=NULL)
        {
            SAFE_FREE(lwo->endoMorph[i].vertTable);
        }
    }

    for (i=0; i<LWO_MAX_TXUV; i++)
    {
        if (lwo->txuv[i].uvTable!=NULL)
        {
            SAFE_FREE(lwo->txuv[i].uvTable);
        }        
        if (lwo->txuv[i].uvProjTable!=NULL)
        {
            SAFE_FREE(lwo->txuv[i].uvProjTable);
        }
        if (lwo->txuv[i].uvProjAddTable!=NULL)
        {
            SAFE_FREE(lwo->txuv[i].uvProjAddTable);
        }
    }

    for (i=0; i<lwo->surfCount; i++)
    {
        SAFE_FREE(lwo->surfTable[i].triTable);
        SAFE_FREE(lwo->surfTable[i].uvTable);
        SAFE_FREE(lwo->surfTable[i].uvVertProjTable);
        SAFE_FREE(lwo->surfTable[i].uvProjTable);
    }

    SAFE_FREE(lwo->surfTable);
    SAFE_FREE(lwo->vertTable);
    SAFE_FREE(lwo->vertBaseTable);
    SAFE_FREE(lwo->normTable);
    SAFE_FREE(lwo->normBaseTable);
    SAFE_FREE(lwo->polyTable);
    SAFE_FREE(lwo->triExTable);

    return OK;
}

/*---------------------------------------------------------------------------- ACCESORS */
Lwo *
LwoGetCurrent(void)
{
    return lwoCurr;
}

File *
LwoGetCurrentFile(void)
{
    return lwoFile;
}