/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/******************************************************************************
 * lwo.h - LWO Object                                                         *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#ifndef _LWO_
#define _LWO_

/*---------------------------------------------------------------------------- DEFINES */
#define LWO_MAX_ENDOMORPH   64
#define LWO_MAX_TXUV        32

/*---------------------------------------------------------------------------- ENUM */
typedef enum
{
    MappingUndef        = 0,
    MappingPlanar       = 1,
    MappingCylindrical  = 2,
    MappingSpherical    = 3,
    MappingCubic        = 4,
    MappingFront        = 5,
    MappingUV           = 6,
    MappingEnvmap       = 7
} MappingType;

typedef enum
{
    AxisX               = 0,
    AxisY               = 1,
    AxisZ               = 2
} AxisType;

typedef enum
{
    FirstPass           = 0,
    SecondPass          = 1
} Pass;

/*---------------------------------------------------------------------------- STRUCT */
typedef struct RenderState
{
    BOOL gouraud;
    BOOL doubleSided;
    BOOL mapping;
    BOOL additive;
    BOOL envmap;
} RenderState;

typedef struct Norm
{
    FLOAT32 x;
    FLOAT32 y;
    FLOAT32 z;
} Norm;

typedef struct Vert
{
    FLOAT32 x;
    FLOAT32 y;
    FLOAT32 z;
#ifdef VERT_ALIGNEMENT
	FLOAT32 _;
#endif
} Vert;

typedef struct Poly
{
    INT32 surfIndex;

    INT32 triOffset;
    INT32 triCount;
} Poly;

typedef struct Tri
{
    INT32 vert1Index;
    INT32 vert2Index;
    INT32 vert3Index;
} Tri;

typedef struct TriEx
{
    INT32 polyIndex;

    INT32 vert1Index;
    INT32 vert2Index;
    INT32 vert3Index;

    Norm norm;
} TriEx;

typedef struct UV
{
    FLOAT32 u;
    FLOAT32 v;
} UV;

typedef struct UVProj
{
    INT32 vertIndex;
    INT32 polyIndex; 

    FLOAT32 u;
    FLOAT32 v;
} UVProj;

typedef struct Trans
{
    FLOAT32 transX;
    FLOAT32 transY;
    FLOAT32 transZ;

    FLOAT32 rotX;
    FLOAT32 rotY;
    FLOAT32 rotZ;

    FLOAT32 stretchX;
    FLOAT32 stretchY;
    FLOAT32 stretchZ;

    FLOAT32 pivotX;
    FLOAT32 pivotY;
    FLOAT32 pivotZ;
} Trans;

typedef struct BBox
{
    FLOAT32 minX;
    FLOAT32 minY;
    FLOAT32 minZ;

    FLOAT32 maxX;
    FLOAT32 maxY;
    FLOAT32 maxZ;
} BBox;

typedef struct EndoMorph
{
    INT8 name[STR_DEFAULT_SIZE];
    float amount;
    Vert *vertTable;
    Norm *normTable;
} EndoMorph;

typedef struct TexUV
{
    INT8 name[STR_DEFAULT_SIZE];

    INT32 vertBaseCount;
    INT32 vertCount;
    Vert *vertTable;
    Vert *vertBaseTable;
    
    UVProj *uvProjTable;

    INT32 uvProjAddCount;
    UVProj *uvProjAddTable;

    INT32 uvCount;
    UV *uvTable;
} TexUV;

typedef struct Surf
{
    INT8 name[STR_DEFAULT_SIZE];

    UINT8 r;
    UINT8 g;
    UINT8 b;

    MappingType mappingType;
    AxisType axisType;
    RenderState renderState;
    BBox bBox;
    
    Tex *tex0;
    Tex *tex1;

    INT32 triCount;
    Tri *triTable;

    INT32 uvCount;
    UV *uvTable;

    INT8 txuvName[STR_DEFAULT_SIZE];
    TexUV *txuv;

    INT32 uvProjCount;
    Vert *uvVertProjTable;
    UV *uvProjTable;

	FLOAT32 sizeX;
	FLOAT32 sizeY;
	FLOAT32 sizeZ;

	FLOAT32 centerX;
	FLOAT32 centerY;
	FLOAT32 centerZ;

	BOOL precaIsSkipped;
	INT32 precaFirstCount;
	INT32 precaLastCount;
	BOOL precaSkip2DClipping;
} Surf;

typedef struct Lwo
{
    INT8 fileName[STR_DEFAULT_SIZE];

    INT32 vertCount;
    Vert *vertTable;
    Vert *vertBaseTable;

    INT32 endoMorphCount;
    EndoMorph endoMorph[LWO_MAX_ENDOMORPH];

    INT32 txuvCount;
    TexUV txuv[LWO_MAX_TXUV];

    BOOL hasNormals;
	INT32 normCount;
    Norm *normTable;
    Norm *normBaseTable;

    INT32 surfCount;
    Surf *surfTable;

    INT32 texCount;
    Tex *texTable;
    
    INT32 polyCount;
    Poly *polyTable;

    INT32 triExCount;
    TriEx *triExTable;

    INT32 applyTrans;
    Trans trans;

    BBox bBox;

    BOOL isCullFace;
    BOOL isDisp;
    BOOL isNull;

	FLOAT32 sizeCoef;
	FLOAT32 offsetUV;
    FLOAT32 dissolveAmount;

    INT32 statSurfTotalCount;
	INT32 statTexTotalCount;
	
	INT32 stripCount;
	INT32 *stripSize;
    INT32 **strip;

	BOOL usePreca;
	INT32 precaFirstCount;
	INT32 precaLastCount;
} Lwo;

/*---------------------------------------------------------------------------- GLOBAL FUNCTIONS */
RESULT LwoLoad(Lwo *lwo, INT8 *fileName, const FLOAT32 sizeCoef, void (*progress)(INT32 i));
RESULT LwoFree(Lwo *lwo);

Lwo *LwoGetCurrent(void);
File *LwoGetCurrentFile(void);

/*---------------------------------------------------------------------------- END OF DECLARATION */
#endif /* _LWO_ */