/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoinit.c - LWO Object Initializer                                         *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "system/file.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lwo/lwoinitnorm.h"
#include "lightwave/lwo/lwoinittri.h"
#include "lightwave/lwo/lwoinituv.h"
#include "lightwave/lwo/lwomorph.h"

/*---------------------------------------------------------------------------- GLOBAL VARIABLES */
INT32 initSurf = 0;
Lwo *initLwo = NULL;

/*---------------------------------------------------------------------------- INIT */
RESULT
LwoInit(void)
{
    initLwo = LwoGetCurrent();
    
    initLwo->isCullFace = TRUE;
	initLwo->trans.stretchX = 1.0f;
    initLwo->trans.stretchY = 1.0f;
    initLwo->trans.stretchZ = 1.0f;
    initLwo->normCount = initLwo->vertCount;
    initLwo->normTable = MemAlloc(sizeof(Norm)*initLwo->normCount);

	initLwo->hasNormals = FALSE;

    for (initSurf=0; initSurf<initLwo->surfCount; initSurf++)
    {
		if (initLwo->surfTable[initSurf].renderState.doubleSided || initLwo->surfTable[initSurf].renderState.additive)
		{
			initLwo->isCullFace = FALSE;
		}

		if (initLwo->surfTable[initSurf].renderState.gouraud || initLwo->surfTable[initSurf].renderState.envmap || initLwo->surfTable[initSurf].mappingType==MappingCubic)
		{
			initLwo->hasNormals = TRUE;
		}
	}
	
	if (initLwo->hasNormals)
	{
		LwoObjBuildNormals();
	}

    for (initSurf=0; initSurf<initLwo->surfCount; initSurf++)
    {
		LwoSurfAllocTriTable();
        
        initLwo->surfTable[initSurf].uvCount = initLwo->vertCount;
        initLwo->surfTable[initSurf].uvTable = MemAlloc(sizeof(UV)*initLwo->surfTable[initSurf].uvCount);

        LwoSurfBuildTriTable();
        
        if (initLwo->surfTable[initSurf].tex0!=NULL)
        {
            LwoSurfBuildUV();
        }
    }

    LwoMorphAlloc();

    if (initLwo->endoMorphCount>0)
    {
        initLwo->isCullFace = FALSE;
    }

	return OK;
}

/*---------------------------------------------------------------------------- SET DISSOLVE */
RESULT
LwoSetDissolve(Lwo *src, const FLOAT32 amount)
{
    if (src==NULL)
    {
        return ERR_INVALID_PTR;
    }

    src->dissolveAmount = amount;

    return OK;
}
