/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoinitnorm.c - LWO Object Normals Initializer                             *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/math.h"
#include "system/file.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lwo/lwoinitnorm.h"
#include "platform/renderer/lwodraw.h"

/*---------------------------------------------------------------------------- NORMALS COMPUTATION */
void 
LwoObjBuildNormals(void)
{
    INT32 norm = 0;
    INT32 vert1Index = 0;
    INT32 vert2Index = 0;
    INT32 vert3Index = 0;
    INT32 *normAdd = NULL;
    FLOAT32 len = 0.0f;
    Norm v1;
    Norm v2;
    Norm out;
    Norm *normVert1 = NULL;
    Norm *normVert2 = NULL;
    Norm *normVert3 = NULL;
    Vert vert1;
    Vert vert2;
    Vert vert3;
    INT32 i = 0;
    INT32 tri = 0;

    normAdd = MemAlloc(initLwo->normCount*sizeof(INT32));

    for (norm=0; norm<initLwo->normCount; norm++)
    {
        normAdd[norm] = 0;

        initLwo->normTable[norm].x = 0.0f;
        initLwo->normTable[norm].y = 0.0f;
        initLwo->normTable[norm].z = 0.0f;
    }

    for (tri=0; tri<initLwo->triExCount; tri++)
    {
        vert1Index = initLwo->triExTable[tri].vert1Index;
        vert2Index = initLwo->triExTable[tri].vert2Index;
        vert3Index = initLwo->triExTable[tri].vert3Index;         
        
        vert1 = initLwo->vertTable[vert1Index];
        vert2 = initLwo->vertTable[vert2Index];
        vert3 = initLwo->vertTable[vert3Index];

        v1.x = vert2.x-vert1.x;
        v1.y = vert2.y-vert1.y;
        v1.z = vert2.z-vert1.z;
        v2.x = vert3.x-vert1.x;
        v2.y = vert3.y-vert1.y;
        v2.z = vert3.z-vert1.z;

        out.x = (v1.y*v2.z)-(v1.z*v2.y);
        out.y = (v1.z*v2.x)-(v1.x*v2.z);
        out.z = (v1.x*v2.y)-(v1.y*v2.x);

		len = (FLOAT32) MathSqrt((out.x*out.x)+(out.y*out.y)+(out.z*out.z));

        if (len==0.0f)
        {
            len = 1.0f;
        }
        
		len *= LwoTransfNormCoef(initLwo->sizeCoef);
		
        out.x /= len;
        out.y /= len;
        out.z /= len;

		initLwo->triExTable[tri].norm = out;
        
        normVert1 = &initLwo->normTable[vert1Index];
        normVert1->x += out.x;
        normVert1->y += out.y;
        normVert1->z += out.z;
        normAdd[vert1Index]++;

        normVert2 = &initLwo->normTable[vert2Index];
        normVert2->x += out.x;
        normVert2->y += out.y;
        normVert2->z += out.z;
        normAdd[vert2Index]++;

        normVert3 = &initLwo->normTable[vert3Index];
        normVert3->x += out.x;
        normVert3->y += out.y;
        normVert3->z += out.z;
        normAdd[vert3Index]++;
    }

    for (norm=0; norm<initLwo->normCount; norm++)
    {
        initLwo->normTable[norm].x /= normAdd[norm];
        initLwo->normTable[norm].y /= normAdd[norm];
        initLwo->normTable[norm].z /= normAdd[norm];
    }

    for (i=0; i<initLwo->endoMorphCount; i++)
    {
        initLwo->endoMorph[i].normTable = MemAlloc(initLwo->normCount*sizeof(Norm));
            
        for (norm=0; norm<initLwo->normCount; norm++)
        {
            normAdd[norm] = 0;

            initLwo->endoMorph[i].normTable[norm].x = 0.0f;
            initLwo->endoMorph[i].normTable[norm].y = 0.0f;
            initLwo->endoMorph[i].normTable[norm].z = 0.0f;
        }

        for (tri=0; tri<initLwo->triExCount; tri++)
        {
            vert1Index = initLwo->triExTable[tri].vert1Index;
            vert2Index = initLwo->triExTable[tri].vert2Index;
            vert3Index = initLwo->triExTable[tri].vert3Index;         
            
            vert1 = initLwo->vertTable[vert1Index];
            vert2 = initLwo->vertTable[vert2Index];
            vert3 = initLwo->vertTable[vert3Index];

            vert1.x += initLwo->endoMorph[i].vertTable[vert1Index].x;
            vert1.y += initLwo->endoMorph[i].vertTable[vert1Index].y;
            vert1.z += initLwo->endoMorph[i].vertTable[vert1Index].z;
            vert2.x += initLwo->endoMorph[i].vertTable[vert2Index].x;
            vert2.y += initLwo->endoMorph[i].vertTable[vert2Index].y;
            vert2.z += initLwo->endoMorph[i].vertTable[vert2Index].z;
            vert3.x += initLwo->endoMorph[i].vertTable[vert3Index].x;
            vert3.y += initLwo->endoMorph[i].vertTable[vert3Index].y;
            vert3.z += initLwo->endoMorph[i].vertTable[vert3Index].z;

            v1.x = vert2.x-vert1.x;
            v1.y = vert2.y-vert1.y;
            v1.z = vert2.z-vert1.z;
            v2.x = vert3.x-vert1.x;
            v2.y = vert3.y-vert1.y;
            v2.z = vert3.z-vert1.z;

            out.x = (v1.y*v2.z)-(v1.z*v2.y);
            out.y = (v1.z*v2.x)-(v1.x*v2.z);
            out.z = (v1.x*v2.y)-(v1.y*v2.x);

            len = (FLOAT32) MathSqrt((out.x*out.x)+(out.y*out.y)+(out.z*out.z));

            if (len==0.0f)
            {
                len = 1.0f;
            }

            len *= LwoTransfNormCoef(initLwo->sizeCoef);

            out.x /= len;
            out.y /= len;
            out.z /= len;

            normVert1 = &initLwo->endoMorph[i].normTable[vert1Index];
            normVert1->x += out.x;
            normVert1->y += out.y;
            normVert1->z += out.z;
            normAdd[vert1Index]++;

            normVert2 = &initLwo->endoMorph[i].normTable[vert2Index];
            normVert2->x += out.x;
            normVert2->y += out.y;
            normVert2->z += out.z;
            normAdd[vert2Index]++;

            normVert3 = &initLwo->endoMorph[i].normTable[vert3Index];
            normVert3->x += out.x;
            normVert3->y += out.y;
            normVert3->z += out.z;
            normAdd[vert3Index]++;
        }

        for (norm=0; norm<initLwo->normCount; norm++)
        {
            initLwo->endoMorph[i].normTable[norm].x /= normAdd[norm];
            initLwo->endoMorph[i].normTable[norm].y /= normAdd[norm];
            initLwo->endoMorph[i].normTable[norm].z /= normAdd[norm];
        }
    }

    SAFE_FREE(normAdd);
}
