/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoinituv.c - LWO Object UV Coordinates Initializer                        *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/math.h"
#include "system/file.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lwo/lwoinituv.h"
#include "platform/renderer/lwodraw.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static FLOAT32 sx = 0.0f;
static FLOAT32 sy = 0.0f;
static FLOAT32 sz = 0.0f;

static FLOAT32 cx = 0.0f;
static FLOAT32 cy = 0.0f;
static FLOAT32 cz = 0.0f;

static FLOAT32 x = 0.0f;
static FLOAT32 y = 0.0f;
static FLOAT32 z = 0.0f;

static INT32 vert1Index = 0;
static INT32 vert2Index = 0;
static INT32 vert3Index = 0;

static TexUV *txuv = NULL;

static Vert vert1;
static Vert vert2;
static Vert vert3;
    
static AxisType axisType;	
	
static FLOAT32 len = 0.0f;
static FLOAT32 lat = 0.0f;

/*---------------------------------------------------------------------------- XZ TO H */
static void 
xzToH(FLOAT32 x, FLOAT32 z, FLOAT32 *h)
{
	if (x==0.0f && z==0.0f)
	{
		*h = 0.0f;
	}
	else 
	{
		if (z==0.0f)
		{
			*h = x<0.0f ? (PI/2.0f):-(PI/2.0f);
		}
		else
		{
			*h = -MathAtan2(x, z);
		}

		if (*h<0.0f)
		{
			*h += PI*2.0f;
		}
	}
}

/*---------------------------------------------------------------------------- XYZ TO HP */
static void 
xyzToHP(FLOAT32 x, FLOAT32 y, FLOAT32 z, FLOAT32 *h, FLOAT32 *p)
{
	if (x==0.0f && z==0.0f) 
	{
		*h = 0.0f;

		if (y!=0.0f)
		{
			*p = y<0.0f ? -(PI/2.0f):(PI/2.0f);
		}
		else
		{
			*p = 0.0f;
		}
    }
    else
	{
       *h = -MathAtan2(x, z);

       if (*h<0.0f)
	   {
		   *h += PI*2.0f;
	   }

       x = MathSqrt(x*x+z*z);

       if (x==0.0f)
	   {
			*p = y<0.0f ? -(PI/2.0f):(PI/2.0f);
	   }
       else
	   {
			*p = MathAtan(y/x);
	   }
    }
}

/*---------------------------------------------------------------------------- PLANAR */
static void
doPlanar(void)
{
	FLOAT32 *uvTable = NULL;

    INT32 vert = 0;
    
    sx = initLwo->surfTable[initSurf].sizeX;
	sy = initLwo->surfTable[initSurf].sizeY;
	sz = initLwo->surfTable[initSurf].sizeZ;

	uvTable = (FLOAT32 *) initLwo->surfTable[initSurf].uvTable;

    cx = initLwo->surfTable[initSurf].centerX;
	cy = initLwo->surfTable[initSurf].centerY;
	cz = initLwo->surfTable[initSurf].centerZ;

	for (vert=0; vert<initLwo->vertCount; vert++)
    {
        x = 0.5f+(initLwo->vertTable[vert].x-cx)/sx;
        y = 0.5f+(initLwo->vertTable[vert].y-cy)/sy;
        z = 0.5f-(initLwo->vertTable[vert].z+cz)/sz;

		switch (initLwo->surfTable[initSurf].axisType)
        {
            case AxisX:
                *(uvTable++) = z;
                *(uvTable++) = y;
                break;

            case AxisY:
                *(uvTable++) = x;
				*(uvTable++) = z;
                break;

            case AxisZ:
                *(uvTable++) = x;
				*(uvTable++) = y;
				break;
        }
    }
}
	
/*---------------------------------------------------------------------------- CUBIC */
static void
doCubic(void)
{
	FLOAT32 f0 = 0.0f;
	FLOAT32 f1 = 0.0f;
	FLOAT32 f2 = 0.0f;
		
	Norm v1;
	Norm v2;
	Norm out;

	vert1 = txuv->vertTable[vert1Index];
    vert2 = txuv->vertTable[vert2Index];
    vert3 = txuv->vertTable[vert3Index];

    v1.x = vert2.x-vert1.x;
    v1.y = vert2.y-vert1.y;
    v1.z = vert2.z-vert1.z;
    v2.x = vert3.x-vert1.x;
    v2.y = vert3.y-vert1.y;
    v2.z = vert3.z-vert1.z;

    out.x = (v1.y*v2.z)-(v1.z*v2.y);
    out.y = (v1.z*v2.x)-(v1.x*v2.z);
    out.z = (v1.x*v2.y)-(v1.y*v2.x);

	len = (FLOAT32) MathSqrt((out.x*out.x)+(out.y*out.y)+(out.z*out.z));

    if (len==0.0f)
    {
        len = 1.0f;
    }
    
	out.x /= len;
    out.y /= len;
    out.z /= len;

	f0 = MathFabs(out.x);
	f1 = MathFabs(out.y);
	f2 = MathFabs(out.z);

	if (f0<f1)
	{
		if (f2<f1)
		{
			axisType = AxisY;
		}
		else
		{
			axisType = AxisZ;
		}
	}
	else if (f2<f0)
	{
		axisType = AxisX;
	}
	else
	{
		axisType = AxisZ;
	}

	x = 0.5f+(txuv->vertTable[vert1Index].x-cx)/sx;
    y = 0.5f+(txuv->vertTable[vert1Index].y-cy)/sy;
    z = 0.5f-(txuv->vertTable[vert1Index].z+cz)/sz;

	if (axisType==AxisX)
	{
		txuv->uvTable[vert1Index].u = z;
	}
	else
	{
		txuv->uvTable[vert1Index].u = x;
	}

	if (axisType==AxisY)
	{
		txuv->uvTable[vert1Index].v = z;
	}
	else
	{
		txuv->uvTable[vert1Index].v = y;
	}

	x = 0.5f+(txuv->vertTable[vert2Index].x-cx)/sx;
    y = 0.5f+(txuv->vertTable[vert2Index].y-cy)/sy;
    z = 0.5f-(txuv->vertTable[vert2Index].z+cz)/sz;

	if (axisType==AxisX)
	{
		txuv->uvTable[vert2Index].u = z;
	}
	else
	{
		txuv->uvTable[vert2Index].u = x;
	}

	if (axisType==AxisY)
	{
		txuv->uvTable[vert2Index].v = z;
	}
	else
	{
		txuv->uvTable[vert2Index].v = y;
	}

	x = 0.5f+(txuv->vertTable[vert3Index].x-cx)/sx;
    y = 0.5f+(txuv->vertTable[vert3Index].y-cy)/sy;
    z = 0.5f-(txuv->vertTable[vert3Index].z+cz)/sz;

	if (axisType==AxisX)
	{
		txuv->uvTable[vert3Index].u = z;
	}
	else
	{
		txuv->uvTable[vert3Index].u = x;
	}

	if (axisType==AxisY)
	{
		txuv->uvTable[vert3Index].v = z;
	}
	else
	{
		txuv->uvTable[vert3Index].v = y;
	}
}

/*---------------------------------------------------------------------------- CYLINDRICAL */
static void
doCylindrical(void)
{
	FLOAT32 a = 0.0f;

	x = txuv->vertTable[vert1Index].x-cx;
    y = txuv->vertTable[vert1Index].y-cy;
    z = -txuv->vertTable[vert1Index].z+cz;

	if (initLwo->surfTable[initSurf].axisType==AxisX)
	{
		xzToH(z, -y, &len);
		a = 0.5f+(x/sx);
	}
	else if (initLwo->surfTable[initSurf].axisType==AxisY)
	{
		xzToH(-x, z, &len);
		a = 0.5f+(y/sy);
	}
	else
	{
		xzToH(-x, -y, &len);
		a = 0.5f+(z/sz);
	}

	while (a<0.0f)
	{
		a += 1.0f;
	}

	txuv->uvTable[vert1Index].u = 1.0f-len/(2*PI);
	txuv->uvTable[vert1Index].v = a;

	x = txuv->vertTable[vert2Index].x-cx;
    y = txuv->vertTable[vert2Index].y-cy;
    z = -txuv->vertTable[vert2Index].z+cz;

	if (initLwo->surfTable[initSurf].axisType==AxisX)
	{
		xzToH(z, -y, &len);
		a = 0.5f+(x/sx);
	}
	else if (initLwo->surfTable[initSurf].axisType==AxisY)
	{
		xzToH(-x, z, &len);
		a = 0.5f+(y/sy);
	}
	else
	{
		xzToH(-x, -y, &len);
		a = 0.5f+(z/sz);
	}

	while (a<0.0f)
	{
		a += 1.0f;
	}

	txuv->uvTable[vert2Index].u = 1.0f-len/(2*PI);
	txuv->uvTable[vert2Index].v = a;

	x = txuv->vertTable[vert3Index].x-cx;
    y = txuv->vertTable[vert3Index].y-cy;
    z = -txuv->vertTable[vert3Index].z+cz;

	if (initLwo->surfTable[initSurf].axisType==AxisX)
	{
		xzToH(z, -y, &len);
		a = 0.5f+(x/sx);
	}
	else if (initLwo->surfTable[initSurf].axisType==AxisY)
	{
		xzToH(-x, z, &len);
		a = 0.5f+(y/sy);
	}
	else
	{
		xzToH(-x, -y, &len);
		a = 0.5f+(z/sz);
	}

	while (a<0.0f)
	{
		a += 1.0f;
	}

	txuv->uvTable[vert3Index].u = 1.0f-len/(2*PI);
	txuv->uvTable[vert3Index].v = a;
}

/*---------------------------------------------------------------------------- SPHERICAL */
static void
doSpherical(void)
{
	x = txuv->vertTable[vert1Index].x-cx;
    y = txuv->vertTable[vert1Index].y-cy;
    z = txuv->vertTable[vert1Index].z-cz;

	if (initLwo->surfTable[initSurf].axisType==AxisX)
	{
		xyzToHP(-z, -x, -y, &len, &lat);
	}
	else if (initLwo->surfTable[initSurf].axisType==AxisY)
	{
		xyzToHP(-x, -y, -z, &len, &lat);
	}
	else
	{
		xyzToHP(-x, z, -y, &len, &lat);
	}

	txuv->uvTable[vert1Index].u = 1.0f-len/(2*PI);
	txuv->uvTable[vert1Index].v = 0.5f-lat/PI;

	x = txuv->vertTable[vert2Index].x-cx;
    y = txuv->vertTable[vert2Index].y-cy;
    z = txuv->vertTable[vert2Index].z-cz;

	if (initLwo->surfTable[initSurf].axisType==AxisX)
	{
		xyzToHP(-z, -x, -y, &len, &lat);
	}
	else if (initLwo->surfTable[initSurf].axisType==AxisY)
	{
		xyzToHP(-x, -y, -z, &len, &lat);
	}
	else
	{
		xyzToHP(-x, z, -y, &len, &lat);
	}

	txuv->uvTable[vert2Index].u = 1.0f-len/(2*PI);
	txuv->uvTable[vert2Index].v = 0.5f-lat/PI;

	x = txuv->vertTable[vert3Index].x-cx;
    y = txuv->vertTable[vert3Index].y-cy;
    z = txuv->vertTable[vert3Index].z-cz;

	if (initLwo->surfTable[initSurf].axisType==AxisX)
	{
		xyzToHP(-z, -x, -y, &len, &lat);
	}
	else if (initLwo->surfTable[initSurf].axisType==AxisY)
	{
		xyzToHP(-x, -y, -z, &len, &lat);
	}
	else
	{
		xyzToHP(-x, z, -y, &len, &lat);
	}

	txuv->uvTable[vert3Index].u = 1.0f-len/(2*PI);
	txuv->uvTable[vert3Index].v = 0.5f-lat/PI;
}

/*---------------------------------------------------------------------------- UV */
static void 
doUV(void)
{
    TexUV *txuv = NULL;
    Poly *poly = NULL;
    INT32 i = 0;
    INT32 j = 0;
    INT32 k = 0;
    INT32 vertIndex = 0;
    INT32 add = 0;
    INT32 polyIndex = 0;
    INT32 inc = 0;
    INT32 *triIndexTable = NULL;
    INT32 vert = 0;

    j = 0;
    txuv = &initLwo->txuv[0];

    while (strcmp(txuv->name, initLwo->surfTable[initSurf].txuvName)!=0)
    {
        j++;
        txuv = &initLwo->txuv[j];
    }

    initLwo->surfTable[initSurf].txuv = txuv;

    add = 0;

    for (i=0; i<txuv->uvProjAddCount; i++)
    {
        vertIndex = txuv->uvProjAddTable[i].vertIndex;
        polyIndex = txuv->uvProjAddTable[i].polyIndex;

        poly = &initLwo->polyTable[polyIndex];

        for (j=poly->triOffset; j<poly->triOffset+poly->triCount; j++)
        {   
            if (initLwo->triExTable[j].vert1Index==vertIndex)
            {
                add++;
            }

            if (initLwo->triExTable[j].vert2Index==vertIndex)
            {
                add++;
            }

            if (initLwo->triExTable[j].vert3Index==vertIndex)
            {
                add++;
            }
        }
    }

    txuv->vertTable = MemAlloc(sizeof(Vert)*(initLwo->vertCount+add));        

    memcpy(txuv->vertTable, initLwo->vertTable, sizeof(Vert)*initLwo->vertCount);

    txuv->vertBaseCount = initLwo->vertCount;
    txuv->vertCount = txuv->vertBaseCount+add;

    txuv->uvTable = MemAlloc(sizeof(UV)*txuv->vertCount);        
    txuv->uvCount = txuv->vertCount;

    for (vert=0; vert<txuv->vertBaseCount; vert++)
    {
        txuv->uvTable[vert].u = txuv->uvProjTable[vert].u;
        txuv->uvTable[vert].v = txuv->uvProjTable[vert].v;
    }
    
    triIndexTable = MemAlloc(sizeof(INT32)*initLwo->triExCount);
        
    inc = 0;

    for (i=0; i<initLwo->triExCount; i++)
    {
        polyIndex = initLwo->triExTable[i].polyIndex;

        if (initLwo->polyTable[polyIndex].surfIndex==initSurf)
        {
            triIndexTable[i] = inc;

            inc++;
        }
        else
        {
            triIndexTable[i] = -1;
        }
    }
    
    k = txuv->vertBaseCount;

    for (i=0; i<txuv->uvProjAddCount; i++)
    {
        vertIndex = txuv->uvProjAddTable[i].vertIndex;
        polyIndex = txuv->uvProjAddTable[i].polyIndex;

        poly = &initLwo->polyTable[polyIndex];

        for (j=poly->triOffset; j<poly->triOffset+poly->triCount; j++)
        {
            inc = triIndexTable[j];

            if (inc!=-1)
            {
                if (initLwo->triExTable[j].vert1Index==vertIndex)
                {
                    initLwo->surfTable[initSurf].triTable[inc].vert1Index = k;

                    txuv->vertTable[k].x = initLwo->vertTable[vertIndex].x;
                    txuv->vertTable[k].y = initLwo->vertTable[vertIndex].y;
                    txuv->vertTable[k].z = initLwo->vertTable[vertIndex].z;

                    txuv->uvTable[k].u = txuv->uvProjAddTable[i].u;
                    txuv->uvTable[k].v = txuv->uvProjAddTable[i].v;

                    k++;
                }

                if (initLwo->triExTable[j].vert2Index==vertIndex)
                {
                    initLwo->surfTable[initSurf].triTable[inc].vert2Index = k;

                    txuv->vertTable[k].x = initLwo->vertTable[vertIndex].x;
                    txuv->vertTable[k].y = initLwo->vertTable[vertIndex].y;
                    txuv->vertTable[k].z = initLwo->vertTable[vertIndex].z;

                    txuv->uvTable[k].u = txuv->uvProjAddTable[i].u;
                    txuv->uvTable[k].v = txuv->uvProjAddTable[i].v;

                    k++;
                }

                if (initLwo->triExTable[j].vert3Index==vertIndex)
                {
                    initLwo->surfTable[initSurf].triTable[inc].vert3Index = k;

                    txuv->vertTable[k].x = initLwo->vertTable[vertIndex].x;
                    txuv->vertTable[k].y = initLwo->vertTable[vertIndex].y;
                    txuv->vertTable[k].z = initLwo->vertTable[vertIndex].z;

                    txuv->uvTable[k].u = txuv->uvProjAddTable[i].u;
                    txuv->uvTable[k].v = txuv->uvProjAddTable[i].v;

                    k++;
                }
            }
        }
    }

    SAFE_FREE(triIndexTable);    
}

/*---------------------------------------------------------------------------- CUBIC+CYLINDRICAL+SPHERICAL */
static void 
doAddVert(MappingType mappingType)
{
    INT32 i = 0;
    INT32 j = 0;
    INT32 k = 0;
    INT32 vertIndex = 0;
    BOOL *isUVUsed = NULL;
	INT32 add = 0;

    j = 0;
    txuv = &initLwo->txuv[0];

    while (strcmp(txuv->name, initLwo->surfTable[initSurf].txuvName)!=0)
    {
        j++;
        txuv = &initLwo->txuv[j];
    }

    initLwo->surfTable[initSurf].txuv = txuv;

    add = 0;

    for (i=0; i<initLwo->vertCount; i++)
    {
        vertIndex = i;

        for (j=0; j<initLwo->triExCount; j++)
        {   
            if (initLwo->triExTable[j].vert1Index==vertIndex)
            {
                add++;
            }

            if (initLwo->triExTable[j].vert2Index==vertIndex)
            {
                add++;
            }

            if (initLwo->triExTable[j].vert3Index==vertIndex)
            {
                add++;
            }
        }
    }

    txuv->vertTable = MemAlloc(sizeof(Vert)*add);

    memcpy(txuv->vertTable, initLwo->vertTable, sizeof(Vert)*initLwo->vertCount);

    txuv->vertBaseCount = initLwo->vertCount;
    txuv->vertCount = add;

    txuv->uvTable = MemAlloc(sizeof(UV)*txuv->vertCount);        
    txuv->uvCount = txuv->vertCount;

	isUVUsed = MemAlloc(sizeof(BOOL)*txuv->vertCount);
	for (i=0; i<txuv->vertCount; i++)
	{
		isUVUsed[i] = FALSE;
	}

    k = txuv->vertBaseCount;

    sx = initLwo->surfTable[initSurf].sizeX;
	sy = initLwo->surfTable[initSurf].sizeY;
	sz = initLwo->surfTable[initSurf].sizeZ;

	cx = initLwo->surfTable[initSurf].centerX;
	cy = initLwo->surfTable[initSurf].centerY;
	cz = initLwo->surfTable[initSurf].centerZ;

	for (j=0; j<initLwo->triExCount; j++)
    {
		vert1Index = initLwo->triExTable[j].vert1Index;
		vert2Index = initLwo->triExTable[j].vert2Index;
		vert3Index = initLwo->triExTable[j].vert3Index;

		if (isUVUsed[vert1Index]==FALSE)
		{
			isUVUsed[vert1Index] = TRUE;
		}
		else
		{
			initLwo->surfTable[initSurf].triTable[j].vert1Index = k;

            txuv->vertTable[k].x = initLwo->vertTable[vert1Index].x;
            txuv->vertTable[k].y = initLwo->vertTable[vert1Index].y;
            txuv->vertTable[k].z = initLwo->vertTable[vert1Index].z;

			vert1Index = k;
            k++;
        }

        if (isUVUsed[vert2Index]==FALSE)
		{
			isUVUsed[vert2Index] = TRUE;
		}
		else
		{
            initLwo->surfTable[initSurf].triTable[j].vert2Index = k;

            txuv->vertTable[k].x = initLwo->vertTable[vert2Index].x;
            txuv->vertTable[k].y = initLwo->vertTable[vert2Index].y;
            txuv->vertTable[k].z = initLwo->vertTable[vert2Index].z;

            vert2Index = k;
			k++;
        }

        if (isUVUsed[vert3Index]==FALSE)
        {
			isUVUsed[vert3Index] = TRUE;
		}
		else
		{
            initLwo->surfTable[initSurf].triTable[j].vert3Index = k;

            txuv->vertTable[k].x = initLwo->vertTable[vert3Index].x;
            txuv->vertTable[k].y = initLwo->vertTable[vert3Index].y;
            txuv->vertTable[k].z = initLwo->vertTable[vert3Index].z;

            vert3Index = k;
			k++;
        }
		
		switch (mappingType)
		{
			case MappingCubic:
				doCubic();
				break;

			case MappingCylindrical:
				doCylindrical();
				break;

			case MappingSpherical:
				doSpherical();
				break;

			default:
				break;
		}
    }

    SAFE_FREE(isUVUsed);
}

/*---------------------------------------------------------------------------- MAPPING */
void 
LwoSurfBuildUV(void)
{
    switch(initLwo->surfTable[initSurf].mappingType)
	{
		case MappingPlanar:
			doPlanar();
			break;
		
		case MappingUV:
			doUV();
			break;
	
		case MappingCubic:
			doAddVert(MappingCubic);
			break;

		case MappingCylindrical:
			doAddVert(MappingCylindrical);
			break;

		case MappingSpherical:
			doAddVert(MappingSpherical);
			break;

		default:
				break;
	}
}