/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwomorph.c - LWO Object Morphing                                           *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "system/file.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lwo/lwomorph.h"

/*---------------------------------------------------------------------------- ALLOC */
void 
LwoMorphAlloc(void)
{
    initLwo->vertBaseTable = MemAlloc(sizeof(Vert)*initLwo->vertCount);
    memcpy(initLwo->vertBaseTable, initLwo->vertTable, sizeof(Vert)*initLwo->vertCount);
    
    if (initLwo->hasNormals)
	{
		initLwo->normBaseTable = MemAlloc(sizeof(Norm)*initLwo->normCount);
		memcpy(initLwo->normBaseTable, initLwo->normTable, sizeof(Norm)*initLwo->normCount);
	}
}

/*---------------------------------------------------------------------------- INIT */
RESULT
LwoMorphInit(Lwo *src, Lwo *dst, const FLOAT32 amount)
{
    INT32 vert = 0;
    Vert *vertSrc = NULL;
    Vert *vertDst = NULL;

    INT32 norm = 0;
    Norm *normSrc = NULL;
    Norm *normDst = NULL;
    
    if (src==NULL || dst==NULL)
    {
        return ERR_INVALID_PTR;
    }

    for (vert=0; vert<src->vertCount; vert++)
    {
        vertSrc = &src->vertBaseTable[vert];
        vertDst = &dst->vertBaseTable[vert];
        
        src->vertTable[vert].x = (vertSrc->x*(1.0f-amount))+(vertDst->x*amount);
        src->vertTable[vert].y = (vertSrc->y*(1.0f-amount))+(vertDst->y*amount);
        src->vertTable[vert].z = (vertSrc->z*(1.0f-amount))+(vertDst->z*amount);
    }
        
    if (src->hasNormals)
    {
        for (norm=0; norm<src->normCount; norm++)
        {
            normSrc = &src->normBaseTable[norm];
            normDst = &dst->normBaseTable[norm];
        
            src->normTable[norm].x = (normSrc->x*(1.0f-amount))+(normDst->x*amount);
            src->normTable[norm].y = (normSrc->y*(1.0f-amount))+(normDst->y*amount);
            src->normTable[norm].z = (normSrc->z*(1.0f-amount))+(normDst->z*amount);
        }
    }
        
    return OK;
}

/*---------------------------------------------------------------------------- MORPH AMOUNT */
void
LwoSetEndoMorphAmount(Lwo *lwo, INT8 *name, float amount)
{
    INT32 i = 0;
    INT8 *nameSearch = NULL;

    while (i<lwo->endoMorphCount)
    {
        nameSearch = name;
            
        if (strstr(lwo->endoMorph[i].name, ".")==NULL && strstr(nameSearch, ".")!=NULL)
        {
            while (*nameSearch!='.')
            {
                nameSearch++;
            }
            nameSearch++;
        }

        if (strcmp(lwo->endoMorph[i].name, nameSearch)==0)
        {
            lwo->endoMorph[i].amount = amount;

            return;
        }

        i++;
    }
}

/*---------------------------------------------------------------------------- MORPH CALCULATE */
void
LwoEndoMorphCalculate(Lwo *lwo)
{
    INT32 i = 0;
    INT32 j = 0;
    float x = 0.0f;
    float y = 0.0f;
    float z = 0.0f;
    float nx = 0.0f;
    float ny = 0.0f;
    float nz = 0.0f;
    
    if (lwo->endoMorphCount==0)
    {
        return;
    }

    for (i=0; i<lwo->vertCount; i++)
    {
        x = lwo->vertBaseTable[i].x;
        y = lwo->vertBaseTable[i].y;
        z = lwo->vertBaseTable[i].z;
        
        for (j=0; j<lwo->endoMorphCount; j++)
        {
            x += lwo->endoMorph[j].vertTable[i].x*lwo->endoMorph[j].amount;
            y += lwo->endoMorph[j].vertTable[i].y*lwo->endoMorph[j].amount;
            z += lwo->endoMorph[j].vertTable[i].z*lwo->endoMorph[j].amount;
        }

        lwo->vertTable[i].x = x;
        lwo->vertTable[i].y = y;
        lwo->vertTable[i].z = z;

        if (lwo->hasNormals)
        {
            nx = lwo->normBaseTable[i].x;
            ny = lwo->normBaseTable[i].y;
            nz = lwo->normBaseTable[i].z;
        
            for (j=0; j<lwo->endoMorphCount; j++)
            {
                x += lwo->endoMorph[j].vertTable[i].x*lwo->endoMorph[j].amount;
                y += lwo->endoMorph[j].vertTable[i].y*lwo->endoMorph[j].amount;
                z += lwo->endoMorph[j].vertTable[i].z*lwo->endoMorph[j].amount;

                nx += lwo->endoMorph[j].normTable[i].x*lwo->endoMorph[j].amount;
                ny += lwo->endoMorph[j].normTable[i].y*lwo->endoMorph[j].amount;
                nz += lwo->endoMorph[j].normTable[i].z*lwo->endoMorph[j].amount;
            }

            lwo->normTable[i].x = nx;
            lwo->normTable[i].y = ny;
            lwo->normTable[i].z = nz;
        }
    }
}