/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoparse.c - LWO Object Parser                                             *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "system/file.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoparse.h"
#include "lightwave/lwo/lwoparsetools.h"
#include "lightwave/lwo/lwoparsesurf.h"
#include "lightwave/lwo/lwoparsepnts.h"
#include "lightwave/lwo/lwoparsepols.h"
#include "lightwave/lwo/lwoparsetex.h"
#include "lightwave/lwo/lwoparsevmap.h"

/*---------------------------------------------------------------------------- PARSER */
RESULT
LwoParse(void (*progress)(INT32 i))
{
    RESULT result = ERR;
	INT32 layr = 0;
	INT32 statSurfCount = 0;
	INT32 statTexCount = 0;
	INT32 percentSurf = 0;
	INT32 percentTex = 0;

    lwoCurr = LwoGetCurrent();
    lwoFile = LwoGetCurrentFile();
    
    if (lwoCurr==NULL || lwoFile==NULL)
    {
        return ERR_INVALID_PTR;
    }

    lwoPtr = (UINT8 *) lwoFile->buffer;

    SKIP_INT8(8);
    
    if (LwoGetUINT32()!=CHUNK('L','W','O','2'))
    {
        ConsoleAdd("Error : %s is not a LWO2 file (LW 6.X+) !", lwoFile->fileName);
        
        return ERR_LWO_VERSION;
    }

    result = LwoParseGetPolyTexCount();    
    
    if (result!=OK)
    {
        return result;
    }

    LwoOrderChunks(CHUNK('P','T','A','G'), CHUNK('P','O','L','S'));

    if (lwoCurr->polyCount<=0)
    {
        ConsoleAdd("Error : %s has a bad polyCount definitions ! (Nurbs & Bones are not supported)", lwoFile->fileName);
        
        return ERR_LWO_INVALID;
    }

    statSurfCount = 0;
	statTexCount = 0;
	
	lwoCurr->polyTable = MemAlloc(sizeof(Poly)*lwoCurr->polyCount);
    lwoCurr->texTable = MemAlloc(sizeof(Tex)*lwoCurr->texCount);

    while (GET_OFFSET<lwoFile->size)
    {
        chunkID = LwoGetUINT32();
        chunkSize = LwoGetUINT32();
            
        switch (chunkID)
        {
		case CHUNK('L','A','Y','R'):
			layr++;
			if (layr>1)
			{
				ConsoleAdd("Error : %s has more than 1 layer !", lwoFile->fileName);
                
                return ERR_LWO_MULTI_LAYER;
			}
			SKIP_INT8(chunkSize);
			break;

        case CHUNK('T','A','G','S'):
            LwoParseSurfCreate();
            break;

        case CHUNK('P','N','T','S'):
            LwoParseVert();
            break;

        case CHUNK('B','B','O','X'):
            LwoParseBounBox();
            break;

        case CHUNK('P','T','A','G'):
            LwoParsePolyGetSurf();
            break;

        case CHUNK('P','O','L','S'):
            LwoParsePoly();             
            break;

        case CHUNK('C','L','I','P'):
            statTexCount++;
			
            percentSurf = (INT32) (((FLOAT32) statSurfCount/(FLOAT32) lwoCurr->statSurfTotalCount)*50.0f);
			percentTex = (INT32) (((FLOAT32) statTexCount/(FLOAT32) lwoCurr->statTexTotalCount)*50.0f);				
			progress(percentSurf+percentTex);

            if (LwoParseTex()==ERR_LWO_TEXTURE)
            {
                return ERR_LWO_TEXTURE;
            }
            break;

        case CHUNK('S','U','R','F'):
			statSurfCount++;                

			percentSurf = (INT32) (((FLOAT32) statSurfCount/(FLOAT32) lwoCurr->statSurfTotalCount)*100.0f);
				
			if (lwoCurr->statTexTotalCount==0)
			{
				progress(percentSurf);
			}
			else
			{
				percentTex = (INT32) (((FLOAT32) statTexCount/(FLOAT32) lwoCurr->statTexTotalCount)*50.0f);
				progress((percentSurf>>1)+percentTex);
			}

            if (LwoParseSurf()==ERR_LWO_TEXTURE)
            {
                return ERR_LWO_TEXTURE;
            }
            break;                      

        case CHUNK('V','M','A','P'):
            if (LwoParseVMAP()!=OK)
            {
                return ERR_LWO_TEXTURE;
            }
            break;

        case CHUNK('V','M','A','D'):
            if (LwoParseVMAD()!=OK)
            {
                return ERR_LWO_TEXTURE;
            }
            break;

        default:
            SKIP_INT8(chunkSize);
            break;
        }
    }

    return OK;
}