/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoparsesurf.c - LWO Object Surface Parser                                 *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "system/file.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoparse.h"
#include "lightwave/lwo/lwoparsetools.h"
#include "lightwave/lwo/lwoparsesurf.h"
#include "lightwave/lwo/lwoparsetex.h"

/*---------------------------------------------------------------------------- SURFACE CREATION */
void 
LwoParseSurfCreate(void)
{
    INT8 a = 0;
    INT32 i = 0;
    INT32 j = 0;
    INT32 surf = 0;
    
    surf = 0;
    i = 0;
    j = 0;

    do
    {
        a = LwoGetUINT8();

        i++;

        if (a==0)                        
        {
            surf++;

            j = 0;
            a = LwoGetUINT8();
            if (a==0)
            {
                i++;
            }
            else
            {
                BACK_INT8(1);
            }
        }
        else
        {
            j++;
        }
    } while (i<chunkSize);

    BACK_INT8(chunkSize);

    lwoCurr->surfCount = surf;
    lwoCurr->surfTable = MemAlloc(sizeof(Surf)*lwoCurr->surfCount);

	for(i=0; i<lwoCurr->surfCount; i++)
	{
		lwoCurr->surfTable[i].tex0 = NULL;
		lwoCurr->surfTable[i].tex1 = NULL;
	}

	surf = 0;
    i = 0;
    j = 0;

    do
    {
        a = LwoGetUINT8();

        lwoCurr->surfTable[surf].name[j] = a;

        i++;

        if (a==0)    
        {
            surf++;

            j = 0;

            a = LwoGetUINT8();

            if (a==0)
            {
                i++;
            }
            else
            {
                BACK_INT8(1);
            }
        }
        else
        {
            j++;
        }
    } while (i<chunkSize);
}

/*---------------------------------------------------------------------------- SURFACE TEXTURE INFO */
RESULT
LwoParseSurfTexInfo(INT32 surf)
{
    INT32 subI = 0;
    INT32 subSubChunkID = 0;
    INT32 subSubChunkSize = 0;
    INT32 proj = 0;
    INT32 tex = 0;

    subI = 6;

    while (subI<subChunkSize)
    {
        subSubChunkID = LwoGetUINT32();
        subSubChunkSize = LwoGetUINT16();
        subI += 6;

        switch (subSubChunkID)
        {
        case CHUNK('A','X','I','S'):
            lwoCurr->surfTable[surf].axisType = LwoGetUINT16();
            subI += subSubChunkSize;
            SKIP_INT8(subSubChunkSize-2);
            break;

        case CHUNK('P','R','O','J'):
            proj = LwoGetUINT16()+1;
            subI += subSubChunkSize;
            SKIP_INT8(subSubChunkSize-2);
            break;

        case CHUNK('T','M','A','P'):
			{
				INT32 j = 0;
                
                j = subSubChunkSize-4;

                while (j>=0)
                {
                    if (lwoPtr[j]=='S' && lwoPtr[j+1]=='I' && lwoPtr[j+2]=='Z' && lwoPtr[j+3]=='E')
                    {
                        INT32 a = 0;
						
						SKIP_INT8(j+sizeof(UINT32));
						
						SKIP_INT8(2);
							
						a = LwoGetUINT32(); lwoCurr->surfTable[surf].sizeX = *(FLOAT32 *)&a;
						a = LwoGetUINT32(); lwoCurr->surfTable[surf].sizeY = *(FLOAT32 *)&a;
						a = LwoGetUINT32(); lwoCurr->surfTable[surf].sizeZ = *(FLOAT32 *)&a;
						
						SKIP_INT8(2);
							
						BACK_INT8(j+((4+1)*sizeof(UINT32)));

						j = 0;
                    }

                    j--;
                }

				j = subSubChunkSize-4;

                while (j>=0)
                {
                    if (lwoPtr[j]=='C' && lwoPtr[j+1]=='N' && lwoPtr[j+2]=='T' && lwoPtr[j+3]=='R')
                    {
                        INT32 a = 0;
						
						SKIP_INT8(j+sizeof(UINT32));
						
						SKIP_INT8(2);
							
						a = LwoGetUINT32(); lwoCurr->surfTable[surf].centerX = *(FLOAT32 *) &a;
						a = LwoGetUINT32(); lwoCurr->surfTable[surf].centerY = *(FLOAT32 *)&a;
						a = LwoGetUINT32(); lwoCurr->surfTable[surf].centerZ = *(FLOAT32 *)&a;
						
						SKIP_INT8(2);
							
						BACK_INT8(j+((4+1)*sizeof(UINT32)));

						j = 0;
                    }

                    j--;
                }
			}

            subI += subSubChunkSize;
            SKIP_INT8(subSubChunkSize);

			break;

        case CHUNK('I','M','A','P'):
            {
                INT32 j = 0;
                
                j = subSubChunkSize-4;

                while (j>=0)
                {
                    if (lwoPtr[j]=='R' && lwoPtr[j+1]=='E' && lwoPtr[j+2]=='F' && lwoPtr[j+3]=='L')
                    {
                        lwoCurr->surfTable[surf].renderState.envmap = TRUE;
                        lwoCurr->hasNormals = TRUE;

                        j = 0;
                    }

                    j--;
                }
                
                subI += subSubChunkSize;
                SKIP_INT8(subSubChunkSize);
            }
            break;

        case CHUNK('I','M','A','G'):
            tex = LwoGetVX()-1;

            if (tex<lwoCurr->texCount)
            {
                if (strlen(lwoCurr->texTable[tex].fileName)==0)
				{
					ConsoleAdd("Error : %s (%s) -> texture problem !", lwoFile->fileName, lwoCurr->surfTable[surf].name);
                
                    /*
                    return ERR_LWO_TEXTURE;
                    */

                    lwoCurr->surfTable[surf].renderState.gouraud = TRUE;
                    lwoCurr->surfTable[surf].renderState.mapping = FALSE;
                    lwoCurr->surfTable[surf].renderState.envmap = FALSE;
                    lwoCurr->surfTable[surf].tex0 = NULL;
                    lwoCurr->surfTable[surf].tex1 = NULL;
                    lwoCurr->hasNormals = TRUE;
				}
				else
                {
                    if (lwoCurr->surfTable[surf].renderState.envmap==TRUE && lwoCurr->surfTable[surf].tex0!=NULL)
				    {
					    lwoCurr->surfTable[surf].tex1 = &lwoCurr->texTable[tex];
				    }
				    else
				    {
					    lwoCurr->surfTable[surf].tex0 = &lwoCurr->texTable[tex];
				        
                        lwoCurr->surfTable[surf].mappingType = proj;
                    }

				    if (lwoCurr->surfTable[surf].tex1!=NULL)
                    {
                        lwoCurr->hasNormals = TRUE;
                    }

				    lwoCurr->surfTable[surf].renderState.mapping = TRUE;
                }
            }

            subI += subSubChunkSize;
            break;

        case CHUNK('V','M','A','P'):
            {
                UINT8 b = 0;
                INT32 j = 0;

                do
                {
                    b = LwoGetUINT8();
                    lwoCurr->surfTable[surf].txuvName[j] = b;
                    j++;
                } while (b!=0);
                
                if (j&1)
                {
                    SKIP_INT8(1);
                }
            }   
            subI += subSubChunkSize;
            break;
        
        default:
            subI += subSubChunkSize;
            SKIP_INT8(subSubChunkSize);
        }
    }

	return OK;
}

/*---------------------------------------------------------------------------- SURFACE */
RESULT
LwoParseSurf(void)
{
    INT32 i = 0;
    INT32 surf = 0;
    INT8 a = 0;
    INT8 name[STR_DEFAULT_SIZE] = {""};
    
    i = 0;

    do
    {
        a = LwoGetUINT8();
        name[i] = a;
        i++;
    } while (a!=0);

    a = LwoGetUINT8();
    i++;

    while (a==0)
    {
        a = LwoGetUINT8();
        i++;
    }
    BACK_INT8(1);
    i--;

    surf = 0;

    while (strcmp(name, lwoCurr->surfTable[surf].name)!=0)
    {
        surf++;
    }

    while (i<chunkSize)
    {
        subChunkID = LwoGetUINT32();
        subChunkSize = LwoGetUINT16();

        i += 6;

        switch (subChunkID)
        {
        case CHUNK('S','M','A','N'):
            if (LwoGetUINT32()!=0)
            {
#ifndef PS2
                lwoCurr->surfTable[surf].renderState.gouraud = TRUE;
                lwoCurr->hasNormals = TRUE;
#endif
            }

            i += subChunkSize;
            break;

        case CHUNK('B','L','O','K'):
            if (LwoParseSurfTexInfo(surf)==ERR_LWO_TEXTURE)
			{
				return ERR_LWO_TEXTURE;
			}

            i += subChunkSize;
            break;

        case CHUNK('S','I','D','E'):
            if (LwoGetUINT16()==3)
            {
                lwoCurr->surfTable[surf].renderState.doubleSided = TRUE;
            }

            i += subChunkSize;
            break;

        case CHUNK('T','R','A','N'):
            {
                INT32 i = 0;

                i = LwoGetUINT32();
                LwoGetVX();

                if (*(FLOAT32 *) &i>0.0f)
                {
                    lwoCurr->surfTable[surf].renderState.additive = TRUE;
                }
            }

            i += subChunkSize;
            break;

        case CHUNK('C','O','L','R'):
            {
                INT32 r = 0;
                INT32 g = 0;
                INT32 b = 0;

                r = LwoGetUINT32();
                g = LwoGetUINT32();
                b = LwoGetUINT32();
                LwoGetVX();

                lwoCurr->surfTable[surf].r = (UINT8) (*(FLOAT32 *) &r*255.0f);
                lwoCurr->surfTable[surf].g = (UINT8) (*(FLOAT32 *) &g*255.0f);
                lwoCurr->surfTable[surf].b = (UINT8) (*(FLOAT32 *) &b*255.0f);
            }

            i+=subChunkSize;
            break;

       default:
            SKIP_INT8(subChunkSize);
            i += subChunkSize;
            break;   
        }
    }   
	
	return OK;
}

