/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoparsetools.c - LWO Object Tools Parser                                  *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "system/mem.h"
#include "texture/tex.h"
#include "system/file.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoparse.h"
#include "lightwave/lwo/lwoparsetools.h"
#include "lightwave/lwo/lwoparsesurf.h"
#include "lightwave/lwo/lwoparsetex.h"

/*---------------------------------------------------------------------------- LOCAL FUNCTIONS */
static UINT16
getVX16(void)
{
    UINT16 a = 0;

    a = (UINT16) (*(lwoPtr-1)<<8);
    a |= LwoGetUINT8();

    return a;
}

static UINT32
getVX24(void)
{
    UINT32 a = 0;

    a = LwoGetUINT8()<<16;
    a |= LwoGetUINT8()<<8;
    a |= LwoGetUINT8();

    return a;
}

/*---------------------------------------------------------------------------- ORDERING */
void
LwoOrderChunks(UINT32 firstChunkID, UINT32 secondChunkID)
{
    INT32 prevOffset = 0;
    INT32 firstOffset = 0;
    INT32 secondOffset = 0;
    UINT32 chunkID = 0;
    INT32 size = 0;
    INT32 firstSize = 0;
    INT32 secondSize = 0;
    INT32 tempSize = 0;
    void *first = NULL;
    void *second = NULL;
    void *temp = NULL;
    
    prevOffset = GET_OFFSET;
    firstOffset = 0;
    secondOffset = 0;

    while (GET_OFFSET<lwoFile->size)
    {
        chunkID = LwoGetUINT32();
        size = LwoGetUINT32();

        if (chunkID==firstChunkID)
        {
            firstOffset = GET_OFFSET-8;
            firstSize = size+8;
        }
        else if (chunkID==secondChunkID)
        {
            secondOffset = GET_OFFSET-8;
            secondSize = size+8;
        }
            
        SKIP_INT8(size);
    }

    SET_OFFSET(prevOffset);

    if (firstOffset>secondOffset)
    {
        first = MemAlloc(firstSize);
        second = MemAlloc(secondSize);

        if ((tempSize=firstOffset-(secondOffset+secondSize))!=0)
        {
            temp = MemAlloc(tempSize);
        }

        memcpy(first, &lwoFile->buffer[firstOffset], firstSize);
        memcpy(second, &lwoFile->buffer[secondOffset], secondSize);

        if (tempSize!=0)
        {
            memcpy(temp, &lwoFile->buffer[secondOffset+secondSize], tempSize);
        }

        memcpy(&lwoFile->buffer[secondOffset], first, firstSize);

        if (tempSize!=0)
        {
            memcpy(&lwoFile->buffer[secondOffset+firstSize], temp, tempSize);
        }
        memcpy(&lwoFile->buffer[secondOffset+firstSize+tempSize], second, secondSize);
            
        SAFE_FREE(first);
        SAFE_FREE(second);

        if (tempSize!=0)
        {
            SAFE_FREE(temp);
        }
    }
}

/*---------------------------------------------------------------------------- POLY & TEXTURE COUNT */
RESULT
LwoParseGetPolyTexCount(void)
{
    INT32 offset = 0;
	INT32 prevOffset = 0;
    INT32 polsOffset = 0;
    INT32 polsSize = 0;
    INT32 size = 0;
    INT32 type = 0;
    INT32 i = 0;
	INT32 j = 0;
    INT32 count = 0;	
    
    if (lwoCurr==NULL)
    {
        return ERR_INVALID_PTR;
    }

    prevOffset = GET_OFFSET;

    lwoCurr->texCount = 0;
	lwoCurr->statSurfTotalCount = 0;
	lwoCurr->statTexTotalCount = 0;

    while (GET_OFFSET<lwoFile->size)
    {
        chunkID = LwoGetUINT32();
        chunkSize = LwoGetUINT32();

        switch (chunkID)
        {
		case CHUNK('S','U','R','F'):
			lwoCurr->statSurfTotalCount++;
			break;

        case CHUNK('C','L','I','P'):
            lwoCurr->statTexTotalCount++;
			lwoCurr->texCount++;
            break;

        case CHUNK('P','O','L','S'):
            polsSize = chunkSize;
            polsOffset = GET_OFFSET;
            break;
        }

        SKIP_INT8(chunkSize);
    }

    SET_OFFSET(polsOffset);

    type = LwoGetUINT32();
    size = polsSize;

    lwoCurr->polyCount = 0;

    if (type==CHUNK('F','A','C','E'))
    {
        i = 4;

        while (i<size)
        {
            offset = GET_OFFSET;

			count = LwoGetUINT16();
            count &= 1023;

            lwoCurr->polyCount++;

			for (j=0; j<count; j++)
			{
				LwoGetVX();
			}

            i += GET_OFFSET-offset;
        }
    }

    SET_OFFSET(prevOffset);

    return OK;
}

/*---------------------------------------------------------------------------- PARSE */
UINT8
LwoGetUINT8(void)
{
    return *(lwoPtr++);
}

UINT16
LwoGetUINT16(void)
{
    UINT16 a = 0;

    a = (UINT16) (LwoGetUINT8()<<8);
    a |= LwoGetUINT8();
    
    return a;
}

UINT32
LwoGetUINT32(void)
{
    UINT32 a = 0;

    a = LwoGetUINT8()<<24;
    a |= LwoGetUINT8()<<16;
    a |= LwoGetUINT8()<<8;
    a |= LwoGetUINT8();
    
    return a;
}

UINT32
LwoGetVX(void)
{
    if (LwoGetUINT8()<0xFF)
    {
        return getVX16();
    }

    return getVX24();
}
