/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwoparsevmap.c - LWO Object Vertex Mapping Parser                          *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "system/file.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoparse.h"
#include "lightwave/lwo/lwoparsetools.h"
#include "lightwave/lwo/lwoparsesurf.h"
#include "lightwave/lwo/lwoparsetex.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT32 subI = 0;

/*---------------------------------------------------------------------------- MORF */
static void
lwoParseMORF(void)    
{
    EndoMorph *endoMorph = NULL;
    UINT8 *oldLwoPtr = NULL;
    UINT32 vert = 0;
    UINT8 b = 0;
    INT32 x = 0;
    INT32 y = 0;
    INT32 z = 0;
    INT32 j = 0;

    endoMorph = &lwoCurr->endoMorph[lwoCurr->endoMorphCount];
    
    subI = 6;
    do
    {
        b = LwoGetUINT8();
        endoMorph->name[subI-6] = b;
        subI++;
    } while (b!=0);

    if (subI&1)
    {
        SKIP_INT8(1);
        subI++;
    }

    endoMorph->vertTable = MemAlloc(lwoCurr->vertCount*sizeof(Vert));
    for (j=0; j<lwoCurr->vertCount; j++)
    {
        endoMorph->vertTable[j].x = 0.0f;
        endoMorph->vertTable[j].y = 0.0f;
        endoMorph->vertTable[j].z = 0.0f;
    }

    while (subI<(chunkSize-6))
    {
        oldLwoPtr = lwoPtr;
        
        vert = LwoGetVX();
        
        x = LwoGetUINT32();  
        y = LwoGetUINT32();
        z = LwoGetUINT32();

        endoMorph->vertTable[vert].x = *(FLOAT32 *)&x;
        endoMorph->vertTable[vert].y = *(FLOAT32 *)&y;
        endoMorph->vertTable[vert].z = -*(FLOAT32 *)&z;

        subI += lwoPtr-oldLwoPtr;
    }

    lwoCurr->endoMorphCount++;
}

/*---------------------------------------------------------------------------- UV */
static void
lwoParseUV(void)
{
    TexUV *txuv = NULL;
    UINT8 b = 0;
    UINT32 vert = 0;
    INT32 u = 0;
    INT32 v = 0;
    INT32 j = 0;
    UINT8 *oldOldLwoPtr = NULL;
    
    txuv = &lwoCurr->txuv[lwoCurr->txuvCount];
    
    subI = 6;
    do
    {
        b = LwoGetUINT8();
        txuv->name[subI-6] = b;
        subI++;
    } while (b!=0);
    
    if (subI&1)
    {
        SKIP_INT8(1);
        subI++;
    }

    txuv->uvProjTable = MemAlloc(sizeof(UVProj)*lwoCurr->vertCount);

    j = subI;

    do
    {
        oldOldLwoPtr = lwoPtr;
        
        vert = LwoGetVX();

        u = LwoGetUINT32();  
        v = LwoGetUINT32();

        txuv->uvProjTable[vert].u = *(FLOAT32 *)&u;
        txuv->uvProjTable[vert].v = *(FLOAT32 *)&v;

        j += lwoPtr-oldOldLwoPtr;
    } while (j<chunkSize);
    
    lwoCurr->txuvCount++;
}

/*---------------------------------------------------------------------------- DISCONTINUOUS UV */
static void
lwoParseDiscUV(void)
{
    INT8 name[STR_DEFAULT_SIZE] = {""};
    TexUV *txuv = NULL;                
    UINT8 b = 0;
    INT32 vert = 0;
    INT32 poly = 0;
    INT32 u = 0;
    INT32 v = 0;
    INT32 j = 0;
    UINT8 *oldLwoPtr = NULL;
    UINT8 *oldOldLwoPtr = NULL;
    INT32 oldSubI = 0;
    
    subI = 6;
    do
    {
        b = LwoGetUINT8();
        name[subI-6] = b;
        subI++;
    } while(b != 0);

    if (subI&1)
    {
        SKIP_INT8(1);
        subI++;
    }

    j = 0;
    txuv = &lwoCurr->txuv[j];

    while (strcmp(txuv->name, name)!=0)
    {
        j++;
        txuv = &lwoCurr->txuv[j];
    }

    oldSubI = subI;
    oldLwoPtr = lwoPtr;
    j = subI;
    txuv->uvProjAddCount = 0;

    do
    {
        oldOldLwoPtr = lwoPtr;
        
        LwoGetVX();
        LwoGetVX();

        SKIP_INT8(8);

        txuv->uvProjAddCount++;

        j += lwoPtr-oldOldLwoPtr;
    } while (j<chunkSize);
    
    subI = oldSubI;
    lwoPtr = oldLwoPtr;

    txuv->uvProjAddTable = MemAlloc(sizeof(UVProj)*txuv->uvProjAddCount);

    for (j=0; j<txuv->uvProjAddCount; j++)
    {
        vert = LwoGetVX();
        poly = LwoGetVX();

        u = LwoGetUINT32();  
        v = LwoGetUINT32();

        txuv->uvProjAddTable[j].vertIndex = vert;
        txuv->uvProjAddTable[j].polyIndex = poly;
        txuv->uvProjAddTable[j].u = *(FLOAT32 *) &u;
        txuv->uvProjAddTable[j].v = *(FLOAT32 *) &v;
    }
}

/*---------------------------------------------------------------------------- VERTEX MAPPING */
RESULT
LwoParseVMAP(void)
{
    INT32 i = 0;
    INT32 subChunkID = 0;
    INT32 subChunkSize = 0;

    while (i<(chunkSize+6))
    {
        subChunkID = LwoGetUINT32();
        subChunkSize = LwoGetUINT16();

        i += 6;

        switch (subChunkID)
        {
        case CHUNK('W','G','H','T'):
            SKIP_INT8(chunkSize-i);

            i += chunkSize;
            break;
        
        case CHUNK('M','O','R','F'):
            lwoParseMORF();
            
            i += chunkSize;
            
            break;
        
        case CHUNK('T','X','U','V'):
            lwoParseUV();
            
            i += chunkSize;
            break;

        default:
            SKIP_INT8(chunkSize);

            i += chunkSize;
            break;                          
        }
    }   

    return OK;
}

/*---------------------------------------------------------------------------- DISCONTINUOUS VERTEX MAPPING */
RESULT
LwoParseVMAD(void)
{
    INT32 i = 0;
    INT32 subChunkID = 0;
    INT32 subChunkSize = 0;

    while (i<chunkSize)
    {
        subChunkID = LwoGetUINT32();
        subChunkSize = LwoGetUINT16();

        i += 6;

        switch (subChunkID)
        {
        case CHUNK('T','X','U','V'):
            lwoParseDiscUV();
            
            i += chunkSize;
            break;

        case CHUNK('M','O','R','F'):  
            ConsoleAdd("Warning : UVMAP mapping AND EndoMorph used simultaneously are not supported !", lwoFile->fileName);
                
            SKIP_INT8(chunkSize - 6);
            
            i += chunkSize;
            break;

        default:
            SKIP_INT8(chunkSize - 6);
            i += chunkSize;
            break;                          
        }
    }   

    return OK;
}
