/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lws.c - LWS Scene                                                          *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/timer.h"
#include "platform/os/console.h"
#include "system/mem.h"
#include "system/file.h"
#include "system/file.h"
#include "system/filepack.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsparse.h"
#include "lightwave/lws/lwsinit.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static File lwsFile;
static Lws *lwsCurr = NULL;

/*---------------------------------------------------------------------------- INIT */
RESULT
LwsLoad(Lws *lws, INT8 *fileName, void (*progress)(INT32 i))
{
    if (lws==NULL || fileName==NULL)
    {
        return ERR_INVALID_PTR;
    }

    strcpy(lws->fileName, fileName);

    lws->timeLoad = 0;
        
    if (FileLoad(&lwsFile, fileName)!=OK)
    {
        ConsoleAdd("Error : %s can not be opened !", lws->fileName);
        
        return ERR_LWS_LOAD;
    }

	lwsCurr = lws;

    lwsCurr->blockCount = 0;
    lwsCurr->lwoCount = 0;
	lwsCurr->offsetUV = 0.0f;
    lwsCurr->focalAmount = 3.2f;
    lwsCurr->triCount = 0;
    lwsCurr->viewLines = FALSE;
	lwsCurr->usePreca = FALSE;

	if (LwsParse(progress)!=OK)
    {
        FileFree(&lwsFile);

        ConsoleAdd("Error : there is a parsing problem with %s !", lwsCurr->fileName);
        
        return ERR;
    }

    if (LwsInit(lws)!=OK)
    {
        FileFree(&lwsFile);

        ConsoleAdd("Error : there is an initialization problem with %s !", lwsCurr->fileName);
        
        return ERR;
    }

    FileFree(&lwsFile);
    
    lwsCurr->timeLoad = 0;

    return OK;
}

/*---------------------------------------------------------------------------- FREE */
RESULT
LwsFree(Lws *lws)
{
    Block *block = NULL;
    
    if (lws==NULL)
    {
        return ERR_INVALID_PTR;
    }

    block = lws->blockObj;

    while (block!=NULL)
    {
        LwoFree(block->lwo);

        block = block->next;
    }

	if (lws->usePreca)
	{
		FileFree(&lws->precaFile);
	}

    return OK;
}

/*---------------------------------------------------------------------------- SET PRECA FILE */
RESULT
LwsSetPreca(Lws *lws, INT8 *fileName)
{
	INT8 precaFileName[STR_DEFAULT_SIZE];
	
	if (lws==NULL || fileName==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (strlen(fileName)>0)
	{
		strcpy(precaFileName, fileName);

		if (FileLoad(&lws->precaFile, precaFileName)==OK)
		{
			/*
			INT32 unpackedSize;
			UINT8 *unpackedBuffer = NULL;

			unpackedSize = FileGetUnpackedSize((UINT8 *) &lws->precaFile.buffer[4]);
			unpackedBuffer = (UINT8 *) MemAlloc(unpackedSize);
    
			FileUnpack((UINT8 *) &lws->precaFile.buffer[4], unpackedBuffer);

			MemFree(lws->precaFile.buffer);
				
			lws->precaFile.buffer = (INT8 *) unpackedBuffer;
			lws->precaFile.size = unpackedSize;			
			*/
			
			lws->usePreca = TRUE;

			ConsoleAdd("Precaching file found");
		}
		else
		{
			lws->usePreca = FALSE;
		}
	}

	return OK;
}		
	
/*---------------------------------------------------------------------------- ACCESSORS */
Lws *
LwsGetCurrent(void)
{
    return lwsCurr;
}

File *
LwsGetCurrentFile(void)
{
    return &lwsFile;
}

