/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/******************************************************************************
 * lws.h - LWS Scene                                                          *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#ifndef _LWS_
#define _LWS_

/*---------------------------------------------------------------------------- DEFINE */
#define LWS_MAX_OBJ         300
#define LWS_MAX_BLOCK       300
#define LWS_MAX_PARENT      200

/*---------------------------------------------------------------------------- ENUM */
typedef enum
{
    MotionUndef,
    MotionObj,
    MotionLight,
    MotionCam
} MotionType;

typedef enum
{
    ParentUndef,
    ParentObj,
    ParentLight,
    ParentCam
} ParentType;

/*---------------------------------------------------------------------------- STRUCT */
typedef struct LwsEndoMorph
{
    INT8 name[STR_DEFAULT_SIZE];
    Env *env;
} LwsEndoMorph;

typedef struct Block
{
    MotionType motionType;

    INT8 name[STR_DEFAULT_SIZE];

    Lwo *lwo;
    Light *light;
    Cam *cam;
    
    ParentType parentType;
    INT32 parentItem;

    Env *envMorph;
    INT32 morphTarget;
    FLOAT32 morphAmount;

    INT32 endoMorphCount;
    LwsEndoMorph endoMorph[LWO_MAX_ENDOMORPH];
    
    INT32 useDissolve;
    Env *envDissolve;
    FLOAT32 dissolveAmount;
    
    INT32 chanCount;
    Env *env;
    Env *envDiss;
    
    FLOAT32 pivotX;
    FLOAT32 pivotY;
    FLOAT32 pivotZ;

    struct Block *next;
} Block;

typedef struct Lws
{
    INT8 fileName[STR_DEFAULT_SIZE];
    INT32 timeLoad;

    FLOAT32 firstFrame;
    FLOAT32 lastFrame;
    FLOAT32 frameStep;
    FLOAT32 framesPerSecond;

    INT32 flag;

    FLOAT32 gridNumber;
	FLOAT32 gridSize;
    FLOAT32 sizeCoef;
    
    FLOAT32 backDropR;
    FLOAT32 backDropG;
    FLOAT32 backDropB;

    FLOAT32 ambientIntens;
    FLOAT32 ambientR;
    FLOAT32 ambientG;
    FLOAT32 ambientB;

    FLOAT32 time;

    Block *blockObj;
    Block *blockLight;
    Block *blockCam;

    INT32 tickBase;
    INT32 tickEnd;

    INT32 lwoCount;
    Lwo *lwoTable[LWS_MAX_OBJ];

    INT32 blockCount;
    Block *blockTable[LWS_MAX_BLOCK];

    Env *envFocal;
    FLOAT32 focalAmount;
    
    INT32 frameResX;
    INT32 frameResY;
    INT32 viewPortResX;
    INT32 viewPortResY;

	FLOAT32 offsetUV;

    BOOL viewLines;
	
	INT32 triCount;

	BOOL usePreca;
	File precaFile;
} Lws;

/*---------------------------------------------------------------------------- GLOBAL FUNCTIONS */
RESULT LwsLoad(Lws *lws, INT8 *fileName, void (*progress)(INT32 i));
RESULT LwsFree(Lws *lws);

RESULT LwsSetPreca(Lws *lws, INT8 *fileName);

Lws *LwsGetCurrent(void);
File *LwsGetCurrentFile(void);

/*---------------------------------------------------------------------------- END OF DECLARATION */
#endif /* _LWS_ */
