/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwsparse.c - LWS Scene Parser                                              *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsparse.h"
#include "lightwave/lws/lwsparsetools.h"
#include "lightwave/lws/lwsinit.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define LWS_MOTION_BLUR (1<<0)
#define LWS_ANTI_ALIASING (1<<1)

/*---------------------------------------------------------------------------- GLOBAL VARIABLES */
UINT8 *lwsPtr = NULL;
INT8 *lwsLinePtr = NULL;
INT8 lwsLine[STR_DEFAULT_SIZE] = {""};
INT8 cmd[STR_DEFAULT_SIZE] = {""};
INT8 param1[STR_DEFAULT_SIZE] = {""};
INT8 param2[STR_DEFAULT_SIZE] = {""};
INT8 param3[STR_DEFAULT_SIZE] = {""};
INT8 param4[STR_DEFAULT_SIZE] = {""};
INT8 param5[STR_DEFAULT_SIZE] = {""};
INT8 param6[STR_DEFAULT_SIZE] = {""};
INT8 param7[STR_DEFAULT_SIZE] = {""};
INT8 param8[STR_DEFAULT_SIZE] = {""};
INT8 param9[STR_DEFAULT_SIZE] = {""};
INT8 param10[STR_DEFAULT_SIZE] = {""};
INT32 paramsCount = 0;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT8 group[STR_DEFAULT_SIZE] = {""};
static Block *currBlock = NULL;
static File *lwsFile = NULL;
static Lws *lws = NULL;

static INT32 objTotalCount = 0;
static INT32 objCount = 0;    

void (*lwsProgress)(INT32 i) = NULL;

/*---------------------------------------------------------------------------- PROGRESS */
static void
lwoProgress(INT32 i)
{
	INT32 p = 0;
	
	p = (INT32) ((FLOAT32) ((objCount+((FLOAT32) i)/100.0f)/(FLOAT32) objTotalCount)*100.0f);

	if (lwsProgress!=NULL)
	{
		lwsProgress(p);
	}
}
				
/*---------------------------------------------------------------------------- ADD BLOCK */
static void 
addBlock(Block **block, MotionType motionType)
{
    INT32 i = 0;
    
    Block *add = NULL;
    Block *node = NULL;

    add = MemAlloc(sizeof(Block));

    *block = add;
    
    add->parentType = ParentUndef;
    add->parentItem = 0;
    add->useDissolve = FALSE;

    add->endoMorphCount = 0;
    for (i=0; i<LWO_MAX_ENDOMORPH; i++)
    {
        add->endoMorph[i].name[0] = 0;
        add->endoMorph[i].env = NULL;
    }

    lws->blockTable[lws->blockCount] = add;
    lws->blockCount++;
    
    switch (motionType)
    {
    case MotionObj:
        if (lws->blockObj==NULL)
        {
            lws->blockObj = add;
            return;
        }
        node = lws->blockObj;
        break;

    case MotionLight:
        if (lws->blockLight==NULL)
        {
            lws->blockLight = add;
            return;
        }
        node = lws->blockLight;
        break;

    case MotionCam:
        if (lws->blockCam==NULL)
        {
            lws->blockCam = add;
            return;
        }
        node = lws->blockCam;
        break;

    case MotionUndef:
        break;
    }

    while (node->next!=NULL)
    {
        node = node->next;
    }
    node->next = add;
}

/*---------------------------------------------------------------------------- ENVELOPPE PARSE */
static void 
envParse(Env *env)
{
    INT32 i = 0;
    Key *baseKey = NULL;

    getLine();

    getLine();   
    LwsComputeParams();
    
    env->nkeys = getINT32(cmd);

    if (env->nkeys==0)
    {
        env->nkeys = 1;
    }

    env->key = MemAlloc(sizeof(Key));
    baseKey = env->key;
    env->key->prev = NULL;

    for (i=0; i<env->nkeys; i++)
    {
        getLine();
        LwsComputeParams();
    
        env->key->value = getFLOAT32(param1);
        env->key->time = getFLOAT32(param2);
        env->key->shape = getINT32(param3);

        switch (env->key->shape)
        {
            case SHAPE_TCB:
                env->key->tension = getFLOAT32(param4);
                env->key->continuity = getFLOAT32(param5);
                env->key->bias = getFLOAT32(param6);
                break;

            case SHAPE_HERM:
            case SHAPE_BEZI:
                env->key->param[0] = getFLOAT32(param7);
                env->key->param[1] = getFLOAT32(param8);
                break;

            case SHAPE_BEZ2:
                env->key->param[0] = getFLOAT32(param4);
                env->key->param[1] = getFLOAT32(param5);
                env->key->param[2] = getFLOAT32(param6);
                env->key->param[3] = getFLOAT32(param7);
                break;
        }

        env->key->next = MemAlloc(sizeof(Key));
        env->key->next->prev = env->key;
        env->key = env->key->next;
    }
    
    if (env->nkeys>0)
    {
        env->key->prev->next = NULL;
    }
    SAFE_FREE(env->key);
    env->key = baseKey;

    getLine();
    LwsComputeParams();

    env->behavior[0] = getINT32(param1);
    env->behavior[1] = getINT32(param2);
}

/*---------------------------------------------------------------------------- PARSER */
RESULT
LwsParse(void (*progress)(INT32 i))
{
    FLOAT32 zoomFactor = 0.0f;
    
    lws = LwsGetCurrent();
    lwsFile = LwsGetCurrentFile();

    if (lws==NULL || lwsFile==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (progress!=NULL)
    {
        objCount = 0;
        objTotalCount = 0;
        
        lwsPtr = (UINT8 *) lwsFile->buffer;
    
        getLine();

        while (getOffset()<lwsFile->size)
        {
            LwsComputeParams();

            if (qstrcmp(cmd, "LoadObjectLayer")==0)
            {
                objTotalCount++;
            }
            else
            if (qstrcmp(cmd, "ViewZoomFactor")==0)
            {
                zoomFactor = getFLOAT32(param1);
            }
            else
            if (qstrcmp(cmd, "GridNumber")==0)
            {
                lws->gridNumber = getFLOAT32(param1);
            }
			else
            if (qstrcmp(cmd, "GridSize")==0)
            {
                lws->gridSize = getFLOAT32(param1);
                lws->sizeCoef = (10.0f/lws->gridSize)*zoomFactor;
            }

            getLine();
        }

        lwsProgress = progress;
		
		lwsProgress(0);
    }

    group[0] = 0x00;
    
    lwsPtr = (UINT8 *) lwsFile->buffer;
    
    getLine();
    while (getOffset()<lwsFile->size)
    {       
        LwsComputeParams();

        if (qstrcmp(cmd, "PreviewFirstFrame")==0)
        {
            lws->firstFrame = (FLOAT32) getINT32(param1);
        }
        else if (qstrcmp(cmd, "PreviewLastFrame")==0)
        {
            lws->lastFrame = (FLOAT32) getINT32(param1);
        }
        else if (qstrcmp(cmd, "PreviewFrameStep")==0)
        {
            lws->frameStep = (FLOAT32) getINT32(param1);
        }
        else if (qstrcmp(cmd, "FramesPerSecond")==0)
        {
            lws->framesPerSecond = (FLOAT32) getINT32(param1);
        }
        else if (qstrcmp(cmd, "LoadObjectLayer")==0)
        {
            INT8 fileName[STR_DEFAULT_SIZE] = {""};
            INT32 i = 0;

            addBlock(&currBlock, MotionObj);

            currBlock->lwo = MemAlloc(sizeof(Lwo));

			i = 0;			
			while (lwsLine[i++]!=' ');
			while (lwsLine[i++]!=' ');

            strcpy(fileName, (INT8 *) &lwsLine[i]);

            i = 0;
            while (fileName[i++]!='.')
            {
                if (fileName[i]=='/')
                {
                    fileName[i] = '\\';
                }
            }

            if (fileName[1]=='\\')
            {
                INT8 fileNameAddDrive[STR_DEFAULT_SIZE];

                fileNameAddDrive[0] = fileName[0];
                fileNameAddDrive[1] = fileName[1];
                fileNameAddDrive[2] = '\\';
                fileNameAddDrive[3] = 0;
                strcat(fileNameAddDrive, &fileName[2]);
                strcpy(fileName, fileNameAddDrive);
            }

            if (LwoLoad(currBlock->lwo, fileName, lws->sizeCoef, lwoProgress)!=OK)
            {
                ConsoleAdd("Error : can not load %s !", fileName);
            
                return ERR;
            }

            lws->lwoTable[lws->lwoCount] = currBlock->lwo;
            lws->lwoCount++;

            if (lwsProgress!=NULL)
            {
                objCount++;
                
                lwsProgress((INT32) (((FLOAT32) objCount/(FLOAT32) objTotalCount)*100.0f));
            }

            lws->triCount += currBlock->lwo->triExCount;
        }
        else if (qstrcmp(cmd, "AddNullObject")==0)
        {
            addBlock(&currBlock, MotionObj);

            currBlock->lwo = MemAlloc(sizeof(Lwo));
			currBlock->lwo->isNull = TRUE;			

			currBlock->lwo->sizeCoef = lws->sizeCoef;
			strcpy(currBlock->lwo->fileName, param1);

            lws->lwoTable[lws->lwoCount] = currBlock->lwo;
            lws->lwoCount++;
        }
        else if (qstrcmp(cmd, "AddLight")==0)
        {
            addBlock(&currBlock, MotionLight);

            currBlock->light = MemAlloc(sizeof(Light));
        }
        else if (qstrcmp(cmd, "AddCamera")==0)
        {
            addBlock(&currBlock, MotionCam);

            currBlock->cam = MemAlloc(sizeof(Cam));
        }
        else if (qstrcmp(cmd, "LightName")==0)
        {
            strcpy(currBlock->light->name, param1);
        }
        else if (qstrcmp(cmd, "CameraName")==0)
        {
            strcpy(currBlock->cam->name, param1);
        }
        else if (qstrcmp(cmd, "ObjectMotion")==0)
        {
            currBlock->motionType = MotionObj;
        }
        else if (qstrcmp(cmd, "LightMotion")==0)
        {
            currBlock->motionType = MotionLight;
        }
        else if (qstrcmp(cmd, "CameraMotion")==0)
        {
            currBlock->motionType = MotionCam;
        }
        else if (qstrcmp(cmd, "LightIntensity")==0)
        {
            if (qstrcmp(param1, "(envelope)")==0)
            {
                currBlock->light->envIntens = MemAlloc(sizeof(Env));

                envParse(currBlock->light->envIntens);
            }
            else
            {   
                currBlock->light->intens = getFLOAT32(param1);
            }
        }
        else if (qstrcmp(cmd, "MotionBlur")==0)
        {
            if (param1[0]=='1')
            {
                lws->flag |= LWS_MOTION_BLUR;
            }
        }
        else if (qstrcmp(cmd, "Antialiasing")==0)
        {
            if (param1[0]=='1')
            {
                lws->flag |= LWS_ANTI_ALIASING;
            }
        }
        else if (qstrcmp(cmd, "FrameSize")==0)
        {
            lws->frameResX = getINT32(param1);
            lws->frameResY = getINT32(param2);
        }
        else if (qstrcmp(cmd, "LightType")==0)
        {
            switch(param1[0])
            {
            case '0':
                currBlock->light->type = LightDistant;
                break;

            case '1':
                currBlock->light->type = LightPoint;
                break;

            case '2':
                currBlock->light->type = LightSpot;
                break;
            }
        }
        else if (qstrcmp(cmd, "PivotPosition")==0)
        {
            currBlock->pivotX = getFLOAT32(param1)*lws->sizeCoef;
            currBlock->pivotY = getFLOAT32(param2)*lws->sizeCoef;
            currBlock->pivotZ = getFLOAT32(param3)*lws->sizeCoef;
        }
        else if (qstrcmp(cmd, "BackdropColor")==0)
        {
            lws->backDropR = getFLOAT32(param1);
            lws->backDropG = getFLOAT32(param2);
            lws->backDropB = getFLOAT32(param3);
        }
        else if (qstrcmp(cmd, "AmbientColor")==0)
        {
            lws->ambientR = getFLOAT32(param1);
            lws->ambientG = getFLOAT32(param2);
            lws->ambientB = getFLOAT32(param3);
        }
        else if (qstrcmp(cmd, "AmbientIntensity")==0)
        {
            lws->ambientIntens = getFLOAT32(param1);
        }
        else if (qstrcmp(cmd, "LightColor")==0)
        {
            currBlock->light->colorR = getFLOAT32(param1);
            currBlock->light->colorG = getFLOAT32(param2);
            currBlock->light->colorB = getFLOAT32(param3);
        }
        else if (qstrcmp(cmd, "NumChannels")==0)
        {
            currBlock->chanCount = getINT32(param1);

            currBlock->env = MemAlloc(currBlock->chanCount*sizeof(Env));
        }
        else if (qstrcmp(cmd, "MorphAmount")==0)
        {
            if (qstrcmp(param1, "(envelope)")==0)
            {
                currBlock->envMorph = MemAlloc(sizeof(Env));

                envParse(currBlock->envMorph);
            }
            else
            {   
                currBlock->morphAmount = getFLOAT32(param1);
            }
        }
        else if (qstrcmp(cmd, "ObjectDissolve")==0)
        {
            currBlock->useDissolve = TRUE;
                
            if (qstrcmp(param1, "(envelope)")==0)
            {
                currBlock->envDissolve = MemAlloc(sizeof(Env));

                envParse(currBlock->envDissolve);
            }
            else
            {   
                currBlock->dissolveAmount = getFLOAT32(param1);
            }
        }
        else if (qstrcmp(param1, "Group")==0)
        {
            getLine();

            getLine();
            LwsComputeParams();

            strcpy(group, &cmd[1]);

            group[strlen(group)-1] = 0x00;
        }
        else if (qstrcmp(param1, "MorfForm")==0)
        {
            INT8 tmp[STR_DEFAULT_SIZE];
            INT32 i = 0;
            
            getLine();
            LwsComputeParams();
            
            i = currBlock->endoMorphCount;
            
            currBlock->endoMorph[i].env = MemAlloc(sizeof(Env));

            strcpy(currBlock->endoMorph[i].name, &cmd[1]);

            if (cmd[strlen(currBlock->endoMorph[i].name)]!='"')
            {
                strcat(currBlock->endoMorph[i].name, " ");
                strcat(currBlock->endoMorph[i].name, param1);
            }

            currBlock->endoMorph[i].name[strlen(currBlock->endoMorph[i].name)-1] = 0x00;
                
            strcpy(tmp, currBlock->endoMorph[i].name);

            strcpy(currBlock->endoMorph[i].name, group);
            strcat(currBlock->endoMorph[i].name, ".");
            strcat(currBlock->endoMorph[i].name, tmp);

            getLine();
            
            envParse(currBlock->endoMorph[i].env);

            currBlock->endoMorphCount++;
        }
        else if (qstrcmp(cmd, "ZoomFactor")==0)
        {
            if (qstrcmp(param1, "(envelope)")==0)
            {
                lws->envFocal = MemAlloc(sizeof(Env));

                envParse(lws->envFocal);
            }
            else
            {   
                lws->focalAmount = getFLOAT32(param1);
            }
        }
        else if (qstrcmp(cmd, "MorphTarget")==0)
        {
            currBlock->morphTarget = getINT32(param1);
        }
        else if (qstrcmp(cmd, "ParentItem")==0)
        {
            {
                INT32 i = 0;
                INT8 *p = NULL;

                p = &param1[6];
                
                if (*p>='1' && *p<='9')
                {
                    i += (*p-'0')*16;
                }
            
                p = &param1[7];
                
                if (*p>='0' && *p<='9')
                {
                    i += (*p)-'0';
                }
                else if (*p>='a' && *p<='f')
                {
                    i += (*p)-'a'+10;
                }
                   
                currBlock->parentItem = i+1;
            }

            switch (param1[0])
            {
            case '1':
                currBlock->parentType = ParentObj;
                break;

            case '2':
                currBlock->parentType = ParentLight;
                break;

            case '3':
                currBlock->parentType = ParentCam;
                break;
            }
        }
        else if (qstrcmp(cmd, "ObjectDissolve")==0)
        {
            currBlock->envDiss = MemAlloc(sizeof(Env));

            envParse(currBlock->envDiss);
        }
        else if(qstrcmp(cmd, "Channel")==0)
        {
            INT32 z = 0;
            
            z = getINT32(param1);            

            if (z>=0 && z<10)
            {
                envParse(&currBlock->env[z]);
            }
        }
        
        getLine();
    }

    return OK;
}
