/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwsparsetools.c - LWS Scene Tools Parser                                   *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsparse.h"
#include "lightwave/lws/lwsinit.h"

/*---------------------------------------------------------------------------- STRCMP */
INT32
qstrcmp(INT8 *s1, INT8 *s2)
{
	while (*s1!=0)
	{
		if (*s2==0)
		{
			return -1;
		}

		if (*s1!=*s2)
		{
			return -1;
		}

		s1++;
		s2++;
	}

	if (*s2!=0)
	{
		return -1;
	}
	
	return 0;
}

/*---------------------------------------------------------------------------- GET OFFSET */
INT32
getOffset(void)
{
	return (INT32) (lwsPtr-(UINT8 *) LwsGetCurrentFile()->buffer);
}

/*---------------------------------------------------------------------------- GET LINE */
void 
getLine(void)
{
	UINT8 *endPtr = (UINT8 *) LwsGetCurrentFile()->buffer+LwsGetCurrentFile()->size;
    
    lwsLinePtr = lwsLine;

	while (lwsPtr<endPtr && *lwsPtr!=0x0D && *lwsPtr!=0x0A)
    {
        *lwsLinePtr = *lwsPtr;

        lwsLinePtr++;

        lwsPtr++;
    }

	*lwsLinePtr = 0;

	lwsPtr++;
    
    if (lwsPtr<endPtr && *lwsPtr==0x0A)
    {
        lwsPtr++;
    }
}

/*---------------------------------------------------------------------------- GET INT32 */
INT32 
getINT32(INT8 *s)
{
    INT32 a = 0;
    BOOL nega = FALSE;

    while (*s==32)
    {
        s++;
    }

    if (*s=='-')
    {
        s++;
        nega = TRUE;
    }
    
    while (*s>='0' && *s<='9')
    {
        a *= 10;
        a += *s-'0';

        s++;
    }

    if (nega==FALSE)
    {
        return a;
    }

    return -a;
}

/*---------------------------------------------------------------------------- GET FLOAT32 */
FLOAT32 
getFLOAT32(INT8 *s)
{
    FLOAT32 a = 0.0f;
    BOOL nega = FALSE;
    INT32 i = 0;
    INT8 *sBase = s;
    
    while (*s==' ')
    {
        s++;
    }

    if (*s=='-')
    {
        s++;
        nega = TRUE;
    }
    
    while (*s>='0' && *s<='9')
    {
        a *= 10.0f;
        a += (FLOAT32) (*s-'0');
        s++;
    }

	if (*s=='e' || *s=='E' || *s=='d' || *s=='D')
    {
#ifdef PS2
        sBase = NULL;
        
        ConsoleAdd("Error : too complex float value !");
        
        return (FLOAT32) 0.0f;
#else
        return (FLOAT32) atof(sBase);
#endif
    }
	
    if (*s=='.')
    {
		s++;

		while (*s>='0' && *s<='9')
		{
			a *= 10.0f;
			a += (FLOAT32) (*s-'0');

			s++;
			i++;
		}

		if (*s=='e' || *s=='E' || *s=='d' || *s=='D')
		{
#ifdef PS2
        sBase = NULL;
        
        ConsoleAdd("Error : too complex float value !");
        
        return (FLOAT32) 0.0f;
#else
        return (FLOAT32) atof(sBase);
#endif
		}

		while (i--!=0)
		{
			a *= 0.1f;
		}
	}

    if (nega==FALSE)
    {
        return a;
    }

    return -a;
}

/*---------------------------------------------------------------------------- COMPUTE PARAMS */
void 
LwsComputeParams(void)
{
    INT8 *ptr = NULL;
    BOOL cont = TRUE;

    lwsLinePtr = lwsLine;
    paramsCount = 0;

    cmd[0] = 0;
    param1[0] = 0;
    param2[0] = 0;
    param3[0] = 0;
    param4[0] = 0;
    param5[0] = 0;
    param6[0] = 0;
    param7[0] = 0;
    param8[0] = 0;
    param9[0] = 0;
    param10[0] = 0;

    while (cont)
    {
        switch (paramsCount)
        {
            case 0: ptr = cmd;      break;
            case 1: ptr = param1;   break;
            case 2: ptr = param2;   break;
            case 3: ptr = param3;   break;
            case 4: ptr = param4;   break;
            case 5: ptr = param5;   break;
            case 6: ptr = param6;   break;
            case 7: ptr = param7;   break;
            case 8: ptr = param8;   break;
            case 9: ptr = param9;   break;
            case 10: ptr = param10; break;
        }

        while (*(lwsLinePtr++)==' ');
        lwsLinePtr--;

        while (*lwsLinePtr!=0 && *lwsLinePtr!=' ')
        {
            *(ptr++) = *(lwsLinePtr++);
        }
        *ptr = 0;

        if (*(lwsLinePtr++)==0)
        {
            cont = FALSE;
        }

        paramsCount++;
    }
}