/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * playlist.c - Playlist                                                      *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/window.h"
#include "system/file.h"
#include "system/filearc.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsplaylist.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define PLAYLIST_MAX_LWS 64

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT32 lwsCount = 0;
static INT32 pos = 0;
static PlayList table[PLAYLIST_MAX_LWS];

/*---------------------------------------------------------------------------- ADD */
RESULT 
PlayListAdd(const INT8 *fileName)
{
    if (fileName==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (lwsCount>=PLAYLIST_MAX_LWS)
    {
        return ERR_PLAYLIST_FULL;
    }

    table[lwsCount].isValid = FALSE;
    strcpy((INT8 *) table[lwsCount].fileName, fileName);

    lwsCount++;

    return OK;
}

/*---------------------------------------------------------------------------- REMOVE ALL */
RESULT 
PlayListRemoveAll(void)
{
    if (lwsCount==0)
    {
        return ERR_PLAYLIST_EMPTY;
    }

    memset(table, 0, sizeof(PlayList)*PLAYLIST_MAX_LWS);

    lwsCount = 0;
    
    return OK;
}

/*---------------------------------------------------------------------------- REMOVE */
RESULT 
PlayListRemove(const INT8 *fileName)
{
    INT32 i = 0;
    
    if (fileName==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (lwsCount==0)
    {
        return ERR_PLAYLIST_EMPTY;
    }

    i = 0;
    while (strcmp(fileName, (INT8 *) table[i].fileName)!=0 && i<=lwsCount)
    {
        i++;
    }

    if (i==lwsCount)
    {
        return ERR_PLAYLIST_MISSING;
    }

    while (i<(PLAYLIST_MAX_LWS-1))
    {
        memcpy(&table[i], &table[i+1], sizeof(PlayList));

        i++;
    }

    memset(&table[PLAYLIST_MAX_LWS-1], 0, sizeof(PlayList));

    return OK;
}

/*---------------------------------------------------------------------------- SET */
RESULT 
PlayListSet(const INT32 i)
{
    if (pos>=0 && pos<lwsCount)
    {
        pos = i;
    }
    else
    {
        return ERR;
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- GET POS */
INT32 
PlayListGetPos(void)
{
    return pos;
}

/*---------------------------------------------------------------------------- NEXT */
RESULT 
PlayListNext(INT32 *i)
{
    if (i==NULL)
    {
        return ERR_INVALID_PTR;
    }
    
    pos++;

    if (pos>=lwsCount)
    {
        pos = 0;
    }

    *i = pos;
    
    return OK;
}

/*---------------------------------------------------------------------------- GET COUNT */
INT32 
PlayListGetCount(void)
{
    return lwsCount;
}

/*---------------------------------------------------------------------------- GET PLAYLIST */
PlayList *
PlayListGet(void)
{
    return &table[pos];
}

/*---------------------------------------------------------------------------- ADD ARCHIVE CONTENT */
RESULT 
PlayListAddArchive(void)
{
	INT32 i = 0;
	INT32 count = 0;
	INT8 fileName[STR_DEFAULT_SIZE];

	if (!FileArcIsPresent())
	{
		return ERR_ARCHIVE;
	}
	
	PlayListRemoveAll();

	FileArcGetNodeCount(&count);

	for (i=0; i<count; i++)
	{
		FileArcGetIndexedFilename(fileName, i);

		if (strstr(fileName, ".lws")!=NULL)
		{
			PlayListAdd(fileName);
		}
		else if (strstr(fileName, ".LWS")!=NULL)
		{
			PlayListAdd(fileName);
		}
	}
	
	return OK;
}