/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * entry.c - Entry                                                            *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/software/soft.h"
#include "platform/software/softdraw.h"
#include "platform/os/ps2/ps2.h"
#include "platform/os/math.h"
#include "platform/os/font.h"
#include "platform/os/ps2/ps2draw.h"
#include "platform/os/ps2/ps2screen.h"
#include "platform/software/softpreca.h"
#include "texture/tex.h"
#include "texture/texrgb.h"
#include "texture/textga.h"
#include "texture/texjpgtiny.h"
#include "system/mem.h"
#include "system/file.h"
#include "system/filearc.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwoparse.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "platform/renderer/lwsdraw.h"
#include "platform/os/console.h"
#include "platform/os/window.h"
#include "platform/os/splash.h"
#include "platform/os/fileio.h"
#include "platform/renderer/matrix.h"
#include "lightwave/lws/lwsplaylist.h"
#include "platform/os/ps2/lib/floatlib/floatlib.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT32 progressCount = 0;
static INT32 scene = 0;
static INT32 sceneCount = 0;

static Window *window = NULL;
    
/*---------------------------------------------------------------------------- CLEAR SCREEN */
static void
clearScreen(void)
{
	SoftViewport viewport;
	
    window = WindowGet();

    /*
	viewport.x1 = window->resX/16;
    viewport.y1 = window->resY/16;
    viewport.x2 = window->resX-window->resX/16;
    viewport.y2 = window->resY-window->resY/16;
	*/

	viewport.x1 = 0;
    viewport.y1 = 0;
    viewport.x2 = window->resX;
    viewport.y2 = window->resY;
    
    SoftDrawSetViewport(&viewport);

    PS2DrawClear(0);
}

/*---------------------------------------------------------------------------- PROGRESS BAR */
static void
progress(INT32 i)
{
	INT32 count = PlayListGetCount();

    i /= count;

    i += (INT32) ((((FLOAT32) progressCount/(FLOAT32) count) * 100.0f));

	PS2DrawSetColor(0, 0, 0);
	clearScreen();
	SplashShow(0);
	WindowDisplayText(20, 0, "Decrunching.. (%d %%)", i);

    PS2ScreenRefresh();
}

/*---------------------------------------------------------------------------- NEXT SCENE */
static void 
nextScene(void)
{
    INT32 j = 0;
    
    if (PlayListGetCount()==0)
    {
        return;
    }

    PlayListNext(&j);

    PlayListGet()->lws->time = 0.0f;
}

/*---------------------------------------------------------------------------- SCENE INIT */
static void 
sceneInit(void)
{
    INT32 i = 0;
    INT32 j = 0;

    scene = 0;
    sceneCount = PlayListGetCount();

    PlayListSet(0);

    i = 0;

    while (i<sceneCount)
    {
        ConsoleAdd("Loading scene %d of %d (%s)", i+1, sceneCount, (INT8 *) PlayListGet()->fileName);
        
        PlayListGet()->lws = MemAlloc(sizeof(Lws));
        PlayListGet()->isValid = FALSE;

        progressCount = i;
    
        if (LwsLoad(PlayListGet()->lws, (INT8 *) PlayListGet()->fileName, progress)==OK)
        {
            PlayListGet()->isValid = TRUE;
			
			LwsSetPreca(PlayListGet()->lws, (INT8 *) SoftPrecaBuildFileName((INT8 *) PlayListGet()->fileName));
		}
    
        PlayListNext(&j);

        i++;
    }
}

/*---------------------------------------------------------------------------- 3D */
static void
init3D(void)
{
    INT8 fileNameArc[STR_DEFAULT_SIZE];

	strcpy(fileNameArc, "host:FRESCO.JNG");
	
	PS2DrawSetColor(0, 0, 0);
	clearScreen();
	SplashShow(0);
	WindowDisplayText(20, 0, "Loading %s, please wait..", fileNameArc);

	PS2ScreenRefresh();
	
	FileArcInit(fileNameArc);

	PlayListAddArchive();
	
	//PlayListRemoveAll();
	//PlayListAdd("C:\\jfyeNGDemo\\sallefeu\\sallefeu.lws");

	sceneInit();
    
    //LwsLoad(&lws, "C:\\voiture\\voiture.lws", progress);
    //LwsLoad(&lws, "C:\\jfyeNGDemo\\john\\john.lws", progress);
    //LwsLoad(&lws, "C:\\jfyeNGDemo\\parent\\parent.lws", progress);    
	//LwsLoad(&lws, "C:\\jfyeNGDemo\\castle\\test.lws", progress);

	PlayListSet(0);
}

/*---------------------------------------------------------------------------- INIT */
static void 
cbInit(void)
{
    init3D();
}

/*---------------------------------------------------------------------------- DRAW */
static void 
cbDraw(void)
{
    Lws *lws = NULL;

	VideoMode *videoMode = NULL;
	
	FLOAT32 frameCount = 0.0f;
	
	PS2DrawSetColor((INT32) (PlayListGet()->lws->backDropR*255.0f), (INT32) (PlayListGet()->lws->backDropG*255.0f), (INT32) (PlayListGet()->lws->backDropB*255.0f));
	clearScreen();
	
	window = WindowGet();

	lws = PlayListGet()->lws;

    lws->viewPortResX = window->resX>>1;
    lws->viewPortResY = window->resY>>1;

	lws->offsetUV += 0.04f;
	if (lws->offsetUV>1.0f)
	{
		lws->offsetUV -= 1.0f;
	}

    LwsEnable(lws);

    frameCount = (FLOAT32) (lws->lastFrame-lws->firstFrame);
            
    lws->time += 5.8f/frameCount;

	videoMode = PS2ScreenGetVideoMode();

	/*
	PS2DrawSetColor(0, 0, 0);
    PS2DrawClear(videoMode->height-3*FontGet()->charHeight);
    
    WindowDisplayText(20, 0, "%dx%d (%d bpp)", videoMode->width, videoMode->height, videoMode->depth);
	WindowDisplayText(20, 1, "%s (%d triangles)", lws->fileName, lws->triCount);
    WindowDisplayText(20, 2, "Frame %d/%d (%d %%)", (INT32) (frameCount*lws->time), (INT32) lws->lastFrame, (INT32) (lws->time*100.0f));

    */
	if (lws->time>=1.0f)
    {
        lws->time = 0.0f;
		nextScene();
    }

	lws = PlayListGet()->lws;

    PS2ScreenRefresh();
}

/*---------------------------------------------------------------------------- FREE */
static void 
cbFree(void)
{
}

/*---------------------------------------------------------------------------- KEY PRESS */
static void 
cbKey(INT32 k)
{
    k = 0;
}

/*---------------------------------------------------------------------------- MAIN */
INT32
main(INT32 argc, INT8 **argv)
{
	TexSettings settings;
    
	argc=argc;
	argv=argv;

    ConsoleInit();

    MemInit();
    
	FileIOInit();
    
	settings.doRGBConvert = TRUE;
	settings.doScalePower2 = TRUE;
	settings.maxPower2 = 128;
    TexInit(&settings);

	TexRGBSet(ARGB555);
	TexRegisterCodec("TGA", TgaLoad);
	TexRegisterCodec("JPG", JpgTinyLoad);
    
    MathInit();
	MatrixInit();

    WindowSetInit(cbInit);
    WindowSetDraw(cbDraw);
    WindowSetFree(cbFree);
    WindowSetKey(cbKey);

    WindowShow();
    
    return 0;
}