/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * math.h - Mathematics                                                       *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/math.h"
#include "platform/os/console.h"
#include "platform/os/ps2/lib/floatlib/floatlib.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define	SIN_COS_TABLE_SIZE		8192
#define SIN_COS_MAX				360.0f

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static FLOAT32 cosTable[SIN_COS_TABLE_SIZE];
static FLOAT32 sinTable[SIN_COS_TABLE_SIZE];

/*---------------------------------------------------------------------------- INIT */
void
MathInit(void)
{
	FLOAT32 a = 0.0f;
	INT32 i = 0;
	
	for (i=0; i<SIN_COS_TABLE_SIZE; i++)
	{
		a = ((FLOAT32) i)*(SIN_COS_MAX/SIN_COS_TABLE_SIZE);
		
		cosTable[i] = (FLOAT32) cosf(RADIAN(a));
		sinTable[i] = (FLOAT32) sinf(RADIAN(a));		
	}

	ConsoleAdd("Initializing math");
}

/*---------------------------------------------------------------------------- COS */
FLOAT32 
MathCos(FLOAT32 value)
{
	INT32 i = 0;
	
	i = (INT32) (DEGREE(value)*(SIN_COS_TABLE_SIZE/SIN_COS_MAX));

	while (i<0)
	{
		i += SIN_COS_TABLE_SIZE;
	}
	while (i>=SIN_COS_TABLE_SIZE)
	{
		i -= SIN_COS_TABLE_SIZE;
	}

	return cosTable[i];
}

/*---------------------------------------------------------------------------- SIN */
FLOAT32 
MathSin(FLOAT32 value)
{
	INT32 i = 0;
	
	i = (INT32) (DEGREE(value)*(SIN_COS_TABLE_SIZE/SIN_COS_MAX));

	while (i<0)
	{
		i += SIN_COS_TABLE_SIZE;
	}
	while (i>=SIN_COS_TABLE_SIZE)
	{
		i -= SIN_COS_TABLE_SIZE;
	}

	return sinTable[i];
}

/*---------------------------------------------------------------------------- ABS */
INT32
MathAbs(INT32 value)
{
	if (value<0)
	{
		return -value;
	}
	else
	{
		return value;
	}
}

/*---------------------------------------------------------------------------- ABS */
FLOAT32
MathFabs(FLOAT32 value)
{
	if (value<0.0f)
	{
		return -value;
	}
	else
	{
		return value;
	}
}

/*---------------------------------------------------------------------------- SQRT */
FLOAT32 
MathSqrt(FLOAT32 value)
{
	return (FLOAT32) sqrtf(value);
}

/*---------------------------------------------------------------------------- FLOOR */
FLOAT32 
MathFloor(FLOAT32 value)
{
	return (FLOAT32) floorf(value);
}

/*---------------------------------------------------------------------------- ATAN */
FLOAT32 MathAtan(FLOAT32 value)
{
	return (FLOAT32) atanf(value);
}

/*---------------------------------------------------------------------------- ATAN 2 */
FLOAT32 MathAtan2(FLOAT32 value1, FLOAT32 value2)
{
	return atan2f(value1, value2);
}

/*---------------------------------------------------------------------------- NORMAL VALUE */
FLOAT32 MathNormCoef(FLOAT32 value)
{
	return value;
}
