/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * memblock.c - Memory Block                                                  *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/memblock.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define HEAP_SIZE						65536*22
#define HEAP_NB_BLOCKS					120000
#define HEAP_MIN_BLOCK_SEARCH_SIZE		512

/*---------------------------------------------------------------------------- LOCAL TYPES */
typedef struct INT128
{
	long lo;
    long hi;
} INT128 __attribute__((aligned(16)));

typedef struct MemBlock
{
    void *pointer;
	
	INT32 size;
	UINT8 used;
} MemBlock;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT128 memPool[HEAP_SIZE] __attribute__((aligned (16)));
static INT128 *memPointer = NULL;

static MemBlock blockTable[HEAP_NB_BLOCKS];
static INT32 currentBlock = 0;
static INT32 allocated = 0;

/*---------------------------------------------------------------------------- MALLOC */
void * 
MemBlockAlloc(const INT32 size)
{
	INT32 i = 0;
	INT32 a = 0;
	INT32 currentMinSize = 0;

	void *memBlock = NULL;
	
	if (memPointer==NULL)
	{
		memPointer = memPool;

		currentBlock = 0;
		for (i=0; i<HEAP_NB_BLOCKS; i++)
		{
			blockTable[i].used = FALSE;
			blockTable[i].pointer = 0;
			blockTable[i].size = 0;
		}
	}

	if (size<HEAP_MIN_BLOCK_SEARCH_SIZE)
	{
		a = -1;
	}
	else		
	{
		a = -1;
		currentMinSize = -1;

		i = 0;
		while (i<currentBlock)
		{
			if (blockTable[i].used==FALSE && size<blockTable[i].size)
			{
				if (a==-1)
				{
					a = i;
					currentMinSize = blockTable[i].size;
				}
				else if (blockTable[i].size<currentMinSize)
				{
					a = i;
					currentMinSize = blockTable[i].size;
				}			
			}
			
			i++;
		}
	}

	if (a==-1)
	{
		memBlock = (void *) memPointer;
		memPointer += (size+15)>>4;

		blockTable[currentBlock].used = TRUE;
		blockTable[currentBlock].pointer = memBlock;
		blockTable[currentBlock].size = size;		

		currentBlock++;

		allocated += size;
		if (size>100000)
		{
			/*
			ConsoleAdd("%d %d", currentBlock, allocated);
			*/
		}
	}
	else
	{
		blockTable[a].used = TRUE;

		memBlock = blockTable[a].pointer;
	}

	return memBlock;
}

/*---------------------------------------------------------------------------- FREE */
void 
MemBlockFree(void *memBlock)
{
	INT32 i = 0;
	
	if (memBlock!=NULL)
	{
		i = 0;
		while (i<currentBlock)
		{
			if (blockTable[i].pointer==memBlock)
			{
				blockTable[i].used = FALSE;

				return;
			}
			
			i++;
		}
	}
}