/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/******************************************************************************
 * ps2.h - PS2 Library                                                        *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 * Based on work of Tony Saveski (t_saveski@yahoo.com)                        *
 ******************************************************************************/
#ifndef _PS2_
#define _PS2_

/*---------------------------------------------------------------------------- DMA REGISTERS */
#define GIF_CHCR	        0x1000A000
#define GIF_MADR	        0x1000A010
#define GIF_QWC		        0x1000A020
#define GIF_TADR            0x1000A030

/*---------------------------------------------------------------------------- GS PRIVILIGED REGISTERS */
#define PMODE               0x12000000
#define SMODE2              0x12000020
#define DISPFB1             0x12000070
#define DISPLAY1            0x12000080
#define DISPFB2             0x12000090
#define DISPLAY2            0x120000A0
#define EXTBUF              0x120000B0
#define EXTDATA             0x120000C0
#define EXTWRITE            0x120000D0
#define BGCOLOR             0x120000E0
#define CSR                 0x12001000
#define IMR                 0x12001010
#define BUSDIR              0x12001040
#define SIGLBLID            0x12001080

/*---------------------------------------------------------------------------- GS GENERAL PURPOSE REGISTERS */
#define PRIM		        0x00
#define RGBAQ		        0x01
#define ST			        0x02
#define UV2			        0x03
#define XYZF2		        0x04
#define XYZ2		        0x05
#define TEX0_1		        0x06
#define TEX0_2		        0x07
#define CLAMP_1		        0x08
#define CLAMP_2		        0x09
#define FOG			        0x0A
#define XYZF3		        0x0C
#define XYZ3		        0x0D
#define TEX1_1		        0x14
#define TEX1_2		        0x15
#define TEX2_1		        0x16
#define TEX2_2		        0x17
#define XYOFFSET_1	        0x18
#define XYOFFSET_2	        0x19
#define PRMODECONT	        0x1A
#define PRMODE		        0x1B
#define TEXCLUT		        0x1C
#define SCANMSK		        0x22
#define MIPTBP1_1	        0x34
#define MIPTBP1_2	        0x35
#define MIPTBP2_1	        0x36
#define MIPTBP2_2	        0x37
#define TEXA		        0x3B
#define FOGCOL		        0x3D
#define TEXFLUSH	        0x3F
#define SCISSOR_1	        0x40
#define SCISSOR_2	        0x41
#define ALPHA_1		        0x42
#define ALPHA_2		        0x43
#define DIMX		        0x44
#define DTHE		        0x45
#define COLCLAMP	        0x46
#define TEST_1		        0x47
#define TEST_2		        0x48
#define PABE		        0x49
#define FBA_1		        0x4A
#define FBA_2		        0x4B
#define FRAME_1		        0x4C
#define FRAME_2		        0x4D
#define ZBUF_1		        0x4E
#define ZBUF_2		        0x4F
#define BITBLTBUF	        0x50
#define TRXPOS		        0x51
#define TRXREG		        0x52
#define TRXDIR		        0x53
#define HWREG		        0x54
#define SIGNAL		        0x60
#define FINISH		        0x61
#define LABEL		        0x62

/*---------------------------------------------------------------------------- GS GENERAL PURPOSE STATES */
#define PSMCT32				0
#define PSMCT24				1
#define PSMCT16				2

#define ATST_NEVER		    0
#define ATST_ALWAYS		    1
#define ATST_LESS		    2
#define ATST_LEQUAL		    3
#define ATST_EQUAL		    4
#define ATST_GEQUAL		    5
#define ATST_GREATER	    6
#define ATST_NOTEQUAL	    7
                            
#define AFAIL_KEEP		    0
#define AFAIL_FBONLY	    1
#define AFAIL_ZBONLY	    2
#define AFAIL_RGBONLY	    3
                            
#define ZTST_NEVER		    0
#define ZTST_ALWAYS		    1
#define ZTST_GEQUAL		    2
#define ZTST_GREATER	    3

#define INTERLACE_NO        0
#define INTERLACE_YES       1

#define PRIM_POINT			0
#define PRIM_LINE			1
#define PRIM_LINE_STRIP		2
#define PRIM_TRI			3
#define PRIM_TRI_STRIP		4
#define PRIM_TRI_FAN		5
#define PRIM_SPRITE			6

#define TEX_MODULATE	    0
#define TEX_DECAL		    1
#define TEX_HILIGHT		    2
#define TEX_HILIGHT2	    3

#define XDIR_EE_GS			0
#define XDIR_GS_EE			1
#define XDIR_GS_GS			2
#define XDIR_DEACTIVATE		3

/*---------------------------------------------------------------------------- GIF */
#define GIF_AD		        0x0E
#define GIF_NOP		        0x0F

/*---------------------------------------------------------------------------- EXTERN ASM */
extern void dma_reset(void);
extern void vuReset(void);
extern void gs_set_imr(void);
extern void gs_set_crtc(UINT8 isInterlaced, UINT8 isNTSC, UINT8 fieldMode);
extern void ps2_flush_cache(int dummy);

/*---------------------------------------------------------------------------- DMA MACROS */
#define GIF_GET_QWC                                     \
    ((volatile UINT32 *) (GIF_QWC))

#define GIF_SET_QWC(WHICH, SIZE)                        \
	*WHICH = (UINT32) SIZE

#define GIF_GET_MADR		                            \
    ((volatile UINT32 *) (GIF_MADR))

#define GIF_SET_MADR(WHICH, ADDR, SPR)                  \
	*WHICH = ((UINT32) (ADDR)<<0) | ((UINT32) (SPR)<<31)

#define GIF_GET_TADR		                            \
    ((volatile UINT32 *) (GIF_TADR))

#define GIF_SET_TADR(WHICH, ADDR, SPR)                  \
	*WHICH = ((UINT32) (ADDR)<<0) | ((UINT32) (SPR)<<31)

#define GIF_GET_CHCR                                    \
    ((volatile UINT32 *) (GIF_CHCR))

#define GIF_SET_CHCR(WHICH, DIR, MOD, ASP, TTE, TIE, STR, TAG) \
	*WHICH =                                            \
	((UINT32) (DIR)<<0) |                               \
	((UINT32) (MOD)<<2) |                               \
	((UINT32) (ASP)<<4) |                               \
	((UINT32) (TTE)<<6) |                               \
	((UINT32) (TIE)<<7) |                               \
	((UINT32) (STR)<<8) |                               \
	((UINT32) (TAG)<<16)

#define DMA_WAIT(WHICH)                                 \
	while ((*WHICH)&(1<<8))

/*---------------------------------------------------------------------------- GIF MACROS */
#define DECLARE_GS_PACKET(name, items)                  \
	UINT64 __attribute__ ((aligned(64))) name[items*2+2]; \
	INT32 name##_offset = 0;                            \
	INT32 name##_size = 0

#define DECLARE_EXTERN_GS_PACKET(name, items)           \
	extern UINT64 __attribute__ ((aligned(64))) name[items*2+2]; \
	extern INT32 name##_offset;                         \
	extern INT32 name##_size

#define BEGIN_GS_PACKET(name)                           \
	name##_offset = 0

#define GIF_TAG(name, nloop, eop, pre, prim, flg, nreg, regs) \
	name##_size = nloop+1;                              \
	name[name##_offset++] =                             \
		((UINT64) (nloop)<<0) |                         \
		((UINT64) (eop)<<15) |                          \
		((UINT64) (pre)<<46) |                          \
		((UINT64) (prim)<<47) |                         \
		((UINT64) (flg)<<58) |                          \
		((UINT64) (nreg)<<60);                          \
	name[name##_offset++] = (UINT64) regs

#define GIF_TAG_AD(name, nloop, eop, pre, prim, flg)    \
	GIF_TAG(name, nloop, eop, pre, prim, flg, 1, GIF_AD)

#define GIF_TAG_IMG(name, qsize)                        \
	GIF_TAG(name, qsize, 1, 0, 0, 2, 0, 0);             \
	name##_size = 1                                     \

#define GIF_DATA_AD(name, reg, dat)                     \
	name[name##_offset++] = (UINT64) dat;               \
	name[name##_offset++] = (UINT64) reg

#define SEND_GS_PACKET(name)                            \
	ps2_flush_cache(0);							        \
	GIF_SET_QWC(((volatile UINT32 *)GIF_QWC), name##_size);                  \
	GIF_SET_MADR(((volatile UINT32 *)GIF_MADR), name, 0);                    \
    GIF_SET_CHCR(((volatile UINT32 *)GIF_CHCR), 1, 0, 0, 0, 0, 1, 0);        \
	DMA_WAIT(((volatile UINT32 *)GIF_CHCR))

/*---------------------------------------------------------------------------- GS MACROS */
#define GS_RESET()                                      \
    *((volatile UINT64 *)CSR) = (UINT64) (1<<9)

#define GS_SET_BGCOLOR(R, G, B)                         \
    *((volatile UINT64 *)BGCOLOR) =                     \
        ((UINT64) (R)<<0) | ((UINT64) (G)<<8) | ((UINT64) (B)<<16)

#define GS_SET_PMODE(EN1, EN2, MMOD, AMOD, SLBG, ALP)   \
    *((volatile UINT64 *) (PMODE)) =                    \
	((UINT64) (EN1)<<0) |                               \
	((UINT64) (EN2)<<1) |                               \
	((UINT64) (001)<<2) |                               \
	((UINT64) (MMOD)<<5) |                              \
	((UINT64) (AMOD)<<6) |                              \
	((UINT64) (SLBG)<<7) |                              \
	((UINT64) (ALP)<<8)

#define GS_SET_DISPLAY2(DX, DY, MAGH, MAGV, DW, DH)     \
    *((volatile UINT64 *) (DISPLAY2)) =                 \
	((UINT64) (DX)<<0) |                                \
	((UINT64) (DY)<<12) |                               \
	((UINT64) (MAGH)<<23) |                             \
	((UINT64) (MAGV)<<27) |                             \
	((UINT64) (DW)<<32) |                               \
	((UINT64) (DH)<<44)

#define GS_SET_DISPFB2(FBP, FBW, PSM, DBX, DBY)         \
	*((volatile UINT64 *) (DISPFB2)) =                  \
	((UINT64) (FBP)<<0)	| \
	((UINT64) (FBW)<<9)	| \
	((UINT64) (PSM)<<15) | \
	((UINT64) (DBX)<<32) | \
	((UINT64) (DBY)<<43)

#define GS_FRAME(FBP, FBW, PSM, FBMSK)                  \
(                                                       \
    ((UINT64) (FBP)<<0) |                               \
	((UINT64) (FBW)<<16) |                              \
	((UINT64) (PSM)<<24) |                              \
	((UINT64) (FBMSK)<<32)                              \
)

#define GS_XYOFFSET(OFX, OFY)	                        \
(                                                       \
    ((UINT64) (OFX)<<0) |                               \
	((UINT64) (OFY)<<32)                                \
)

#define GS_BITBLTBUF(SBP, SBW, SPSM, DBP, DBW, DPSM)    \
(                                                       \
	((UINT64) (SBP)<<0) |                               \
	((UINT64) (SBW)<<16) |                              \
	((UINT64) (SPSM)<<24) |                             \
	((UINT64) (DBP)<<32) |                              \
	((UINT64) (DBW)<<48) |                              \
	((UINT64) (DPSM)<<56)                               \
)

#define GS_TRXPOS(SSAX, SSAY, DSAX, DSAY, DIR)	        \
(                                                       \
	((UINT64) (SSAX)<<0) |                              \
	((UINT64) (SSAY)<<16) |                             \
	((UINT64) (DSAX)<<32) |                             \
	((UINT64) (DSAY)<<48) |                             \
	((UINT64) (DIR)<<59)                                \
)

#define GS_TRXREG(RRW, RRH)	                            \
(                                                       \
    ((UINT64) (RRW)<<0)	|                               \
    ((UINT64) (RRH)<<32)                                \
)

#define GS_TRXDIR(XDIR)	                                \
(                                                       \
    (UINT64) (XDIR)                                     \
)

#define GS_SCISSOR(X0, X1, Y0, Y1)                      \
(                                                       \
    ((UINT64) (X0)<<0) |                                \
	((UINT64) (X1)<<16) |                               \
	((UINT64) (Y0)<<32) |                               \
	((UINT64) (Y1)<<48)                                 \
)

#define GS_TEST(ATE, ATST, AREF,AFAIL, DATE, DATM, ZTE, ZTST) \
(                                                       \
    ((UINT64) (ATE)<<0) |                               \
	((UINT64) (ATST)<<1) |                              \
	((UINT64) (AREF)<<4) |                              \
	((UINT64) (AFAIL)<<12) |                            \
	((UINT64) (DATE)<<14) |                             \
	((UINT64) (DATM)<<15) |                             \
	((UINT64) (ZTE)<<16) |                              \
	((UINT64) (ZTST)<<17)                               \
)

#define GS_ALPHA(A, B, C, D, FIX)                       \
(                                                       \
    ((UINT64) (A)<<0) |                                 \
	((UINT64) (B)<<2) |                                 \
	((UINT64) (C)<<4) |                                 \
	((UINT64) (D)<<6) |                                 \
	((UINT64) (FIX)<<32)                                \
)

#define GS_RGBAQ(R, G, B, A, Q)                         \
(                                                       \
	((UINT64) (R)<<0) |                                 \
	((UINT64) (G)<<8) |                                 \
	((UINT64) (B)<<16) |                                \
	((UINT64) (A)<<24) |                                \
	((UINT64) (Q)<<32)                                  \
)

#define GS_RGB128Q(Q)			                        \
(                                                       \
	((UINT64) (128)<<0) |                               \
	((UINT64) (128)<<8) |                               \
	((UINT64) (128)<<16) |                              \
	((UINT64) (0x80)<<24) |                             \
	((UINT64) (Q)<<32)                                  \
)

#define GS_Q(Q)											\
(                                                       \
	((UINT64) (0x80)<<24) |                             \
	((UINT64) (Q)<<32)                                  \
)

#define GS_XYZ2(X, Y, Z)                            	\
(                                                       \
	((INT64) (X)<<0) |                                  \
	((INT64) (Y)<<16) |                                 \
	((UINT64) (Z)<<32)                                  \
)

#define GS_UV(U, V)	                                    \
(                                                       \
	((UINT64) (U)<<0) |                                 \
	((UINT64) (V)<<16)                                  \
)

#define GS_ST(S, T)	                                    \
(                                                       \
	((UINT64) (S)<<0) |                                 \
	((UINT64) (T)<<32)                                  \
)

#define GS_PRIM(PRI, IIP, TME, FGE, ABE, AA1, FST, CTXT, FIX) \
(                                                       \
	((UINT64) (PRI)<<0) |                               \
	((UINT64) (IIP)<<3) |                               \
	((UINT64) (TME)<<4) |                               \
	((UINT64) (FGE)<<5) |                               \
	((UINT64) (ABE)<<6) |                               \
	((UINT64) (AA1)<<7) |                               \
	((UINT64) (FST)<<8) |                               \
	((UINT64) (CTXT)<<9) |                              \
	((UINT64) (FIX)<<10)                                \
)

#define GS_TEX0(TBP0, TBW, PSM, TW, TH, TCC, TFX, CBP, CPSM, CSM, CSA, CLD) \
(                                                       \
	((UINT64) (TBP0)<<0) |                              \
	((UINT64) (TBW)<<14) |                              \
	((UINT64) (PSM)<<20) |                              \
	((UINT64) (TW)<<26) |                               \
	((UINT64) (TH)<<30) |                               \
	((UINT64) (TCC)<<34) |                              \
	((UINT64) (TFX)<<35) |                              \
	((UINT64) (CBP)<<37) |                              \
	((UINT64) (CPSM)<<51) |                             \
	((UINT64) (CSM)<<55) |                              \
	((UINT64) (CSA)<<56) |                              \
	((UINT64) (CLD)<<61)                                \
)

/*---------------------------------------------------------------------------- DMA PACKET */
#define DMA_BUFFER_SIZE         (128/8)*16384

/*---------------------------------------------------------------------------- END OF DECLARATION */
#endif /* _PS2_ */
