/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * ps2draw.c - PS2 Draw Manager                                               *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 * Based on work of Tony Saveski (t_saveski@yahoo.com)                        *
 ******************************************************************************/
#include "platform/os/window.h"
#include "platform/os/font.h"
#include "platform/software/soft.h"
#include "platform/software/softdraw.h"
#include "platform/os/ps2/ps2.h"
#include "platform/os/ps2/ps2draw.h"
#include "platform/os/ps2/ps2drawtex.h"
#include "platform/os/ps2/ps2screen.h"

/*---------------------------------------------------------------------------- EXTERN VARIABLES */
DECLARE_EXTERN_GS_PACKET(packet, DMA_BUFFER_SIZE);

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static UINT8 colorR = 0;
static UINT8 colorG = 0;
static UINT8 colorB = 0;
static VideoMode *videoMode = NULL;

static Font *charFont = NULL;

static FLOAT32 zScale = 0.0f;

/*---------------------------------------------------------------------------- Z-SCALE */
void
PS2DrawSetZScale(FLOAT32 scale)
{
    zScale = scale;
}

/*---------------------------------------------------------------------------- PEN COLOR */
void
PS2DrawSetColor(UINT8 r, UINT8 g, UINT8 b)
{
    colorR = r;
    colorG = g;
    colorB = b;
}

/*---------------------------------------------------------------------------- Z-BUFFER ENABLING */
void 
PS2DrawEnableZBuffer(BOOL enable)
{
    if (enable)
    {
        BEGIN_GS_PACKET(packet);
        GIF_TAG_AD(packet, 2, 1, 0, 0, 0);
        GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(PS2ScreenGetVideoRAM()->zBufferAddr/8192, videoMode->width/64, PSMCT16, 0));
		GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GEQUAL));
        SEND_GS_PACKET(packet);
    }
    else
    {
        BEGIN_GS_PACKET(packet);
        GIF_TAG_AD(packet, 1, 1, 0, 0, 0);
        GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_ALWAYS));
        SEND_GS_PACKET(packet);
    }
}

/*---------------------------------------------------------------------------- CLEAR */
void
PS2DrawClear(INT32 y)
{
    videoMode = PS2ScreenGetVideoMode();
    
    BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, 2, 1, 0, 0, 0);
	GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(PS2ScreenGetVideoRAM()->zBufferAddr/8192, videoMode->width/64, PSMCT16, 0));	
    GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_ALWAYS));
    SEND_GS_PACKET(packet);
	
    PS2DrawFillRect(0, y, videoMode->width, videoMode->height);

    BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, 1, 1, 0, 0, 0);
    GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GREATER));
    SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- SET PIXEL */
void 
PS2DrawSetPixel(INT16 x, INT16 y)
{
    x += videoMode->originX;
	y += videoMode->originY;

	BEGIN_GS_PACKET(packet);
	GIF_TAG_AD(packet, 3, 1, 0, 0, 0);

	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_POINT, 0, 0, 0, 0, 0, 0, 0, 0));
	GIF_DATA_AD(packet, RGBAQ, GS_RGBAQ(colorR, colorG, colorB, 0xFF, 0));
	GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x<<4, y<<4, 0));

	SEND_GS_PACKET(packet);  
}

/*---------------------------------------------------------------------------- FILL RECT */
void
PS2DrawFillRect(INT16 x1, INT16 y1, INT16 x2, INT16 y2)
{
    videoMode = PS2ScreenGetVideoMode();
    
    x1 += videoMode->originX;
    x2 += videoMode->originX;
    y1 += videoMode->originY;
    y2 += videoMode->originY;

	BEGIN_GS_PACKET(packet);
	GIF_TAG_AD(packet, 4, 1, 0, 0, 0);

	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_SPRITE, 0, 0, 0, 0, 0, 0, 0, 0));
	GIF_DATA_AD(packet, RGBAQ, GS_RGBAQ(colorR, colorG, colorB, 0xFF, 0));
	GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x1<<4, y1<<4, 0.0f));
	GIF_DATA_AD(packet, XYZ2, GS_XYZ2((x2+1)<<4, (y2+1)<<4, 0.0f));

	SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- SET CHAR INFO */
RESULT 
PS2SetFont(Font *font)
{
	if (font==NULL)
	{
		return ERR_INVALID_PTR;
	}
	
	charFont = font;

	return OK;
}

/*---------------------------------------------------------------------------- DRAW CHAR */
void
PS2DrawChar(INT16 x, INT16 y, INT16 u, INT16 v)
{
	BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, 1, 1, 0, 0, 0);
    GIF_DATA_AD(packet, TEST_1, GS_TEST(1, ATST_LESS, 1, AFAIL_KEEP, 0, 0, 1, ZTST_ALWAYS));
    SEND_GS_PACKET(packet);
	
    x += videoMode->originX;
	y += videoMode->originY;

	PS2DrawTexUpload((UINT16) charFont->width, (UINT16) charFont->height, (UINT32) charFont->data);

	BEGIN_GS_PACKET(packet);
	GIF_TAG_AD(packet, 6, 1, 0, 0, 0);

	GIF_DATA_AD(packet, TEX0_1, GS_TEX0(PS2ScreenGetVideoRAM()->texBuffer/256, charFont->width/64, PSMCT16, PS2DrawTexGetWH(charFont->width), PS2DrawTexGetWH(charFont->height), 1, TEX_DECAL, 0, 0, 0, 0, 0));
	
	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_SPRITE, 0, 1, 0, 0, 0, 1, 0, 0));

	GIF_DATA_AD(packet, UV2, GS_UV(u<<4, (charFont->height-v-1)<<4));
	GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x<<4, y<<4, 0));

	GIF_DATA_AD(packet, UV2, GS_UV((u+8)<<4, (charFont->height-v-charFont->charHeight)<<4));
	GIF_DATA_AD(packet, XYZ2, GS_XYZ2((x+8)<<4, (y+10)<<4, 0));

	SEND_GS_PACKET(packet);

	BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, 1, 1, 0, 0, 0);
    GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GREATER));
    SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- DRAW TRIANGLE BUFFER */
void
PS2DrawTriangleBuffer(Vertex *triBuffer, INT32 count, UINT8 r, UINT8 g, UINT8 b)
{
    INT16 x = 0;
    INT16 y = 0;
    INT32 size = 0;
	INT16 originX = 0;
	INT16 originY = 0;
    
    if (count<=0)
    {
        return;
    }

	videoMode = PS2ScreenGetVideoMode();

    originX = videoMode->originX<<4;
	originY = videoMode->originX<<4;
    
    count *= 3;

    BEGIN_GS_PACKET(packet);
	
    size = 4;

	GIF_TAG_AD(packet, size, 1, 0, 0, 0);
    
	GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(PS2ScreenGetVideoRAM()->zBufferAddr/8192, videoMode->width/64, PSMCT16, 0));
	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_TRI, 0, 0, 0, 0, 0, 0, 0, 0));    
	GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GREATER));
    GIF_DATA_AD(packet, RGBAQ, GS_RGBAQ(r, g, b, 0x80, 0));
    
    while (count>0)
    {
        if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x = originX+(INT16) (triBuffer->x*16.0f);
			y = originY+(INT16) (triBuffer->y*16.0f);

			GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x, y, 65535-(UINT16) (triBuffer->z*zScale)));
        
			size += 1;

			count--;
			triBuffer++;
		}
    }

    BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, size, 1, 0, 0, 0);
	
	SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- DRAW TRIANGLE BUFFER */
void
PS2DrawTriangleBufferAdd(Vertex *triBuffer, INT32 count, UINT8 r, UINT8 g, UINT8 b)
{
    INT16 x = 0;
    INT16 y = 0;
    INT32 size = 0;
	INT16 originX = 0;
	INT16 originY = 0;
    
    if (count<=0)
    {
        return;
    }

	videoMode = PS2ScreenGetVideoMode();

    originX = videoMode->originX<<4;
	originY = videoMode->originX<<4;
    
    count *= 3;

    BEGIN_GS_PACKET(packet);
	
    size = 6;

	GIF_TAG_AD(packet, size, 1, 0, 0, 0);
    
	GIF_DATA_AD(packet, RGBAQ, GS_RGBAQ(r, g, b, 0x80, 0));
    GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(PS2ScreenGetVideoRAM()->zBufferAddr/8192, videoMode->width/64, PSMCT16, 0));
	GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GEQUAL));
	GIF_DATA_AD(packet, COLCLAMP, 1);
	GIF_DATA_AD(packet, ALPHA_1, GS_ALPHA(0, 2, 2, 1, 128));
	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_TRI, 0, 0, 0, 1, 0, 0, 0, 0));	
	
	while (count>0)
    {
        if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x = originX+(INT16) (triBuffer->x*16.0f);
			y = originY+(INT16) (triBuffer->y*16.0f);

			GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x, y, 65535-(UINT16) (triBuffer->z*zScale)));
        
			size += 1;

			count--;
			triBuffer++;
		}
    }

    BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, size, 1, 0, 0, 0);
	
	SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- DRAW TEXTURED TRIANGLE BUFFER */
void 
PS2DrawTriangleBufferWithTex(Vertex *triBuffer, INT32 count, FLOAT32 w, FLOAT32 h, UINT32 *data)
{
    INT16 x = 0;
    INT16 y = 0;
    INT32 size = 0;
	UINT32 s = 0;
	UINT32 t = 0;
	UINT32 q = 0;
	INT16 originX = 0;
	INT16 originY = 0;

	if (count<=0)
    {
        return;
    }

	videoMode = PS2ScreenGetVideoMode();
    
    originX = videoMode->originX<<4;
	originY = videoMode->originX<<4;
    
    PS2DrawTexUpload(w, h, (UINT32) data);

	count *= 3;

	BEGIN_GS_PACKET(packet);
	
    size = 5;

	GIF_TAG_AD(packet, size, 1, 0, 0, 0);

	GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GREATER));
	GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(PS2ScreenGetVideoRAM()->zBufferAddr/8192, videoMode->width/64, PSMCT16, 0));
	GIF_DATA_AD(packet, TEX0_1, GS_TEX0(PS2ScreenGetVideoRAM()->texBuffer/256, videoMode->width/64, PSMCT16, PS2DrawTexGetWH(w), PS2DrawTexGetWH(h), 1, TEX_DECAL, 0, 0, 0, 0, 0));
	GIF_DATA_AD(packet, TEX1_1, 1<<5);
	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_TRI, 0, 1, 0, 0, 0, 0, 0, 0));
	
	while (count>0)
    {
        if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x = originX+(INT16) (triBuffer->x*16.0f);
			y = originY+(INT16) (triBuffer->y*16.0f);

			* ((FLOAT32 *) &s) = triBuffer->u;
			* ((FLOAT32 *) &t) = triBuffer->v;
			* ((FLOAT32 *) &q) = triBuffer->q;

			GIF_DATA_AD(packet, RGBAQ, GS_RGB128Q(q));
			GIF_DATA_AD(packet, ST, GS_ST(s, t));
			GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x, y, 65535-(UINT16) (triBuffer->z*zScale)));
        
			size += 3;

			count--;
			triBuffer++;			
		}
    }

    BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, size, 1, 0, 0, 0);
    
    SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- DRAW ADDITIVE TEXTURED TRIANGLE BUFFER */
void 
PS2DrawTriangleBufferWithTexAdd(Vertex *triBuffer, INT32 count, FLOAT32 w, FLOAT32 h, UINT32 *data)
{
    INT16 x = 0;
    INT16 y = 0;
    INT32 size = 0;
	UINT32 s = 0;
	UINT32 t = 0;
	UINT32 q = 0;
	INT16 originX = 0;
	INT16 originY = 0;

    if (count<=0)
    {
        return;
    }

	videoMode = PS2ScreenGetVideoMode();

	originX = videoMode->originX<<4;
	originY = videoMode->originX<<4;

	PS2DrawTexUpload(w, h, (UINT32) data);

	count *= 3;

	BEGIN_GS_PACKET(packet);
	
    size = 7;

	GIF_TAG_AD(packet, size, 1, 0, 0, 0);

	GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(PS2ScreenGetVideoRAM()->zBufferAddr/8192, videoMode->width/64, PSMCT16, 1));
	GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_NEVER, 0, AFAIL_KEEP, 0, 0, 1, ZTST_GEQUAL));
	GIF_DATA_AD(packet, TEX0_1, GS_TEX0(PS2ScreenGetVideoRAM()->texBuffer/256, videoMode->width/64, PSMCT16, PS2DrawTexGetWH(w), PS2DrawTexGetWH(h), 1, TEX_DECAL, 0, 0, 0, 0, 0));
	GIF_DATA_AD(packet, TEX1_1, 1<<5);
	GIF_DATA_AD(packet, COLCLAMP, 1);
	GIF_DATA_AD(packet, ALPHA_1, GS_ALPHA(0, 2, 2, 1, 128));
	GIF_DATA_AD(packet, PRIM, GS_PRIM(PRIM_TRI, 0, 1, 0, 1, 0, 0, 0, 0));	
	
	while (count>0)
    {
        if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x = originX+(INT16) (triBuffer->x*16.0f);
			y = originY+(INT16) (triBuffer->y*16.0f);

			* ((FLOAT32 *) &s) = triBuffer->u;
			* ((FLOAT32 *) &t) = triBuffer->v;
			* ((FLOAT32 *) &q) = triBuffer->q;

			GIF_DATA_AD(packet, RGBAQ, GS_Q(q));
			GIF_DATA_AD(packet, ST, GS_ST(s, t));
			GIF_DATA_AD(packet, XYZ2, GS_XYZ2(x, y, 65535-(UINT16) (triBuffer->z*zScale)));
        
			size += 3;

			count--;
			triBuffer++;			
		}
    }

	BEGIN_GS_PACKET(packet);
    GIF_TAG_AD(packet, size, 1, 0, 0, 0);
    
    SEND_GS_PACKET(packet);
}