/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * ps2screen.c - PS2 Screen Manager                                           *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 * Based on work of Tony Saveski (t_saveski@yahoo.com)                        *
 ******************************************************************************/
#include "platform/os/ps2/ps2.h"
#include "platform/os/ps2/ps2screen.h"
#include "platform/os/font.h"
#include "platform/software/soft.h"
#include "platform/software/softdraw.h"
#include "platform/os/ps2/ps2draw.h"
#include "platform/software/softtransf.h"

/*---------------------------------------------------------------------------- EXTERN VARIABLES */
DECLARE_EXTERN_GS_PACKET(packet, DMA_BUFFER_SIZE);

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define DEFAULT_BGCOLOR_R       0
#define DEFAULT_BGCOLOR_G       0
#define DEFAULT_BGCOLOR_B       0

#define NTSC			        2
#define PAL		                3

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static VideoRAM videoRam;
static VideoMode videoMode;

/*---------------------------------------------------------------------------- PAL/NTSC */
static BOOL
isNTSC(void)
{
    if (*((INT8 *) 0x1FC80000 - 0xAE)!='E')
    {
        return TRUE;
    }

	return FALSE;
}

/*---------------------------------------------------------------------------- HARDWARE RESET */
static void
reset(void)
{
    dma_reset();
	
	GS_RESET();
    
    vuReset();
	
	__asm__
    ("
        sync.p
		nop
	");

    gs_set_imr();
}

/*---------------------------------------------------------------------------- SCREEN INIT */
static void
screenInit(void)
{
    SoftViewport viewport;
	
	if (isNTSC())
    {
        videoMode.width = 640;
        videoMode.height = 224;
        videoMode.depth = 16;
        videoMode.hmag = 4;

        gs_set_crtc(INTERLACE_NO, NTSC, 0);
    }
    else
    {
        videoMode.width = 640;
        videoMode.height = 256;
        videoMode.depth = 16;
        videoMode.hmag = 4;

        gs_set_crtc(INTERLACE_NO, PAL, 0);
    }

    GS_SET_PMODE(0, 1, 1, 1, 0, 0xFF);

    GS_SET_DISPLAY2(656, 36, videoMode.hmag-1, 0, videoMode.width*videoMode.hmag-1, videoMode.height-1);

    GS_SET_BGCOLOR(DEFAULT_BGCOLOR_R, DEFAULT_BGCOLOR_G, DEFAULT_BGCOLOR_B);

    viewport.x1 = 0;
	viewport.y1 = 0;
	viewport.x2 = videoMode.width;
	viewport.y2 = videoMode.height;	
	SoftDrawSetViewport(&viewport);
}

/*---------------------------------------------------------------------------- GS VRAM INIT */
static void
vramInit(void)
{
    INT32 size = 0;
    
    videoRam.offset = 0;
    videoRam.bufferIsFront = TRUE;

    size = videoMode.width*videoMode.height*videoMode.depth/8;

    BEGIN_GS_PACKET(packet);
	GIF_TAG_AD(packet, 7, 1, 0, 0, 0);

    GIF_DATA_AD(packet, PRMODECONT, 1);

	GIF_DATA_AD(packet, ZBUF_1, GS_FRAME(videoRam.offset/8192, videoMode.width/64, PSMCT16, 0));
    videoRam.zBufferAddr = videoRam.offset;
	videoRam.offset += size;

	GIF_DATA_AD(packet, FRAME_1, GS_FRAME(videoRam.offset/8192, videoMode.width/64, PSMCT16, 0));		
	videoRam.frontBuffer = videoRam.offset;
	videoRam.offset += size;

	videoRam.backBuffer = videoRam.offset;
	videoRam.offset += size;

	videoMode.originX = 1024;
    videoMode.originY = 1024;
    GIF_DATA_AD(packet, XYOFFSET_1, GS_XYOFFSET(videoMode.originX<<4, videoMode.originY<<4));

	GIF_DATA_AD(packet, SCISSOR_1, GS_SCISSOR(0, videoMode.width-1, 0, videoMode.height-1));

    videoRam.texBuffer = videoRam.offset;
	videoRam.offset += size;

    GIF_DATA_AD(packet, TEST_1, GS_TEST(0, ATST_ALWAYS, 0, AFAIL_KEEP, 0, 0, 1, ZTST_ALWAYS));
	GIF_DATA_AD(packet, ALPHA_1, GS_ALPHA(0, 1, 0, 1, 0));

	SEND_GS_PACKET(packet);
}

/*---------------------------------------------------------------------------- INIT */
RESULT
PS2ScreenInit(void)
{
    INT32 i = 0;

	reset();

    screenInit();
    
    vramInit();

    for (i=0; i<2; i++)
    {
        PS2DrawSetColor(DEFAULT_BGCOLOR_R, DEFAULT_BGCOLOR_G, DEFAULT_BGCOLOR_B);
        PS2DrawClear(0);
    
        PS2ScreenRefresh();
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- REFRESH */
void
PS2ScreenWaitVBL(void)
{
    *((volatile UINT64 *)CSR) = *((volatile UINT64 *)CSR) & 8;

	while (!(*((volatile UINT64 *)CSR)&8));
}

void 
PS2ScreenRefresh(void)
{
    UINT32 front = 0;
    UINT32 back = 0;
    
    *((volatile unsigned long *) 0x120000E0) = (unsigned long) (0);
	
	PS2ScreenWaitVBL();

	*((volatile unsigned long *) 0x120000E0) = (unsigned long) (255);
    
    if (videoRam.bufferIsFront)
    {
        front = videoRam.frontBuffer;
        back = videoRam.backBuffer;

        videoRam.bufferIsFront = FALSE;
    }
    else
    {
        back = videoRam.frontBuffer;
        front = videoRam.backBuffer;

        videoRam.bufferIsFront = TRUE;
    }

    BEGIN_GS_PACKET(packet);
	GIF_TAG_AD(packet, 1, 1, 0, 0, 0);

	GIF_DATA_AD(packet, FRAME_1, GS_FRAME(front/8192, videoMode.width/64, PSMCT16, 0));

	SEND_GS_PACKET(packet);

    GS_SET_DISPFB2(back/8192, videoMode.width/64, PSMCT16, 0, 0);
}

/*---------------------------------------------------------------------------- ACCESSORS */
VideoMode *
PS2ScreenGetVideoMode(void)
{
    return &videoMode;
}

VideoRAM *
PS2ScreenGetVideoRAM(void)
{
    return &videoRam;
}