/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * window.c - Output Window                                                   *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/window.h"
#include "platform/os/console.h"
#include "platform/os/font.h"
#include "platform/software/soft.h"
#include "platform/os/ps2/ps2.h"
#include "platform/os/ps2/ps2screen.h"
#include "platform/os/ps2/ps2draw.h"
#include "platform/os/ps2/ps2drawtex.h"
#include "system/mem.h"

/*---------------------------------------------------------------------------- EXTERN */
extern int vsprintf (char *str, const char *format, va_list args);

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static void (*funcInit)(void) = NULL;
static void (*funcDraw)(void) = NULL;
static void (*funcFree)(void) = NULL;
static void (*funcKey)(INT32 key) = NULL;
static Window window;
static BOOL destroy = FALSE;

/*---------------------------------------------------------------------------- INIT (CALLBACK) */
RESULT
WindowSetInit(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcInit = func;

    return OK;
}

/*---------------------------------------------------------------------------- DRAW (CALLBACK) */
RESULT
WindowSetDraw(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcDraw = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- FREE (CALLBACK) */
RESULT
WindowSetFree(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcFree = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- KEY (CALLBACK) */
RESULT
WindowSetKey(void (*func)(INT32 param))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcKey = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- SHOW */
RESULT
WindowShow(void)
{
    VideoMode *videoMode = NULL;
    
    PS2ScreenInit();

    videoMode = PS2ScreenGetVideoMode();

    window.resX = videoMode->width;
    window.resY = videoMode->height;

	FontInit();

    funcInit();

	ConsoleAdd("Starting..");

    destroy = FALSE;
        
    while (!destroy)
    {
        funcDraw();
    }
    
    FontFree();
	
	funcFree();

    return OK;
}

/*---------------------------------------------------------------------------- GET */
Window *
WindowGet(void)
{
    return &window;
}

/*---------------------------------------------------------------------------- DISPLAY TEXT */
void
WindowDisplayText(INT32 x, INT32 negY, const INT8 *msg, ...)
{
    INT32 charHeight = 0;
	
	INT8 str[STR_DEFAULT_SIZE];
	va_list argList;
    
    if (msg==NULL)
    {
        return;
    }

	va_start(argList, msg);
    vsprintf(str, msg, argList);
    va_end(argList);

	charHeight = FontGet()->charHeight;

	FontDisplayText(x, window.resY-charHeight-charHeight*negY, str);
}