#include <windows.h>
#include <commctrl.h>
#include <config.h>
/******************************************************************************
 * console.c - Console                                                        *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/win32/rsc/resource.h"
#include "platform/os/gui.h"
#include "platform/os/fileio.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define LOG_FILE	"jfyeNG.log"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT8 logFile[STR_DEFAULT_SIZE] = {""};

/*---------------------------------------------------------------------------- BUILD FILENAME */
static
buildFilename(void)
{
    strcpy(logFile, FileIOGetCurrentDirectory());    
    strcat(logFile, LOG_FILE);	
}

/*---------------------------------------------------------------------------- INIT */
RESULT 
ConsoleInit(void)
{
	buildFilename();
	
	DeleteFile(logFile);
		 
	return OK;
}

/*---------------------------------------------------------------------------- ADD */
RESULT
ConsoleAdd(const INT8 *msg, ...)
{
	FILE *file = NULL;
	HWND hwnd = NULL;
    INT8 str[STR_DEFAULT_SIZE] = {""};
    va_list argList;
    
    if (msg==NULL)
    {
        return ERR_INVALID_PTR;
    }

    va_start(argList, msg);
    vsprintf(str, msg, argList);
    va_end(argList);

    hwnd = (HWND) UserInterfaceGetID();

    if (hwnd!=NULL)
    {
        SendDlgItemMessage(hwnd, IDC_CONSOLE, LB_ADDSTRING, 0, (LPARAM) &str[0]);
		SendDlgItemMessage(hwnd, IDC_CONSOLE, LB_SETCURSEL, SendDlgItemMessage(hwnd, IDC_CONSOLE, LB_GETCOUNT, 0, 0)-1, 0);

        EnableWindow(GetDlgItem(hwnd, IDC_CLEAR_CONSOLE), TRUE);

        UpdateWindow(hwnd);
    }

	buildFilename();
	
	SetFileAttributes(logFile, FILE_ATTRIBUTE_NORMAL);
	
    file = fopen(logFile,"a+");
    
    if (file!=NULL)
	{
		fprintf(file, "%s\n", str);

		fclose(file);
	}
	
	return OK;
}

/*---------------------------------------------------------------------------- PROGRESS SET */
RESULT
ConsoleProgressBarSet(INT32 i)
{
    HWND hwnd = NULL;
    
    if (i<0)
    {
        i = 0;
    }
    else if (i>100)
    {
        i = 100;
    }
    
    hwnd = (HWND) UserInterfaceGetID();

    if (hwnd!=NULL)
    {
        SendDlgItemMessage(hwnd, (LPARAM) MAKEINTRESOURCE(IDC_PROGRESS_BAR), PBM_SETPOS, i, 0);
        
        UpdateWindow(hwnd);
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- FREE */
RESULT
ConsoleFree(void)
{
	return OK;
}
