/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * main.c - Main                                                              *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "platform/os/window.h"
#include "platform/renderer/matrix.h"
#include "platform/os/timer.h"
#include "platform/os/gui.h"
#include "platform/os/math.h"
#include "system/file.h"
#include "system/filearc.h"
#include "system/filepack.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "texture/textga.h"
#include "texture/texjpg.h"
#include "texture/texjpgtiny.h"
#include "texture/texrgb.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "platform/renderer/lwsdraw.h"
#include "system/snapshot.h"
#include "platform/os/entry.h"
#include "lightwave/lws/lwsplaylist.h"
#include "system/cmdline.h"
#include "platform/os/music.h"
#include "platform/software/soft.h"
#include "platform/software/softpreca.h"
#include "platform/software/softdraw.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define RENDER_NAME				"Software"

#define MSG_INVALID_SCENE       "invalid scene (check log window)"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static UINT8 renderName[] = {RENDER_NAME};

static BOOL viewLines = FALSE;
static INT32 tick = 0;
static INT32 invalidTime = 0;
static INT32 scene = 0;
static INT32 sceneCount = 0;
static INT32 offsetTick = 0;
static FLOAT32 framesPerSecond = 0.0f;
static FLOAT32 lastTime = 0.0f;
static FLOAT32 currentTime = 0.0f;
static INT32 progressCount = 0;
static INT32 displayFPS = 0;
static BOOL displayText = TRUE;
static SoftViewport viewport;
	    
/*---------------------------------------------------------------------------- PROGRESS */
static void
progress(INT32 i)
{
    INT32 count = PlayListGetCount();

    i /= count;

    i += (INT32) ((((float) progressCount/(float) count) * 100.0f));

    ConsoleProgressBarSet(i);
}

/*---------------------------------------------------------------------------- FPS GET */
static INT32 
fpsGet(INT32 tickCount)
{
    INT32 res = 0;
    
    currentTime = (FLOAT32) tickCount/1000.0f;
        
    framesPerSecond += 1.0f;

    if (currentTime-lastTime>1.0f)
    {
        lastTime = currentTime;
        res = (INT32) framesPerSecond;
        framesPerSecond = 0.0f;
    }

    return res;
}

/*---------------------------------------------------------------------------- SNAPSHOT */
static void 
snapShot(void)
{
    Tex tex;
    Window *window = NULL;
    GLvoid *pixels = NULL;

    window = WindowGet();
    
    pixels = MemAlloc(window->resX*window->resY*sizeof(INT32));
    
    /*
	glReadPixels(0, 0, window->resX, window->resY, GL_BGR_EXT, GL_UNSIGNED_BYTE, pixels);
	*/
	    
    tex.depth = 24;
    tex.buffer = (INT32 *) pixels;
    tex.width = (UINT16) window->resX;
    tex.height = (UINT16) window->resY;
    strcpy(tex.fileName, SnapshotGetFileName());
    
    SnapshotSave(&tex);

    SAFE_FREE(pixels);

    ConsoleAdd("Snapshot created (%s)", tex.fileName);
}

/*---------------------------------------------------------------------------- NEXT */
static void 
nextScene(void)
{
    INT32 j = 0;
    
    if (PlayListGetCount()==0)
    {
        return;
    }

    PlayListNext(&j);

    PlayListGet()->lws->tickBase = 0;
    tick = 0;
	invalidTime = 0;

	TimerInit();
}

/*---------------------------------------------------------------------------- INIT */
static void 
sceneInit(void)
{
    INT32 i = 0;
    INT32 j = 0;
	
    viewLines = FALSE;
    
    scene = 0;
    sceneCount = PlayListGetCount();

    PlayListSet(0);

    i = 0;

    while (i<sceneCount)
    {
        ConsoleAdd("Loading %s", (INT8 *) PlayListGet()->fileName);
            
        PlayListGet()->lws = MemAlloc(sizeof(Lws));
        PlayListGet()->isValid = FALSE;

        progressCount = i;
    
        if (LwsLoad(PlayListGet()->lws, (INT8 *) PlayListGet()->fileName, progress)==OK)
        {
            PlayListGet()->isValid = TRUE;
        }

		LwsSetPreca(PlayListGet()->lws, (INT8 *) SoftPrecaBuildFileName((INT8 *) PlayListGet()->fileName));

		SoftPrecaInit(FALSE, 0, 0, NULL, 0, 0);
    
        PlayListNext(&j);

        i++;
    }

    ConsoleProgressBarSet(0);
}

/*---------------------------------------------------------------------------- INIT */
static void 
cbInit(void)
{    
    ConsoleAdd("Managing textures");

    TexUploadAllToHard();

	tick = 0;

	TimerInit();

	offsetTick = 0;

	invalidTime = 0;
	
	PlayListSet(0);
    
    ConsoleFree();
    
    displayFPS = 0;
    
    MusicPlay();

    displayText = TRUE;

	ConsoleAdd("Playing");
}

/*---------------------------------------------------------------------------- DISPLAY INIT */
static void
drawDisplayInit(void)
{
    /*
	glClear(GL_DEPTH_BUFFER_BIT);
    glDisable(GL_BLEND);
    glEnable(GL_COLOR_MATERIAL);
    glDisable(GL_LIGHTING);
    glColorMaterial(GL_FRONT_AND_BACK, GL_DIFFUSE);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_POLYGON_SMOOTH);
    glColor4f(1.0f, 1.0f, 0.9f, 1.0f);
	*/
}

extern void writeCpcFile();
extern void initCpcFile();

/*---------------------------------------------------------------------------- DRAW */
static void 
cbDraw(void)
{    
    FLOAT32 frameCount = 0.0f;
    FLOAT32 frameStep = 0.0f;
    Lws *lws = NULL;

	if (PlayListGetCount()==0)
    {
        /*
		glClearColor(1.0f, 0.0f, 0.0f, 0.0f);
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		*/
		
        drawDisplayInit();
        WindowDisplayText(8, 0, MSG_INVALID_SCENE);
    }
    else if (PlayListGet()->isValid==FALSE)
    {
        /*
		glClearColor(1.0f, 0.0f, 0.0f, 0.0f);
        glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
		*/

		drawDisplayInit();
        WindowDisplayText(8, 0, MSG_INVALID_SCENE);
        
        if (invalidTime==0)
		{
			invalidTime = 0;
		}
		else if (invalidTime<0)
		{
			nextScene();
		}
    }
    else
    {
        lws = PlayListGet()->lws;

        SoftDrawGetViewport()->buffer = WindowGet()->buffer;
		SoftDrawClearViewport((UINT8) (lws->backDropR*255.0f), (UINT8) (lws->backDropG*255.0f), (UINT8) (lws->backDropB*255.0f), 0);

		/*
		glClearColor(lws->backDropR, lws->backDropG, lws->backDropB, 0.0f);
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

		glDepthFunc(GL_LEQUAL);
		glDepthRange(0.0f, 1.0f);
		glEnable(GL_DEPTH_TEST);
		*/

		tick = TimerGetTime();

        {
            INT32 fps = 0;

            fps = fpsGet(tick);

            if (fps>0)
            {
                displayFPS = fps;
            }
        }
            
        if (tick>=lws->tickEnd && lws->tickBase>0)
        {
			writeCpcFile( );
			
			nextScene();
        }

        lws = PlayListGet()->lws;
		
		frameCount = (FLOAT32) (lws->lastFrame-lws->firstFrame);
            
        if (lws->tickBase==0)
        {
            frameStep = (1000.0f/(FLOAT32) lws->frameStep);

            lws->tickBase = tick;
            lws->tickEnd = lws->tickBase+(INT32) ((frameCount/(FLOAT32) lws->framesPerSecond)*frameStep);
        }

        lws->time = ((FLOAT32) (tick-lws->tickBase))/((FLOAT32) (lws->tickEnd-lws->tickBase));
		lws->viewPortResX = WindowGet()->resX;
        lws->viewPortResY = WindowGet()->resY;

		lws->offsetUV = (FLOAT32) ((offsetTick-tick)%2000)/2000.0f;
		
        lws->viewLines = viewLines;
        
        LwsEnable(lws);

        if (displayText)
		{
			drawDisplayInit();
			WindowDisplayText(8, 0, "Keys: <space bar> skip scene, <L> lines mode, <S> snapshot, <T> text");
			WindowDisplayText(8, 1, lws->fileName);
			WindowDisplayText(8, 2, "Frame %d/%d (%d %%)", (INT32) (frameCount*lws->time), (INT32) lws->lastFrame, (INT32) (lws->time*100.0f));
			WindowDisplayText(8, 3, "%d triangles", lws->triCount);
			WindowDisplayText(8, 4, "%d fps", displayFPS);
		}
        
        if (viewLines)
        {
            WindowDisplayText(8, 5, "Lines mode");
        }
    }
}

/*---------------------------------------------------------------------------- FREE */
static void 
cbFree(void)
{    
    INT32 i = 0;
    INT32 j = 0;
    
    MusicStop();
    
    PlayListSet(0);

    i = 0;

    while (i<sceneCount)
    {
        ConsoleAdd("Freeing %s", (INT8 *) PlayListGet()->fileName);
        
        LwsFree(PlayListGet()->lws);
    
        PlayListNext(&j);

        i++;
    }

    ConsoleAdd("textures");        
    TexRemoveAllFromHard();	
    TexFreeAll();

    ConsoleAdd("misc");
    FileArcFree();
    MemFreeAll();

    ConsoleAdd("Done.");
}

/*---------------------------------------------------------------------------- KEY PRESS */
static void 
cbKey(INT32 k)
{   
    switch (k)
    {
        case ' ':
            nextScene();
            break;

        case 'l':
        case 'L':
            if (viewLines)
            {
                viewLines = FALSE;
            }
            else
            {
                viewLines = TRUE;
            }
            break;

        case 's':
        case 'S':
            snapShot();
            break;

		case 't':
		case 'T':
			if (displayText==TRUE)
			{
				displayText = FALSE;
			}
			else
			{
				displayText = TRUE;
			}
			break;
    }
}

/*---------------------------------------------------------------------------- API NAME */
UINT8 *
GetRenderName(void)
{
	return renderName;
}

/*---------------------------------------------------------------------------- MAIN */
void 
Main(void)
{
    Window *window = NULL;
	TexSettings settings;
    
    ConsoleAdd("");
	
	//CmdLineParse();
    
    MathInit();
	MatrixInit();

	settings.doRGBConvert = TRUE;
	settings.doScalePower2 = FALSE;
	settings.maxPower2 = 1024;
    TexInit(&settings);
	TexRGBSet(ABGR8888);

    TexRegisterCodec("TGA", TgaLoad);
    TexRegisterCodec("JPG", JpgLoad);
    
    SnapshotRegisterCodec(TgaSave);

    sceneInit();
    
    ConsoleAdd("Initializing window");
    
    window = WindowGet();    
    strcpy((INT8 *) window->title, "jfyeNG - Playing..");
        
    viewport.x1 = (INT16) (0);
    viewport.y1 = (INT16) (0);
    viewport.x2 = (INT16) (window->resX);
    viewport.y2 = (INT16) (window->resY);
	SoftDrawSetViewport(&viewport);
	
	WindowSetInit(cbInit);
    WindowSetDraw(cbDraw);
    WindowSetFree(cbFree);
    WindowSetKey(cbKey);

	initCpcFile();

	WindowShow();
}
