/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//-----------------------------------------------------------------------------
// File: SwitchScreenMode.cpp
//
// Desc: This sample demonstrates how to switch between windowed and
//       full-screen exclusive DDraw cooperative levels.
//
// Copyright (c) 1999-2000 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
#define STRICT
#include <windows.h>
#include <ddraw.h>
#include <mmsystem.h>
#include "platform/os/win32/rsc/resource.h"
#include "resource.h"
#include "ddutil.h"
#include "config.h"
#include "platform/os/window.h"
#include "cpcConvert.h"

//-----------------------------------------------------------------------------
// Defines, constants, and global variables
//-----------------------------------------------------------------------------
#define SAFE_DELETE(p)  { if(p) { delete (p);     (p)=NULL; } }
#define SAFE_RELEASE(p) { if(p) { (p)->Release(); (p)=NULL; } }

int SCREEN_WIDTH =0;
int SCREEN_HEIGHT =0;
#define SCREEN_BPP      32

#define SPRITE_DIAMETER 48
#define NUM_SPRITES     25

struct SPRITE_STRUCT
{
    FLOAT                fPosX; 
    FLOAT                fPosY;
    FLOAT                fVelX; 
    FLOAT                fVelY;
};

CDisplay*            g_pDisplay        = NULL;
CSurface*            g_pLogoSurface    = NULL;  
CSurface*            g_pTextSurface    = NULL;  
RECT                 g_rcWindow;            
RECT                 g_rcViewport;          
RECT                 g_rcScreen;            
BOOL                 g_bWindowed   = TRUE;  
BOOL                 g_bActive     = FALSE;
DWORD                g_dwLastTick;

static INT32 *YTable = NULL;




//-----------------------------------------------------------------------------
// Function-prototypes
//-----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam );
HRESULT WinInit( HINSTANCE hInst, HWND* phWnd, INT8 *title, HACCEL* phAccel, HWND parent );
HRESULT InitDirectDraw( HWND hWnd );
VOID    FreeDirectDraw();
HRESULT InitDirectDrawMode( HWND hWnd, BOOL bWindowed );
HRESULT ProcessNextFrame( HWND hWnd );
HRESULT DisplayFrame();
HRESULT RestoreSurfaces();



/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static void (*funcInit)(void) = NULL;
static void (*funcDraw)(void) = NULL;
static void (*funcFree)(void) = NULL;
static void (*funcKey)(INT32 key) = NULL;
static INT32 *buffer = NULL;
static INT32 bufferPitch = 0;
static INT32 lBpp = 0;

//-----------------------------------------------------------------------------
// Name: WinMain()
// Desc: Entry point to the program. Initializes everything and calls
//       UpdateFrame() when idle from the message pump.
//-----------------------------------------------------------------------------

extern "C" INT32*
DXWindowGetBuffer(void)
{
	return buffer;
}

extern "C" INT32
DXWindowGetBufferPitch(void)
{
	return bufferPitch;
}


/*---------------------------------------------------------------------------- SET INIT */
extern "C" RESULT
DXWindowSetInit(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcInit = func;

    return OK;
}

/*---------------------------------------------------------------------------- SET DRAW */
extern "C" RESULT
DXWindowSetDraw(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcDraw = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- SET FREE */
extern "C" RESULT
DXWindowSetFree(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcFree = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- SET KEY */
extern "C" RESULT
DXWindowSetKey(void (*func)(INT32 param))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcKey = func;
    
    return OK;
}

extern "C" 
int DXWindowShow( HINSTANCE hInst, HWND parent, INT8* title, INT32 width, INT32 height, INT32 isFullScreen)
{
    MSG	   msg;
    HWND   hWnd;
    HACCEL hAccel;

	SCREEN_WIDTH = width;
	SCREEN_HEIGHT = height;

    funcInit();
	
    srand( GetTickCount() );

    if( FAILED( WinInit( hInst, &hWnd,title, &hAccel, parent ) ) )
        return FALSE;

    g_bWindowed = TRUE;
	if (isFullScreen)
	{
		g_bWindowed = FALSE;
	}

	YTable = new INT32[height];
	    
	if( FAILED( InitDirectDraw( hWnd) ) )
    {
        MessageBox( hWnd, TEXT("DirectDraw init failed. ")
                    TEXT("The sample will now exit. "), TEXT("DirectDraw Sample"), 
                    MB_ICONERROR | MB_OK );
        return FALSE;
    }

	g_dwLastTick = timeGetTime();

    while( TRUE )
    {
        // Look for messages, if none are found then 
        // update the state and display it
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            if( 0 == GetMessage(&msg, NULL, 0, 0 ) )
            {
                // WM_QUIT was posted, so exit
				UnregisterClass(TEXT("SwitchScreenMode"), hInst);
					
				funcFree();

                return (int)msg.wParam;
            }

            // Translate and dispatch the message
            if( 0 == TranslateAccelerator( hWnd, hAccel, &msg ) )
            {
                TranslateMessage( &msg ); 
                DispatchMessage( &msg );
            }
        }
        else
        {
            if( g_bActive )
            {
                // Move the sprites, blt them to the back buffer, then 
                // flip or blt the back buffer to the primary buffer
                if( FAILED( ProcessNextFrame( hWnd ) ) )
                {
                    SAFE_DELETE( g_pDisplay );

                    MessageBox( hWnd, TEXT("Displaying the next frame failed. ")
                                TEXT("The sample will now exit. "), TEXT("DirectDraw Sample"), 
                                MB_ICONERROR | MB_OK );
                    return FALSE;
                }
            }
            else
            {
                // Go to sleep if we have nothing else to do
                WaitMessage();

                // Ignore time spent inactive 
                g_dwLastTick = timeGetTime();
            }
        }
    }

	delete YTable;
}




//-----------------------------------------------------------------------------
// Name: WinInit()
// Desc: Init the window
//-----------------------------------------------------------------------------
HRESULT WinInit( HINSTANCE hInst, HWND* phWnd, INT8 *title, HACCEL* phAccel, HWND parent )
{
    WNDCLASSEX wc;
    HWND       hWnd;
    HACCEL     hAccel;

    // Register the window class
    wc.cbSize        = sizeof(wc);
    wc.lpszClassName = TEXT("SwitchScreenMode");
    wc.lpfnWndProc   = MainWndProc;
    wc.style         = CS_VREDRAW | CS_HREDRAW;
    wc.hInstance     = hInst;
    wc.hIcon         = LoadIcon(hInst, MAKEINTRESOURCE(IDI_JFYENG));
    wc.hIconSm       = LoadIcon(hInst, MAKEINTRESOURCE(IDI_JFYENG));
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH) GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName  = NULL;
    wc.cbClsExtra    = 0;
    wc.cbWndExtra    = 0;

    if( RegisterClassEx( &wc ) == 0 )
        return E_FAIL;

    // Load keyboard accelerators
    hAccel = LoadAccelerators( hInst, MAKEINTRESOURCE(IDR_MAIN_ACCEL) );

    // Calculate the proper size for the window given a client of 640x480
    DWORD dwFrameWidth    = GetSystemMetrics( SM_CXSIZEFRAME );
    DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
    DWORD dwMenuHeight    = 0;
    DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );
    DWORD dwWindowWidth   = SCREEN_WIDTH  + dwFrameWidth * 2;
    DWORD dwWindowHeight  = SCREEN_HEIGHT + dwFrameHeight * 2 + 
                            dwMenuHeight + dwCaptionHeight;

    
    // Create and show the main window
    DWORD dwStyle = WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX;    
	hWnd = CreateWindowEx( 0, TEXT("SwitchScreenMode"), 
                           (char*) title,
                           dwStyle, (GetSystemMetrics(SM_CXSCREEN)-dwWindowWidth)>>1, (GetSystemMetrics(SM_CYSCREEN)-dwWindowHeight)>>1,
  	                       dwWindowWidth, dwWindowHeight, parent, NULL, hInst, NULL );
    if( hWnd == NULL )
    	return E_FAIL;

    lBpp = GetDeviceCaps(GetDC(NULL), BITSPIXEL);
	
	ShowWindow( hWnd, SW_SHOW );
    UpdateWindow( hWnd );

    // Save the window size/pos for switching modes
    GetWindowRect( hWnd, &g_rcWindow );

    *phWnd   = hWnd;
    *phAccel = hAccel;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitDirectDraw()
// Desc: Create the DirectDraw object, and init the surfaces
//-----------------------------------------------------------------------------
HRESULT InitDirectDraw( HWND hWnd )
{
    HRESULT hr;

    // Initialize all the surfaces we need
    if( FAILED( hr = InitDirectDrawMode( hWnd, g_bWindowed ) ) )
        return hr;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: InitDirectDrawMode()
// Desc: Called when the user wants to toggle between full-screen and windowed 
//       to create all the needed DDraw surfaces and set the coop level
//-----------------------------------------------------------------------------
HRESULT InitDirectDrawMode( HWND hWnd, BOOL bWindowed )
{
    HRESULT		        hr;
    LPDIRECTDRAWPALETTE pDDPal   = NULL; 

    // Release all existing surfaces
    SAFE_DELETE( g_pLogoSurface );
    SAFE_DELETE( g_pTextSurface );
    SAFE_DELETE( g_pDisplay );

    // The back buffer and primary surfaces need to be created differently 
    // depending on if we are in full-screen or windowed mode
    g_pDisplay = new CDisplay();

    if( bWindowed )
    {
        if( FAILED( hr = g_pDisplay->CreateWindowedDisplay( hWnd, SCREEN_WIDTH, 
                                                            SCREEN_HEIGHT ) ) )
            return hr;

        // Add the system menu to the window's style
		/*
		DWORD dwStyle = GetWindowLong( hWnd, GWL_STYLE );
        dwStyle |= WS_SYSMENU;
        SetWindowLong( hWnd, GWL_STYLE, dwStyle );
		*/

		DWORD dwFrameWidth    = GetSystemMetrics( SM_CXSIZEFRAME );
		DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
		DWORD dwMenuHeight    = 0;
		DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );
		DWORD dwWindowWidth   = SCREEN_WIDTH  + dwFrameWidth * 2;
		DWORD dwWindowHeight  = SCREEN_HEIGHT + dwFrameHeight * 2 + 
							                    dwMenuHeight + dwCaptionHeight;
		SetWindowPos(hWnd, NULL, (GetSystemMetrics(SM_CXSCREEN)-dwWindowWidth)>>1, (GetSystemMetrics(SM_CYSCREEN)-dwWindowHeight)>>1, dwWindowWidth, dwWindowHeight, NULL);

        /*
		// Show the menu in windowed mode 
#ifdef _WIN64
        HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr( hWnd, GWLP_HINSTANCE );
#else
        HINSTANCE hInst = (HINSTANCE) GetWindowLong( hWnd, GWL_HINSTANCE );
#endif	
		*/
    }
    else
    {
        if( FAILED( hr = g_pDisplay->CreateFullScreenDisplay( hWnd, SCREEN_WIDTH, 
                                                              SCREEN_HEIGHT, SCREEN_BPP ) ) )
        {
            MessageBox( hWnd, TEXT("This display card does not support 640x480x8. "),
                        TEXT("DirectDraw Sample"), MB_ICONERROR | MB_OK );
            return hr;
        }

        // Disable the menu in full-screen since we are 
        // using a palette and a menu would look bad 
        /*
		SetMenu( hWnd, NULL );
		*/

        // Remove the system menu from the window's style
        /*
		DWORD dwStyle = GetWindowLong( hWnd, GWL_STYLE );
        dwStyle &= ~WS_SYSMENU;
        SetWindowLong( hWnd, GWL_STYLE, dwStyle );       
		*/
    }

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: FreeDirectDraw()
// Desc: Release all the DirectDraw objects
//-----------------------------------------------------------------------------
VOID FreeDirectDraw()
{
    SAFE_DELETE( g_pLogoSurface );
    SAFE_DELETE( g_pTextSurface );
    SAFE_DELETE( g_pDisplay );
}




//-----------------------------------------------------------------------------
// Name: MainWndProc()
// Desc: The main window procedure
//-----------------------------------------------------------------------------
LRESULT CALLBACK MainWndProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch (msg)
    {
        case WM_KEYDOWN:
            switch( LOWORD(wParam) )
            {
                case VK_ESCAPE:
                    // Received key/menu command to exit app
            	    PostMessage( hWnd, WM_CLOSE, 0, 0 );
                    return 0L;

				default:
					funcKey((int) wParam);

					return 0L;
            }
            break; // Continue with default processing

        case WM_SYSKEYDOWN:
            switch( LOWORD(wParam) )
            {
                case VK_RETURN:
					// Toggle the fullscreen/window mode
					if( g_bWindowed )
						GetWindowRect( hWnd, &g_rcWindow );

					g_bWindowed = !g_bWindowed;

					if( FAILED( InitDirectDrawMode( hWnd, g_bWindowed ) ) )
					{
						SAFE_DELETE( g_pDisplay );

						MessageBox( hWnd, TEXT("InitDirectDraw() failed. ")
									TEXT("The sample will now exit. "), TEXT("DirectDraw Sample"), 
									MB_ICONERROR | MB_OK );
                		PostMessage( hWnd, WM_CLOSE, 0, 0 );
					}
					
					lBpp = GetDeviceCaps(GetDC(NULL), BITSPIXEL);

					return 0L;
            }
            break; // Continue with default processing

        case WM_GETMINMAXINFO:
            {
                // Don't allow resizing in windowed mode.  
                // Fix the size of the window to 640x480 (client size)
                MINMAXINFO* pMinMax = (MINMAXINFO*) lParam;

                DWORD dwFrameWidth    = GetSystemMetrics( SM_CXSIZEFRAME );
                DWORD dwFrameHeight   = GetSystemMetrics( SM_CYSIZEFRAME );
                DWORD dwMenuHeight    = 0;
                DWORD dwCaptionHeight = GetSystemMetrics( SM_CYCAPTION );

                pMinMax->ptMinTrackSize.x = SCREEN_WIDTH  + dwFrameWidth * 2;
                pMinMax->ptMinTrackSize.y = SCREEN_HEIGHT + dwFrameHeight * 2 + 
                                            dwMenuHeight + dwCaptionHeight;

                pMinMax->ptMaxTrackSize.x = pMinMax->ptMinTrackSize.x;
                pMinMax->ptMaxTrackSize.y = pMinMax->ptMinTrackSize.y;
            }
            return 0L;

        case WM_MOVE:
            // Retrieve the window position after a move.  
            if( g_pDisplay )
                g_pDisplay->UpdateBounds();

            return 0L;

        case WM_SIZE:
            // Check to see if we are losing our window...
            if( SIZE_MAXHIDE==wParam || SIZE_MINIMIZED==wParam )
                g_bActive = FALSE;
            else
                g_bActive = TRUE;

            if( g_pDisplay )
                g_pDisplay->UpdateBounds();
            break;

        case WM_SETCURSOR:
            // Hide the cursor if in fullscreen 
            if( !g_bWindowed )
            {
                SetCursor( NULL );
                return TRUE;
            }
            break; // Continue with default processing

        case WM_QUERYNEWPALETTE:
            if( g_pDisplay && g_pDisplay->GetFrontBuffer() )            
            {
                // If we are in windowed mode with a desktop resolution in 8 bit 
                // color, then the palette we created during init has changed 
                // since then.  So get the palette back from the primary 
                // DirectDraw surface, and set it again so that DirectDraw 
                // realises the palette, then release it again. 
                LPDIRECTDRAWPALETTE pDDPal = NULL; 
                g_pDisplay->GetFrontBuffer()->GetPalette( &pDDPal );
                g_pDisplay->GetFrontBuffer()->SetPalette( pDDPal );
                SAFE_RELEASE( pDDPal );
            }
            break;

        case WM_EXITMENULOOP:
            // Ignore time spent in menu
            g_dwLastTick = timeGetTime();
            break;

        case WM_EXITSIZEMOVE:
            // Ignore time spent resizing
            g_dwLastTick = timeGetTime();
            break;

        case WM_SYSCOMMAND:
            // Prevent moving/sizing and power loss in fullscreen mode
            switch( wParam )
            {
                case SC_MOVE:
                case SC_SIZE:
                case SC_MAXIMIZE:
                case SC_MONITORPOWER:
                    if( !g_bWindowed )
                        return TRUE;
            }
            break;
            
        case WM_DESTROY:
            // Cleanup and close the app
            FreeDirectDraw();
            PostQuitMessage( 0 );
            return 0L;
    }

    return DefWindowProc(hWnd, msg, wParam, lParam);
}




//-----------------------------------------------------------------------------
// Name: ProcessNextFrame()
// Desc: Move the sprites, blt them to the back buffer, then 
//       flip or blt the back buffer to the primary buffer
//-----------------------------------------------------------------------------
HRESULT ProcessNextFrame( HWND hWnd )
{
    HRESULT hr;

    // Figure how much time has passed since the last time
    DWORD dwCurrTick = timeGetTime();
    DWORD dwTickDiff = dwCurrTick - g_dwLastTick;

    // Don't update if no time has passed 
    if( dwTickDiff == 0 )
        return S_OK; 

    g_dwLastTick = dwCurrTick;

    // Check the cooperative level before rendering
    if( FAILED( hr = g_pDisplay->GetDirectDraw()->TestCooperativeLevel() ) )
    {
        switch( hr )
        {
            case DDERR_EXCLUSIVEMODEALREADYSET:
            case DDERR_NOEXCLUSIVEMODE:
                // Do nothing because some other app has exclusive mode
                Sleep(10);
                return S_OK;

            case DDERR_WRONGMODE:
                // The display mode changed on us. Update the
                // DirectDraw surfaces accordingly
                return InitDirectDrawMode( hWnd, g_bWindowed );
        }
        return hr;
    }

    // Display the sprites on the screen
    if( FAILED( hr = DisplayFrame() ) )
    {
        if( hr != DDERR_SURFACELOST )
            return hr;

        // The surfaces were lost so restore them 
        RestoreSurfaces();
    }

    return S_OK;
}



//-----------------------------------------------------------------------------
// Name: DisplayFrame()
// Desc: Blts a the sprites to the back buffer, then it blts or flips the 
//       back buffer onto the primary buffer.
//-----------------------------------------------------------------------------
extern "C" int writeCPCOutput = 0;

int add = 0;
HRESULT DisplayFrame()
{
    HRESULT hr;
	DDSURFACEDESC2 ddsd;
	INT32 lResX = 0;
	INT32 lResY = 0;
    
    // Fill the back buffer with black, ignoring errors until the flip
    g_pDisplay->Clear( 0 );

    // Blt the help text on the backbuffer, ignoring errors until the flip
    //g_pDisplay->Blt( 10, 10, g_pTextSurface, NULL );
	
	LPDIRECTDRAWSURFACE7 pDDS = g_pDisplay->GetBackBuffer();

	ZeroMemory( &ddsd,sizeof(ddsd) );
    ddsd.dwSize = sizeof(ddsd);

	RECT rect;
	rect.left = 0;
	rect.top = 0;
	rect.right = ddsd.dwWidth;
	rect.bottom = ddsd.dwHeight;
	
	// Lock the surface to directly write to the surface memory 
    if( FAILED( hr = pDDS->Lock( &rect, &ddsd, DDLOCK_SURFACEMEMORYPTR, NULL ) ) )
        return hr;

	int* pDDSColor = (int*) ddsd.lpSurface;

	bufferPitch = ddsd.lPitch/(SCREEN_BPP/8);
	buffer = pDDSColor;

	funcDraw();

	lResX = SCREEN_WIDTH;
	lResY = SCREEN_HEIGHT;

	if (writeCPCOutput)
	{
		cpcConvert( lResX, lResY, (unsigned int *) buffer, bufferPitch );
	}

	/*
	TODO

	if (g_bWindowed)
	{
		unsigned char CoulR, CoulG, CoulB, *p32Page;
		long lX, lY;
		
		if(SCREEN_BPP==32 && lBpp==24)
		{
			p32Page = (unsigned char*)pDDSColor;

			for(lY=0; lY<lResY; lY++)
			{
				for(lX = 0; lX < lResX; lX++)
				{
					*(p32Page+(lX*3)  ) = *(p32Page+(lX<<2)  );
					*(p32Page+(lX*3)+1) = *(p32Page+(lX<<2)+1);
					*(p32Page+(lX*3)+2) = *(p32Page+(lX<<2)+2);
				}

				p32Page += bufferPitch * (SCREEN_BPP/8);
			}
		}
		else
		if(SCREEN_BPP==32 && lBpp==16)
		{
			p32Page = (unsigned char*)pDDSColor;

			for(lY=0; lY<lResY; lY++)
			{
				for(lX = 0; lX < lResX; lX++)
				{
					CoulB = *(p32Page+(lX<<2)  )>>3;
					CoulG = *(p32Page+(lX<<2)+1)>>2;
					CoulR = *(p32Page+(lX<<2)+2)>>3;
					
					*(unsigned int*)(p32Page+(lX<<2)) =
						((unsigned int)CoulR<<11)+((unsigned int)CoulG<<5)+CoulB;
				}

				p32Page += bufferPitch*(SCREEN_BPP/8);
			}
		}
	}

	if(lBpp==16)
	{
		// convert RGB565 to RGB555
		if (ddsd.ddpfPixelFormat.dwGBitMask != 0x7E0)
		{// conversion routine greatly improved by Xbarr/Syndrome
			long *pAsmPage = (long*) pDDSColor; // "long" is not the real type of the pointer
			long lResX_ASM = lResX;
			long lResY_ASM = lResY;

			for (int iLoop=0; iLoop<lResY; iLoop++) 
				YTable[iLoop] = (long) iLoop * ddsd.lPitch/(SCREEN_BPP/8);	
			
			_asm
			{
				mov		edx,lResY_ASM
ConvertY:		dec		edx
				cmp		edx,-1
				je		ConvertEnd
				mov		ecx,lResX_ASM
				sal		ecx,1
ConvertX:		sub		ecx,4
				jl		ConvertXEnd
				mov		edi,YTable[edx*4]
				add		edi,pAsmPage
				add		edi,ecx
				mov		ebx,[edi]
				mov		eax,ebx
				and		ebx,11111111110000001111111111000000b
				and		eax,00000000000111110000000000011111b
				shr		ebx,1
				or		eax,ebx
				mov		[edi],eax
				jmp		short ConvertX
ConvertXEnd:	jmp		short ConvertY
ConvertEnd:
			}
		}
	}
	*/

	// Unlock the surface
    pDDS->Unlock(NULL);

    // Flip or blt the back buffer onto the primary buffer
    if( FAILED( hr = g_pDisplay->Present() ) )
        return hr;

    return S_OK;
}




//-----------------------------------------------------------------------------
// Name: RestoreSurfaces()
// Desc: Restore all the surfaces, and redraw the sprite surfaces.
//-----------------------------------------------------------------------------
HRESULT RestoreSurfaces()
{
    HRESULT hr;
    
	if( FAILED( hr = g_pDisplay->GetDirectDraw()->RestoreAllSurfaces() ) )
        return hr;

    if( g_bWindowed )
    {
        // No need to re-create the surface, just re-draw it.
        if( FAILED( hr = g_pTextSurface->DrawText( NULL, "windowed", 
                                                   0, 0, RGB(0,0,0), RGB(255, 255, 0) ) ) )
            return hr;
    }
    else
    {
        // No need to re-create the surface, just re-draw it.
        if( FAILED( hr = g_pTextSurface->DrawText( NULL, "fullscreen", 
                                                   0, 0, RGB(0,0,0), RGB(255, 255, 0) ) ) )
            return hr;
    }

    return S_OK;
}
