/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * font.c - Font                                                              *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/font.h"
#include "platform/renderer/matrix.h"
#include "platform/os/window.h"
#include "platform/os/console.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define FONT_MAX_CHARS              256
#define CHAR_HEIGHT                 16
#define FONT_NAME                   "Tahoma"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static Font font;

static HFONT oldFont = NULL;
static INT32 listID = 0;
static HDC dc = 0;

/*---------------------------------------------------------------------------- CREATE */
static void 
fontCreate(void)
{
    HFONT f = NULL;
	
	dc = GetDC((HWND) WindowGet()->handle);

    listID = glGenLists(FONT_MAX_CHARS);	

    f = CreateFont(CHAR_HEIGHT, 0, 0, 0, FW_BOLD, FALSE, FALSE, FALSE, ANSI_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, FF_DONTCARE | DEFAULT_PITCH, FONT_NAME);

    oldFont = (HFONT) SelectObject(dc, f);

    wglUseFontBitmaps(dc, 0, FONT_MAX_CHARS-1, listID);
}

/*---------------------------------------------------------------------------- RELEASE */
static void
fontRelease(void)
{
    glDeleteLists(listID, FONT_MAX_CHARS);

	SelectObject(dc, oldFont);
}

/*---------------------------------------------------------------------------- SET POS */
static void
fontSetPos(INT32 x, INT32 y)
{
    glPushAttrib(GL_TRANSFORM_BIT | GL_VIEWPORT_BIT);

    MatrixMode(GL_PROJECTION);
    MatrixPush();	
    MatrixLoadIdentity();
    MatrixMode(GL_MODELVIEW);
    MatrixPush();
    MatrixLoadIdentity();

    y = WindowGet()->resY-CHAR_HEIGHT-y;
    glViewport(x-1, y-1, 0, 0);

    glRasterPos4f(0, 0, 0, 1);

    MatrixPop();
    MatrixMode(GL_PROJECTION);
    MatrixPop();
    glPopAttrib();
}

/*---------------------------------------------------------------------------- INIT */
RESULT
FontInit(void)
{
	ConsoleAdd("Initializing font");

	fontCreate();

	font.charHeight = CHAR_HEIGHT;
	
	return OK;
}

/*---------------------------------------------------------------------------- FREE */
RESULT
FontFree(void)
{
	fontRelease();
	
	return OK;
}

/*---------------------------------------------------------------------------- GET */
Font *
FontGet(void)
{
	return &font;
}

/*---------------------------------------------------------------------------- DISPLAY TEXT */
void
FontDisplayText(INT32 x, INT32 y, INT8 *msg)
{
 	if (msg==NULL)
    {
        return;
    }

	fontSetPos(x, y);

    glPushAttrib(GL_LIST_BIT);

    glListBase(listID);

    glCallLists(strlen(msg), GL_UNSIGNED_BYTE, msg);

    glPopAttrib();
}