/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * window.c - Output Window                                                   *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "platform/os/font.h"
#include "platform/os/win32/rsc/resource.h"
#include "platform/os/gui.h"
#include "platform/os/window.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static void (*funcInit)(void) = NULL;
static void (*funcDraw)(void) = NULL;
static void (*funcFree)(void) = NULL;
static void (*funcKey)(INT32 key) = NULL;
static HWND hwnd = NULL;
static RECT rect;
static HDC dc = NULL;
static HGLRC rc = NULL;
static HINSTANCE inst = NULL;
static WNDCLASS wc;
static Window window;
static BOOL destroy = FALSE;

/*---------------------------------------------------------------------------- RESIZE */
static void 
resizeWin(INT32 resX, INT32 resY)
{
    if (resY==0)
    {
        resY = 1;
    }

    glViewport(0, 0, resX, resY);
    
    window.resX = resX;
    window.resY = resY;
}

/*---------------------------------------------------------------------------- DESTROY */
static void 
destroyWin(void)
{
    if (rc!=NULL)
    {
        wglMakeCurrent(NULL, NULL);
        wglDeleteContext(rc);
    }
    
    if (dc!=NULL)
    {
        ReleaseDC(hwnd, dc);
    }

    UnregisterClass(wc.lpszClassName, inst);

    DestroyWindow(hwnd);

    hwnd = NULL;
}

/*---------------------------------------------------------------------------- CALLBACK */
static LRESULT CALLBACK 
winProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
    PAINTSTRUCT ps;
    
    switch (msg)
    {
    case WM_SIZE:
        if (!window.isFullScreen)
        {
            resizeWin(LOWORD(lParam), HIWORD(lParam));
            GetClientRect(hwnd, &rect);
        }
        return FALSE;

    case WM_PAINT:
        BeginPaint(hwnd, &ps);
        EndPaint(hwnd, &ps);
        return FALSE;

    case WM_KEYDOWN:
        switch(wParam)
        {
        case VK_ESCAPE:
            destroy = TRUE;
            break;

        default:
            funcKey((INT32) wParam);
            break;
        }
        return FALSE;

    case WM_DESTROY:
        destroy = TRUE;
        return FALSE;
    }

    return DefWindowProc(hwnd, msg, wParam, lParam);
}

/*---------------------------------------------------------------------------- SET INIT */
RESULT
WindowSetInit(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcInit = func;

    return OK;
}

/*---------------------------------------------------------------------------- SET DRAW */
RESULT
WindowSetDraw(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcDraw = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- SET FREE */
RESULT
WindowSetFree(void (*func)(void))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcFree = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- SET KEY */
RESULT
WindowSetKey(void (*func)(INT32 param))
{
    if (func==NULL)
    {
        return ERR_INVALID_PTR;
    }

    funcKey = func;
    
    return OK;
}

/*---------------------------------------------------------------------------- SHOW */
RESULT
WindowShow(void)
{
    DWORD style = 0;
    PIXELFORMATDESCRIPTOR pfd; 
    INT32 pf = 0; 
    MSG msg;
    INT32 posX = 0;
    INT32 posY = 0;
    HWND parent = NULL;

    inst = GetModuleHandle(NULL);
    parent = (HWND) UserInterfaceGetID();
    
    memset(&wc, 0, sizeof(WNDCLASS));
    wc.style = CS_HREDRAW|CS_VREDRAW;
    wc.lpfnWndProc = winProc;
    wc.hInstance = inst;
    wc.hIcon = LoadIcon(NULL, IDI_WINLOGO);
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = GetStockObject(BLACK_BRUSH);
    wc.lpszClassName = "window";
    RegisterClass(&wc);

    if (!window.isFullScreen)
    {
        style = WS_OVERLAPPEDWINDOW | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

        posX = (GetSystemMetrics(SM_CXSCREEN)-window.resX)>>1;
        posY = (GetSystemMetrics(SM_CYSCREEN)-window.resY)>>1;
    }
    else
    {
        style = WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

        posX = 0;
        posY = 0;
    }

    rect.left = 0;
    rect.right = window.resX;
    rect.top = 0;
    rect.bottom = window.resY;
    AdjustWindowRect(&rect, style, FALSE);

    hwnd = CreateWindow(wc.lpszClassName, (INT8 *) window.title, style, posX, posY, rect.right-rect.left, rect.bottom-rect.top, parent, NULL, inst, NULL);
                            
    if (hwnd==NULL)
    {
        return ERR;
    }

    window.handle = (UINT32) hwnd;
	
	SendMessage(hwnd, WM_SETICON, ICON_SMALL, (INT32) LoadIcon(inst, (char*)IDI_JFYENG));
	SendMessage(hwnd, WM_SETICON, ICON_BIG, (INT32) LoadIcon(inst, (char*)IDI_JFYENG));

    GetClientRect(hwnd, &rect);

	dc = GetDC(hwnd);

    pfd.nSize = sizeof(PIXELFORMATDESCRIPTOR);
    pfd.nVersion = 1;
    pfd.dwFlags = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER;
    pfd.dwLayerMask = PFD_MAIN_PLANE;
    pfd.iPixelType = PFD_TYPE_RGBA;
    pfd.cColorBits = 32;
    pfd.cDepthBits = 32;
    pfd.cAccumBits = 0;
    pfd.cStencilBits = 32;

    pf = ChoosePixelFormat(dc, &pfd);
    
    if (!pf)
    { 
        return ERR;
    }

    if (!SetPixelFormat(dc, pf, &pfd))
    { 
        return ERR;
    } 

    rc = wglCreateContext(dc);
    wglMakeCurrent(dc, rc);

    resizeWin(window.resX, window.resY);

    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT);
    SwapBuffers(dc);

    FontInit();
            
    funcInit();

    EnableWindow(parent, FALSE);

    if (window.isFullScreen)
    {
        DEVMODE devMode;
        
        memset(&devMode, 0, sizeof(DEVMODE));

        EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &devMode);

        devMode.dmPelsWidth = window.resX;
        devMode.dmPelsHeight = window.resY;
        
        ChangeDisplaySettings(&devMode, CDS_FULLSCREEN);

        ShowCursor(FALSE);
    }
    
    ShowWindow(hwnd, SW_SHOW);
    
    destroy = FALSE;
        
    while (!destroy)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            funcDraw();
    
			glFlush();

            SwapBuffers(dc);
        }
    }

    if (window.isFullScreen)
    {
        ChangeDisplaySettings(NULL, 0);

        ShowCursor(TRUE);
    }
        
    ShowWindow(hwnd, SW_HIDE);    
    EnableWindow(parent, TRUE);
    SetFocus(parent);
    
    FontFree();
    
    funcFree();

    destroyWin();    

    return OK;
}

/*---------------------------------------------------------------------------- GET */
Window *
WindowGet(void)
{
    return &window;
}

/*---------------------------------------------------------------------------- DISPLAY TEXT */
void
WindowDisplayText(INT32 x, INT32 negY, const INT8 *msg, ...)
{
    INT32 charHeight = 0;
	INT8 str[STR_DEFAULT_SIZE];
    va_list argList;
    
    if (msg==NULL)
    {
        return;
    }

    va_start(argList, msg);
    vsprintf(str, msg, argList);
    va_end(argList);

	charHeight = FontGet()->charHeight;

	FontDisplayText(x, window.resY-charHeight*2-charHeight*negY, str);
}