/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * music.c - Music                                                            *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/win32/lib/fmod/fmod.h"
#include "platform/os/console.h"
#include "platform/os/music.h"
#include "system/file.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static File file;
static FMUSIC_MODULE *mod = NULL;
static FSOUND_STREAM *stream = NULL;
static loaded = FALSE;
static INT8 audioFileName[STR_DEFAULT_SIZE] = {""};
    
/*---------------------------------------------------------------------------- SET FILE */
void
MusicSetFile(INT8 *fileName)
{
    if (fileName!=NULL)
    {
        strcpy(audioFileName, fileName);
    }
}

/*---------------------------------------------------------------------------- PLAY */
RESULT
MusicPlay(void)
{
    if (strlen(audioFileName)==0)
    {
        return ERR;
    }
    
    ConsoleAdd("Loading audio file");
    
    if (FileLoad(&file, audioFileName)!=OK)
    {
        ConsoleAdd("Error while opening audio file");

        return ERR_OPEN_FILE;
    }

    FSOUND_Init(44100, 32, 0);

    stream = NULL;
	mod = FMUSIC_LoadSongMemory(file.buffer, file.size);
	
    if (mod)
    {
        FMUSIC_PlaySong(mod);
    }
    else
    {
		stream = FSOUND_Stream_OpenFile(file.buffer, FSOUND_NORMAL | FSOUND_MPEGACCURATE | FSOUND_LOADMEMORY, file.size);

        FSOUND_Stream_Play(FSOUND_FREE, stream);

        if (!stream)
        {
            ConsoleAdd("Error while opening audio file");
            
            return ERR_OPEN_FILE;
        }
    }

    return OK;
}

/*---------------------------------------------------------------------------- STOP */
RESULT
MusicStop(void)
{
    if (strlen(audioFileName)==0)
    {
        return ERR;
    }
    
    if(mod)
    {
        FMUSIC_StopSong(mod);
    }
    else if (stream)
    {
        FSOUND_Stream_Close(stream);
    }
    
    FSOUND_Close();

    FileFree(&file);

    loaded = FALSE;

    return OK;
}