/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <config.h>
/******************************************************************************
 * lwodraw.c - LWO Object Drawer                                              *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwomorph.h"
#include "platform/renderer/lwodraw.h"
#include "platform/renderer/lwodrawstate.h"
#include "platform/software/softpreca.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define	PRECA_TRI_TABLE_SIZE	256*1024

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static BOOL usePreca = FALSE;
static Tri precaTriTable[PRECA_TRI_TABLE_SIZE];
static INT32 precaTriCount;

/*---------------------------------------------------------------------------- PRECA */
static void
buildPrecaTriTable(Tri *src, INT32 count)
{
	INT32 i = 0;

	precaTriCount = 0;

	for (i=0; i<count; i++)
	{
		if (SoftPrecaGetValue(1)==1)
		{
			precaTriTable[precaTriCount] = src[i];

			precaTriCount++;
		}
	}
}

/*---------------------------------------------------------------------------- RENDER */
static void 
render(Lwo *lwo, RenderType renderType)
{
    INT32 surf = 0;
    INT32 skip = 0;
	BOOL skip2DClipping = FALSE;

    glNormalPointer(GL_FLOAT, 0, lwo->normTable);
    
    for (surf=0; surf<lwo->surfCount; surf++)
    {
        skip = FALSE;
		if (usePreca)
		{
			if (renderType==RenderOpacity && !lwo->surfTable[surf].renderState.additive)
			{
				if (SoftPrecaGetValue(1)==0)
				{
					skip = TRUE;
				}
			}
		}
		
		if (!skip)
		{
			skip2DClipping = FALSE;
			if (usePreca)
			{
				if (!lwo->surfTable[surf].renderState.additive && renderType==RenderOpacity)
				{
					if (SoftPrecaGetValue(1)==1)
					{
						skip2DClipping = TRUE;
					}
				}
			}
			
			if (renderType==RenderTransparency)
			{
				if (lwo->surfTable[surf].renderState.additive || lwo->surfTable[surf].tex1!=NULL)
				{
					LwoBeginState(lwo, surf, renderType);
				
					glDrawElements(GL_TRIANGLES, lwo->surfTable[surf].triCount*3, GL_UNSIGNED_INT, lwo->surfTable[surf].triTable);

					LwoEndState(lwo, surf);
				}
			}
			else
			{				
				if (!lwo->surfTable[surf].renderState.additive)
				{
					LwoBeginState(lwo, surf, renderType);

					if (usePreca==TRUE)
					{
						buildPrecaTriTable(lwo->surfTable[surf].triTable, lwo->surfTable[surf].triCount);

						glDrawElements(GL_TRIANGLES, precaTriCount*3, GL_UNSIGNED_INT, precaTriTable);
					}
					else
					{
						glDrawElements(GL_TRIANGLES, lwo->surfTable[surf].triCount*3, GL_UNSIGNED_INT, lwo->surfTable[surf].triTable);
					}

					LwoEndState(lwo, surf);
				}
			}
        }
    }
}

/*---------------------------------------------------------------------------- APPLY TRANSF */
RESULT
LwoApplyTransf(Trans *trans, const FLOAT32 sizeCoef)
{
    if (trans==NULL)
    {
        return ERR_INVALID_PTR;
    }

    MatrixTranslate(trans->transX, trans->transY, trans->transZ);
    
    MatrixRotate(-trans->rotY, 0.0f, 1.0f, 0.0f);
    MatrixRotate(-trans->rotX, 1.0f, 0.0f, 0.0f);
    MatrixRotate(trans->rotZ, 0.0f, 0.0f, 1.0f);
    
    MatrixTranslate(trans->pivotX, trans->pivotY, trans->pivotZ);
    
    MatrixScale(trans->stretchX*sizeCoef, trans->stretchY*sizeCoef, trans->stretchZ*sizeCoef);

    return OK;
}

/*---------------------------------------------------------------------------- ENABLE */
RESULT
LwoEnable(Lwo *lwo, Pass pass)
{
    if (lwo==NULL)  
    {
        return ERR_INVALID_PTR;
    }

    if (lwo->isDisp)
    {
        return LWO_SKIPPED;
    }

    if (pass==FirstPass)
    {
        LwoEndoMorphCalculate(lwo);

        if (lwo->dissolveAmount<0.0f)
        {
            lwo->dissolveAmount = 0.0f;
        }
        else if (lwo->dissolveAmount>1.0f)
        {
            lwo->dissolveAmount = 1.0f;
        }
    }
    
    if (lwo->applyTrans)
    {
        MatrixPush(); 

        LwoApplyTransf(&lwo->trans, lwo->sizeCoef);
    }

    if (lwo->trans.stretchX!=1.0f || lwo->trans.stretchY!=1.0f || lwo->trans.stretchZ!=1.0f || lwo->endoMorphCount>0)
    {
        glEnable(GL_NORMALIZE);
    }
    else
    {
        glDisable(GL_NORMALIZE);
    }

    glCullFace(GL_FRONT);

    glDisable(GL_DITHER);

	if (lwo->isCullFace)
	{
		glEnable(GL_CULL_FACE);
	}
	else
	{
		glDisable(GL_CULL_FACE);
	}

    glEnable(GL_DEPTH_TEST);
    
    if (pass==FirstPass)
	{
		glDisable(GL_BLEND);
	
		glDepthMask(GL_TRUE);
		glDepthFunc(GL_LESS);

		render(lwo, RenderOpacity);
	}
	else
	if (pass==SecondPass)
	{	
		glDepthMask(GL_FALSE);
		glDepthFunc(GL_LEQUAL);

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);

		render(lwo, RenderTransparency);
	}

    glDepthMask(GL_TRUE);

    if (lwo->applyTrans)
    {
        MatrixPop();
    }

    return OK;
}

/*---------------------------------------------------------------------------- ENABLE PRECA */
void 
LwoTransfEnablePreca(BOOL enable)
{
	usePreca = enable;
}

/*---------------------------------------------------------------------------- NORM COEF */
FLOAT32 LwoTransfNormCoef(FLOAT32 value)
{
	return 1.0f/value;
}
