/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <config.h>
/******************************************************************************
 * lwodrawstate.c - LWO Object State Enabler                                  *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "platform/renderer/lwodraw.h"
#include "platform/renderer/lwodrawstate.h"

/*---------------------------------------------------------------------------- BEGIN STATE */
void
LwoBeginState(Lwo *lwo, INT32 surf, RenderType renderType)
{
    if (lwo->surfTable[surf].mappingType!=MappingUndef)
	{
		MatrixMode(GL_TEXTURE);
		MatrixPush();	
	}

	if ((lwo->surfTable[surf].renderState.envmap && renderType==RenderOpacity && lwo->surfTable[surf].tex1==NULL) || (lwo->surfTable[surf].tex1!=NULL && renderType==RenderTransparency))
    {
        glEnableClientState(GL_VERTEX_ARRAY);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
        glEnableClientState(GL_NORMAL_ARRAY);
        glEnable(GL_TEXTURE_2D);
        glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
        glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
        glEnable(GL_TEXTURE_GEN_S);
        glEnable(GL_TEXTURE_GEN_T); 
		glDisable(GL_COLOR_MATERIAL);
        
        if (lwo->dissolveAmount!=1.0f)
        {
            glColor4f(1.0f, 1.0f, 1.0f, lwo->dissolveAmount);
        }
            
        if (lwo->surfTable[surf].tex1!=NULL && renderType==RenderTransparency)
		{
			glVertexPointer(3, GL_FLOAT, 0, lwo->vertTable);
                
            glNormalPointer(GL_FLOAT, 0, lwo->normTable);
            
            glBindTexture(GL_TEXTURE_2D, (UINT32) lwo->surfTable[surf].tex1->id);
		} 
		else if (lwo->surfTable[surf].mappingType!=MappingUV)
        {
            if (lwo->surfTable[surf].renderState.gouraud)
            {
                glEnable(GL_LIGHTING);
            }
            else
            {
                glDisable(GL_LIGHTING);
            }
                
            glVertexPointer(3, GL_FLOAT, 0, lwo->vertTable);
                
            glNormalPointer(GL_FLOAT, 0, lwo->normTable);
            
            glBindTexture(GL_TEXTURE_2D, (UINT32) lwo->surfTable[surf].tex0->id);

            //glDrawElements(GL_TRIANGLES, lwo->surfTable[surf].triCount*3, GL_UNSIGNED_INT, lwo->surfTable[surf].triTable);
        }
    }
    else
    {	
		glDisable(GL_TEXTURE_2D);
		glEnableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_NORMAL_ARRAY);
		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_COLOR_MATERIAL);
		glDisable(GL_LIGHTING);
    
		if (lwo->surfTable[surf].renderState.gouraud)
		{
			glEnable(GL_LIGHTING);
        
			glEnableClientState(GL_NORMAL_ARRAY);
    
			glShadeModel(GL_SMOOTH);                    
			glEnable(GL_POLYGON_SMOOTH);
		}
		
		if (lwo->surfTable[surf].mappingType==MappingUV || lwo->surfTable[surf].mappingType==MappingCubic || lwo->surfTable[surf].mappingType==MappingCylindrical || lwo->surfTable[surf].mappingType==MappingSpherical)
		{
			glVertexPointer(3, GL_FLOAT, 0, lwo->surfTable[surf].txuv->vertTable);
    
			glTexCoordPointer(2, GL_FLOAT, 0, lwo->surfTable[surf].txuv->uvTable);
		}
		else
		{
			glVertexPointer(3, GL_FLOAT, 0, lwo->vertTable);
    
			if (lwo->surfTable[surf].mappingType!=MappingUndef)
			{
				glTexCoordPointer(2, GL_FLOAT, 0, lwo->surfTable[surf].uvTable);
			}
		}
        
		if (lwo->surfTable[surf].renderState.envmap && lwo->surfTable[surf].tex1==NULL)
		{
			if (lwo->surfTable[surf].tex0!=NULL)
			{
				glEnable(GL_TEXTURE_2D);

				glEnableClientState(GL_NORMAL_ARRAY);
    
				glBindTexture(GL_TEXTURE_2D, (UINT32) lwo->surfTable[surf].tex0->id);

				glEnable(GL_TEXTURE_GEN_S);
				glEnable(GL_TEXTURE_GEN_T); 
				glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
				glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP); 
			}
		}
		else if (lwo->surfTable[surf].renderState.mapping)
		{
			if (lwo->surfTable[surf].tex0!=NULL)
			{
				glEnable(GL_TEXTURE_2D);

				glEnableClientState(GL_TEXTURE_COORD_ARRAY);
            
				glBindTexture(GL_TEXTURE_2D, (UINT32) lwo->surfTable[surf].tex0->id);

				switch (lwo->surfTable[surf].tex0->scrollType)
				{
				case ScrollUndef:
					MatrixTranslate(0.0f, 0.0f, 0.0f);
					break;

				case ScrollUp:
					MatrixTranslate(0.0f, lwo->offsetUV, 0.0f);
					break;

				case ScrollDown:
					MatrixTranslate(0.0f, -lwo->offsetUV, 0.0f);
					break;

				case ScrollLeft:
					MatrixTranslate(-lwo->offsetUV, 0.0f, 0.0f);
					break;

				case ScrollRight:
					MatrixTranslate(lwo->offsetUV, 0.0f, 0.0f);
					break;
				}
        
				if (lwo->surfTable[surf].renderState.gouraud)
				{
					glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
				}
			}
		}

		if (lwo->surfTable[surf].mappingType==MappingUndef || lwo->surfTable[surf].renderState.gouraud)
		{
			glEnable(GL_COLOR_MATERIAL);

			glColor3ub(lwo->surfTable[surf].r, lwo->surfTable[surf].g, lwo->surfTable[surf].b);
		}

		if (lwo->dissolveAmount!=1.0f)
		{
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			glColor4f(1.0f, 1.0f, 1.0f, lwo->dissolveAmount);
		}
	}
}

/*---------------------------------------------------------------------------- END STATE */
void
LwoEndState(Lwo *lwo, INT32 surf)
{
    if (lwo->surfTable[surf].mappingType!=MappingUndef)
	{
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

		MatrixPop();
		MatrixMode(GL_MODELVIEW);
	}
}





















