/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <config.h>
/******************************************************************************
 * lwsdraw.c - LWS Scene Drawer                                               *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "platform/renderer/lwodraw.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lwo/lwomorph.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "platform/software/softpreca.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsinit.h"

/*---------------------------------------------------------------------------- LOCAL MACROS */
#define GET_INT8 (*(lwoPtr++))

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static FLOAT32 time = 0.0f;
static INT32 transCount = 0;
static Trans transTable[LWS_MAX_PARENT];
            
/*---------------------------------------------------------------------------- GET TRANS */
static void 
getTrans(Trans *trans, Block *block, FLOAT32 sizeCoef)
{
    trans->transX = EnvEval(&block->env[0], time)*sizeCoef;
    trans->transY = EnvEval(&block->env[1], time)*sizeCoef;
    trans->transZ = -EnvEval(&block->env[2], time)*sizeCoef;

    trans->rotX = DEGREE(EnvEval(&block->env[4], time));
    trans->rotY = DEGREE(EnvEval(&block->env[3], time));
    trans->rotZ = DEGREE(EnvEval(&block->env[5], time));

    trans->stretchX = EnvEval(&block->env[6], time);
    trans->stretchY = EnvEval(&block->env[7], time);
    trans->stretchZ = EnvEval(&block->env[8], time);

    trans->pivotX = -block->pivotX;
    trans->pivotY = -block->pivotY;
    trans->pivotZ = block->pivotZ;
}

/*---------------------------------------------------------------------------- SET PERSPECTIVE */
static void 
lwsSetPerspective(Lws *lws)
{
    FLOAT32 focal = 0.0f;
    FLOAT32 ratio = 0.0f;
    
    MatrixMode(GL_PROJECTION);
    MatrixLoadIdentity();

    if (lws->envFocal==NULL)
    {
        focal = lws->focalAmount;
    }
    else
    {
        focal = EnvEval(lws->envFocal, time);
    }

    ratio = (FLOAT32) lws->frameResX/(FLOAT32) lws->frameResY;
    
	MatrixFrustum(-ratio/focal, ratio/focal, -1.0f/focal, 1.0f/focal, 1.0f, 99999999999999.9f);
}

/*---------------------------------------------------------------------------- DISPLAY OBJECTS */
static void
lwsDisplayObjects(Lws *lws, Pass pass)
{
	Block *block = NULL;
	BOOL skip = FALSE;    
	INT32 i = 0;
	FLOAT32 amount = 0.0f;

    block = lws->blockObj;

    while (block!=NULL)
    {
        skip = FALSE;
		if (lws->usePreca)
		{
			if (SoftPrecaGetValue(1)==0)
			{
				skip = TRUE;
			}
		}

		if (!skip)
		{			
			for (i=0; i<block->endoMorphCount; i++)
			{
				amount = EnvEval(block->endoMorph[i].env, time);
            
				LwoSetEndoMorphAmount(block->lwo, block->endoMorph[i].name, amount);
			}
        
			if (block->morphTarget>0 && !block->lwo->isNull)
			{
				if (block->envMorph==NULL)
				{
					amount = block->morphAmount;
				}
				else
				{
					amount = EnvEval(block->envMorph, time);
				}

				LwoMorphInit(block->lwo, lws->lwoTable[block->morphTarget-1], amount);
			}

			if (block->useDissolve)
			{
				if (block->envDissolve==NULL)
				{
					amount = 1.0f-block->dissolveAmount;
				}
				else
				{
					amount = 1.0f-EnvEval(block->envDissolve, time);
				}

				LwoSetDissolve(block->lwo, amount);
			}

			MatrixPush();

			if (block->parentItem>0)
			{
				Block *blockParent = NULL;
            
				transCount = 0;
            
				blockParent = lws->blockTable[block->parentItem-1];

				if (blockParent->parentItem==block->parentItem)
				{
					blockParent = NULL;
				}

				while (blockParent!=NULL)
				{
					getTrans(&transTable[transCount], blockParent, lws->sizeCoef);
					transCount++;

					if (blockParent->parentItem==0)
					{
						blockParent = NULL;
					}
					else
					{
						blockParent = lws->blockTable[blockParent->parentItem-1];
					}
				}

				while (--transCount>=0)
				{
					LwoApplyTransf(&transTable[transCount], 1.0f);
				}   
			}

			if (!block->lwo->isNull)
			{
				getTrans(&block->lwo->trans, block, lws->sizeCoef);

				block->lwo->offsetUV = lws->offsetUV;
					
				LwoApplyTransf(&block->lwo->trans, lws->sizeCoef);

				block->lwo->applyTrans = 0;

				LwoEnable(block->lwo, pass);
			}

			MatrixPop();
		}

        block = block->next;
    }
}

/*---------------------------------------------------------------------------- ENABLE */
RESULT
LwsEnable(Lws *lws)
{
    Block *block = NULL;
	INT32 i = 0;

    UINT32 lightID = GL_LIGHT0;

    if (lws==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (lws->viewLines)
    {
        glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    }
    else
    {
        glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    }
    
    lwsSetPerspective(lws);
    
    MatrixMode(GL_MODELVIEW);
    MatrixLoadIdentity();
    
    time = lws->time;
    
    LwoTransfEnablePreca(FALSE);
    if (lws->usePreca)
	{
		SoftPrecaSetFrame((UINT8 *) lws->precaFile.buffer, time);
		LwoTransfEnablePreca(TRUE);
	}
	
	time *= ((FLOAT32) lws->lastFrame/(FLOAT32) lws->framesPerSecond);

    block = lws->blockCam;

    block->cam->transX = EnvEval(&block->env[0], time)*lws->sizeCoef;
    block->cam->transY = EnvEval(&block->env[1], time)*lws->sizeCoef;
    block->cam->transZ = EnvEval(&block->env[2], time)*lws->sizeCoef;

    block->cam->rotX = DEGREE(EnvEval(&block->env[4], time));
    block->cam->rotY = DEGREE(EnvEval(&block->env[3], time));
    block->cam->rotZ = DEGREE(EnvEval(&block->env[5], time));

    CamEnable(block->cam);
    
    block = lws->blockCam;

    if (block->parentItem>0)
    {
        Block *blockParent = NULL;
        Trans *trans = NULL;
            
        transCount = 0;
    
        blockParent = lws->blockTable[block->parentItem-1];

        if (blockParent->parentItem==block->parentItem)
        {
            blockParent = NULL;
        }

        while (blockParent!=NULL)
        {
            getTrans(&transTable[transCount], blockParent, lws->sizeCoef);
            transCount++;

            if (blockParent->parentItem==0)
            {
                blockParent = NULL;
            }
            else
            {
                blockParent = lws->blockTable[blockParent->parentItem-1];
            }
        }

        while (--transCount>=0)
        {
            trans = &transTable[transCount];
                
            MatrixRotate(-trans->rotZ, 0.0f, 0.0f, 1.0f);
            MatrixRotate(trans->rotX, 1.0f, 0.0f, 0.0f);
            MatrixRotate(trans->rotY, 0.0f, 1.0f, 0.0f);
            
            MatrixTranslate(-trans->transX, -trans->transY, -trans->transZ);
        }
    }

	lightID = GL_LIGHT0;

    for (i=0; i<8; i++)
    {
        glDisable(lightID);

        lightID++;
    }
    
    block = lws->blockLight;

    LightAmbientSet(lws->ambientR, lws->ambientG, lws->ambientB, lws->ambientIntens);

    lightID = GL_LIGHT0;

    while (block!=NULL)
    {
        block->light->transX = EnvEval(&block->env[0], time)*lws->sizeCoef;
        block->light->transY = EnvEval(&block->env[1], time)*lws->sizeCoef;
        block->light->transZ = -EnvEval(&block->env[2], time)*lws->sizeCoef;

        block->light->rotX = DEGREE(EnvEval(&block->env[3], time));
        block->light->rotY = DEGREE(EnvEval(&block->env[4], time));
        block->light->rotZ = DEGREE(EnvEval(&block->env[5], time));

        block->light->sizeCoef = lws->sizeCoef;
        block->light->glID = lightID;

        if (block->light->envIntens!=NULL)
        {
            block->light->intens = EnvEval(block->light->envIntens, time);
        }

        lightID++;

        LightEnable(block->light);
        block = block->next;
    }

    {
        FLOAT32 t[4] = {0.0f, 0.0f, 0.0f, 0.0f};
    
        t[0] = 0.0f;
        t[1] = 0.0f;
        t[2] = 0.0f;
        t[3] = 1.0f;
        glMaterialfv(GL_FRONT, GL_SPECULAR, t);

        t[0] = 0.0f;
        t[1] = 0.0f;
        t[2] = 0.0f;
        t[3] = 1.0f;
        glMaterialfv(GL_FRONT, GL_EMISSION, t);

        t[0] = 0.0f;
        glMaterialfv(GL_FRONT, GL_SHININESS, t);

        t[0] = 1.0f;
        t[1] = 1.0f;
        t[2] = 1.0f;
        t[3] = 1.0f;
        glMaterialfv(GL_FRONT, GL_DIFFUSE, t);

        t[0] = 1.0f;
        t[1] = 1.0f;
        t[2] = 1.0f;
        t[3] = 1.0f;
        glMaterialfv(GL_FRONT, GL_AMBIENT, t);
    }

    lwsDisplayObjects(lws, FirstPass);
	lwsDisplayObjects(lws, SecondPass);
    
    block = lws->blockLight;
    while (block!=NULL)
    {
        LightDisable(block->light);

        block = block->next;
    }

    return OK;
}
