/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <config.h>
/******************************************************************************
 * lwsdrawlight.c - LWS Scene Light Manager                                   *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"

/*---------------------------------------------------------------------------- AMBIENT SET */
void 
LightAmbientSet(float r, float g, float b, float intens)
{
    FLOAT32 t[4];
    
    t[0] = r*intens;
    t[1] = g*intens;
    t[2] = b*intens;
    t[3] = 1.0f;

    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, t);
}

/*---------------------------------------------------------------------------- ENABLE */
RESULT
LightEnable(Light *light)
{
    FLOAT32 t[4]={0.0f, 0.0f, 0.0f, 0.0f};
    
    if (light==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (light->glID<(GL_LIGHT0+0) || light->glID>(GL_LIGHT0+7))
    {
        return ERR;
    }

    MatrixPush();

    switch (light->type)
    {
    case LightSpot:
        t[0] = light->colorR*light->intens;
        t[1] = light->colorG*light->intens;
        t[2] = light->colorB*light->intens;
        glLightfv(light->glID, GL_DIFFUSE, t);  
        
        glLightf(light->glID, GL_CONSTANT_ATTENUATION, 1.0f);
        glLightf(light->glID, GL_LINEAR_ATTENUATION, 0.0f);
        glLightf(light->glID, GL_QUADRATIC_ATTENUATION, 0.0f);
        glLightf(light->glID, GL_SPOT_CUTOFF, 180.0f);

        t[0] = light->transX;
        t[1] = light->transY;
        t[2] = -light->transZ;
        t[3] = 1.0f;
        glLightfv(light->glID, GL_POSITION, t);
        break;

    case LightDistant:
        t[0] = light->colorR;
        t[1] = light->colorG;
        t[2] = light->colorB;
        t[3] = light->intens;
        glLightfv(light->glID, GL_DIFFUSE, t);  
    
        MatrixRotate(light->rotX, 0.0f, -1.0f, 0.0f);
        MatrixRotate(light->rotY, -1.0f, 0.0f, 0.0f);
        MatrixRotate(light->rotZ, 0.0f, 0.0f, 1.0f);
        
        t[0] = 0.0f;
        t[1] = 0.0f;
        t[2] = -1.0f;
        t[3] = 0.0f;
        glLightfv(light->glID, GL_POSITION, t);
        break;

    case LightPoint:    
        t[0] = light->colorR*light->intens;
        t[1] = light->colorG*light->intens;
        t[2] = light->colorB*light->intens;
        t[3] = 1.0f;
        glLightfv(light->glID, GL_DIFFUSE, t);  
        
        t[0] = 0.0f;
        t[1] = 0.0f;
        t[2] = 0.0f;
        t[3] = 1.0f;
        glLightfv(light->glID, GL_SPECULAR, t);  
        
        glLightf(light->glID, GL_CONSTANT_ATTENUATION, 1.0f);
        glLightf(light->glID, GL_LINEAR_ATTENUATION, 0.0f);
        glLightf(light->glID, GL_QUADRATIC_ATTENUATION, 0.0f);
        glLightf(light->glID, GL_SPOT_CUTOFF, 180.0f);
        glLightf(light->glID, GL_SPOT_EXPONENT, 0.0f);

        t[0] = light->transX;
        t[1] = -light->transY;
        t[2] = light->transZ;
        t[3] = 1.0f;
        glLightfv(light->glID, GL_POSITION, t);
        break;

    default:
        return ERR;
    }

    MatrixPop();

    glEnable(light->glID);
    light->isEnabled = TRUE;
    
    return OK;
}

/*---------------------------------------------------------------------------- DISABLE */
RESULT
LightDisable(Light *light)
{
    if (light==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (light->glID<(GL_LIGHT0+0) || light->glID>(GL_LIGHT0+7))
    {
        return ERR;
    }

    glDisable(light->glID);
    light->isEnabled = FALSE;
    
    return OK;
}





















