/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <config.h>
/******************************************************************************
 * matrix.c - Matrix                                                          *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "platform/os/console.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static T_MATRIX getMatrix;

/*---------------------------------------------------------------------------- INIT */
void
MatrixInit(void)
{
}

/*---------------------------------------------------------------------------- LOAD IDENTITY */
void
MatrixLoadIdentity(void)
{
    glLoadIdentity();
}

/*---------------------------------------------------------------------------- ROTATE */
void 
MatrixRotate(FLOAT32 angle, FLOAT32 x, FLOAT32 y, FLOAT32 z)
{
    glRotatef(angle, x, y, z);
}

/*---------------------------------------------------------------------------- TRANSLATE */
void
MatrixTranslate(FLOAT32 x, FLOAT32 y, FLOAT32 z)
{
    glTranslatef(x, y, z);
}

/*---------------------------------------------------------------------------- SCALE */
void
MatrixScale(FLOAT32 x, FLOAT32 y, FLOAT32 z)
{
    glScalef(x, y, z);
}

/*---------------------------------------------------------------------------- VIEWPORT */
void
MatrixFrustum(FLOAT32 left, FLOAT32 right, FLOAT32 bottom, FLOAT32 top, FLOAT32 zNear, FLOAT32 zFar)
{
    glFrustum(left, right, bottom, top, zNear, zFar);
}

void
MatrixOrtho(FLOAT32 left, FLOAT32 right, FLOAT32 bottom, FLOAT32 top, FLOAT32 zNear, FLOAT32 zFar)
{
    glOrtho(left, right, bottom, top, zNear, zFar);
}

/*---------------------------------------------------------------------------- MULTIPLICATION */
void
MatrixMult(T_MATRIX *m)
{
    glMultMatrixf((FLOAT32 *) m);
}

/*---------------------------------------------------------------------------- VECTOR MULTIPLICATION */
void 
MatrixVectMult(T_VEC *res, T_VEC *in, T_MATRIX *mat)
{
    (*res)[0] = (*in)[0]*(*mat)[0][0] + (*in)[1]*(*mat)[1][0] + (*in)[2]*(*mat)[2][0] + (*mat)[3][0];
    (*res)[1] = (*in)[0]*(*mat)[0][1] + (*in)[1]*(*mat)[1][1] + (*in)[2]*(*mat)[2][1] + (*mat)[3][1];
    (*res)[2] = (*in)[0]*(*mat)[0][2] + (*in)[1]*(*mat)[1][2] + (*in)[2]*(*mat)[2][2] + (*mat)[3][2];
}

void 
MatrixVectMultWithW(T_VEC *res, T_VEC *in, T_MATRIX *mat)
{
    (*res)[0] = (*in)[0]*(*mat)[0][0] + (*in)[1]*(*mat)[1][0] + (*in)[2]*(*mat)[2][0] + (*mat)[3][0];
    (*res)[1] = (*in)[0]*(*mat)[0][1] + (*in)[1]*(*mat)[1][1] + (*in)[2]*(*mat)[2][1] + (*mat)[3][1];
    (*res)[2] = (*in)[0]*(*mat)[0][2] + (*in)[1]*(*mat)[1][2] + (*in)[2]*(*mat)[2][2] + (*mat)[3][2];
    (*res)[3] = (*in)[0]*(*mat)[0][3] + (*in)[1]*(*mat)[1][3] + (*in)[2]*(*mat)[2][3] + (*mat)[3][3];
}

void
MatrixVectMultTable(T_VEC *dst, INT32 dstPad, T_MATRIX *mat, T_VEC *src, INT32 srcPad, INT32 count)
{
	dst;
	dstPad;
	mat;
	src;
	srcPad;
	count;
}

void 
MatrixVectMultWithWTable(T_VEC *dst, INT32 dstPad, T_MATRIX *mat, T_VEC *src, INT32 srcPad, INT32 count)
{
	dst;
	dstPad;
	mat;
	src;
	srcPad;
	count;
}

/*---------------------------------------------------------------------------- GET */
T_MATRIX *
MatrixGet(INT32 mode)
{
    switch(mode)
    {
    case MATRIX_MODELVIEW:
        glGetFloatv(GL_MODELVIEW_MATRIX, (FLOAT32 *) &getMatrix);
		break;
        
    case MATRIX_PROJECTION:
        glGetFloatv(GL_PROJECTION_MATRIX, (FLOAT32 *) &getMatrix);
		break;

    case MATRIX_TEXTURE:
        glGetFloatv(GL_TEXTURE_MATRIX, (FLOAT32 *) &getMatrix);
		break;
    }

    return &getMatrix;
}

/*---------------------------------------------------------------------------- SET CONTENT OF CURRENT MATRIX */
void 
MatrixLoad(T_MATRIX *m)
{
    glLoadMatrixf((FLOAT32 *) m);
}

/*---------------------------------------------------------------------------- STACK */
void
MatrixPush(void)
{
    glPushMatrix();
}

void
MatrixPop(void)
{
    glPopMatrix();
}

/*---------------------------------------------------------------------------- MODE */
void
MatrixMode(INT32 mode)
{
    glMatrixMode(mode);
}

/*---------------------------------------------------------------------------- DUMP TO CONSOLE */
void 
MatrixDump(T_MATRIX *mat)
{
    INT32 l = 0;

    for (l=0; l<4; l++)
    {
        ConsoleAdd("%d %d %d %d", (INT32)((*mat)[l][0]*100.0f), (INT32)((*mat)[l][1]*100.0f), (INT32)((*mat)[l][2]*100.0f), (INT32)((*mat)[l][3]*100.0f));
    }
    
    ConsoleAdd("");
}