/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwodraw.c - LWO Object drawer                                              *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/ps2/ps2.h"
#include "platform/os/font.h"
#include "platform/software/soft.h"
#include "platform/os/ps2/ps2draw.h"
#include "platform/os/ps2/ps2screen.h"
#include "platform/renderer/matrix.h"
#include "system/file.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lwo/lwomorph.h"
#include "platform/renderer/lwodraw.h"
#include "platform/renderer/lwodrawstate.h"
#include "platform/software/softtransf.h"
#include "platform/software/softpreca.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define MAX_VERTICES_PER_LIST       3000

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static Vertex vertexBufferObj[MAX_VERTICES_PER_LIST] __attribute__((aligned (16)));
static Vertex vertexBufferSurf[MAX_VERTICES_PER_LIST] __attribute__((aligned (16)));
static Vertex *triangleBuffer = NULL;

/*---------------------------------------------------------------------------- RENDER */
static void 
render(Lwo *lwo, RenderType renderType)
{
    SoftTransfTriBufferParam param;
	
	BOOL doCulling = FALSE;
	BOOL doTexClipping = FALSE;
	BOOL skip = FALSE;
	BOOL skip2DClipping = FALSE;

	INT32 surf = 0;
    INT16 w = 0;
    INT16 h = 0;
    
    INT8 r = 0;
    INT8 g = 0;
    INT8 b = 0;

	INT32 vertCount = 0;
    
    BOOL envmap = FALSE;

    UINT32 *texBuffer = NULL;
    FLOAT32 *uvTable = NULL;

    Vertex *vertexBuffer = NULL;

    INT32 triCount = 0;    

	for (surf=0; surf<lwo->surfCount; surf++)
    {   
        skip = FALSE;
		if (lwo->usePreca)
		{
			if (renderType==RenderOpacity && !lwo->surfTable[surf].renderState.additive)
			{
				if (SoftPrecaGetValue(1)==0)
				{
					skip = TRUE;
				}
			}
		}
		
		if (!skip)
		{
			skip2DClipping = FALSE;
			if (lwo->usePreca)
			{
				if (!lwo->surfTable[surf].renderState.additive && renderType==RenderOpacity)
				{
					if (SoftPrecaGetValue(1)==1)
					{
						skip2DClipping = TRUE;
					}
				}
			}
				
			if (lwo->surfTable[surf].mappingType!=MappingUndef)
			{
				switch (lwo->surfTable[surf].tex0->scrollType)
				{
				case ScrollUndef:
					SoftTransfSetOffsetUV(0.0f, 0.0f);
					break;

				case ScrollDown:
					SoftTransfSetOffsetUV(0.0f, lwo->offsetUV);
					break;

				case ScrollUp:
					SoftTransfSetOffsetUV(0.0f, -lwo->offsetUV);
					break;

				case ScrollRight:
					SoftTransfSetOffsetUV(-lwo->offsetUV, 0.0f);
					break;

				case ScrollLeft:
					SoftTransfSetOffsetUV(lwo->offsetUV, 0.0f);
					break;
				}
			}
			
			if (renderType==RenderTransparency)
			{
				if (lwo->surfTable[surf].renderState.additive || lwo->surfTable[surf].tex1!=NULL)
				{
					SoftTransfEnablePreca(FALSE);
								
					texBuffer = NULL;
					uvTable = NULL;
					envmap = FALSE;
        
					if (lwo->surfTable[surf].mappingType!=MappingUndef)
					{
						if (lwo->surfTable[surf].tex1==NULL)
						{
							texBuffer = (UINT32 *) lwo->surfTable[surf].tex0->buffer;

							w = (INT16) lwo->surfTable[surf].tex0->width;
							h = (INT16) lwo->surfTable[surf].tex0->height;
						}
						else
						{
							texBuffer = (UINT32 *) lwo->surfTable[surf].tex1->buffer;

							w = (INT16) lwo->surfTable[surf].tex1->width;
							h = (INT16) lwo->surfTable[surf].tex1->height;
						}

						doTexClipping = TRUE;

						if (lwo->surfTable[surf].mappingType==MappingUV || lwo->surfTable[surf].mappingType==MappingCubic || lwo->surfTable[surf].mappingType==MappingCylindrical || lwo->surfTable[surf].mappingType==MappingSpherical)
						{
							uvTable = (FLOAT32 *) lwo->surfTable[surf].txuv->uvTable;

							SoftTransfBuildVertexBuffer((FLOAT32 *) lwo->surfTable[surf].txuv->vertTable, NULL, vertexBufferSurf, lwo->surfTable[surf].txuv->vertCount, doTexClipping);

							vertCount = lwo->surfTable[surf].txuv->vertCount;

							vertexBuffer = vertexBufferSurf;
						}
						else
						{
							if (lwo->surfTable[surf].renderState.envmap)
							{
								uvTable = NULL;
								envmap = TRUE;
							}
							else
							if (lwo->surfTable[surf].mappingType!=MappingUndef)
							{
								uvTable = (FLOAT32 *) lwo->surfTable[surf].uvTable;
							}

							vertCount = lwo->vertCount;

							vertexBuffer = vertexBufferObj;
						}

						SoftTransfComputeUVVertexBuffer(vertexBuffer, vertCount, uvTable, envmap);
					}
					else
					{
						r = lwo->surfTable[surf].r;
						g = lwo->surfTable[surf].g;
						b = lwo->surfTable[surf].b;

						doTexClipping = FALSE;

						vertexBuffer = vertexBufferObj;
					}

					doCulling = !lwo->surfTable[surf].renderState.doubleSided;
						
					if (lwo->surfTable[surf].triCount>0)
					{	
						param.vertexBuffer = vertexBuffer;
						param.uvBuffer = uvTable;
						param.indexBuffer = (INT32 *) lwo->surfTable[surf].triTable;
						param.srcTriCount = lwo->surfTable[surf].triCount;
						param.dstTriBuffer = (Vertex **) &triangleBuffer;
						param.dstBufferCount = &triCount;
						param.envmap = envmap;
						param.doCulling = doCulling;
						param.doTexClipping = doTexClipping;
						param.skip2DClipping = skip2DClipping;

						SoftTransfBuildTriBuffer(&param);
						
						if (triCount>0)
						{
							if (lwo->surfTable[surf].mappingType!=MappingUndef)
							{
								PS2DrawTriangleBufferWithTexAdd(triangleBuffer, triCount, (FLOAT32) w, (FLOAT32) h, texBuffer);
							}
							else
							{
								PS2DrawTriangleBufferAdd(triangleBuffer, triCount, r, g, b);
							}
						}
					}
				}
			}
			else
			{
				if (!lwo->surfTable[surf].renderState.additive)
				{
					SoftTransfEnablePreca(lwo->usePreca);
								
					texBuffer = NULL;
					uvTable = NULL;
					envmap = FALSE;
        
					if (lwo->surfTable[surf].mappingType!=MappingUndef)
					{
						texBuffer = (UINT32 *) lwo->surfTable[surf].tex0->buffer;

						w = (INT16) lwo->surfTable[surf].tex0->width;
						h = (INT16) lwo->surfTable[surf].tex0->height;

						doTexClipping = TRUE;

						if (lwo->surfTable[surf].mappingType==MappingUV || lwo->surfTable[surf].mappingType==MappingCubic || lwo->surfTable[surf].mappingType==MappingCylindrical || lwo->surfTable[surf].mappingType==MappingSpherical)
						{
							uvTable = (FLOAT32 *) lwo->surfTable[surf].txuv->uvTable;

							SoftTransfBuildVertexBuffer((FLOAT32 *) lwo->surfTable[surf].txuv->vertTable, NULL, vertexBufferSurf, lwo->surfTable[surf].txuv->vertCount, doTexClipping);

							vertCount = lwo->surfTable[surf].txuv->vertCount;

							vertexBuffer = vertexBufferSurf;
						}
						else
						{
							if (lwo->surfTable[surf].renderState.envmap && lwo->surfTable[surf].tex1==NULL)
							{
								uvTable = NULL;
								envmap = TRUE;
							}
							else
							if (lwo->surfTable[surf].mappingType!=MappingUndef)
							{
								uvTable = (FLOAT32 *) lwo->surfTable[surf].uvTable;
							}

							vertCount = lwo->vertCount;

							vertexBuffer = vertexBufferObj;
						}

						SoftTransfComputeUVVertexBuffer(vertexBuffer, vertCount, uvTable, envmap);
					}
					else
					{
						r = lwo->surfTable[surf].r;
						g = lwo->surfTable[surf].g;
						b = lwo->surfTable[surf].b;

						doTexClipping = FALSE;

						vertexBuffer = vertexBufferObj;
					}
					
					doCulling = !lwo->surfTable[surf].renderState.doubleSided;
						
					if (lwo->surfTable[surf].triCount>0)
					{	
						param.vertexBuffer = vertexBuffer;
						param.uvBuffer = uvTable;
						param.indexBuffer = (INT32 *) lwo->surfTable[surf].triTable;
						param.srcTriCount = lwo->surfTable[surf].triCount;
						param.dstTriBuffer = (Vertex **) &triangleBuffer;
						param.dstBufferCount = &triCount;
						param.envmap = envmap;
						param.doCulling = doCulling;
						param.skip2DClipping = skip2DClipping;
						
						if (SoftPrecaIsEnabled())
						{
							param.doTexClipping = FALSE;
						}
						else
						{
							param.doTexClipping = doTexClipping;
						}

						SoftTransfBuildTriBuffer(&param);

						if (triCount>0)
						{
							if (lwo->surfTable[surf].mappingType!=MappingUndef)
							{
								PS2DrawTriangleBufferWithTex(triangleBuffer, triCount, (FLOAT32) w, (FLOAT32) h, texBuffer);
							}
							else
							{
								PS2DrawTriangleBuffer(triangleBuffer, triCount, r, g, b);
							}
						}
					}
				}
			}
		}
    }
}

/*---------------------------------------------------------------------------- APPLY TRANSF */
RESULT
LwoApplyTransf(Trans *trans, const FLOAT32 sizeCoef)
{
    if (trans==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (sizeCoef==0.0f)
	{
	}

    MatrixTranslate(-trans->pivotX/sizeCoef, -trans->pivotY/sizeCoef, trans->pivotZ/sizeCoef);
    
    MatrixRotate(-trans->rotZ, 0.0f, 0.0f, 1.0f);
	MatrixRotate(-trans->rotX, 1.0f, 0.0f, 0.0f);
	MatrixRotate(-trans->rotY-180.0f, 0.0f, 1.0f, 0.0f);
    
	MatrixScale(trans->stretchX, trans->stretchY, trans->stretchZ);

    MatrixTranslate(-trans->transX, trans->transY, trans->transZ);

	return OK;
}

/*---------------------------------------------------------------------------- ENABLE */
RESULT
LwoEnable(Lwo *lwo, Pass pass)
{
	BOOL envmap = FALSE;
	INT32 i = 0;
	
    if (lwo==NULL)  
    {
        return ERR_INVALID_PTR;
    }

    if (lwo->isDisp)
    {
        return LWO_SKIPPED;
    }

    if (pass==FirstPass)
    {
        LwoEndoMorphCalculate(lwo);

        if (lwo->dissolveAmount<0.0f)
        {
            lwo->dissolveAmount = 0.0f;
        }
        else if (lwo->dissolveAmount>1.0f)
        {
            lwo->dissolveAmount = 1.0f;
        }
    }
    
    if (lwo->applyTrans)
    {
        MatrixPush(); 

        LwoApplyTransf(&lwo->trans, lwo->sizeCoef);
    }

    envmap = FALSE;
		
	for (i=0; i<lwo->surfCount; i++)
	{
		if (lwo->surfTable[i].renderState.envmap)
		{
			i = lwo->surfCount;

			envmap = TRUE;
		}
	}

	if (envmap)
	{
		SoftTransfBuildVertexBuffer((FLOAT32 *) lwo->vertTable, (FLOAT32 *) lwo->normTable, vertexBufferObj, lwo->vertCount, TRUE);
	}
	else
	{
		SoftTransfBuildVertexBuffer((FLOAT32 *) lwo->vertTable, NULL, vertexBufferObj, lwo->vertCount, TRUE);
	}
	
	render(lwo, RenderOpacity);
	render(lwo, RenderTransparency);

    if (lwo->applyTrans)
    {
        MatrixPop();
    }

    return OK;
}

/*---------------------------------------------------------------------------- ENABLE PRECA */
void 
LwoTransfEnablePreca(BOOL enable)
{	
	enable = FALSE;
}

/*---------------------------------------------------------------------------- NORM COEF */
FLOAT32 LwoTransfNormCoef(FLOAT32 value)
{
	return value;
}
