/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * lwsdraw.c - LWS Scene drawer                                               *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "system/file.h"
#include "texture/tex.h"
#include "platform/os/font.h"
#include "platform/software/soft.h"
#include "lightwave/lwo/lwo.h"
#include "platform/renderer/lwodraw.h"
#include "lightwave/lwo/lwoinit.h"
#include "lightwave/lwo/lwomorph.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "platform/os/window.h"
#include "platform/software/softpreca.h"
#include "platform/software/softtransf.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsinit.h"

/*---------------------------------------------------------------------------- LOCAL MACROS */
#define GET_INT8 (*(lwoPtr++))

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static FLOAT32 time = 0.0f;
static INT32 transCount = 0;
static Trans transTable[LWS_MAX_PARENT];

static UINT8 *checkTable = NULL;
            
/*---------------------------------------------------------------------------- PRECA IS USED */
static BOOL
isUsed(INT32 first, INT32 last)
{
	INT32 i = 0;
	
	for (i=first; i<last; i++)
	{
		if (checkTable[i]==1)
		{
			return TRUE;
		}
	}

	return FALSE;
}

/*---------------------------------------------------------------------------- GET TRANS */
static void 
getTrans(Trans *trans, Block *block, FLOAT32 sizeCoef)
{
    trans->transX = -EnvEval(&block->env[0], time)*sizeCoef;
    trans->transY = EnvEval(&block->env[1], time)*sizeCoef;
    trans->transZ = -EnvEval(&block->env[2], time)*sizeCoef;

    trans->pivotX = block->pivotX;
    trans->pivotY = block->pivotY;
    trans->pivotZ = block->pivotZ;

    trans->rotX = DEGREE(EnvEval(&block->env[4], time));
    trans->rotY = 180.0f+DEGREE(EnvEval(&block->env[3], time));
    trans->rotZ = -DEGREE(EnvEval(&block->env[5], time));

    trans->stretchX = EnvEval(&block->env[6], time)*sizeCoef;
    trans->stretchY = EnvEval(&block->env[7], time)*sizeCoef;
    trans->stretchZ = EnvEval(&block->env[8], time)*sizeCoef;
}

/*---------------------------------------------------------------------------- SET PERSPECTIVE */
static void 
lwsSetPerspective(Lws *lws)
{
    FLOAT32 focal = 0.0f;
    FLOAT32 ratio = 0.0f;
    
    MatrixMode(MATRIX_PROJECTION);
    MatrixLoadIdentity();

    if (lws->envFocal==NULL)
    {
        focal = lws->focalAmount;
    }
    else
    {
        focal = EnvEval(lws->envFocal, time);
    }

    ratio = (FLOAT32) lws->frameResX/(FLOAT32) lws->frameResY;
    
	MatrixFrustum(-ratio/focal, ratio/focal, -1.0f/focal, 1.0f/focal, 1.0f, 99999999999999.9f);

    MatrixScale((FLOAT32) -lws->viewPortResX/2, (FLOAT32) lws->viewPortResY/2, 1.0f);
}

/*---------------------------------------------------------------------------- DISPLAY OBJECTS */
static void
lwsDisplayObjects(Lws *lws, Pass pass)
{
	Block *block = NULL;
	BOOL skip = FALSE;    
	INT32 i = 0;
	FLOAT32 amount = 0.0f;

	Cam *cam = NULL;
    Trans *trans = NULL;
	BOOL isNull = FALSE;
	
	block = lws->blockCam;

    block->cam->transX = EnvEval(&block->env[0], time)*lws->sizeCoef;
    block->cam->transY = EnvEval(&block->env[1], time)*lws->sizeCoef;
    block->cam->transZ = EnvEval(&block->env[2], time)*lws->sizeCoef;

    block->cam->rotX = DEGREE(EnvEval(&block->env[4], time));
    block->cam->rotY = -DEGREE(EnvEval(&block->env[3], time));
    block->cam->rotZ = DEGREE(EnvEval(&block->env[5], time));

    cam = block->cam;        

    block = lws->blockObj;

    while (block!=NULL)
    {
        skip = FALSE;
		if (lws->usePreca)
		{
			if (SoftPrecaGetValue(1)==0)
			{
				skip = TRUE;
			}
		}

		if (!skip)
		{			
			MatrixMode(MATRIX_MODELVIEW);
			MatrixLoadIdentity();

			for (i=0; i<block->endoMorphCount; i++)
			{
				amount = EnvEval(block->endoMorph[i].env, time);
            
				LwoSetEndoMorphAmount(block->lwo, block->endoMorph[i].name, amount);
			}
        
			if (block->morphTarget>0 && !block->lwo->isNull)
			{
				if (block->envMorph==NULL)
				{
					amount = block->morphAmount;
				}
				else
				{
					amount = EnvEval(block->envMorph, time);
				}

				LwoMorphInit(block->lwo, lws->lwoTable[block->morphTarget-1], amount);
			}

			if (block->useDissolve)
			{
				if (block->envDissolve==NULL)
				{
					amount = 1.0f-block->dissolveAmount;
				}
				else
				{
					amount = 1.0f-EnvEval(block->envDissolve, time);
				}

				LwoSetDissolve(block->lwo, amount);
			}

			isNull = block->lwo->isNull;
        
			if (!isNull)
			{
				getTrans(&block->lwo->trans, block, lws->sizeCoef);

				block->lwo->offsetUV = lws->offsetUV;
					
				LwoApplyTransf(&block->lwo->trans, lws->sizeCoef);
			}
        
			if (block->parentItem>0)
			{
				Block *blockParent = NULL;
            
				transCount = 0;
            
				blockParent = lws->blockTable[block->parentItem-1];

				if (blockParent->parentItem==block->parentItem)
				{
					blockParent = NULL;
				}

				while (blockParent!=NULL)
				{
					getTrans(&transTable[transCount], blockParent, 1.0f);
					transCount++;

					if (blockParent->parentItem==0)
					{
						blockParent = NULL;
					}
					else
					{
						blockParent = lws->blockTable[blockParent->parentItem-1];
					}
				}

				for (i=0; i<transCount; i++)
				{
					trans = &transTable[i];
			
					MatrixTranslate(-trans->pivotX, -trans->pivotY, trans->pivotZ);
    
					MatrixRotate(-trans->rotZ, 0.0f, 0.0f, 1.0f);
					MatrixRotate(-trans->rotX, 1.0f, 0.0f, 0.0f);
					MatrixRotate(-trans->rotY-180.0f, 0.0f, 1.0f, 0.0f);
					
					MatrixScale(trans->stretchX, trans->stretchY, trans->stretchZ);

					MatrixTranslate(-trans->transX*lws->sizeCoef, trans->transY*lws->sizeCoef, trans->transZ*lws->sizeCoef);
				}
			}

			if (!isNull)
			{
				cam->sizeCoef = lws->sizeCoef;
				CamEnable(cam);

				block->lwo->applyTrans = 0;
    
				block->lwo->usePreca = lws->usePreca;
				LwoEnable(block->lwo, pass);
			}
		}

        block = block->next;
    }
}

/*---------------------------------------------------------------------------- ENABLE */
RESULT
LwsEnable(Lws *lws)
{
    Block *block = NULL;
	
	Lwo *lwo = NULL;
	INT32 surf = 0;
	UINT8 additive = 0;
	
    INT32 i = 0;
    
    if (lws==NULL)
    {
        return ERR_INVALID_PTR;
    }

	/*
	PS2DrawSetZScale(3.0f);
	*/

	lwsSetPerspective(lws);

	time = lws->time;

    SoftTransfEnablePreca(FALSE);
	if (lws->usePreca)
	{
		SoftPrecaSetFrame((UINT8 *) lws->precaFile.buffer, lws->time);
		SoftTransfEnablePreca(TRUE);
	}
	
	if (SoftPrecaIsEnabled())
	{
		SoftPrecaReset();
	}
    
    time *= ((FLOAT32) lws->lastFrame/(FLOAT32) lws->framesPerSecond);

    lwsDisplayObjects(lws, FirstPass);
	lwsDisplayObjects(lws, SecondPass);
    
	if (SoftPrecaIsEnabled())
	{
		checkTable = SoftPrecaComputeCheckTable();
		
		block = lws->blockObj;

		while (block!=NULL)
		{
			lwo = block->lwo;

			if (isUsed(lwo->precaFirstCount, lwo->precaLastCount))
			{
				SoftPrecaAddValue(1, 1);
				
				if (!block->lwo->isNull)
				{
					for (surf=0; surf<lwo->surfCount; surf++)
					{
						if (!lwo->surfTable[surf].renderState.additive)
						{
							if (isUsed(lwo->surfTable[surf].precaFirstCount, lwo->surfTable[surf].precaLastCount))
							{
								SoftPrecaAddValue(1, 1);

								if (lwo->surfTable[surf].precaSkip2DClipping==TRUE)
								{
									SoftPrecaAddValue(1, 1);
								}
								else
								{
									SoftPrecaAddValue(0, 1);
								}

								for (i=lwo->surfTable[surf].precaFirstCount; i<lwo->surfTable[surf].precaLastCount; i++)
								{
									SoftPrecaAddValue(checkTable[i], 1);
								}
							}
							else
							{
								SoftPrecaAddValue(0, 1);
							}
						}
					}
				}
			}
			else
			{
				additive = 0;				
				for (i=0; i<lwo->surfCount; i++)
				{
					if (lwo->surfTable[i].renderState.additive)
					{
						additive = 1;					

						i = lwo->surfCount;
					}
				}

				if (additive==0)
				{				
					SoftPrecaAddValue(0, 1);
				}
				else
				{
					SoftPrecaAddValue(1, 1);

					for (i=0; i<lwo->surfCount; i++)
					{
						if (!lwo->surfTable[i].renderState.additive)
						{
							SoftPrecaAddValue(0, 1);
						}
					}
				}
			}

			block = block->next;
		}
	}

	return OK;
}
