/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#define NO_MATH_H
#include <config.h>
/******************************************************************************
 * softdraw.c - Software Draw                                                 *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/window.h"
#include "platform/software/soft.h"
#include "platform/software/softdraw.h"
#include "platform/software/softpreca.h"
#include "platform/software/softdrawtriflat.h"
#include "platform/software/softdrawtriflatadd.h"
#include "platform/software/softdrawtritex.h"
#include "platform/software/softdrawtritexadd.h"

/*---------------------------------------------------------------------------- GLOBAL VARIABLES */
FLOAT32 x1 = 0.0f;
FLOAT32 y1 = 0.0f;
FLOAT32 z1 = 0.0f;
FLOAT32 s1 = 0.0f;
FLOAT32 t1 = 0.0f;
FLOAT32 q1 = 0.0f;
FLOAT32 x2 = 0.0f;
FLOAT32 y2 = 0.0f;
FLOAT32 z2 = 0.0f;
FLOAT32 s2 = 0.0f;
FLOAT32 t2 = 0.0f;
FLOAT32 q2 = 0.0f;
FLOAT32 x3 = 0.0f;
FLOAT32 y3 = 0.0f;
FLOAT32 z3 = 0.0f;
FLOAT32 s3 = 0.0f;
FLOAT32 t3 = 0.0f;
FLOAT32 q3 = 0.0f;

INT32 color = 0;
INT32 bufferPitch = 0;
INT32 zBufferPitch = 0;
INT32 *buffer = NULL;
UINT32 *zBuffer = NULL;
UINT32 *texBuffer = NULL;
INT32 texWidth = 0;
INT32 texHeight = 0;
INT32 texPitch = 0;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static SoftViewport viewport;

/*---------------------------------------------------------------------------- SET CLIP */
RESULT 
SoftDrawSetViewport(SoftViewport *v)
{
    INT16 tmp = 0;

	if (v==NULL)
    {
        return ERR_INVALID_PTR;
    }
	
	if (v->x2<v->x1)
    {
        tmp = v->x1;
        v->x1 = v->x2;
        v->x2 = tmp;
    }    
    if (v->y2<v->y1)
    {
        tmp = v->y1;
        v->y1 = v->y2;
        v->y2 = tmp;
    }

	viewport = *v;

	return OK;
}

/*---------------------------------------------------------------------------- GET CLIP */
SoftViewport*
SoftDrawGetViewport(void)
{
	return &viewport;
}

/*---------------------------------------------------------------------------- CLEAR VIEWPORT */
void 
SoftDrawClearViewport(UINT8 r, UINT8 g, UINT8 b, UINT32 a)
{
	INT32 *buffer = NULL;
	UINT32 *zBuffer = NULL;
	INT32 count = 0;
	
	INT32 color = (a<<24)+(r<<16)+(g<<8)+b;
	
	if (SoftPrecaIsEnabled())
	{
		buffer = SoftPrecaGet()->buffer;
		count = SoftPrecaGet()->width*SoftPrecaGet()->height;
		memset(buffer, (UINT8) 0, count*sizeof(UINT32));

		zBuffer = SoftPrecaGet()->zBuffer;	
		count = SoftPrecaGet()->width*SoftPrecaGet()->height;
		memset(zBuffer, (UINT8) -1, count*sizeof(UINT32));
	}
	else
	{
		buffer = viewport.buffer;	
		count = WindowGet()->bufferPitch*WindowGet()->resY;
		while (count>0)
		{
			*buffer++ = color;		
			count--;
		}

		zBuffer = WindowGet()->zBuffer;	
		count = WindowGet()->zBufferPitch*WindowGet()->resY;
		memset(zBuffer, (UINT8) -1, count*sizeof(UINT32));
	}
}

/*---------------------------------------------------------------------------- DRAW TRIANGLE BUFFER */
void
SoftDrawTriangleBuffer(Vertex *triBuffer, INT32 count, UINT8 r, UINT8 g, UINT8 b)
{
	if (count<=0)
    {
        return;
    }

	if (SoftPrecaIsEnabled())
	{
		bufferPitch = SoftPrecaGet()->width;
		zBufferPitch = SoftPrecaGet()->width;
		buffer = SoftPrecaGet()->buffer;
		zBuffer = SoftPrecaGet()->zBuffer;
	}
	else
	{		 
		bufferPitch = WindowGet()->bufferPitch;
		zBufferPitch = WindowGet()->zBufferPitch;
		buffer = WindowGet()->buffer;
		zBuffer = WindowGet()->zBuffer;
	}
	
	color = (255<<24)+(r<<16)+(g<<8)+b;
	
	count *= 3;

	while (count>0)
    {
		if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			if (SoftPrecaIsEnabled())
			{
				color = triBuffer->precaCount;
			}
			
			x1 = triBuffer->x;
			y1 = triBuffer->y;
			z1 = triBuffer->z;
			triBuffer++;

			x2 = triBuffer->x;
			y2 = triBuffer->y;
			z2 = triBuffer->z;
			triBuffer++;

			x3 = triBuffer->x;
			y3 = triBuffer->y;
			z3 = triBuffer->z;
			triBuffer++;

			SoftDrawTriFlat();

			count -= 3;
		}
    }
}

/*---------------------------------------------------------------------------- DRAW ADDITIVE TRIANGLE BUFFER */
void
SoftDrawTriangleBufferAdd(Vertex *triBuffer, INT32 count, UINT8 r, UINT8 g, UINT8 b)
{
	if (count<=0)
    {
        return;
    }

	bufferPitch = WindowGet()->bufferPitch;
	zBufferPitch = WindowGet()->zBufferPitch;
	buffer = WindowGet()->buffer;
	zBuffer = WindowGet()->zBuffer;
	
	color = (255<<24)+(r<<16)+(g<<8)+b;
	
	count *= 3;

	while (count>0)
    {
		if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x1 = triBuffer->x;
			y1 = triBuffer->y;
			z1 = triBuffer->z;
			triBuffer++;

			x2 = triBuffer->x;
			y2 = triBuffer->y;
			z2 = triBuffer->z;
			triBuffer++;

			x3 = triBuffer->x;
			y3 = triBuffer->y;
			z3 = triBuffer->z;
			triBuffer++;

			SoftDrawTriFlatAdd();

			count -= 3;
		}
    }
}

/*---------------------------------------------------------------------------- DRAW TEXTURED TRIANGLE BUFFER */
void 
SoftDrawTriangleBufferWithTex(Vertex *triBuffer, INT32 count, FLOAT32 w, FLOAT32 h, UINT32 *data)
{
    if (count<=0)
    {
        return;
    }

	bufferPitch = WindowGet()->bufferPitch;
	zBufferPitch = WindowGet()->zBufferPitch;
	buffer = WindowGet()->buffer;
	zBuffer = WindowGet()->zBuffer;	
	texBuffer = data;
	texPitch = (INT32) w;
	texWidth = ((INT32) w)-1;
	texHeight = ((INT32) h)-1;
	
	count *= 3;

	while (count>0)
    {
        if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x1 = triBuffer->x;
			y1 = triBuffer->y;
			z1 = triBuffer->z;
			s1 = triBuffer->u*texWidth;
			t1 = triBuffer->v*texHeight;
			q1 = triBuffer->q;
			triBuffer++;

			x2 = triBuffer->x;
			y2 = triBuffer->y;
			z2 = triBuffer->z;
			s2 = triBuffer->u*texWidth;
			t2 = triBuffer->v*texHeight;
			q2 = triBuffer->q;
			triBuffer++;

			x3 = triBuffer->x;
			y3 = triBuffer->y;
			z3 = triBuffer->z;
			s3 = triBuffer->u*texWidth;
			t3 = triBuffer->v*texHeight;
			q3 = triBuffer->q;
			triBuffer++;

			SoftDrawTriTex();

			count -= 3;	
		}
    }
}

/*---------------------------------------------------------------------------- DRAW ADDITIVE TEXTURED TRIANGLE BUFFER */
void 
SoftDrawTriangleBufferWithTexAdd(Vertex *triBuffer, INT32 count, FLOAT32 w, FLOAT32 h, UINT32 *data)
{
    if (count<=0)
    {
        return;
    }

	bufferPitch = WindowGet()->bufferPitch;
	zBufferPitch = WindowGet()->zBufferPitch;
	buffer = WindowGet()->buffer;
	zBuffer = WindowGet()->zBuffer;	
	texBuffer = data;
	texPitch = (INT32) w;
	texWidth = ((INT32) w)-1;
	texHeight = ((INT32) h)-1;

	count *= 3;

	while (count>0)
    {
        if (triBuffer->skip)
		{
			count -= 3;
			triBuffer += 3;
		}
		else
		{		
			x1 = triBuffer->x;
			y1 = triBuffer->y;
			z1 = triBuffer->z;
			s1 = triBuffer->u*texWidth;
			t1 = triBuffer->v*texHeight;
			q1 = triBuffer->q;
			triBuffer++;

			x2 = triBuffer->x;
			y2 = triBuffer->y;
			z2 = triBuffer->z;
			s2 = triBuffer->u*texWidth;
			t2 = triBuffer->v*texHeight;
			q2 = triBuffer->q;
			triBuffer++;

			x3 = triBuffer->x;
			y3 = triBuffer->y;
			z3 = triBuffer->z;
			s3 = triBuffer->u*texWidth;
			t3 = triBuffer->v*texHeight;
			q3 = triBuffer->q;
			triBuffer++;

			SoftDrawTriTexAdd();

			count -= 3;	
		}
    }
}
