/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * softdrawbilinear.c - Software Bilinear Filtering                           *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/software/soft.h"
#include "platform/software/softdrawbilinear.h"

/*---------------------------------------------------------------------------- LOCAL MACROS */
#define	R(texel)		((texel>>16)&255)
#define	G(texel)		((texel>>8)&255)
#define	B(texel)		(texel&255)

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static UINT32 coefUpLeft = 0;
static UINT32 coefUpRight = 0;
static UINT32 coefDownLeft = 0;
static UINT32 coefDownRight = 0;

static UINT32 texelUpLeft = 0;
static UINT32 texelUpRight = 0;
static UINT32 texelDownLeft = 0;
static UINT32 texelDownRight = 0;

static UINT32 r = 0;
static UINT32 g = 0;
static UINT32 b = 0;

static UINT32 u = 0;
static UINT32 v = 0;
static UINT32 vPitch = 0;
static UINT32 offset = 0;

static UINT32 uCoef = 0;
static UINT32 vCoef = 0;
static UINT32 uCoefInv = 0;
static UINT32 vCoefInv = 0;

/*---------------------------------------------------------------------------- BILINEAR GET TEXEL */
UINT32
SoftDrawBilinearGetTexel(UINT32 *buffer, UINT32 pitch, UINT32 height, UINT32 u8, UINT32 v8)
{
	u = u8>>8;
	v = v8>>8;
	vPitch = v*pitch;
	
	offset = vPitch+u;

	texelUpLeft = buffer[offset];

	if (u<(pitch-1))
	{
		texelUpRight = buffer[offset+1];
	}
	else
	{
		texelUpRight = buffer[vPitch];
	}

	if (v<height)
	{
		texelDownLeft = buffer[offset+pitch];
	}
	else
	{
		texelDownLeft = buffer[u];
	}
	
	if (u<(pitch-1))
	{
		if (v<height)
		{
			texelDownRight = buffer[offset+pitch+1];
		}
		else
		{
			texelDownRight = buffer[u+1];
		}
	}
	else
	{
		if (v<height)
		{
			texelDownRight = buffer[vPitch];
		}
		else
		{
			texelDownRight = buffer[0];
		}
	}

	uCoef = u8&255;
	vCoef = v8&255;	
	uCoefInv = 256-uCoef;
	vCoefInv = 256-vCoef;
	
	coefUpLeft = uCoefInv*vCoefInv;
	coefUpRight = uCoef*vCoefInv;
	coefDownLeft = uCoefInv*vCoef;
	coefDownRight = uCoef*vCoef;

	r = (R(texelUpLeft)*coefUpLeft+R(texelUpRight)*coefUpRight+R(texelDownLeft)*coefDownLeft+R(texelDownRight)*coefDownRight)>>16;
	g = (G(texelUpLeft)*coefUpLeft+G(texelUpRight)*coefUpRight+G(texelDownLeft)*coefDownLeft+G(texelDownRight)*coefDownRight)>>16;
	b = (B(texelUpLeft)*coefUpLeft+B(texelUpRight)*coefUpRight+B(texelDownLeft)*coefDownLeft+B(texelDownRight)*coefDownRight)>>16;

	return (r<<16)+(g<<8)+b;
}
