/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#define NO_MATH_H
#include <config.h>
/******************************************************************************
 * softdrawtriflat.c - Software Draw Flat Triangle                            *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/window.h"
#include "platform/software/soft.h"
#include "platform/software/softdraw.h"
#include "platform/software/softdrawtriflat.h"

/*---------------------------------------------------------------------------- EXTERN VARIABLES */
extern FLOAT32 x1;
extern FLOAT32 y1;
extern FLOAT32 z1;
extern FLOAT32 x2;
extern FLOAT32 y2;
extern FLOAT32 z2;
extern FLOAT32 x3;
extern FLOAT32 y3;
extern FLOAT32 z3;

extern INT32 color;
extern INT32 bufferPitch;
extern INT32 zBufferPitch;
extern INT32 *buffer;
extern UINT32 *zBuffer;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static FLOAT32 tmp = 0.0f;
static FLOAT32 leftX = 0.0f;
static FLOAT32 rightX = 0.0f;
static FLOAT32 leftDeltaX = 0.0f;
static FLOAT32 rightDeltaX = 0.0f;
static FLOAT32 secondDeltaX = 0.0f;
static FLOAT32 leftZ = 0.0f;
static FLOAT32 rightZ = 0.0f;
static FLOAT32 leftDeltaZ = 0.0f;
static FLOAT32 rightDeltaZ = 0.0f;
static FLOAT32 secondDeltaZ = 0.0f;
static FLOAT32 start = 0.0f;
static FLOAT32 end = 0.0f;
static INT32 x = 0;
static INT32 y = 0;
static UINT32 z = 0;
static INT32 bufferOffset = 0;
static INT32 zBufferOffset = 0;
static INT32 zy = 0;
static INT32 xSize = 0;
static INT32 ySize = 0;
static INT32 leftXFixed = 0;
static INT32 rightXFixed = 0;
static INT32 leftDeltaXFixed = 0;
static INT32 rightDeltaXFixed = 0;
static INT32 leftZFixed = 0;
static INT32 rightZFixed = 0;
static INT32 leftDeltaZFixed = 0;
static INT32 rightDeltaZFixed = 0;
static INT32 deltaZFixed = 0;
static BOOL ok = FALSE;
static INT32 *bufferLine;
static UINT32 *zBufferLine;

/*---------------------------------------------------------------------------- DRAW FLAT TRIANGLE */
void
SoftDrawTriFlat(void)
{
	if (y2<y1)
	{
		tmp = y1;
		y1 = y2;
		y2 = tmp;
		tmp = x1;
		x1 = x2;
		x2 = tmp;
		tmp = z1;
		z1 = z2;
		z2 = tmp;
	}

	if (y3<y2)
	{
		tmp = y2;
		y2 = y3;
		y3 = tmp;
		tmp = x2;
		x2 = x3;
		x3 = tmp;
		tmp = z2;
		z2 = z3;
		z3 = tmp;
	}

	if (y2<y1)
	{
		tmp = y1;
		y1 = y2;
		y2 = tmp;
		tmp = x1;
		x1 = x2;
		x2 = tmp;
		tmp = z1;
		z1 = z2;
		z2 = tmp;
	}

	if (y1==y2)
	{
		if (y3==y2)
		{
			y3 += 0.0000001f;
		}

		leftX = x1;
		leftZ = z1;
		rightX = x2;
		rightZ = z2;

		leftDeltaX = (x3-x1)/(y3-y1);
		leftDeltaZ = (z3-z1)/(y3-y1);
		rightDeltaX = (x3-x2)/(y3-y2);
		rightDeltaZ = (z3-z2)/(y3-y2);

		end = y3;
		
		if (leftDeltaX>rightDeltaX)
		{
			tmp = leftX;
			leftX = rightX;
			rightX = tmp;
			tmp = leftZ;
			leftZ = rightZ;
			rightZ = tmp;
		}
	}	
	else
	{
		if (y3==y1)
		{
			y3 += 0.0000001f;
		}

		leftX = x1;
		leftZ = z1;
		rightX = x1;
		rightZ = z1;
		leftDeltaX = (x2-x1)/(y2-y1);
		leftDeltaZ = (z2-z1)/(y2-y1);
		rightDeltaX = (x3-x1)/(y3-y1);
		rightDeltaZ = (z3-z1)/(y3-y1);

		end = y2;
	}
		
	if (leftDeltaX>rightDeltaX)
	{
		tmp = leftDeltaX;
		leftDeltaX = rightDeltaX;
		rightDeltaX = tmp;
		tmp = leftDeltaZ;
		leftDeltaZ = rightDeltaZ;
		rightDeltaZ = tmp;
	}
	
	if (leftX>rightX)
	{			
		tmp = rightX;
		rightX = leftX;
		leftX = tmp;
		tmp = leftDeltaX;
		leftDeltaX = rightDeltaX;
		rightDeltaX = tmp;
		tmp = rightZ;
		rightZ = leftZ;
		leftZ = tmp;
		tmp = leftDeltaZ;
		leftDeltaZ = rightDeltaZ;
		rightDeltaZ = tmp;
	}

	start = y1;

	ok = FALSE;

	leftXFixed = (INT32) ((leftX*COEF_X_FLOAT32)+0.5f);
	leftZFixed = (INT32) ((leftZ*COEF_Z_FLOAT32)+0.5f);

	rightXFixed = (INT32) ((rightX*COEF_X_FLOAT32)+0.5f);
	rightZFixed = (INT32) ((rightZ*COEF_Z_FLOAT32)+0.5f);

	leftDeltaXFixed = (INT32) (leftDeltaX*COEF_X_FLOAT32);
	leftDeltaZFixed = (INT32) (leftDeltaZ*COEF_Z_FLOAT32);

	rightDeltaXFixed = (INT32) (rightDeltaX*COEF_X_FLOAT32);
	rightDeltaZFixed = (INT32) (rightDeltaZ*COEF_Z_FLOAT32);
		
	do
	{
		y = (INT32) start;
		ySize = (INT32) end-y;

		zy = y*zBufferPitch;
		y *= bufferPitch;
		
		while (ySize>0)
		{
			x = (leftXFixed+32768)>>16;
			
			bufferOffset = y+x;
			zBufferOffset = zy+x;

			xSize = ((rightXFixed+32768)>>16)-x;

			if (xSize>0)
			{
				deltaZFixed = (rightZFixed-leftZFixed)/xSize;
				z = leftZFixed;
				
				bufferLine = &buffer[bufferOffset];			
				zBufferLine = &zBuffer[zBufferOffset];
				
				while (xSize>0)
				{
					if (z<=*zBufferLine)
					{
						*bufferLine = color;

						*zBufferLine = z;						
					}

					z += deltaZFixed;
					
					bufferLine++;
					zBufferLine++;
						
					xSize--;
				}
			}

			leftXFixed += leftDeltaXFixed;
			leftZFixed += leftDeltaZFixed;

			rightXFixed += rightDeltaXFixed;
			rightZFixed += rightDeltaZFixed;

			y += bufferPitch;
			zy += zBufferPitch;

			ySize--;
		}

		if (ok==TRUE)
		{
			ok = FALSE;
		}
		else
		{
			if (y1!=y2 && y2!=y3)
			{
				ok = TRUE;
				
				secondDeltaX = (x3-x2)/(y3-y2);
				secondDeltaZ = (z3-z2)/(y3-y2);
				
				if (leftDeltaX<secondDeltaX)
				{
					leftXFixed = (INT32) ((x2*COEF_X_FLOAT32)+0.5f);
					leftZFixed = (INT32) ((z2*COEF_Z_FLOAT32)+0.5f);
					
					leftDeltaXFixed = (INT32) (secondDeltaX*COEF_X_FLOAT32);
					leftDeltaZFixed = (INT32) (secondDeltaZ*COEF_Z_FLOAT32);					
				}
				else
				{
					rightXFixed = (INT32) ((x2*COEF_X_FLOAT32)+0.5f);
					rightZFixed = (INT32) ((z2*COEF_Z_FLOAT32)+0.5f);

					rightDeltaXFixed = (INT32) (secondDeltaX*COEF_X_FLOAT32);
					rightDeltaZFixed = (INT32) (secondDeltaZ*COEF_Z_FLOAT32);
				}
				
				start = end;
				end = y3;
			}
			else
			{
				ok = FALSE;
			}
		}
	} while (ok==TRUE);
}