/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * softpreca.c - Software Precaching                                          *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "system/mem.h"
#include "system/file.h"
#include "system/filepack.h"
#include "platform/software/softpreca.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define PRECA_EXTENSION				"PRK"
#define PRECA_BUFFER_SIZE			1024*1024*16
#define PRECA_CHECK_TABLE_SIZE		256*1024

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static BOOL enabled = FALSE;

static UINT32 precaTriCount = 0;
static SoftPreca preca;

static INT8 fileName[STR_DEFAULT_SIZE];

static UINT8 *buffer = NULL;
static UINT32 bitsOffset = 0;
static UINT32 bitsTableOffset = 0;

static UINT32 speed = 0;

static INT32 precaWidth = 0;
static INT32 precaHeight = 0;

static UINT8 checkTable[PRECA_CHECK_TABLE_SIZE];

/*---------------------------------------------------------------------------- ADD VALUE TO BUFFER */
void
SoftPrecaAddValue(UINT32 value, UINT32 count)
{
	UINT32 offset = 0;
    UINT8 mask = 0;
    UINT32 i = 0;

    for (i=1; i<=count; i++)
	{
		offset = bitsOffset>>3;

		mask = (UINT8) (1<<(7-(bitsOffset-(offset<<3))));
		
		if ((value&(1<<(count-i)))>0)
		{
			buffer[offset] |= mask;
		}
		else
		{
			buffer[offset] &= (UINT8) (0xFF^mask);
		}
        
		bitsOffset++;
	}
}

/*---------------------------------------------------------------------------- GET VALUE FROM BUFFER */
UINT32
SoftPrecaGetValue(UINT32 count)
{
    UINT32 value = 0;
    UINT32 offset = 0;
    UINT32 i = 0;

    for (i=1; i<=count; i++)
    {
        offset = bitsOffset>>3;

        value += (((buffer[offset]>>(7-(bitsOffset-(offset<<3))))&1)<<(count-i));

        bitsOffset++;
    }
    
    return value;
}

/*---------------------------------------------------------------------------- ADD FRAME OFFSET */
void
SoftPrecaOffsetAdd(void)
{
	UINT32 oldBitsOffset = 0;
	UINT32 currentBitsOffset= 0;
	
	currentBitsOffset = bitsOffset;

	oldBitsOffset = bitsOffset;

	bitsOffset = bitsTableOffset;
	SoftPrecaAddValue(currentBitsOffset/*-32signature*/, 32);
	bitsTableOffset = bitsOffset;

	bitsOffset = oldBitsOffset;
}

/*---------------------------------------------------------------------------- INIT */
void
SoftPrecaInit(BOOL enable, INT32 width, INT32 height, INT8 *sceneFileName, UINT32 speed, UINT32 framesCount)
{
	INT32 size = 0;
	INT32 len = 0;
	
	enabled = enable;

	precaTriCount = 1;
	precaWidth = width;
	precaHeight = height;
	
	if (enabled)
	{
		size = precaWidth*precaHeight*sizeof(INT32);			
		preca.buffer = (INT32 *) MemAlloc(size);
		preca.zBuffer = (UINT32 *) MemAlloc(size);

		preca.width = precaWidth;
		preca.height = precaHeight;

		strcpy(fileName, sceneFileName);
		len = strlen(fileName);
		
		fileName[len-3] = PRECA_EXTENSION[0];
		fileName[len-2] = PRECA_EXTENSION[1];
		fileName[len-1] = PRECA_EXTENSION[2];

		buffer = (UINT8 *) MemAlloc(PRECA_BUFFER_SIZE);
		
		bitsOffset = 0;

		SoftPrecaAddValue(PRECA_EXTENSION[0], 8);
		SoftPrecaAddValue(PRECA_EXTENSION[1], 8);
		SoftPrecaAddValue(PRECA_EXTENSION[2], 8);
		SoftPrecaAddValue('1', 8);

		SoftPrecaAddValue(framesCount, 32);
		SoftPrecaAddValue(speed, 32);

		bitsTableOffset = bitsOffset;
		bitsOffset += (framesCount/speed)*sizeof(UINT32)*8;
	}
}

/*---------------------------------------------------------------------------- COMPUTE FRAME */
UINT8 *
SoftPrecaComputeCheckTable(void)
{
	UINT32 i = 0;
	UINT32 color = 0;

	SoftPrecaOffsetAdd();

	memset(checkTable, 0, precaTriCount);
		
	for (i=0; i<(UINT32) preca.width*preca.height; i++)
	{
		color = preca.buffer[i];

		if (color!=0)
		{		
			checkTable[color] = 1;
		}
	}

	return checkTable;
}

/*---------------------------------------------------------------------------- SET CURRENT FRAME */
void 
SoftPrecaSetFrame(UINT8 *fileBuffer, FLOAT32 time)
{
	UINT32 framesCount = 0;
	
	buffer = fileBuffer;
	bitsOffset = 0;

	bitsOffset += 32;

	framesCount = SoftPrecaGetValue(32);
	speed = SoftPrecaGetValue(32);

	bitsOffset += ((UINT32) (time*(FLOAT32) (framesCount/speed)))*sizeof(UINT32)*8;

	bitsOffset = SoftPrecaGetValue(32);
}

/*---------------------------------------------------------------------------- FREE */
void
SoftPrecaFree(void)
{
	File file;

/*
	UINT8 *packedBuffer = NULL;
    INT32 packedSize = 0;    
	*/
	
	file.buffer = (INT8 *) buffer;

	/*
	FilePack(&buffer[4], ((bitsOffset>>3)+1)-4, &packedBuffer, &packedSize);

	memcpy(&buffer[4], packedBuffer, packedSize);
	file.size = packedSize+4;
	*/
	file.size = (bitsOffset>>3)+1;

	strcpy(file.fileName, fileName);

	FileWrite(&file);
	
	MemFree(preca.buffer);
	MemFree(preca.zBuffer);
	MemFree(buffer);
}

/*---------------------------------------------------------------------------- IS ENABLED */
BOOL
SoftPrecaIsEnabled(void)
{
	return enabled;
}

/*---------------------------------------------------------------------------- RESET */
void
SoftPrecaReset(void)
{
	precaTriCount = 1;
}

/*---------------------------------------------------------------------------- ADD +1 */
void
SoftPrecaCountInc(void)
{
	precaTriCount++;
}

/*---------------------------------------------------------------------------- GET */
UINT32
SoftPrecaCountGet(void)
{
	return precaTriCount;
}

/*---------------------------------------------------------------------------- GET PRECA */
SoftPreca *
SoftPrecaGet(void)
{
	return &preca;
}

/*---------------------------------------------------------------------------- BUILD PRK FILENAME */
INT8 *
SoftPrecaBuildFileName(INT8 *sceneFileName)
{
	INT32 len = 0;
	
	strcpy(fileName, sceneFileName);
	len = strlen(fileName);
	
	fileName[len-3] = PRECA_EXTENSION[0];
	fileName[len-2] = PRECA_EXTENSION[1];
	fileName[len-1] = PRECA_EXTENSION[2];

	return fileName;
}