/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * softtransf.c - Software Transformations                                    *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "platform/os/window.h"
#include "platform/os/math.h"
#include "platform/software/soft.h"
#include "platform/software/softpreca.h"
#include "platform/software/softtransf.h"
#include "platform/software/softtransfproj.h"
#include "platform/software/softtransf2dclip.h"
#include "platform/software/softdraw.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define CLIPPING_Z3D                1.0f

#ifdef PS2
#define MAX_TRIANGLES_PER_LIST      2500
#else
#define MAX_TRIANGLES_PER_LIST      50000
#endif

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static Vertex tmpBuffer[MAX_TRIANGLES_PER_LIST*3];
static Vertex bufferProj[MAX_TRIANGLES_PER_LIST*3];
static INT32 tmpBufferCount = 0;

static T_MATRIX *matrixModelView = NULL;

static BOOL clipTex = FALSE;
static BOOL usePreca = FALSE;

static FLOAT32 uOffset = 0.0f;
static FLOAT32 vOffset = 0.0f;

/*---------------------------------------------------------------------------- VERTEX BUFFER TRANSFORMATIONS */
void 
SoftTransfBuildVertexBuffer(FLOAT32 *vertices, FLOAT32 *normals, Vertex *dst, INT32 count, BOOL clipTex)
{
	T_VEC vTrans;
	T_VEC vNormal;
	INT32 i = 0;
	INT32 stride = 0;

#ifdef VERT_ALIGNEMENT
	stride = sizeof(FLOAT32);
#endif
    
    matrixModelView = (T_MATRIX *) MatrixGet(MATRIX_MODELVIEW);

    MatrixVectMultTable((T_VEC *) dst, sizeof(Vertex)-sizeof(T_VEC), matrixModelView, (T_VEC *) vertices, stride, count);

    for (i=0; i<count; i++)
    {
		dst->z = -dst->z;
		dst->q = 0.0f;
		
#ifdef PS2
		dst->y = -dst->y;
#else
        dst->x = -dst->x;
#endif

		dst++;
    }

	if (normals!=NULL)
	{
		dst -= count;
		
		for (i=0; i<count; i++)
		{
			vNormal[0] = normals[i*3];
            vNormal[1] = normals[i*3+1];
            vNormal[2] = normals[i*3+2];
            
            MatrixVectMultEnvMap(&vTrans, &vNormal, matrixModelView);

            dst->nx = vTrans[0];
            dst->ny = vTrans[1];
            // dst->nz = vTrans[2];

			dst++;
		}
	}

	dst -= count;

	SoftTransfProjectionInit(clipTex);

	SoftTransfDoProjection(bufferProj, sizeof(Vertex)-sizeof(T_VEC), dst, sizeof(Vertex)-sizeof(T_VEC), count);
}

/*---------------------------------------------------------------------------- COMPLETE UV INFOS ON VERTEX BUFFER */
void 
SoftTransfComputeUVVertexBuffer(Vertex *dst, INT32 count, FLOAT32 *uvBuffer, BOOL envmap)
{
	INT32 i = 0;
	
	for (i=0; i<count; i++)
	{
		if (uvBuffer!=NULL)
		{
			dst->u = (uvBuffer[i*2]+uOffset)*bufferProj[i].q;
			dst->v = (uvBuffer[i*2+1]+vOffset)*bufferProj[i].q;
		}
		else if (envmap)
		{
			dst->u = (0.5f+dst->nx*0.499999f)*bufferProj[i].q;
			dst->v = (0.5f+dst->ny*0.499999f)*bufferProj[i].q;
		}		

		dst++;
    }
}

/*---------------------------------------------------------------------------- TRIANGLE BUFFER CONSTRUCTION */
void
SoftTransfBuildTriBuffer(SoftTransfTriBufferParam *param)
{
    BOOL useTex = FALSE;
	
	INT32 index1 = 0;
    INT32 index2 = 0;
    INT32 index3 = 0;
	
	INT32 *indexBuffer = NULL;
	Vertex *vertexBuffer = NULL;
	
	Vertex *triBuffer = NULL;
	
	Vertex *swap = NULL;
    Vertex *vert1 = NULL;
    Vertex *vert2 = NULL;
    Vertex *vert3 = NULL;

	FLOAT32 div = 0.0f;

	FLOAT32 q1 = 0.0f;
	FLOAT32 q2 = 0.0f;
	FLOAT32 q3 = 0.0f;

    FLOAT32 culling = 0.0f;

	INT32 i = 0;

	BOOL skip = FALSE;

	triBuffer = tmpBuffer;

	indexBuffer = param->indexBuffer;
	vertexBuffer = param->vertexBuffer;
	useTex = param->doTexClipping;
	
	for (i=0; i<param->srcTriCount; i++)
    {
        skip = FALSE;
		if (usePreca==TRUE)
		{
			if (SoftPrecaGetValue(1)==0)
			{
				skip = TRUE;
			}
		}
		
		if (!skip)
		{
			index1 = indexBuffer[i*3];
			index2 = indexBuffer[i*3+1];
			index3 = indexBuffer[i*3+2];

			vert1 = &vertexBuffer[index1];
			vert2 = &vertexBuffer[index2];
			vert3 = &vertexBuffer[index3];

			if (usePreca==TRUE)
			{
				skip = FALSE;
			}
			else
			{
				skip = FALSE;

				if (vert1->z<CLIPPING_Z3D && vert2->z<CLIPPING_Z3D && vert3->z<CLIPPING_Z3D)
				{
					skip = TRUE;
				}

				if (param->doCulling)
				{
					culling = vert3->x*((vert1->z*vert2->y)-(vert1->y*vert2->z))
							+ vert3->y*((vert1->x*vert2->z)-(vert1->z*vert2->x))
							+ vert3->z*((vert1->y*vert2->x)-(vert1->x*vert2->y));

					if (culling>0.0f)
					{
						skip = TRUE;
					}
				}
			}
			
			if (skip)
			{
			}
			else
			{
				if (vert1->z>=CLIPPING_Z3D && vert2->z>=CLIPPING_Z3D && vert3->z>=CLIPPING_Z3D)
				{
					triBuffer[0] = bufferProj[index1];
					triBuffer[1] = bufferProj[index2];
					triBuffer[2] = bufferProj[index3];
					
					triBuffer[0].skip = FALSE;
					
					if (useTex)
					{
						triBuffer[0].u = vert1->u;
						triBuffer[0].v = vert1->v;
						triBuffer[1].u = vert2->u;
						triBuffer[1].v = vert2->v;
						triBuffer[2].u = vert3->u;
						triBuffer[2].v = vert3->v;
					}

					if (SoftPrecaIsEnabled())
					{
						triBuffer[0].precaCount = SoftPrecaCountGet();
					}
					
					triBuffer += 3;
				}
				else
				{
					vert1->q = bufferProj[index1].q;
					vert2->q = bufferProj[index2].q;
					vert3->q = bufferProj[index3].q;
					
					if (vert3->z>vert2->z)
					{
						swap = vert2;
						vert2 = vert3;
						vert3 = swap;
					}		
					if (vert2->z>vert1->z)
					{
						swap = vert1;
						vert1 = vert2;
						vert2 = swap;
					}
					if (vert3->z>vert2->z)
					{
						swap = vert2;
						vert2 = vert3;
						vert3 = swap;
					}
					
					if (vert3->z<CLIPPING_Z3D && vert2->z>=CLIPPING_Z3D)
					{
						triBuffer[0] = *vert1;
						triBuffer[1] = *vert2;						
						triBuffer[0].skip = FALSE;
					
						if (vert1->z==vert3->z)
						{
							div = (vert1->z-CLIPPING_Z3D)/(vert1->z-vert3->z+0.0000001f);
						}
						else
						{
							div = (vert1->z-CLIPPING_Z3D)/(vert1->z-vert3->z);
						}
							
						triBuffer[2].x = vert1->x+(vert3->x-vert1->x)*div;
						triBuffer[2].y = vert1->y+(vert3->y-vert1->y)*div;
						triBuffer[2].z = CLIPPING_Z3D;					
						
						if (useTex)
						{
							q1 = 1.0f/vert1->q;
							q2 = 1.0f/vert2->q;
							q3 = 1.0f/vert3->q;
							
							triBuffer[0].u *= q1;
							triBuffer[0].v *= q1;
							triBuffer[1].u *= q2;
							triBuffer[1].v *= q2;
							triBuffer[2].u = vert1->u*q1+(vert3->u*q3-vert1->u*q1)*div;
							triBuffer[2].v = vert1->v*q1+(vert3->v*q3-vert1->v*q1)*div;
						}
						
						if (SoftPrecaIsEnabled())
						{
							triBuffer[0].precaCount = SoftPrecaCountGet();
						}

						SoftTransfDoVertexProjection(triBuffer, sizeof(Vertex)-sizeof(T_VEC));
						
						triBuffer += 3;

						triBuffer[0] = *vert2;
						triBuffer[0].skip = FALSE;

						if (vert2->z==vert3->z)
						{
							div = (vert2->z-CLIPPING_Z3D)/(vert2->z-vert3->z+0.0000001f);
						}
						else
						{
							div = (vert2->z-CLIPPING_Z3D)/(vert2->z-vert3->z);
						}
							
						triBuffer[2].x = vert2->x+(vert3->x-vert2->x)*div;
						triBuffer[2].y = vert2->y+(vert3->y-vert2->y)*div;
						triBuffer[2].z = CLIPPING_Z3D;

						if (useTex)
						{
							triBuffer[0].u *= q2;
							triBuffer[0].v *= q2;								
							triBuffer[2].u = vert2->u*q2+(vert3->u*q3-vert2->u*q2)*div;
							triBuffer[2].v = vert2->v*q2+(vert3->v*q3-vert2->v*q2)*div;
						}

						if (SoftPrecaIsEnabled())
						{
							triBuffer[0].precaCount = triBuffer[-3].precaCount;
						}

						SoftTransfDoVertexProjection(triBuffer, sizeof(Vertex)-sizeof(T_VEC));

						triBuffer[1] = triBuffer[-1];
						
						triBuffer += 3;
					}
					else if (vert2->z<CLIPPING_Z3D && vert1->z>=CLIPPING_Z3D)
					{
						triBuffer[0] = *vert1;
						triBuffer[0].skip = FALSE;

						if (vert1->z==vert3->z)
						{
							div = (vert1->z-CLIPPING_Z3D)/(vert1->z-vert3->z+0.0000001f);
						}
						else
						{
							div = (vert1->z-CLIPPING_Z3D)/(vert1->z-vert3->z);
						}

						triBuffer[1].x = vert1->x+(vert3->x-vert1->x)*div;
						triBuffer[1].y = vert1->y+(vert3->y-vert1->y)*div;
						triBuffer[1].z = CLIPPING_Z3D;

						if (useTex)
						{
							q1 = 1.0f/vert1->q;
							q3 = 1.0f/vert3->q;
							
							triBuffer[0].u *= q1;
							triBuffer[0].v *= q1;
							triBuffer[1].u = vert1->u*q1+(vert3->u*q3-vert1->u*q1)*div;
							triBuffer[1].v = vert1->v*q1+(vert3->v*q3-vert1->v*q1)*div;
						}

						if (vert1->z==vert2->z)
						{
							div = (vert1->z-CLIPPING_Z3D)/(vert1->z-vert2->z+0.0000001f);
						}
						else
						{
							div = (vert1->z-CLIPPING_Z3D)/(vert1->z-vert2->z);
						}

						triBuffer[2].x = vert1->x+(vert2->x-vert1->x)*div;
						triBuffer[2].y = vert1->y+(vert2->y-vert1->y)*div;
						triBuffer[2].z = CLIPPING_Z3D;

						if (useTex)
						{
							q2 = 1.0f/vert2->q;
							
							triBuffer[2].u = vert1->u*q1+(vert2->u*q2-vert1->u*q1)*div;
							triBuffer[2].v = vert1->v*q1+(vert2->v*q2-vert1->v*q1)*div;
						}

						if (SoftPrecaIsEnabled())
						{
							triBuffer[0].precaCount = SoftPrecaCountGet();
						}

						SoftTransfDoVertexProjection(triBuffer, sizeof(Vertex)-sizeof(T_VEC));
						
						triBuffer += 3;
					}
				}
			}
		}
			
		if (SoftPrecaIsEnabled())
		{
			SoftPrecaCountInc();
		}
	}

	tmpBufferCount = (triBuffer-tmpBuffer)/3;

	*param->dstTriBuffer = tmpBuffer;

	if (tmpBufferCount==0)
	{
		*param->dstBufferCount = 0;
	}
	else
	{
		if (SoftPrecaIsEnabled())
		{
			*param->dstBufferCount = SoftTransf2DClipBuffer(tmpBuffer, tmpBufferCount, clipTex);

			if (*param->dstBufferCount==tmpBufferCount)
			{
				/*
				param->skip2DClipping = TRUE;
				*/
				param->skip2DClipping = FALSE;
			}
			else
			{
				param->skip2DClipping = FALSE;
			}
		}
		else
		{
			if (param->skip2DClipping==TRUE)
			{
				*param->dstBufferCount = tmpBufferCount;
			}
			else
			{
				*param->dstBufferCount = SoftTransf2DClipBuffer(tmpBuffer, tmpBufferCount, useTex);
			}
		}
	}
}

/*---------------------------------------------------------------------------- ENABLE PRECA */
void 
SoftTransfEnablePreca(BOOL enable)
{
	usePreca = enable;
}

/*---------------------------------------------------------------------------- SET UV OFFSET */
void 
SoftTransfSetOffsetUV(FLOAT32 u, FLOAT32 v)
{
	uOffset = u;
	vOffset = v;
}