/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * softtransf2dclip.c - Software Transformations 2D Clipping                  *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/software/soft.h"
#include "platform/software/softdraw.h"
#include "platform/software/softpreca.h"
#include "platform/software/softtransf2dclip.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static FLOAT32 clipX1 = 0;
static FLOAT32 clipY1 = 0;
static FLOAT32 clipX2 = 0;
static FLOAT32 clipY2 = 0;

static Vertex *tmpBuffer = NULL;
static INT32 tmpBufferCount = 0;

static BOOL clipTex = FALSE;

static Vertex *vSwap = NULL;
static Vertex *v1 = NULL;
static Vertex *v2 = NULL;
static Vertex *v3 = NULL;
static Vertex *src = NULL;
static Vertex *add = NULL;

/*---------------------------------------------------------------------------- CLIP LEFT 2D */
static void
clipLeft(void)
{
    INT32 i = 0;    
    FLOAT32 mul = 0.0f;
	
	src = tmpBuffer;
	add = &src[tmpBufferCount*3];

	for (i=0; i<tmpBufferCount; i++)
    {
		v1 = src;
		v2 = &src[1];
		v3 = &src[2];

		if (v1->x<clipX1 || v2->x<clipX1 || v3->x<clipX1)
		{			
			src->skip = TRUE;
			
			if (v2->x<v1->x)
			{
				vSwap = v1;
				v1 = v2;
				v2 = vSwap;
			}
			if (v3->x<v2->x)
			{
				vSwap = v2;
				v2 = v3;
				v3 = vSwap;
			}
			if (v2->x<v1->x)
			{
				vSwap = v1;
				v1 = v2;
				v2 = vSwap;
			}

			if (v3->x>=clipX1)
			{
				if (v2->x<clipX1)
				{
					add->skip = FALSE;
					add->precaCount = src->precaCount;
					
					add->x = v3->x;
					add->y = v3->y;
					add->z = v3->z;
					if (clipTex)
					{
						add->u = v3->u;
						add->v = v3->v;
						add->q = v3->q;
					}
					add++;

					if (v3->x==v1->x)
					{
						mul = (v3->x-clipX1)/(v3->x-v1->x+0.000001f);
					}
					else
					{
						mul = (v3->x-clipX1)/(v3->x-v1->x);
					}

					add->x = clipX1;
					add->y = v3->y+mul*(v1->y-v3->y);
					add->z = v3->z+mul*(v1->z-v3->z);
					if (clipTex)
					{
						add->u = v3->u+mul*(v1->u-v3->u);
						add->v = v3->v+mul*(v1->v-v3->v);
						add->q = v3->q+mul*(v1->q-v3->q);
					}
					add++;

					if (v3->x==v2->x)
					{
						mul = (v3->x-clipX1)/(v3->x-v2->x+0.000001f);
					}
					else
					{
						mul = (v3->x-clipX1)/(v3->x-v2->x);
					}

					add->x = clipX1;
					add->y = v3->y+mul*(v2->y-v3->y);
					add->z = v3->z+mul*(v2->z-v3->z);
					if (clipTex)
					{
						add->u = v3->u+mul*(v2->u-v3->u);
						add->v = v3->v+mul*(v2->v-v3->v);
						add->q = v3->q+mul*(v2->q-v3->q);
					}
					add++;
				}
				else if (v1->x<clipX1)
				{
					add->skip = FALSE;
					add->precaCount = src->precaCount;
					
					add->x = v3->x;
					add->y = v3->y;
					add->z = v3->z;
					if (clipTex)
					{
						add->u = v3->u;
						add->v = v3->v;
						add->q = v3->q;
					}
					add++;

					add->x = v2->x;
					add->y = v2->y;
					add->z = v2->z;
					if (clipTex)
					{
						add->u = v2->u;
						add->v = v2->v;
						add->q = v2->q;
					}
					add++;

					if (v3->x==v1->x)
					{
						mul = (v3->x-clipX1)/(v3->x-v1->x+0.000001f);
					}
					else
					{
						mul = (v3->x-clipX1)/(v3->x-v1->x);
					}

					add->x = clipX1;
					add->y = v3->y+mul*(v1->y-v3->y);
					add->z = v3->z+mul*(v1->z-v3->z);
					if (clipTex)
					{
						add->u = v3->u+mul*(v1->u-v3->u);
						add->v = v3->v+mul*(v1->v-v3->v);
						add->q = v3->q+mul*(v1->q-v3->q);
					}
					add++;

					add->skip = FALSE;
					add->precaCount = src->precaCount;
					
					if (v2->x==v1->x)
					{
						mul = (v2->x-clipX1)/(v2->x-v1->x+0.000001f);
					}
					else
					{
						mul = (v2->x-clipX1)/(v2->x-v1->x);
					}

					add->x = clipX1;
					add->y = v2->y+mul*(v1->y-v2->y);
					add->z = v2->z+mul*(v1->z-v2->z);
					if (clipTex)
					{
						add->u = v2->u+mul*(v1->u-v2->u);
						add->v = v2->v+mul*(v1->v-v2->v);
						add->q = v2->q+mul*(v1->q-v2->q);
					}
					add++;

					*add = *(add-3);
					add++;

					*add = *(add-3);
					add++;
				}
			}
		}

		src += 3;
    }

    tmpBufferCount = (add-tmpBuffer)/3;
}

/*---------------------------------------------------------------------------- CLIP UP 2D */
static void
clipUp(void)
{
    INT32 i = 0;    
    FLOAT32 mul = 0.0f;
	
    src = tmpBuffer;
	add = &src[tmpBufferCount*3];

	for (i=0; i<tmpBufferCount; i++)
    {
        if (src->skip==FALSE)
		{
			v1 = src;
			v2 = &src[1];
			v3 = &src[2];

			if (v1->y<clipY1 || v2->y<clipY1 || v3->y<clipY1)
			{
				src->skip = TRUE;

				if (v2->y<v1->y)
				{
					vSwap = v1;
					v1 = v2;
					v2 = vSwap;
				}
				if (v3->y<v2->y)
				{
					vSwap = v2;
					v2 = v3;
					v3 = vSwap;
				}
				if (v2->y<v1->y)
				{
					vSwap = v1;
					v1 = v2;
					v2 = vSwap;
				}

				if (v3->y>=clipY1)
				{
					if (v2->y<clipY1)
					{
						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						add->x = v3->x;
						add->y = v3->y;
						add->z = v3->z;
						if (clipTex)
						{
							add->u = v3->u;
							add->v = v3->v;
							add->q = v3->q;
						}
						add++;

						if (v3->y==v1->y)
						{
							mul = (v3->y-clipY1)/(v3->y-v1->y+0.000001f);
						}
						else
						{
							mul = (v3->y-clipY1)/(v3->y-v1->y);
						}
        
						add->x = v3->x+mul*(v1->x-v3->x);
						add->y = clipY1;
						add->z = v3->z+mul*(v1->z-v3->z);
						if (clipTex)
						{
							add->u = v3->u+mul*(v1->u-v3->u);
							add->v = v3->v+mul*(v1->v-v3->v);
							add->q = v3->q+mul*(v1->q-v3->q);
						}
						add++;

						if (v3->y==v2->y)
						{
							mul = (v3->y-clipY1)/(v3->y-v2->y+0.000001f);
						}
						else
						{
							mul = (v3->y-clipY1)/(v3->y-v2->y);
						}
        
						add->x = v3->x+mul*(v2->x-v3->x);
						add->y = clipY1;
						add->z = v3->z+mul*(v2->z-v3->z);
						if (clipTex)
						{
							add->u = v3->u+mul*(v2->u-v3->u);
							add->v = v3->v+mul*(v2->v-v3->v);
							add->q = v3->q+mul*(v2->q-v3->q);
						}
						add++;
					}
					else if (v1->y<clipY1)
					{
						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						add->x = v3->x;
						add->y = v3->y;
						add->z = v3->z;
						if (clipTex)
						{
							add->u = v3->u;
							add->v = v3->v;
							add->q = v3->q;
						}
						add++;

						add->x = v2->x;
						add->y = v2->y;
						add->z = v2->z;
						if (clipTex)
						{
							add->u = v2->u;
							add->v = v2->v;
							add->q = v2->q;
						}
						add++;

						if (v3->y==v1->y)
						{
							mul = (v3->y-clipY1)/(v3->y-v1->y+0.000001f);
						}
						else
						{
							mul = (v3->y-clipY1)/(v3->y-v1->y);
						}
						
						add->x = v3->x + mul*(v1->x-v3->x);
						add->y = clipY1;
						add->z = v3->z + mul*(v1->z-v3->z);
						if (clipTex)
						{
							add->u = v3->u+mul*(v1->u-v3->u);
							add->v = v3->v+mul*(v1->v-v3->v);
							add->q = v3->q+mul*(v1->q-v3->q);
						}
						add++;

						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						if (v2->y==v1->y)
						{
							mul = (v2->y-clipY1)/(v2->y-v1->y+0.000001f);
						}
						else
						{
							mul = (v2->y-clipY1)/(v2->y-v1->y);
						}
        
						add->x = v2->x+mul*(v1->x-v2->x);
						add->y = clipY1;
						add->z = v2->z+mul*(v1->z-v2->z);
						if (clipTex)
						{
							add->u = v2->u+mul*(v1->u-v2->u);
							add->v = v2->v+mul*(v1->v-v2->v);
							add->q = v2->q+mul*(v1->q-v2->q);
						}
						add++;

						*add = *(add-3);
						add++;

						*add = *(add-3);
						add++;
					}
				}
			}
		}

		src += 3;
    }

    tmpBufferCount = (add-tmpBuffer)/3;
}

/*---------------------------------------------------------------------------- CLIP RIGHT 2D */
static void
clipRight(void)
{
    INT32 i = 0;    
    FLOAT32 mul = 0.0f;
	
	src = tmpBuffer;
	add = &src[tmpBufferCount*3];

	for (i=0; i<tmpBufferCount; i++)
    {
        if (src->skip==FALSE)
		{
			v1 = src;
			v2 = &src[1];
			v3 = &src[2];

			if (v1->x>=clipX2 || v2->x>=clipX2 || v3->x>=clipX2)
			{
				src->skip = TRUE;

				if (v2->x<v1->x)
				{
					vSwap = v1;
					v1 = v2;
					v2 = vSwap;
				}
				if (v3->x<v2->x)
				{
					vSwap = v2;
					v2 = v3;
					v3 = vSwap;
				}
				if (v2->x<v1->x)
				{
					vSwap = v1;
					v1 = v2;
					v2 = vSwap;
				}

				if (v1->x<clipX2)
				{
					if (v2->x>=clipX2)
					{
						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						add->x = v1->x;
						add->y = v1->y;
						add->z = v1->z;
						if (clipTex)
						{
							add->u = v1->u;
							add->v = v1->v;
							add->q = v1->q;
						}
						add++;

						if (v2->x==v1->x)
						{
							mul = (clipX2-v1->x)/(v2->x-v1->x+0.000001f);
						}
						else
						{
							mul = (clipX2-v1->x)/(v2->x-v1->x);
						}

						add->x = clipX2;
						add->y = v1->y+mul*(v2->y-v1->y);
						add->z = v1->z+mul*(v2->z-v1->z);
						if (clipTex)
						{
							add->u = v1->u+mul*(v2->u-v1->u);
							add->v = v1->v+mul*(v2->v-v1->v);
							add->q = v1->q+mul*(v2->q-v1->q);
						}
						add++;

						if (v3->x==v1->x)
						{
							mul = (clipX2-v1->x)/(v3->x-v1->x+0.000001f);
						}
						else
						{
							mul = (clipX2-v1->x)/(v3->x-v1->x);
						}

						add->x = clipX2;
						add->y = v1->y+mul*(v3->y-v1->y);
						add->z = v1->z+mul*(v3->z-v1->z);
						if (clipTex)
						{
							add->u = v1->u+mul*(v3->u-v1->u);
							add->v = v1->v+mul*(v3->v-v1->v);
							add->q = v1->q+mul*(v3->q-v1->q);
						}
						add++;
					}
					else if (v3->x>=clipX2)
					{
						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						add->x = v1->x;
						add->y = v1->y;
						add->z = v1->z;
						if (clipTex)
						{
							add->u = v1->u;
							add->v = v1->v;
							add->q = v1->q;
						}
						add++;

						add->x = v2->x;
						add->y = v2->y;
						add->z = v2->z;
						if (clipTex)
						{
							add->u = v2->u;
							add->v = v2->v;
							add->q = v2->q;
						}
						add++;

						if (v3->x==v1->x)
						{
							mul = (clipX2-v1->x)/(v3->x-v1->x+0.000001f);
						}
						else
						{
							mul = (clipX2-v1->x)/(v3->x-v1->x);
						}
						
						add->x = clipX2;
						add->y = v1->y+mul*(v3->y-v1->y);
						add->z = v1->z+mul*(v3->z-v1->z);
						if (clipTex)
						{
							add->u = v1->u+mul*(v3->u-v1->u);
							add->v = v1->v+mul*(v3->v-v1->v);
							add->q = v1->q+mul*(v3->q-v1->q);
						}
						add++;

						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						if (v3->x==v2->x)
						{
							mul = (clipX2-v2->x)/(v3->x-v2->x+0.000001f);
						}
						else
						{
							mul = (clipX2-v2->x)/(v3->x-v2->x);
						}

						add->x = clipX2;
						add->y = v2->y+mul*(v3->y-v2->y);
						add->z = v2->z+mul*(v3->z-v2->z);
						if (clipTex)
						{
							add->u = v2->u+mul*(v3->u-v2->u);
							add->v = v2->v+mul*(v3->v-v2->v);
							add->q = v2->q+mul*(v3->q-v2->q);
						}
						add++;

						*add = *(add-3);
						add++;

						*add = *(add-3);
						add++;
					}
				}
			}
        }

		src += 3;
    }

    tmpBufferCount = (add-tmpBuffer)/3;
}

/*---------------------------------------------------------------------------- CLIP DOWN 2D */
static void
clipDown(void)
{
    INT32 i = 0;    
    FLOAT32 mul = 0.0f;
	
	src = tmpBuffer;
	add = &src[tmpBufferCount*3];

	for (i=0; i<tmpBufferCount; i++)
    {
        if (src->skip==FALSE)
		{
			v1 = src;
			v2 = &src[1];
			v3 = &src[2];

			if (v1->y>=clipY2 || v2->y>=clipY2 || v3->y>=clipY2)
			{
				src->skip = TRUE;
			
				if (v2->y<v1->y)
				{
					vSwap = v1;
					v1 = v2;
					v2 = vSwap;
				}
				if (v3->y<v2->y)
				{
					vSwap = v2;
					v2 = v3;
					v3 = vSwap;
				}
				if (v2->y<v1->y)
				{
					vSwap = v1;
					v1 = v2;
					v2 = vSwap;
				}

				if (v1->y<clipY2)
				{
					if (v2->y>=clipY2)
					{
						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						add->x = v1->x;
						add->y = v1->y;
						add->z = v1->z;
						if (clipTex)
						{
							add->u = v1->u;
							add->v = v1->v;
							add->q = v1->q;
						}
						add++;

						if (v2->y==v1->y)
						{
							mul = (clipY2-v1->y)/(v2->y-v1->y+0.000001f);
						}
						else
						{
							mul = (clipY2-v1->y)/(v2->y-v1->y);
						}

						add->x = v1->x+mul*(v2->x-v1->x);
						add->y = clipY2;
						add->z = v1->z+mul*(v2->z-v1->z);
						if (clipTex)
						{
							add->u = v1->u+mul*(v2->u-v1->u);
							add->v = v1->v+mul*(v2->v-v1->v);
							add->q = v1->q+mul*(v2->q-v1->q);
						}
						add++;

						if (v3->y==v1->y)
						{
							mul = (clipY2-v1->y)/(v3->y-v1->y+0.000001f);
						}
						else
						{
							mul = (clipY2-v1->y)/(v3->y-v1->y);
						}

						add->x = v1->x+mul*(v3->x-v1->x);
						add->y = clipY2;
						add->z = v1->z+mul*(v3->z-v1->z);
						if (clipTex)
						{
							add->u = v1->u+mul*(v3->u-v1->u);
							add->v = v1->v+mul*(v3->v-v1->v);
							add->q = v1->q+mul*(v3->q-v1->q);
						}
						add++;
					}
					else if (v3->y>=clipY2)
					{
						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						add->x = v1->x;
						add->y = v1->y;
						add->z = v1->z;
						if (clipTex)
						{
							add->u = v1->u;
							add->v = v1->v;
							add->q = v1->q;
						}
						add++;

						add->x = v2->x;
						add->y = v2->y;
						add->z = v2->z;
						if (clipTex)
						{
							add->u = v2->u;
							add->v = v2->v;
							add->q = v2->q;
						}
						add++;

						if (v3->y==v1->y)
						{
							mul = (clipY2-v1->y)/(v3->y-v1->y+0.000001f);
						}
						else
						{
							mul = (clipY2-v1->y)/(v3->y-v1->y);
						}

						add->x = v1->x+mul*(v3->x-v1->x);
						add->y = clipY2;
						add->z = v1->z+mul*(v3->z-v1->z);
						if (clipTex)
						{
							add->u = v1->u+mul*(v3->u-v1->u);
							add->v = v1->v+mul*(v3->v-v1->v);
							add->q = v1->q+mul*(v3->q-v1->q);
						}
						add++;

						add->skip = FALSE;
						add->precaCount = src->precaCount;
									
						if (v3->y==v2->y)
						{
							mul = (clipY2-v2->y)/(v3->y-v2->y+0.000001f);
						}
						else
						{
							mul = (clipY2-v2->y)/(v3->y-v2->y);
						}

						add->x = v2->x+mul*(v3->x-v2->x);
						add->y = clipY2;
						add->z = v2->z+mul*(v3->z-v2->z);
						if (clipTex)
						{
							add->u = v2->u+mul*(v3->u-v2->u);
							add->v = v2->v+mul*(v3->v-v2->v);
							add->q = v2->q+mul*(v3->q-v2->q);
						}
						add++;

						*add = *(add-3);
						add++;

						*add = *(add-3);
						add++;
					}
				}
			}
        }

		src += 3;
    }

    tmpBufferCount = (add-tmpBuffer)/3;
}

/*---------------------------------------------------------------------------- 2D CLIP */
INT32
SoftTransf2DClipBuffer(Vertex *buffer, INT32 triCount, BOOL texture)
{
	SoftViewport *viewport = NULL;
	
	viewport = SoftDrawGetViewport();
			
	clipX1 = (FLOAT32) viewport->x1;
	clipY1 = (FLOAT32) viewport->y1;
	clipX2 = (FLOAT32) viewport->x2;
	clipY2 = (FLOAT32) viewport->y2;

	tmpBuffer = buffer;
	tmpBufferCount = triCount;
	clipTex = texture;

	clipLeft();
	clipUp();
	clipRight();
	clipDown();

	return tmpBufferCount;
}