/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * softtransfproj.c - Software Transformations Projection                     *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/renderer/matrix.h"
#include "platform/os/window.h"
#include "platform/software/soft.h"
#include "platform/software/softpreca.h"
#include "platform/software/softtransfproj.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static T_MATRIX *matrixProjection = NULL;

static FLOAT32 centerWidth = 0.0f;
static FLOAT32 centerHeight = 0.0f;

static BOOL clipTex = FALSE;

/*---------------------------------------------------------------------------- INIT PROJECTION */
void
SoftTransfProjectionInit(BOOL texture)
{
	clipTex = texture;
	
	matrixProjection = (T_MATRIX *) MatrixGet(MATRIX_PROJECTION);
    (*matrixProjection)[3][2] = 1.0f;

    if (SoftPrecaIsEnabled())
	{
		centerWidth = (FLOAT32) (SoftPrecaGet()->width>>1);
		centerHeight = (FLOAT32) (SoftPrecaGet()->height>>1);
	}
	else
	{
		centerWidth = (FLOAT32) (WindowGet()->resX>>1);
		centerHeight = (FLOAT32) (WindowGet()->resY>>1);
	}
}

/*---------------------------------------------------------------------------- DO PROJECTION */
void
SoftTransfDoProjection(Vertex *dst, INT32 dstPad, Vertex *src, INT32 srcPad, INT32 count)
{
	FLOAT32 w = 0.0f;
	FLOAT32 wDiv = 0.0f;
	INT32 i = 0;

	if (count<=0)
	{
		return;
	}
	
	MatrixVectMultWithWTable((T_VEC *) dst, dstPad, matrixProjection, (T_VEC *) src, srcPad, count);
	
	for (i=0; i<count; i++)
	{
		w = dst->q;
		if (w!=0.0f && w!=1.0f)
		{
			wDiv = (FLOAT32) (1.0f/w);
			
			dst->x *= wDiv;
			dst->y *= wDiv;

			dst->q = wDiv;
		}
		else
		{
			dst->q = 1.0f;
		}

		dst->x += centerWidth;
		dst->y = -dst->y+centerHeight;
		dst->z = -dst->z;

		dst++;
	}
}

/*---------------------------------------------------------------------------- DO PROJECTION FOR A VERTEX */
void
SoftTransfDoVertexProjection(Vertex *dst, INT32 dstPad)
{
	FLOAT32 w = 0.0f;
	FLOAT32 wDiv = 0.0f;
	INT32 i = 0;

	MatrixVectMultWithWTable((T_VEC *) dst, dstPad, matrixProjection, (T_VEC *) dst, dstPad, 3);
	
	for (i=0; i<3; i++)
	{
		w = dst->q;
		if (w!=0.0f && w!=1.0f)
		{
			wDiv = (FLOAT32) (1.0f/w);
			
			dst->x *= wDiv;
			dst->y *= wDiv;
			dst->q = wDiv;
		}

		dst->x += centerWidth;
		dst->y = -dst->y+centerHeight;
		dst->z = -dst->z;

		dst->u *= dst->q;
		dst->v *= dst->q;

		dst++;
	}
}