/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * cmdline.c - Command Line                                                   *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "platform/os/window.h"
#include "system/file.h"
#include "system/filearc.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsplaylist.h"
#include "system/cmdline.h"
#include "platform/os/music.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT8 cmdLine[512] = {""};

/*---------------------------------------------------------------------------- CONSOLE OUTPUT */
static void
showHelp(void)
{
    ConsoleAdd(" --help: this help");
	ConsoleAdd(" --version: version description");
	ConsoleAdd(" -f: fullscreen at startup");
	ConsoleAdd("");
	
	exit(0);
}

static void
showVersion(void)
{
    ConsoleAdd("jfyeNG");
    ConsoleAdd("");

	exit(0);
}

/*---------------------------------------------------------------------------- ARCHIVE PARSING */
static void
arcParse(void)
{
    INT32 i = 0;
    INT32 j = 0;
    INT32 k = 0;
    INT32 len = 0;
    INT8 fileName[STR_DEFAULT_SIZE] = {""};
    INT8 fileNameMinus[STR_DEFAULT_SIZE] = {""};

    FileArcGetNodeCount(&j);

    for (i=0; i<j; i++)
    {
        FileArcGetIndexedFilename(fileName, i);

        len = (INT32) strlen(fileName);

        for (k=0; k<len; k++)
        {
            if (fileName[k]>='A' && fileName[k]<='Z')
            {
                fileNameMinus[k] = (INT8) (fileName[k]+('a'-'A'));
            }
            else
            {
                fileNameMinus[k] = fileName[k];
            }
        }

        if (strstr(fileNameMinus, ".lws")!=NULL)
        {
            PlayListAdd(fileName);
        }
    }
}

/*---------------------------------------------------------------------------- ACCESSORS */
INT8 *
CmdLineGet(void)
{
    return cmdLine;
}

void
CmdLineSet(const INT8 *newCmdLine)
{
    if (newCmdLine==NULL)
    {
        return;
    }
    
    strcpy(cmdLine, newCmdLine);
}

/*---------------------------------------------------------------------------- COMMAND LINE PARSING */
void
CmdLineParse(void)
{
    INT8 cmd[STR_DEFAULT_SIZE] = {""};
    INT32 i = 0;
    INT32 j = 0;
    INT32 len = 0;
    BOOL skip = FALSE;

    ConsoleAdd("jfyeNG - Newtek LightWave replayer");
    ConsoleAdd("Copyright (C) 2003  Arnaud STORQ (norecess@planet-d.net)");
    
    WindowGet()->isFullScreen = FALSE;
	
    len = (INT32) strlen(cmdLine);
    
    if (len>0)
    {
        i = 0;        

        do
        {
            j = 0;
            skip = FALSE;

            while (cmdLine[i]!=' ' && i<len && skip==FALSE)
            {
                if (cmdLine[i]=='"')
                {
                    j = 0;
                    i++;
                    
                    while (cmdLine[i]!='"')
                    {
                        cmd[j] = cmdLine[i];

                        i++;
                        j++;
                    }

                    i++;

                    skip = TRUE;
                }
                else
                {
                    cmd[j] = cmdLine[i];

                    i++;
                    j++;
                }
            }

            cmd[j] = 0;

            i++;

            if (strcmp(cmd, "--help")==0)
            {
                showHelp();                
			}
			else if (strcmp(cmd, "--version")==0)
            {
            	showVersion();
			}
			else if(strcmp(cmd, "-f")==0)
            {
                WindowGet()->isFullScreen = TRUE;
            }
            else if(strstr(cmd, ".jng")!=NULL)
            {
                arcParse();
            }
        } while (i<len);
    }
}