/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * file.c - File I/O                                                          *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "platform/os/settings.h"
#include "platform/os/fileio.h"
#include "system/file.h"
#include "system/filearc.h"
#include "system/mem.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static INT8 prevPath[STR_DEFAULT_SIZE] = "";

/*---------------------------------------------------------------------------- LOAD */
RESULT
FileLoad(File *file, INT8 *fileName)
{
    INT8 fileWithLWPath[STR_DEFAULT_SIZE] = {""};	
	INT32 handle = 0;
    void *buffer = NULL;
    INT32 size = 0;
	INT32 i = 0;
    INT8 tmp[STR_DEFAULT_SIZE] = {""};
    
    if (file==NULL || fileName==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (fileName[0]=='\\')
    {
        strcpy(tmp, fileName);
        strcpy(fileName, "C:");
        strcat(fileName, tmp);
    }

	if (fileName[1]==':' && fileName[2]!='\\')
    {
        strcpy(tmp, fileName);
		fileName[2] = '\\';
		fileName[3] = 0;
        strcat(fileName, &tmp[2]);
    }

    strcpy(file->fileName, fileName);

    SettingsGetValue(SETTINGS_KEY_LW_PATH, fileWithLWPath);
	strcat(fileWithLWPath, "\\");
	strcat(fileWithLWPath, fileName);

    if (FileArcIsPresent()==TRUE)
    {
        if (FileArcGetFile(&buffer, fileName, &size)==OK)
		{
			strcpy(tmp, fileName);
			
			i = strlen(tmp)-1;
			while (tmp[i]!='\\' && tmp[i]!='/' && i>0)
			{
				i--;
			}

			if (i==0)
			{
				strcpy(prevPath, "");
			}
			else
			{
				tmp[i+1] = 0;

				strcpy(prevPath, tmp);
			}
		}
		else if (FileArcGetFile(&buffer, fileWithLWPath, &size)!=OK)
        {
            strcpy(tmp, prevPath);
			strcat(tmp, fileName);
			
			if (FileArcGetFile(&buffer, tmp, &size)!=OK)
			{
				file->size = 0;
				file->buffer = NULL;
    
				if (strstr(fileName, "prk")!=NULL)
				{
				}
				else if (strstr(fileName, "PRK")!=NULL)
				{
				}
				else
				{
					ConsoleAdd("Error while getting %s from archive file !", fileName);
				}

				return ERR_ARCHIVE;
			}
        }
    }
    else
    {   
        handle = FileIOOpen(fileName, FILE_IO_READ_ONLY);
        
        if (handle<=0)
        {
            handle = FileIOOpen(fileWithLWPath, FILE_IO_READ_ONLY);

			if (handle<=0)
			{
				file->size = 0;
				file->buffer = NULL;
            
				if (strstr(fileName, "prk")!=NULL)
				{
				}
				else if (strstr(fileName, "PRK")!=NULL)
				{
				}
				else
				{					
					ConsoleAdd("Error while opening %s !", fileName);
				}
            
				return ERR_OPEN_FILE;
			}
        }

		size = FileIOGetSize(handle);
        
        buffer = MemAlloc(size);

        if (buffer==NULL)
        {
            FileIOClose(handle);
            
            file->size = 0;
            file->buffer = NULL;
            
            ConsoleAdd("Not enough memory");
            
            return ERR_NOT_ENOUGH_MEM;
        }

        FileIORead(handle, buffer, size);

        FileIOClose(handle);
    }

    file->size = size;
    file->buffer = buffer;

    return OK;
}

/*---------------------------------------------------------------------------- WRITE */
RESULT
FileWrite(File *file)
{
	INT32 handle = 0;
    
    if (file==NULL)
    {
        return ERR_INVALID_PTR;
    }
        
    handle = FileIOOpen(file->fileName, FILE_IO_WRITE_ONLY);
    
    if (handle<=0)
    {
        ConsoleAdd("Cannot write file");
        
        return ERR_WRITE_FILE;
    }

    FileIOWrite(handle, file->buffer, file->size);

    FileIOClose(handle);
        
    return OK;
}

/*---------------------------------------------------------------------------- FREE */
RESULT
FileFree(File *file)
{
    if (file==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (file->buffer!=NULL)
    {
        SAFE_FREE(file->buffer);

        file->size = 0;
        file->buffer = NULL;
    }

    return OK;
}
