/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * filepack.c - File Packer                                                   *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "system/filepack.h"
#include "platform/os/console.h"
#include "system/mem.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define PACK_SIGNATURE      "PAK1"
#define NODES_COUNT         256

/*---------------------------------------------------------------------------- LOCAL STRUCT */
typedef struct node
{
	INT32 freq;
	UINT8 value;

    struct node *father;
	struct node *left;
	struct node *right;

    INT32 bitCount;
    UINT32 bitValue;

    BOOL active;
} node;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static UINT32 checksum = 0;
    
static node *nodeTable[NODES_COUNT];
static INT32 nodeTableCount = 0;

static node *packNodeTable[NODES_COUNT];

static UINT32 maxSize = 0;
static UINT32 bitsOffset = 0;
static UINT8 *buffer = NULL;

static BOOL failed = FALSE;

static INT32 depth = 0;
static node *unpackNodeTable = NULL;
        
/*---------------------------------------------------------------------------- CHECKSUM */
static UINT32
computeChecksum(UINT8 *src, INT32 srcSize)
{
    UINT32 a = 0;
    
    a = 0;
    
    while (srcSize>0)
    {
        srcSize--;

        a += (UINT32) src[srcSize];
    }

    return a;
}

/*---------------------------------------------------------------------------- BITS STREAM */
static void
readBitsInit(UINT8 *src)
{
    bitsOffset = 0;

    buffer = src;
}

static UINT32
readBits(UINT32 count)
{
    UINT32 value = 0;
    UINT32 offset = 0;
    UINT32 i = 0;

    for (i=1; i<=count; i++)
    {
        offset = bitsOffset>>3;

        value += (((buffer[offset]>>(7-(bitsOffset-(offset<<3))))&1)<<(count-i));

        bitsOffset++;
    }
    
    return value;
}

static void
writeBitsInit(UINT8 *dst, INT32 srcSize)
{
    maxSize = srcSize;

    bitsOffset = 0;

    buffer = dst;
}

static void 
writeBits(UINT32 value, UINT32 count)
{
    UINT32 offset = 0;
    UINT8 mask = 0;
    UINT32 i = 0;

    if ((bitsOffset>>3)>=maxSize)
    {
        failed = TRUE;
        return;
    }

    for (i=1; i<=count; i++)
	{
		offset = bitsOffset>>3;

		mask = (UINT8) (1<<(7-(bitsOffset-(offset<<3))));
		
		if ((value&(1<<(count-i)))>0)
		{
			buffer[offset] |= mask;
		}
		else
		{
			buffer[offset] &= (UINT8) (0xFF^mask);
		}
        
		bitsOffset++;
	}
}

/*---------------------------------------------------------------------------- NODES PACK */
static void
nodeTableClear(void)
{
    UINT32 i = 0;
    node *n = NULL;

    for (i=0; i<NODES_COUNT; i++)
    {
        n = (node *) MemAlloc(sizeof(node));

        n->value = (UINT8) i;
        n->freq = 0;

        n->father = NULL;
        n->left = NULL;
        n->right = NULL;

        nodeTable[i] = n;

        packNodeTable[i] = n;
    }
}

static void
nodeTableComputeFreq(UINT8 *src, INT32 srcSize)
{
    UINT8 a = 0;
    
    while (srcSize>0)
    {
        srcSize--;

        a = src[srcSize];

        nodeTable[a]->freq++;
    }
}

static void
nodeTableSort(void)
{
    node *n = NULL;
    INT32 i = 0;

    while (i<(NODES_COUNT-1))
    {
        if (nodeTable[i+1]->freq>nodeTable[i]->freq)
        {
            n = nodeTable[i+1];
            nodeTable[i+1] = nodeTable[i];
            nodeTable[i] = n;

            i = -1;
        }
        
        i++;
    }
}

static INT32
nodeTableGetCount(void)
{
    INT32 i = 0;
    
    while (i<NODES_COUNT && nodeTable[i]->freq>0)
    {
        i++;
    }

    return i;
}

static void
nodeTableBuildTree(void)
{
    node *n = NULL;
    INT32 i = nodeTableCount;
    INT32 j = 0;

    i--;
    
    while (i>0)
    {
        n = (node *) MemAlloc(sizeof(node));

        n->value = 0;
        n->freq = nodeTable[i-1]->freq+nodeTable[i]->freq;

        n->father = NULL;
        n->left = nodeTable[i-1];
        n->right = nodeTable[i];

        nodeTable[i-1]->father = n;
        nodeTable[i]->father = n;

        i--;

        nodeTable[i] = n;

        if (i>0)
        {
            j = i;

            while (j>0 && nodeTable[j-1]->freq<nodeTable[j]->freq)
            {
                n = nodeTable[j-1];
                nodeTable[j-1] = nodeTable[j];
                nodeTable[j] = n;

                j--;
            }
        }
    }
}

static void
nodeTableBuildBits(void)
{
    node *n = NULL;
    INT32 i = 0;
    
    for (i=0; i<NODES_COUNT; i++)
    {
        n = packNodeTable[i];

        n->bitCount = 0;
        n->bitValue = 0;

        while (n->father!=NULL)
        {
            if (n->father->right==n)
            {
                packNodeTable[i]->bitValue |= 1<<packNodeTable[i]->bitCount;
            }

            packNodeTable[i]->bitCount++;

            n = n->father;
        }
    }
}

static void
nodeTableComputeDepth(void)
{
    INT32 i = 0;
    
    for (i=0; i<NODES_COUNT; i++)
    {
        if (packNodeTable[i]->bitCount>depth)
        {
            depth = packNodeTable[i]->bitCount;
        }
    }
}

static void
writeTableCount(void)
{
    INT32 i = 0;

    for (i=0; i<NODES_COUNT; i++)
    {
        if (packNodeTable[i]->bitCount>0)
        {
            writeBits(1, 1);
        }
        else
        {
            writeBits(0, 1);
        }
    }
}

static void
writeTableConvert(void)
{
    INT32 i = 0;
    INT32 bitCount = 0;

    for (i=0; i<NODES_COUNT; i++)
    {
        bitCount = packNodeTable[i]->bitCount;
        
        if (bitCount> 0)
		{
			writeBits(bitCount, depth);
			writeBits(packNodeTable[i]->bitValue, bitCount);
		}
    }
}

static void
writePacked(UINT8 *src, INT32 srcSize)
{
    INT32 i = 0;
    UINT8 value = 0;
    INT32 bitCount = 0;

    for (i=0; i<srcSize; i++)
    {
        value = src[i];
        
        bitCount = packNodeTable[value]->bitCount;

        while (bitCount>0)
        {
            bitCount--;

            if (((packNodeTable[value]->bitValue>>bitCount)&1)==0)
            {
                bitsOffset++;
            }
            else
            {
                writeBits(1, 1);
            }
        }
    }
}

/*---------------------------------------------------------------------------- NODES UNPACK */
static void
readNodeTableInit(void)
{
    INT32 i = 0;
    node *n = NULL;

    for (i=0; i<NODES_COUNT; i++)
    {
        n = (node *) MemAlloc(sizeof(node));

        n->value = (UINT8) i;
        n->freq = 0;

        n->father = NULL;
        n->left = NULL;
        n->right = NULL;

        if (readBits(1)==1)
        {
            n->active = TRUE;
        }
        else
        {
            n->active = FALSE;
        }

        nodeTable[i] = n;
    }
}

static void
readNodeTable(void)
{
    INT32 i = 0;
    
    for (i=0; i<NODES_COUNT; i++)
    {
        if (nodeTable[i]->active)
        {
            nodeTable[i]->bitCount = readBits(depth);
            nodeTable[i]->bitValue = readBits(nodeTable[i]->bitCount);
        }
    }
}

static void
nodeTableRebuildTree(void)
{
    UINT32 i = 0;
    INT32 bitCount = 0;
    UINT32 bitValue = 0;
    node *n = NULL;

    unpackNodeTable = (node *) MemAlloc(sizeof(node));

    unpackNodeTable->freq = 0;
    unpackNodeTable->left = NULL;
    unpackNodeTable->right = NULL;
    
    for (i=0; i<NODES_COUNT; i++)
    {
        bitCount = nodeTable[i]->bitCount;

        if (bitCount>0)
        {
            bitValue = nodeTable[i]->bitValue;

            n = unpackNodeTable;

            do
            {
                bitCount--;

                if (((bitValue>>bitCount)&1)==0)
                {
                    if (n->left==NULL)
                    {
                        n->left = (node *) MemAlloc(sizeof(node));

                        n->left->freq = 0;
                        n->left->left = NULL;
                        n->left->right = NULL;
                    }

                    n = n->left;
                }
                else
                {
                    if (n->right==NULL)
                    {
                        n->right = (node *) MemAlloc(sizeof(node));

                        n->right->freq = 0;
                        n->right->left = NULL;
                        n->right->right = NULL;
                    }

                    n = n->right;
                }
            } while (bitCount>0);

            n->value = (UINT8) i;

            if ((n->value&1)==0)
            {
                n->freq = 1;
            }
            else
            {
                n->freq = 2;
            }
        }
    }
}

static void
readPacked(UINT8 *dst, INT32 dstSize)
{
    INT32 dstOffset = 0;
    node *n = NULL;

    while (dstOffset<dstSize)
    {
        n = unpackNodeTable;

        while (n->freq==0)
        {
            if (readBits(1)==0)
            {
                n = n->left;
            }
            else
            {
                n = n->right;
            }
        }
        
        dst[dstOffset] = n->value;

        dstOffset++;
    }
}

/*---------------------------------------------------------------------------- PACK */
void 
FilePack(UINT8 *src, INT32 srcSize, UINT8 **dst, INT32 *dstSize)
{
    checksum = computeChecksum(src, srcSize);
    
    nodeTableClear();

    nodeTableComputeFreq(src, srcSize);

    nodeTableSort();

    nodeTableCount = nodeTableGetCount();
    
    nodeTableBuildTree();

    nodeTableBuildBits();

    *dst = (UINT8 *) MemAlloc(srcSize);
    
    failed = FALSE;
    
    writeBitsInit(*dst, srcSize);

    writeBits(PACK_SIGNATURE[0], 8);
    writeBits(PACK_SIGNATURE[1], 8);
    writeBits(PACK_SIGNATURE[2], 8);
    writeBits(PACK_SIGNATURE[3], 8);

    writeBits(0, 32); /* checksum */
    writeBits(0, 32); /* size */

    writeTableCount();

    nodeTableComputeDepth();
        
    writeBits(depth, 8);

    writeTableConvert();

    writePacked(src, srcSize);

    writeBits(0, 8);

    *dstSize = bitsOffset>>3;
    
    bitsOffset = 4*8;
    writeBits(checksum, 32);
    writeBits(srcSize, 32);
}

INT32
FileGetUnpackedSize(UINT8 *src)
{
    readBitsInit(src);

    if (readBits(8)!=(UINT8) PACK_SIGNATURE[0]) return 0;
    if (readBits(8)!=(UINT8) PACK_SIGNATURE[1]) return 0;
    if (readBits(8)!=(UINT8) PACK_SIGNATURE[2]) return 0;
    if (readBits(8)!=(UINT8) PACK_SIGNATURE[3]) return 0;

    readBits(32); /* checksum */
    return readBits(32); /* size */
}

void
FileUnpack(UINT8 *src, UINT8 *dst)
{
    INT32 dstSize = 0;
    
    readBitsInit(src);

    if (readBits(8)!=(UINT8) PACK_SIGNATURE[0]) return;
    if (readBits(8)!=(UINT8) PACK_SIGNATURE[1]) return;
    if (readBits(8)!=(UINT8) PACK_SIGNATURE[2]) return;
    if (readBits(8)!=(UINT8) PACK_SIGNATURE[3]) return;

    checksum = readBits(32);
    dstSize = readBits(32);

    readNodeTableInit();
    
    depth = readBits(8);

    readNodeTable();

    nodeTableRebuildTree();

    readPacked(dst, dstSize);
}