/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * mem.c - Memory                                                             *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/memblock.h"
#include "system/mem.h"

/*---------------------------------------------------------------------------- LOCAL ENUM */
typedef enum
{
    MemLastUndef,
    MemLastAdd,
    MemLastFree
} MemLast;

/*---------------------------------------------------------------------------- LOCAL STRUCT */
typedef struct Mem
{
    void *buffer;
    INT32 size;

    struct Mem *next;
    struct Mem *prev;
} Mem;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static Mem *tree = NULL;
static Mem *lastUsed = NULL;
static MemLast last = MemLastUndef;
static INT32 memUsed = 0;

/*---------------------------------------------------------------------------- ADD NODE */
static void 
addNode(void *buffer, INT32 size)
{
    Mem *add = NULL;
    Mem *node = NULL;

    add = (Mem *) MemBlockAlloc(sizeof(Mem));

    add->next = NULL;
    add->prev = NULL;
    add->buffer = buffer;
    add->size = size;

    if (last==MemLastAdd)
    {
        lastUsed->next = add;
        add->prev = lastUsed;
    }
    else if (tree==NULL)
    {
        tree = add;
    }
    else
    {
        node = tree;

        while (node->next!=NULL)
        {
            node = node->next;
        }
        
        add->prev = node;
        node->next = add;
    }

    memUsed += size;

    last = MemLastAdd;
    lastUsed = add;
}

/*---------------------------------------------------------------------------- FREE NODE */
static void 
freeNode(void *buffer)
{
    Mem *node = NULL;
    Mem *prevNode = NULL;
    
    last = MemLastFree;
        
    if (tree==NULL)
	{
		return;
	}

    if (tree->next==NULL)
	{
        if (tree->buffer==buffer)
		{
			memUsed -= tree->size;

            MemBlockFree(buffer);
			MemBlockFree(tree);
			tree = NULL;
		}

        return;
	}

    node = tree;

	while (node->buffer!=buffer)
	{
		prevNode = node;
		node = node->next;
	}
	
	if (prevNode==NULL)
	{
		tree = node->next;                
	}
	else
	{
		prevNode->next = node->next;
	}

    memUsed -= node->size;

    MemBlockFree(buffer);
    MemBlockFree(node);
}

/*---------------------------------------------------------------------------- MEM INIT */
void
MemInit(void)
{
    if (tree!=NULL)
    {
        return;
    }

    tree = NULL;
    last = MemLastUndef;

    memUsed = 0;
}

/*---------------------------------------------------------------------------- MEM FREE ALL */
void
MemFreeAll(void)
{
    Mem *next = NULL;
    Mem *node = NULL;

    node = tree;

    while (node!=NULL)
    {
        next = node->next;

        MemBlockFree(node->buffer);

        memUsed -= node->size;

        MemBlockFree(node);

        node = next;
    }

    tree = NULL;
    last = MemLastUndef;

    memUsed = 0;
}

/*---------------------------------------------------------------------------- MEM ALLOC */
void *
MemAlloc(const INT32 size)
{
    void *buffer = NULL;

    buffer = MemBlockAlloc(size);

    if (buffer!=NULL)
    {
        memset(buffer, 0, size);

        addNode(buffer, size);
    }

    return buffer;
}

/*---------------------------------------------------------------------------- MEM FREE */
void 
MemFree(void *buffer)
{
    if (buffer!=NULL)
    {
        freeNode(buffer);
    }
}