/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * texjpg.c - JPEG Texture                                                    *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/win32/lib/jpeglib/jpeglib.h"
#include "platform/os/console.h"
#include <setjmp.h>
#include "system/file.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "texture/texjpg.h"

/*---------------------------------------------------------------------------- LOCAL STRUCT */
struct my_error_mgr
{
    struct jpeg_error_mgr pub;

    jmp_buf setjmp_buffer;
};

typedef struct my_error_mgr *my_error_ptr;

/*---------------------------------------------------------------------------- LOCAL FUNCTIONS */
static void
my_error_exit (j_common_ptr cinfo)
{
    my_error_ptr myerror = (my_error_ptr) cinfo->err;

    (*cinfo->err->output_message) (cinfo);

    longjmp(myerror->setjmp_buffer, 1);
}

static void 
put_scanline_someplace(void *buffer, void *scanline, int rowSize, int whichLine)
{
	UINT8 *src = NULL;
    UINT8 *dst = NULL;
    INT32 i = 0;

    src = (UINT8 *) scanline;
    dst = (UINT8 *) buffer + whichLine*((rowSize)/3)*sizeof(INT32);

	for (i=0; i<rowSize; i+=3)
	{
		*dst++ = *src++;
		*dst++ = *src++;
		*dst++ = *src++;
		*dst++ = 255;
	}
}

/*---------------------------------------------------------------------------- LOAD */
RESULT
JpgLoad(Tex *tex)
{
    struct jpeg_decompress_struct cinfo;  
    struct my_error_mgr jerr;

    JSAMPARRAY buffer;
    int row_stride;

    INT32 i = 0;
    File file;

    FILE *tempFile = NULL;    

    if (tex==NULL)
    {
        return ERR_INVALID_PTR;
    }

    tempFile = tmpfile();
    
    if (tempFile==NULL)
    {
        ConsoleAdd("Error : can not open temporary file (JPEG) !");
        
        return ERR;
    }

    if (FileLoad(&file, tex->fileName)!=OK)
    {
        ConsoleAdd("Error : can not open %s !", tex->fileName);
        
        return ERR_OPEN_FILE;
    }

    fwrite(file.buffer, file.size, 1, tempFile);

    fseek(tempFile, 0, SEEK_SET);
            
    FileFree(&file);

    cinfo.err = jpeg_std_error(&jerr.pub);
    jerr.pub.error_exit = my_error_exit;

    if (setjmp(jerr.setjmp_buffer))
    {
        jpeg_destroy_decompress(&cinfo);

        fclose(tempFile);

        ConsoleAdd("Error while decompressing JPEG image with %s !", tex->fileName);
        
        return ERR;
    }
  
    jpeg_create_decompress(&cinfo);

    jpeg_stdio_src(&cinfo, tempFile);

    jpeg_read_header(&cinfo, TRUE);
  
    jpeg_start_decompress(&cinfo);
  
    row_stride = cinfo.output_width*cinfo.output_components;
  
    tex->width = cinfo.output_width;
    tex->height = cinfo.output_height;
    tex->depth = 32;
    tex->buffer = MemAlloc(tex->width*tex->height*sizeof(INT32));
    
    if (tex->buffer==NULL)
    {
        return ERR_NOT_ENOUGH_MEM;
    }
    
    i = 0;
    
    buffer = (*cinfo.mem->alloc_sarray)	((j_common_ptr) &cinfo, JPOOL_IMAGE, row_stride, 1);
    
    while (cinfo.output_scanline<cinfo.output_height)
    {
        jpeg_read_scanlines(&cinfo, buffer, 1);

        i++;
        
        put_scanline_someplace(tex->buffer, *buffer, row_stride, cinfo.output_height-i);
    }

    jpeg_finish_decompress(&cinfo);

    jpeg_destroy_decompress(&cinfo);

    fclose(tempFile);
    
    return OK;
}
