/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * texrgb.c - Texture RGB Converter                                           *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "texture/texrgb.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define LOG_TXT_RGB_SET		"Texture RGB format set to %s"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static RGBType rgbType = ARGB8888;
static UINT8 *buffer = NULL;
static UINT8 *src = NULL;
static UINT8 *dst = NULL;
static INT32 width = 0;
static INT32 height = 0;
static INT32 x = 0;
static INT32 y = 0;
static UINT8 a = 0;
static UINT8 r = 0;
static UINT8 g = 0;
static UINT8 b = 0;    
static UINT32 rgb = 0;
        
/*---------------------------------------------------------------------------- ABGR 8888 */
static void
toABGR8888(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT32));
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            a = *(src++);
			
            *(dst++) = b;
			*(dst++) = g;
			*(dst++) = r;
			*(dst++) = a;
        }
    }
}

/*---------------------------------------------------------------------------- RGB 888 */
static void
toRGB888(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT8)*3);
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            src++;
			
            *(dst++) = r;
			*(dst++) = g;
			*(dst++) = b;
        }
    }
}

/*---------------------------------------------------------------------------- BGR 888 */
static void
toBGR888(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT8)*3);
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            src++;
			
            *(dst++) = b;
			*(dst++) = g;
			*(dst++) = r;
        }
    }
}

/*---------------------------------------------------------------------------- RGB 565 */
static void
toRGB565(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT16));
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            src++;
			
            rgb = (UINT16) ((b>>3)<<11)+((g>>2)<<5)+(r>>3);

            *(dst++) = (UINT8) (rgb&255);
            *(dst++) = (UINT8) (rgb>>8);
        }
    }
}
    
/*---------------------------------------------------------------------------- BGR 565 */
static void
toBGR565(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT16));
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            src++;
			
            rgb = (UINT16) ((r>>3)<<11)+((g>>2)<<5)+(b>>3);

            *(dst++) = (UINT8) (rgb&255);
            *(dst++) = (UINT8) (rgb>>8);
        }
    }
}
    
/*---------------------------------------------------------------------------- ARGB 555 */
static void
toARGB555(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT16));
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            src++;
			
            rgb = (UINT16) ((b>>3)<<10)+((g>>3)<<5)+(r>>3);

            *(dst++) = (UINT8) (rgb&255);
            *(dst++) = (UINT8) (rgb>>8);
        }
    }
}
    
/*---------------------------------------------------------------------------- ABGR 555 */
static void
toABGR555(void)
{    
    buffer = MemAlloc(width*height*sizeof(UINT16));
    dst = buffer;

    for (y=0; y<height; y++)
    {
        for (x=0; x<width; x++)
        {
            r = *(src++);
			g = *(src++);
            b = *(src++);
            src++;
			
            rgb = (UINT16) ((r>>3)<<10)+((g>>3)<<5)+(b>>3);

            *(dst++) = (UINT8) (rgb&255);
            *(dst++) = (UINT8) (rgb>>8);
        }
    }
}
    
/*---------------------------------------------------------------------------- RGB TYPE */
void 
TexRGBSet(RGBType type)
{
    rgbType = type;

	switch(type)
	{
		case ARGB8888:
			ConsoleAdd(LOG_TXT_RGB_SET, "ARGB8888");
			break;
        
		case ABGR8888:
			ConsoleAdd(LOG_TXT_RGB_SET, "ABGR8888");
			break;
	
		case RGB888:
			ConsoleAdd(LOG_TXT_RGB_SET, "RGB888");
			break;
	
		case BGR888:
			ConsoleAdd(LOG_TXT_RGB_SET, "BGR888");
			break;
	
		case RGB565:
			ConsoleAdd(LOG_TXT_RGB_SET, "RGB565");
			break;
    
		case BGR565:
			ConsoleAdd(LOG_TXT_RGB_SET, "BGR565");
			break;
    
		case ARGB555:
			ConsoleAdd(LOG_TXT_RGB_SET, "ARGB555");
			break;
    
		case ABGR555:
			ConsoleAdd(LOG_TXT_RGB_SET, "ABGR555");
			break;
	}
}

/*---------------------------------------------------------------------------- CONVERTER */
RESULT 
TexRGBConvert(Tex *tex)
{
    if (tex==NULL)
    {
        return ERR_INVALID_PTR;
    }

    src = (UINT8 *) tex->buffer;

    width = tex->width;
    height = tex->height;

    switch (rgbType)
    {
    case ARGB8888:
        return OK;
        
    case ABGR8888:
        toABGR8888();
        tex->depth = 32;
        break;
	
	case RGB888:
        toRGB888();
        tex->depth = 24;
        break;
	
	case BGR888:
        toBGR888();
        tex->depth = 24;
        break;
	
	case RGB565:
        toRGB565();
        tex->depth = 16;
        break;
    
	case BGR565:
        toBGR565();
        tex->depth = 16;
        break;
    
	case ARGB555:
        toARGB555();
        tex->depth = 16;
        break;
    
	case ABGR555:
        toABGR555();
        tex->depth = 16;
        break;
    }

    if (buffer==NULL)
    {
        return ERR;
    }
    
    MemFree(tex->buffer);
    tex->buffer = (INT32 *) buffer;
    
    return OK;
}