/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * texrgb.c - Texture RGB Converter                                           *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/console.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "texture/texrgb.h"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static UINT32 *buffer = NULL;
static UINT32 *src = NULL;
static INT32 width = 0;
static INT32 height = 0;
static UINT32 rgb = 0;
static INT32 max = 0;
        
/*---------------------------------------------------------------------------- GET POWER 2 */
static INT32
getPower2(INT32 a)
{
	INT32 bit = 0;
	INT32 lastBit = 0;
	INT32 bitCmp = 0;
	INT32 i = 0;

	for (i=0; i<32; i++)
	{
		bitCmp = 1<<i;

		if (a&bitCmp)
		{
			bit++;
			lastBit = bitCmp;
		}
	}

	if (bit!=1)
	{
		a = lastBit<<1;
	}

	if (a>max)
	{
		a = max;
	}

	return a;
}

/*---------------------------------------------------------------------------- ABGR 8888 */
static void
scale(INT32 newX, INT32 newY)
{    
    UINT32 *dst = NULL;

	INT32 x = 0;
	INT32 y = 0;

	FLOAT32 ratioX = 0.0f;
	FLOAT32 ratioY = 0.0f;

	ratioX = (FLOAT32) width/(FLOAT32) newX;
	ratioY = (FLOAT32) height/(FLOAT32) newY;

	buffer = MemAlloc(newX*newY*sizeof(INT32));
    dst = buffer;

    for (y=0; y<newY; y++)
    {
        for (x=0; x<newX; x++)
        {
            rgb = src[(INT32) ((FLOAT32) y*ratioY)*width+(INT32) ((FLOAT32) x*ratioX)];

			*(dst++) = rgb;
        }
    }
}

/*---------------------------------------------------------------------------- SCALER */
RESULT 
TexScalePower2(Tex *tex)
{
 	INT32 newX = 0;
	INT32 newY = 0;
	
	if (tex==NULL)
    {
        return ERR_INVALID_PTR;
    }

    src = (UINT32 *) tex->buffer;

    width = tex->width;
    height = tex->height;

    newX = getPower2(width);
	newY = getPower2(height);

	if (newX>newY)
	{
		newY = newX;
	}
	else
	{
		if (newY>newX)
		{
			newX = newY;
		}
	}

	if (newX!=width || newY!=height)
	{
		scale(newX, newY);

		if (buffer==NULL)
		{
			return ERR;
		}
	
		tex->width = newX;
		tex->height = newY;

		MemFree(tex->buffer);
		tex->buffer = (INT32 *) buffer;
	}
    
    return OK;
}

/*---------------------------------------------------------------------------- SET MAX POWER 2 VALUE */
void
TexScaleSetMaxPower2(INT32 maxPower2)
{
	max = maxPower2;

	ConsoleAdd("Max texture size set to %d", max);
}