/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <config.h>
/******************************************************************************
 * textga.c - TGA Texture                                                     *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "platform/os/fileio.h"
#include "system/mem.h"
#include "texture/tex.h"
#include "system/file.h"
#include "texture/textga.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define TGA_NO_IMAGE_DATA                           0
#define TGA_UNCOMPRESSED_COLOR_MAPPED_IMAGES        1
#define TGA_UNCOMPRESSED_RGB_IMAGES                 2
#define TGA_UNCOMPRESSED_BLACK_WHITE_IMAGES         3
#define TGA_RLE_COLOR_MAPPED_IMAGES                 9
#define TGA_RLE_RGB_IMAGES                          10
#define TGA_COMPRESSED_BLACK_WHITE_IMAGES           11

#define TGA_COPYRIGHT_WIDTH							45
#define TGA_COPYRIGHT_HEIGHT						19
#define TGA_COPYRIGHT_DEPTH							8
#define TGA_COPYRIGHT_BORDER_WIDTH					4
#define TGA_COPYRIGHT_BORDER_HEIGHT					4
#define TGA_COPYRIGHT_PIXEL_COLOR_R					255
#define TGA_COPYRIGHT_PIXEL_COLOR_G					255
#define TGA_COPYRIGHT_PIXEL_COLOR_B					230
                    
/*---------------------------------------------------------------------------- LOCAL STRUCT */
typedef struct Desc
{
    UINT8 nbCharsIdentField;
    UINT8 colorMapType;
    UINT8 imgTypeCode;
    UINT8 colorMapOriginLo;
    UINT8 colorMapOriginHi;
    UINT8 colorMapLenghtLo;
    UINT8 colorMapLenghtHi;
    UINT8 colorMapEntrySize;
    UINT8 originXLo;
    UINT8 originXHi;
    UINT8 originYLo;
    UINT8 originYHi;
    UINT8 widthLo;
    UINT8 widthHi;
    UINT8 heightLo;
    UINT8 heightHi;
    UINT8 depth;
    UINT8 imgDesc;
} Desc;

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static Tex *tga = NULL;
static UINT8 *buffer = NULL;
static UINT8 *src = NULL;
static UINT8 *dst = NULL;
static Desc *desc = NULL;

/*---------------------------------------------------------------------------- LOCAL COPYRIGHT BITMAP */
static UINT8 raw[] =
{
	0, 1, 1, 0, 0, 0, 1, 24, 0, 0, 0, 0, 45, 0, 19, 0, 8, 0, 252, 254, 
    252, 0, 0, 0, 168, 170, 168, 84, 85, 84, 255, 0, 255, 153, 0, 153, 90, 135, 11, 94, 
    85, 0, 134, 196, 255, 78, 132, 194, 201, 121, 136, 153, 12, 106, 106, 27, 224, 72, 18, 153, 
    201, 179, 121, 153, 83, 9, 0, 128, 255, 0, 77, 153, 0, 255, 255, 0, 153, 153, 255, 255, 
    0, 153, 153, 0, 71, 71, 255, 43, 43, 153, 60, 102, 150, 38, 64, 94, 178, 102, 255, 107, 
    61, 153, 20, 255, 192, 12, 153, 115, 198, 255, 112, 119, 153, 67, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 128, 
    128, 128, 128, 128, 128, 128, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 
    1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 
    0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 
    1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 
    0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 
    0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 
    1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 
    1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 
    0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 
    1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 
    1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 1, 1, 
    0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 
    0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 
    0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 
    0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 
    0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 
    1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 
    1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 
    1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 
    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
    0, 239, 1, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 0, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    0, 0, 0, 0, 0, 0, 0, 80, 97, 105, 110, 116, 32, 83, 104, 111, 112, 32, 80, 114, 
    111, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 32, 
    32, 32, 32, 32, 32, 32, 32, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 105, 6, 0, 0, 
    0, 0, 0, 0, 84, 82, 85, 69, 86, 73, 83, 73, 79, 78, 45, 88, 70, 73, 76, 69, 
    46, 0
};

/*---------------------------------------------------------------------------- ADD COPYRIGHT */
static void
addCopyright(Tex *tex)
{
	INT32 posX = 0;
	INT32 posY = 0;
	INT32 x = 0;
	INT32 y = 0;
	INT32 offset = 0;
	UINT8 *buffer = NULL;

	posX = tex->width-TGA_COPYRIGHT_WIDTH-TGA_COPYRIGHT_BORDER_WIDTH;
	posY = TGA_COPYRIGHT_BORDER_HEIGHT;
	buffer = (UINT8 *) tex->buffer;
	
	for (y=0; y<TGA_COPYRIGHT_HEIGHT; y++)
	{
		for (x=0; x<TGA_COPYRIGHT_WIDTH; x++)
		{
			if (raw[18+768+(y*TGA_COPYRIGHT_WIDTH)+x]>0)
			{
				offset = ((posY+y)*tex->width+posX+x)*(tex->depth>>3);
				
				buffer[offset] = TGA_COPYRIGHT_PIXEL_COLOR_B;
				buffer[offset+1] = TGA_COPYRIGHT_PIXEL_COLOR_G;
				buffer[offset+2] = TGA_COPYRIGHT_PIXEL_COLOR_R;
			}
		}
	}
}

/*---------------------------------------------------------------------------- FLIP */
static RESULT
tgaFlip32(void)
{
	INT32 x = 0;
	INT32 y = 0;
	INT32 yScanline = 0;
	UINT32 *buffer32 = NULL;
	UINT32 texel = 0;
    
	buffer32 = (UINT32 *) tga->buffer;

	for (y=0; y<(tga->height>>1); y++)
	{
		for (x=0; x<tga->width; x++)
		{
			yScanline = tga->width*y;

			texel = buffer32[yScanline+x];

			buffer32[yScanline+x] = buffer32[(tga->width*(tga->height-1))-yScanline+x];

			buffer32[(tga->width*(tga->height-1))-yScanline+x] = texel;
		}
	}

	return OK;
}

/*---------------------------------------------------------------------------- RAW8 */
static RESULT
tgaRaw8(void)
{
    INT32 i = 0;
	UINT8 *pal = NULL;

    if (tga==NULL)
    {
        return ERR;
    }

    pal = &buffer[sizeof(Desc)];
	src = &buffer[sizeof(Desc)+3*256];
    dst = (UINT8 *)tga->buffer;
    
    for (i=0; i<tga->width*tga->height; i++)
	{
		*dst++ = pal[*src*3+2];
		*dst++ = pal[*src*3+1];
		*dst++ = pal[*src*3];
        *dst++ = 255;

        src++;
    }

    return OK;
}

/*---------------------------------------------------------------------------- RLE8 */
static RESULT
tgaRle8(void)
{
    UINT8 r = 0;
	UINT8 g = 0;
	UINT8 b = 0;
	INT32 i = 0;
    UINT8 j = 0;
	UINT8 *pal = NULL;
	
    if (tga==NULL)
    {
        return ERR;
    }

    pal = &buffer[sizeof(Desc)];
	src = &buffer[sizeof(Desc)+3*256];
    dst = (UINT8 *) tga->buffer;

    i = 0;

    while (i<tga->width*tga->height)
    {
        j = *src++;

        if (j>127)
        {
            j -= 127;
            i += j;

			r = pal[*src*3+2];
			g = pal[*src*3+1];
			b = pal[*src*3];

			src++;

            while (j>0)
            {
                *dst++ = r;
                *dst++ = g;
                *dst++ = b;
                *dst++ = 255;

                j--;
            }
        }
        else
        {
            j++;
            i += j;

            while (j>0)
            {
              	*dst++ = pal[*src*3+2];
				*dst++ = pal[*src*3+1];
				*dst++ = pal[*src*3];
				*dst++ = 255;
            
                src++;
				
				j--;
            }
        }   
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- RAW16 */
static RESULT
tgaRaw16(void)
{
    UINT16 a = 0;
	INT32 i = 0;

    if (tga==NULL)
    {
        return ERR;
    }

    src = &buffer[sizeof(Desc)];
	dst = (UINT8 *) tga->buffer;
    
    for (i=0; i<tga->width*tga->height; i++)
	{
        a = (UINT16) ((*(src+1)<<8)+*src);

		*dst++ = (UINT8) (((a>>10)&31)<<3);
        *dst++ = (UINT8) (((a>>5)&31)<<3);
        *dst++ = (UINT8) ((a&31)<<3);
        *dst++ = 255;

        src += 2;
    }

    return OK;
}

/*---------------------------------------------------------------------------- RLE16 */
static RESULT
tgaRle16(void)
{
    UINT16 a = 0;
	UINT8 r = 0;
	UINT8 g = 0;
	UINT8 b = 0;
	INT32 i = 0;
    UINT8 j = 0;

    if (tga==NULL)
    {
        return ERR;
    }

    src = &buffer[sizeof(Desc)];
	dst = (UINT8 *) tga->buffer;
    
	i = 0;

    while (i<tga->width*tga->height)
    {
        j = *src++;

        if (j>127)
        {
            j -= 127;
            i += j;

            a = (UINT16) ((*(src+1)<<8)+*src);

			r = (UINT8) (((a>>10)&31)<<3);
			g = (UINT8) (((a>>5)&31)<<3);
			b = (UINT8) ((a&31)<<3);
        
			while (j>0)
            {
                *dst++ = r;
                *dst++ = g;
                *dst++ = b;
                *dst++ = 255;

                j--;
            }

            src += 2;
        }
        else
        {
            j++;

            i += j;

            while (j-->0)
            {
                a = (UINT16) ((*(src+1)<<8)+*src);

				*dst++ = (UINT8) (((a>>10)&31)<<3);
				*dst++ = (UINT8) (((a>>5)&31)<<3);
				*dst++ = (UINT8) ((a&31)<<3);
				*dst++ = 255;

				src += 2;
            }
        }   
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- RAW24 */
static RESULT
tgaRaw24(void)
{
    INT32 x = 0;
    INT32 y = 0;

    if (tga==NULL)
    {
        return ERR;
    }

    src = &buffer[sizeof(Desc)];
	dst = (UINT8 *) tga->buffer;
    
    dst += (tga->height-1)*(tga->width)*sizeof(INT32);

    for (y=0; y<tga->height; y++)
    {
        for (x=0; x<tga->width; x++)
        {
            *dst++ = *(src+2);
            *dst++ = *(src+1);
            *dst++ = *src;
            *dst++ = 255;

            src += 3;
        }

        dst -= tga->width*2*sizeof(INT32);
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- RLE24 */
static RESULT
tgaRle24(void)
{
    INT32 i = 0;
    UINT8 j = 0;

    if (tga==NULL)
    {
        return ERR;
    }

    src = &buffer[sizeof(Desc)];
	dst = (UINT8 *) tga->buffer;
    
	i = 0;

    while (i<tga->width*tga->height)
    {
        j = *src++;

        if (j>127)
        {
            j -= 127;
            i += j;

            while (j>0)
            {
                *dst++ = *(src+2);
                *dst++ = *(src+1);
                *dst++ = *src;
                *dst++ = 255;

                j--;
            }

            src += 3;
        }
        else
        {
            j++;

            i += j;

            while (j>0)
            {
                *dst++ = *(src+2);
                *dst++ = *(src+1);
                *dst++ = *src;
                *dst++ = 255;
            
                src += 3;

                j--;
            }
        }   
    }

	return OK;
}

/*---------------------------------------------------------------------------- RAW32 */
static RESULT
tgaRaw32(void)
{
    INT32 x = 0;
    INT32 y = 0;

    if (tga==NULL)
    {
        return ERR;
    }

    src = &buffer[sizeof(Desc)];
	dst = (UINT8 *) tga->buffer;
    
    dst += (tga->height-1)*(tga->width)*sizeof(INT32);

    for (y=0; y<tga->height; y++)
    {
        for (x=0; x<tga->width; x++)
        {
            *dst++ = *(src+2);
            *dst++ = *(src+1);
            *dst++ = *src;
            *dst++ = *(src+3);

            src += 4;
        }

        dst -= tga->width*2*sizeof(INT32);
    }
    
    return OK;
}

/*---------------------------------------------------------------------------- RLE32 */
static RESULT
tgaRle32(void)
{
    INT32 i = 0;
    UINT8 j = 0;

    if (tga==NULL)
    {
        return ERR;
    }

    src = &buffer[sizeof(Desc)];
	dst = (UINT8 *) tga->buffer;
    
	i = 0;

    while (i<tga->width*tga->height)
    {
        j = *src++;

        if (j>127)
        {
            j -= 127;
            i += j;

            while (j>0)
            {
                *dst++ = *(src+2);
                *dst++ = *(src+1);
                *dst++ = *src;
                *dst++ = *(src+3);

                j--;
            }

            src += 4;
        }
        else
        {
            j++;

            i += j;

            while (j>0)
            {
                *dst++ = *(src+2);
                *dst++ = *(src+1);
                *dst++ = *src;
                *dst++ = *(src+3);
            
                src += 4;

                j--;
            }
        }   
    }

	return OK;
}

/*---------------------------------------------------------------------------- LOAD */
RESULT
TgaLoad(Tex *tex)
{
    BOOL rle = 0;
	INT32 depth = 0;
    
    File file;

    if (tex==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (FileLoad(&file, tex->fileName)!=OK)
    {
        return ERR_OPEN_FILE;
    }

    buffer = (UINT8 *) file.buffer;

    tga = tex;
    desc = (Desc *) file.buffer;
    
    depth = desc->depth;
		
	tga->width = (desc->widthHi<<8)+desc->widthLo;
    tga->height = (desc->heightHi<<8)+desc->heightLo;
	tga->depth = desc->depth;
    
    tga->buffer = MemAlloc(tga->width*tga->height*sizeof(INT32));
    
    if (tga->buffer==NULL)
    {
        return ERR_NOT_ENOUGH_MEM;
    }
    
    if (desc->imgTypeCode==TGA_RLE_RGB_IMAGES || (desc->imgTypeCode!=TGA_UNCOMPRESSED_COLOR_MAPPED_IMAGES && depth==8))
    {
        rle = TRUE;
    }
    else
    {
        rle = FALSE;
    }

    switch (depth)
    {
	case 8:	    if (rle) {tgaRle8();} else {tgaRaw8();} break;
	case 16:	if (rle) {tgaRle16();} else {tgaRaw16();} break;
	case 24:	if (rle) {tgaRle24();} else {tgaRaw24(); tgaFlip32();} break;
	case 32:    if (rle) {tgaRle32();} else {tgaRaw32(); tgaFlip32();} break;

    default:
        buffer = NULL;
        FileFree(&file);

        return ERR_TGA_BAD_DEPTH;
    }

    if ((desc->imgDesc&32)!=0)
    {
	    tgaFlip32();
    }

	tga = NULL;
    buffer = NULL;

    FileFree(&file);
    
    return OK;
}

/*---------------------------------------------------------------------------- SAVE */
RESULT
TgaSave(Tex *tex)
{
    File file;
    
    if (tex==NULL || tex->buffer==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (tex->depth!=24)
    {
        return ERR_TGA_BAD_DEPTH;
    }

	addCopyright(tex);

    file.size = sizeof(Desc)+(tex->width*tex->height*3);
    
    file.buffer = (INT8 *) MemAlloc(file.size);

    strcpy(file.fileName, tex->fileName);
    
    desc = (Desc *) file.buffer;
    
    desc->imgTypeCode = TGA_UNCOMPRESSED_RGB_IMAGES;
    desc->widthLo = (UINT8) (tex->width&255);
    desc->widthHi = (UINT8) (tex->width>>8);
    desc->heightLo = (UINT8) (tex->height&255);
    desc->heightHi = (UINT8) (tex->height>>8);
    desc->depth = 24;

    memcpy(&file.buffer[sizeof(Desc)], tex->buffer, tex->height*tex->width*3);
    
    FileWrite(&file);

    SAFE_FREE(file.buffer);

    return OK;
}

/*---------------------------------------------------------------------------- FLIP */
RESULT
TgaFlip(Tex *tex)
{
    if (tex==NULL)
    {
        return ERR_INVALID_PTR;
    }

    if (tex->buffer==NULL)
    {
        return ERR_INVALID_PTR;
    }

    tga = tex;
    tgaFlip32();

    return OK;
}