
// File Packer version 1.1
// All code by Arnaud STORQ a.k.a. "No Recess / Prune"
// More infos on http://norecess.planet-d.net
// Free to use, do what you want with.. (but please credits me)

#ifndef __ARCHIVER_HEADER__
#define __ARCHIVER_HEADER__

#include <windows.h>
#include <stdio.h>

enum VERSIONINFO
{
	ARCHIVE_VERSION = 0,
	ARCHIVE_SUBVERSION = 1
};

#define MAX_STACK 16384
#define ARCHIVE_VERSION_CONTENT 1
#define ARCHIVE_SUBVERSION_CONTENT 0

enum ARCHIVERESULT
{
	ARCHIVE_OK = 0,
	ARCHIVE_ERROR = 1,
	ARCHIVE_INVALIDPARAM = 2,
	ARCHIVE_FILEDOESNOTEXIST = 3,
	ARCHIVE_NOTENOUGHMEM = 4,
	ARCHIVE_IOERROR = 5,
	ARCHIVE_UNKNOWNPAKTYPE = 6
};

enum STACKTYPE
{
	STACK_ALL = 0,
	STACK_GETRAWDATAS = 1,
	STACK_PACK = 2
};

struct STACKSTRUCT
{
	void *p;
	STACKTYPE StackType;
};

struct ARCHIVEINFO
{
	DWORD Codec;
	DWORD RawSize;
	DWORD PackedSize;
	char szFilename[256];
};

struct ARCHIVENODE
{
	DWORD dwStat;
	ARCHIVENODE *pFather;
	ARCHIVENODE *pSonLeft;
	ARCHIVENODE *pSonRight;
	BYTE cLetter;
};

struct DICTIONNARYNODE
{
	ARCHIVENODE *pNode;
	DWORD dwNbBits;
	DWORD dwValue;
	BOOL blActive;
};

class CArchiver
{
public:
	CArchiver();
	~CArchiver();

	ARCHIVERESULT AddFile(char *szFilename);
	ARCHIVERESULT RemoveFile(char *szFilename);
	ARCHIVERESULT ExtractFile(char *szFilename, char *szDestFilename);
	ARCHIVERESULT Load(char *szFilename);
	ARCHIVERESULT Save(char *szFilename);
	
	ARCHIVERESULT GetRawDatas(char *szFilename, void **p, DWORD *dwSize);
	
	DWORD GetSize();
	DWORD GetFileCount();
	ARCHIVERESULT GetFileInfo(DWORD dwFile, ARCHIVEINFO *pArchiveInfo);

	DWORD GetVersion(VERSIONINFO VersionInfo);

	ARCHIVERESULT FreeStack(STACKTYPE StackType);

protected:
	ARCHIVERESULT Realloc(DWORD dwNewSize);

	ARCHIVERESULT ToBeDeleted(void *p, STACKTYPE StackType);
	
	ARCHIVERESULT CleanDictionnary();
	ARCHIVERESULT PackDatas(BYTE **p, DWORD dwRawSize, DWORD *dwPackedSize);
	ARCHIVERESULT UnpackDatas(BYTE **p, DWORD dwRawSize, DWORD dwPackedSize);
	
	DWORD GetBits(DWORD dwGetNbBits);
	DWORD Get8BitsMax(DWORD dwGetNbBits);
	void WriteBits(DWORD dwValue, DWORD dwWriteNbBits);
	void Write8BitsMax(DWORD dwValue, DWORD dwWriteNbBits);
	BYTE *pGetBuffer, *pWriteBuffer;
	DWORD dwGetPosBytes, dwGetPosBits;
	DWORD dwWritePosBytes, dwWritePosBits;
	DWORD dwWriteBitsMaxSize;
	BOOL blPakFailed;

	ARCHIVENODE *pUnpackTree;
	
	BYTE *pBuffer;
	DWORD dwBufferSize;

	STACKSTRUCT DeleteStack[MAX_STACK];
	DWORD DeleteStackCounter;

	ARCHIVENODE *Dico[256];
	DWORD DicoNodesCount;

	DICTIONNARYNODE FinalDico[256];
	DWORD FinalDicoNodesCount;
};

#endif