/*
jfyeNG - a Newtek LightWave scene player
Copyright (C) 1998-2003  Arnaud STORQ a.k.a. No Recess/CONDENSE (norecess@planet-d.net)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <commdlg.h>
#include <shlobj.h>
#include <shellapi.h>
#include <config.h>
/******************************************************************************
 * gui.c - Graphic User Interface                                             *
 ******************************************************************************
 * Project      : jfyeNG                                                      *
 * Author       : Arnaud Storq (norecess@planet-d.net)                        *
 ******************************************************************************/
#include "rsc/resource.h"
#include "platform/os/gui.h"
#include "platform/os/settings.h"
#include "platform/os/splash.h"
#include "platform/os/console.h"
#include "platform/os/window.h"
#include "platform/os/fileio.h"
#include "system/mem.h"
#include "system/file.h"
#include "system/filearc.h"
#include "texture/tex.h"
#include "lightwave/lwo/lwo.h"
#include "lightwave/lws/lwsinterp.h"
#include "platform/renderer/lwsdrawlight.h"
#include "platform/renderer/lwsdrawcam.h"
#include "lightwave/lws/lws.h"
#include "lightwave/lws/lwsplaylist.h"
#include "platform/os/main.h"
#include "platform/os/music.h"

/*---------------------------------------------------------------------------- LOCAL DEFINES */
#define DOC_FILE        "jfyeNG.TXT"

/*---------------------------------------------------------------------------- LOCAL VARIABLES */
static BOOL destroy = FALSE;
static HWND hwnd = NULL;
static INT8 sceneDir[STR_DEFAULT_SIZE];
static INT8 audioDir[STR_DEFAULT_SIZE];
static INT8 archiveFileName[STR_DEFAULT_SIZE] = {""};
static BOOL useArchive = FALSE;
static BOOL usePreca = FALSE;

/*---------------------------------------------------------------------------- TO LOWER CONVERSION */
static void 
toLower(INT8 *str)
{
    INT32 i = 0;
    INT32 len = strlen(str);
    
    for (i=0; i<len; i++)
    {
        if (str[i]>='A' && str[i]<='Z')
        {
            str[i] = (INT8) (str[i]+('a'-'A'));
        }
    }   
}

/*---------------------------------------------------------------------------- SCENE PLAYLIST */
static void
playListDoAdds(void)
{
    INT32 count = 0;
    INT32 i = 0;
    INT8 file[STR_DEFAULT_SIZE];
    
    PlayListRemoveAll();
    
    count = SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETCOUNT, 0, 0);
	
	for (i=0; i<count; i++)
	{
		SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETTEXT, i, (LPARAM) file);

        PlayListAdd(file);
	}
}

/*---------------------------------------------------------------------------- ADD SCENE BUTTON */
static void
cbAdd(void)
{
    INT32 i = 0;
	OPENFILENAME openFilename;
    INT8 fileName[STR_DEFAULT_SIZE]= {""};
    INT8 baseFileName[STR_DEFAULT_SIZE]= {""};

	memset(&openFilename, 0, sizeof(OPENFILENAME));
    
	openFilename.lStructSize = sizeof(OPENFILENAME);
	openFilename.hwndOwner = hwnd;
	openFilename.lpstrFilter = "All jfyeNG supported files\0*.LWS;*.LVL;*.JNG\0Newtek LightWave scene (*.LWS)\0*.LWS\0SkyRacer level (*.LVL)\0*.LVL\0jfyeNG archive (*.JNG)\0*.JNG\0\0";
	openFilename.lpstrCustomFilter = (LPSTR) NULL;
	openFilename.nFilterIndex = 1;
	openFilename.lpstrFile = fileName;
	openFilename.nMaxFile = STR_DEFAULT_SIZE;
	openFilename.lpstrFileTitle = "";	
	openFilename.nMaxFileTitle = 0;
	openFilename.lpstrInitialDir = sceneDir;
	openFilename.lpstrTitle = "Add to the playlist..";
	openFilename.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_EXPLORER;
    
	GetOpenFileName(&openFilename);
	
	strcpy(baseFileName, fileName);
    
    i = strlen(fileName);
    
    if (i>0)
	{
		toLower(fileName);
        
        if (strstr(fileName, "lvl")!=NULL)
        {
            SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_ADDSTRING, 0, (LPARAM) &baseFileName[0]);
        }
        else if (strstr(fileName, "jng")!=NULL)
        {
            File file;
            INT8 *buffer = NULL;
            INT32 count = 0;
            INT32 j = 0;
            INT32 size = 0;
            INT8 *name = NULL;
            INT8 baseName[STR_DEFAULT_SIZE]= {""};

            SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_RESETCONTENT, 0, 0);
            
            FileLoad(&file, fileName);
            buffer = file.buffer;
    
            count = *(DWORD*)&buffer[4];
            j = 8;

            for (i=0; i<count; i++)
            {
                j += size;
                size = *(DWORD*)&buffer[j];

		        name = &buffer[j+28-8];
                strcpy(baseName, name);
                
                toLower(name);
                    
                if (strstr(name, "lws")!=NULL)
                {
                    SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_INSERTSTRING, 0, (LPARAM) &baseName[0]);
                }
                else if (strstr(name, "lvl")!=NULL)
                {
                    SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_INSERTSTRING, 0, (LPARAM) &baseName[0]);
                }
                else if (strstr(name, "cam")!=NULL)
                {
                }
                else if (strstr(name, "lwo")!=NULL)
                {
                }
                else if (strstr(name, "tga")!=NULL)
                {
                }
                else if (strstr(name, "jpg")!=NULL)
                {
                }
            }

            FileFree(&file);

            strcpy(archiveFileName, baseFileName);
            useArchive = TRUE;

            EnableWindow(GetDlgItem(hwnd, IDC_ADD), FALSE);
        }
        else
        {
            SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_ADDSTRING, 0, (LPARAM) &baseFileName[0]);
        }

		strcpy(sceneDir, fileName);
		
		i = strlen(sceneDir)-4;

		while (!(sceneDir[i]=='\\' || sceneDir[i]=='/'))
        {
            i--;
        }

		sceneDir[i] = 0;		

		/*
		SettingsSetValue(SETTINGS_KEY_SCENE_PATH, sceneDir);
		*/

        SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_SETCURSEL, SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETCOUNT, 0, 0)-1, 0);
    
        EnableWindow(GetDlgItem(hwnd, IDC_START), TRUE);
        EnableWindow(GetDlgItem(hwnd, IDC_REMOVE), TRUE);
	}
}

/*---------------------------------------------------------------------------- REMOVE SCENE BUTTON */
static void
cbRemove(void)
{
    INT32 i = 0;
    INT32 buffer[64];

    if (useArchive)
    {
        SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_RESETCONTENT, 0, 0);
        
        EnableWindow(GetDlgItem(hwnd, IDC_ADD), TRUE);

        EnableWindow(GetDlgItem(hwnd, IDC_START), FALSE);
        EnableWindow(GetDlgItem(hwnd, IDC_REMOVE), FALSE);

        useArchive = FALSE;

        return;
    }

    i = SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETSELCOUNT, 0, 0);

    if (i>0)
	{
		SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETSELITEMS, 64, (LPARAM) buffer);
    
    	do
		{
			i--;

			SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_DELETESTRING, buffer[i], 0);
		} while (i>0);
	}
	else
	{
		i = SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETCOUNT, 0, 0);

		if (i>0)
		{
			SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_DELETESTRING, i-1, 0);
		}
	}

    if (SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETCOUNT, 0, 0)>0)
    {
        EnableWindow(GetDlgItem(hwnd, IDC_START), TRUE);
        EnableWindow(GetDlgItem(hwnd, IDC_REMOVE), TRUE);
    }
    else
    {
        EnableWindow(GetDlgItem(hwnd, IDC_START), FALSE);
        EnableWindow(GetDlgItem(hwnd, IDC_REMOVE), FALSE);
    }

    SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_SETCURSEL, SendDlgItemMessage(hwnd, IDC_SCENE_LIST, LB_GETCOUNT, 0, 0)-1, 0);
}

/*---------------------------------------------------------------------------- SET LW PATH */
static void
cbSetLWPath(void)
{
	BROWSEINFO browseInfo;
	LPITEMIDLIST itemIDList;
	INT8 lwPath[STR_DEFAULT_SIZE];
  
	memset(&browseInfo, 0, sizeof(BROWSEINFO));
	browseInfo.hwndOwner = hwnd;
	browseInfo.pszDisplayName = lwPath;
	browseInfo.lpszTitle = "Set your Newtek LightWave Content directory";
	browseInfo.ulFlags = BIF_RETURNONLYFSDIRS | BIF_DONTGOBELOWDOMAIN;
	browseInfo.lpfn = NULL;
	itemIDList = SHBrowseForFolder(&browseInfo);

	SHGetPathFromIDList(itemIDList, lwPath);
	
	if (strlen(lwPath)>0)
	{
		/*
		SettingsSetValue(SETTINGS_KEY_LW_PATH, lwPath);
		*/

		SendDlgItemMessage(hwnd, IDC_LW_PATH, WM_SETTEXT, 0, (LPARAM) &lwPath[0]);

        EnableWindow(GetDlgItem(hwnd, IDC_REMOVE_LW_PATH), TRUE);
	}
}

/*---------------------------------------------------------------------------- REMOVE LW PATH BUTTON */
static void
cbRemoveLWPath(void)
{
    SendDlgItemMessage(hwnd, IDC_LW_PATH, WM_SETTEXT, 0, (LPARAM) "");

    /*
	SettingsSetValue(SETTINGS_KEY_LW_PATH, "");
	*/

    EnableWindow(GetDlgItem(hwnd, IDC_REMOVE_LW_PATH), FALSE);
}

/*---------------------------------------------------------------------------- START */
static void
cbStart(void)
{
    playListDoAdds();
    
    ConsoleInit();
    MemInit();

    if (useArchive)
    {
        FileArcInit(archiveFileName);
    }
        
    Main();
}

/*---------------------------------------------------------------------------- GUI CALLBACK */
static BOOL CALLBACK 
DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    INT8 str[STR_DEFAULT_SIZE] = {""};
    
    switch(uMsg)
    {
    case WM_CLOSE:
    case WM_DESTROY:
        /*
		SettingsSetValue(SETTINGS_KEY_RESOLUTION, str);
		*/

        destroy = TRUE;
        break;

    case WM_COMMAND:
        switch(wParam)
        {
		case IDC_START:
            cbStart();
            break;

        case IDC_ADD:
            cbAdd();
            break;

        case IDC_REMOVE:
            cbRemove();
            break;

        case IDC_SET_LW_PATH:
			cbSetLWPath();
			break;

        case IDC_REMOVE_LW_PATH:
            cbRemoveLWPath();
            break;

        }
        break;
    }
    
    hwndDlg;
    uMsg;
    lParam;    
    
    return FALSE;
}

/*---------------------------------------------------------------------------- GUI INIT */
static void
init(void)
{
    INT8 lwPath[STR_DEFAULT_SIZE];
	
    EnableWindow(GetDlgItem(hwnd, IDC_START), FALSE);
    EnableWindow(GetDlgItem(hwnd, IDC_REMOVE), FALSE);
    EnableWindow(GetDlgItem(hwnd, IDC_REMOVE_LW_PATH), FALSE);

    /*
	SettingsGetValue(SETTINGS_KEY_SCENE_PATH, sceneDir);
	SettingsGetValue(SETTINGS_KEY_AUDIO_PATH, audioDir);
	*/
	strcpy(sceneDir, "c:\\");

	SendMessage(GetDlgItem(hwnd, IDC_CORRECT), BM_SETCHECK, BST_CHECKED, 0);
	SendMessage(GetDlgItem(hwnd, IDC_SPEED_2), BM_SETCHECK, BST_CHECKED, 0);

    strcpy(lwPath, "");
	/*
	SettingsGetValue(SETTINGS_KEY_LW_PATH, lwPath);
	*/
	SendDlgItemMessage(hwnd, IDC_LW_PATH, WM_SETTEXT, 0, (LPARAM) &lwPath[0]);
	if (strlen(lwPath)>0)
	{
		EnableWindow(GetDlgItem(hwnd, IDC_REMOVE_LW_PATH), TRUE);
	}

    /*
	SettingsGetValue(SETTINGS_KEY_RESOLUTION, res);
	SendMessage(GetDlgItem(hwnd, IDC_RES), CB_SETCURSEL, atoi(res), 2);

    SettingsGetValue(SETTINGS_KEY_FULLSCREEN, fullScreen);
    if (fullScreen[0]=='1')
    {
        CheckDlgButton(hwnd, IDC_FULLSCREEN, BST_CHECKED);
    }

    SettingsGetValue(SETTINGS_KEY_BILINEAR, bilinear);
    if (bilinear[0]=='1')
    {
        CheckDlgButton(hwnd, IDC_BILINEAR, BST_CHECKED);
    }

    SettingsGetValue(SETTINGS_KEY_PRECA, preca);
    if (preca[0]=='1')
    {
        CheckDlgButton(hwnd, IDC_PRECA, BST_CHECKED);
    }

    if (strcmp(GetRenderName(), "Software")!=0)
	{
		ShowWindow(GetDlgItem(hwnd, IDC_BILINEAR), SW_HIDE);
	}
	*/
		
	ConsoleAdd("jfyeNGPrecacher (c) Arnaud Storq 1998-2003");
    ConsoleAdd("More informations available at http://norecess.planet-d.net");
    ConsoleAdd("Build date: %s, %s", __DATE__, __TIME__);
	ConsoleAdd("Use this tool to build a .PRK file and improve performances when rendering");
	ConsoleAdd("");
    ConsoleAdd("Welcome !!");
}    

/*---------------------------------------------------------------------------- GUI */
void
UserInterfaceShow(void)
{
    MSG msg;
    HICON icon;
    HINSTANCE instance;

    instance = GetModuleHandle(NULL);
    
    hwnd = CreateDialog(instance, (LPCTSTR) MAKEINTRESOURCE(IDD_GUI), NULL, (DLGPROC) &DialogProc);

    icon = LoadIcon(instance, (LPCTSTR) MAKEINTRESOURCE(IDI_JFYENG));
	SendMessage(hwnd, WM_SETICON, ICON_SMALL, (LPARAM) icon);
	SendMessage(hwnd, WM_SETICON, ICON_BIG, (LPARAM) icon);

	ShowWindow(hwnd, SW_SHOWNORMAL);
	UpdateWindow(hwnd);

    init();

    destroy = FALSE;
	
    while (!destroy)
	{
		if (PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
		{
			if (GetMessage(&msg, NULL, 0, 0))
			{
				if (TranslateAccelerator(hwnd, NULL, &msg)==0)
				{
					TranslateMessage(&msg); 
					DispatchMessage(&msg);
				}
			}
		}
	}

    DestroyWindow(hwnd);
}

INT32
UserInterfaceGetID(void)
{
    return (INT32) hwnd;
}

BOOL 
UserInterfaceIsPrecaChecked(void)
{
	return usePreca;
}

/*---------------------------------------------------------------------------- PRECA PROGRESS SET */
RESULT
UserInterfacePrecaProgressBarSet(INT32 i)
{
    if (i<0)
    {
        i = 0;
    }
    else if (i>100)
    {
        i = 100;
    }
    
    SendDlgItemMessage(hwnd, (LPARAM) MAKEINTRESOURCE(IDC_PRECA_PROGRESS_BAR), PBM_SETPOS, i, 0);
        
    UpdateWindow(hwnd);
    
    return OK;
}